@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: fdlist.h 1732 2013-01-11 12:57:19Z michael $
+ *  $Id: fdlist.h 1736 2013-01-13 09:31:46Z michael $
  */
 
 #ifndef INCLUDED_fdlist_h
@@ -92,7 +92,7 @@ typedef struct _fde {
   struct _fde *hnext;
 } fde_t;
 
-#define FD_HASH_SIZE CLIENT_HEAP_SIZE
+#define FD_HASH_SIZE 1024
 
 extern int number_fd;
 extern int hard_fdlimit;
