@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: hash.h 1169 2011-08-12 18:45:03Z michael $
+ *  $Id: hash.h 1826 2013-04-15 09:09:09Z michael $
  */
 
 #ifndef INCLUDED_hash_h
@@ -32,24 +32,21 @@
 
 struct Client;
 struct Channel;
-struct ResvChannel;
 struct UserHost;
 
-enum {
+enum
+{
   HASH_TYPE_ID,
   HASH_TYPE_CLIENT,
   HASH_TYPE_CHANNEL,
-  HASH_TYPE_USERHOST,
-  HASH_TYPE_RESERVED
+  HASH_TYPE_USERHOST
 };
 
-extern void init_hash(void);
+extern void hash_init(void);
 extern void hash_add_client(struct Client *);
 extern void hash_del_client(struct Client *);
 extern void hash_add_channel(struct Channel *);
 extern void hash_del_channel(struct Channel *);
-extern void hash_add_resv(struct ResvChannel *);
-extern void hash_del_resv(struct ResvChannel *);
 extern void hash_add_id(struct Client *);
 extern void hash_del_id(struct Client *);
 extern void hash_add_userhost(struct UserHost *);
@@ -61,7 +58,6 @@ extern struct Client *hash_find_client(const char *);
 extern struct Client *hash_find_server(const char *);
 extern struct Channel *hash_find_channel(const char *);
 extern void *hash_get_bucket(int, unsigned int);
-extern struct ResvChannel *hash_find_resv(const char *);
 
 extern void free_list_task(struct ListTask *, struct Client *);
 extern void safe_list_channels(struct Client *, struct ListTask *, int);
