@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: ircd.h 1871 2013-04-26 08:53:04Z michael $
+ *  $Id: ircd.h 1858 2013-04-25 15:00:52Z michael $
  */
 
 #ifndef INCLUDED_ircd_h
@@ -27,10 +27,7 @@
 
 #include "ircd_defs.h"
 #include "config.h"
-#include "memory.h"
-#include "list.h"
 
-struct Client;
 
 struct SetOptions
 {
@@ -71,9 +68,6 @@ struct ServerStatistics
   unsigned int    is_abad; /* bad auth requests */
 };
 
-extern struct ServerStatistics ServerStats;
-
-
 struct Counter
 {
   uint64_t totalrestartcount; /* Total client count ever */
@@ -88,15 +82,15 @@ struct Counter
   unsigned int max_loc_cli;   /* XXX This is redundant - Max local client count */
 };
 
-extern struct SetOptions GlobalSetOptions; /* defined in ircd.c */
-
 struct ServerState_t
 {
   int foreground;
 };
 
-extern struct ServerState_t server_state;
 
+#ifdef HAVE_LIBGEOIP
+extern GeoIP *geoip_ctx;
+#endif
 extern char **myargv;
 extern const char *infotext[];
 extern const char *serno;
@@ -106,6 +100,9 @@ extern const char *pidFileName;
 extern int dorehash;
 extern int doremotd;
 extern struct Counter Count;
+extern struct ServerStatistics ServerStats;
+extern struct SetOptions GlobalSetOptions; /* defined in ircd.c */
+extern struct ServerState_t server_state;
 extern struct timeval SystemTime;
 #define CurrentTime SystemTime.tv_sec
 extern int default_server_capabs;
@@ -114,11 +111,6 @@ extern unsigned int splitchecking;
 extern unsigned int split_users;
 extern unsigned int split_servers;
 
-extern dlink_list unknown_list;       /* unknown clients ON this server only        */
-extern dlink_list local_client_list;  /* local clients only ON this server          */
-extern dlink_list serv_list;          /* local servers to this server ONLY          */
-extern dlink_list global_serv_list;   /* global servers on the network              */
-extern dlink_list oper_list;          /* our opers, duplicated in local_client_list */
 extern int rehashed_klines;
 extern void set_time(void);
 #endif
