@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: listener.h 1123 2011-02-06 21:57:50Z michael $
+ *  $Id: listener.h 1798 2013-03-31 17:09:50Z michael $
  */
 
 #ifndef INCLUDED_listener_h
@@ -36,7 +36,7 @@ struct Client;
 
 struct Listener
 {
-  dlink_node	   listener_node;      /* list node pointer */
+  dlink_node	   node;      /* list node pointer */
   fde_t            fd;                 /* file descriptor */
   int              port;               /* listener IP port */
   int              ref_count;          /* number of connection references */
