@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: log.h 1762 2013-01-18 19:21:54Z michael $
+ *  $Id: log.h 2337 2013-07-03 13:00:46Z michael $
  */
 
 #ifndef INCLUDED_s_log_h
@@ -33,6 +33,8 @@ enum log_type {
   LOG_TYPE_KLINE,
   LOG_TYPE_DLINE,
   LOG_TYPE_GLINE,
+  LOG_TYPE_XLINE,
+  LOG_TYPE_RESV,
   LOG_TYPE_OPER,
   LOG_TYPE_USER,
   LOG_TYPE_DEBUG,
