@@ -19,27 +19,19 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: memory.h 1011 2009-09-18 10:14:09Z michael $
+ *  $Id: memory.h 1664 2012-11-18 14:33:47Z michael $
  */
 
 #ifndef _I_MEMORY_H
 #define _I_MEMORY_H
 
 #include "ircd_defs.h"
-#include "config.h"
 
 extern void outofmemory(void);
 
 extern void *MyMalloc(size_t);
 extern void *MyRealloc(void *, size_t);
 extern void MyFree(void *);
-extern void _DupString(char **, const char *);
-
-#define DupString(x,y) _DupString(&x, y)
-
-#ifndef NDEBUG
-extern void mem_frob(void *, int);
-#else
-#define mem_frob(x, y)
-#endif
+extern void *xstrdup(const char *);
+extern void *xstrndup(const char *, size_t);
 #endif /* _I_MEMORY_H */
