@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: packet.h 1321 2012-03-30 11:23:16Z michael $
+ *  $Id: packet.h 1798 2013-03-31 17:09:50Z michael $
  */
 
 #ifndef INCLUDED_packet_h
@@ -46,11 +46,6 @@
 #define MAX_FLOOD 5
 #define MAX_FLOOD_BURST MAX_FLOOD * 8
 
-struct Callback;
-
-void *iorecv_default(va_list);
-extern struct Callback *iorecv_cb;
-
 PF read_packet;
 PF flood_recalc;
 void flood_endgrace(struct Client *);
