@@ -19,35 +19,13 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: resv.h 1685 2012-12-19 20:24:52Z michael $
+ *  $Id: resv.h 2173 2013-06-03 19:40:02Z michael $
  */
 
 #ifndef INCLUDED_resv_h
 #define INCLUDED_resv_h
 
-struct ResvChannel
-{
-  dlink_node node;
-  struct ResvChannel *hnext;
-  time_t hold;          /* Hold action until this time (calendar time) */
-  /* +1 for \0 */
-  char name[CHANNELLEN + 1];
-  char *reason;
-  int	conf;		/* 1 if set from ircd.conf, 0 if from elsewhere */
-  int count; /* How many times this item has been matched */
-};
-
-extern dlink_list nresv_items;
-extern dlink_list resv_channel_list;
-
-extern struct ConfItem *create_channel_resv(char *, char *, int);
-extern struct ConfItem *create_nick_resv(char *, char *, int);
-
-extern int delete_channel_resv(struct ResvChannel *);
-
-extern void clear_conf_resv(void);
-extern void report_resv(struct Client *);
-
-extern int valid_wild_card_simple(const char *);
-extern struct ResvChannel *match_find_resv(const char *);
+extern struct MaskItem *create_resv(const char *, const char *, const dlink_list *);
+extern int resv_find_exempt(const struct Client *, const struct MaskItem *);
+extern struct MaskItem *match_find_resv(const char *);
 #endif  /* INCLUDED_resv_h */
