@@ -19,19 +19,17 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: rsa.h,v 7.11 2002/07/11 09:23:19 androsyn Exp $
+ *  $Id: rsa.h 1313 2012-03-27 11:09:31Z michael $
  */
 #ifndef INCLUDED_rsa_h
-#define INCLUDED_rsa_h 1
+#define INCLUDED_rsa_h
+
+#include "config.h"
 
 #ifdef HAVE_LIBCRYPTO
 extern void report_crypto_errors(void);
-extern int verify_private_key(void);
-extern int generate_challenge( char ** r_challenge, char ** r_response, RSA * key );
-extern int crypt_data(char **out, char *in, int len, char *key);
-extern int decrypt_data(char **out, char *in, int len, char *key);
-extern int get_randomness( unsigned char * buf, int length );
+extern int generate_challenge(char **, char **, RSA *);
+extern int get_randomness(unsigned char *, int);
 #endif
-
 #endif /* INCLUDED_rsa_h */
 
