@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_bsd.h 1000 2009-08-24 13:37:39Z michael $
+ *  $Id: s_bsd.h 2631 2013-12-08 18:33:35Z michael $
  */
 
 #ifndef INCLUDED_s_bsd_h
@@ -27,7 +27,6 @@
 
 #include "config.h"       
 #include "fdlist.h"
-#include "hook.h"
 
 /* Type of IO */
 #define	COMM_SELECT_READ		1
@@ -37,11 +36,9 @@
 #define SELECT_DELAY    500
 
 struct Client;
-struct AccessItem;
+struct MaskItem;
 struct Listener;
 
-extern struct Callback *setup_socket_cb;
-
 extern void add_connection(struct Listener *, struct irc_ssaddr *, int);
 extern void close_connection(struct Client *);
 extern void report_error(int, const char *, const char *, int);
@@ -62,7 +59,6 @@ extern int comm_accept(struct Listener *, struct irc_ssaddr *pn);
 /* These must be defined in the network IO loop code of your choice */
 extern void init_netio(void);
 extern void comm_setselect(fde_t *, unsigned int, PF *, void *, time_t);
-extern void init_comm(void);
 extern int read_message (time_t, unsigned char);
 extern void comm_select(void);
 extern void check_can_use_v6(void);
