@@ -19,16 +19,16 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_misc.h 1878 2013-04-27 10:38:26Z michael $
+ *  $Id: s_misc.h 1847 2013-04-23 16:42:02Z michael $
  */
 
 #ifndef INCLUDED_s_misc_h
 #define INCLUDED_s_misc_h
 
-extern char *date(time_t);
+extern const char *date(time_t);
 extern const char *smalldate(time_t);
 #ifdef HAVE_LIBCRYPTO
-extern char *ssl_get_cipher(const SSL *);
+extern const char *ssl_get_cipher(const SSL *);
 #endif
 
 /* Just blindly define our own MIN/MAX macro */
