@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: send.h 1762 2013-01-18 19:21:54Z michael $
+ *  $Id: send.h 2564 2013-11-17 18:20:52Z michael $
  */
 
 #ifndef INCLUDED_send_h
@@ -27,67 +27,19 @@
 
 #include "fdlist.h"
 
-
-/*
- * struct decls
- */
-struct Callback;
-struct Channel;
-struct Client;
-struct dlink_list;
-
-extern void *iosend_default(va_list);
-extern struct Callback *iosend_cb;
-
-/* send.c prototypes */
-
-extern void sendq_unblocked(fde_t *, struct Client *);
-extern void send_queued_write(struct Client *);
-extern void send_queued_all(void);
-extern void sendto_one(struct Client *, const char *, ...) AFP(2,3);
-extern void sendto_channel_butone(struct Client *, struct Client *,
-                                  struct Channel *, unsigned int,
-                                  const char *, ...) AFP(5,6);
-extern void sendto_common_channels_local(struct Client *, int,
-                                         const char *, ...) AFP(3,4);
-extern void sendto_channel_local(int, int, struct Channel *,
-                                 const char *, ...) AFP(4,5);
-extern void sendto_channel_local_butone(struct Client *, int, struct Channel *,
-                                        const char *, ...) AFP(4,5);
-extern void sendto_channel_remote(struct Client *, struct Client *, int,
-                                  const unsigned int, const unsigned int,
-                                  struct Channel *, const char *, ...) AFP(7,8);
-extern void sendto_server(struct Client *,
-                          const unsigned int,
-                          const unsigned int, const char *, ...) AFP(4,5);
-extern void sendto_match_butone(struct Client *, struct Client *, 
-                                char *, int, const char *, ...) AFP(5,6);
-extern void sendto_match_servs(struct Client *, const char *, int,
-                               const char *, ...) AFP(4,5);
-extern void sendto_realops_flags(unsigned int, int,
-                                 const char *, ...) AFP(3,4);
-extern void sendto_globops_flags(unsigned int, int, const char *, ...) AFP(3,4);
-extern void sendto_wallops_flags(unsigned int, struct Client *,
-                                 const char *, ...) AFP(3,4);
-extern void ts_warn(const char *, ...) AFP(1,2);
-
-extern void sendto_anywhere(struct Client *, struct Client *, 
-                            const char *, ...) AFP(3,4);
-extern void kill_client(struct Client *, struct Client *,
-                        const char *, ... ) AFP(3,4);
-extern void kill_client_ll_serv_butone(struct Client *, struct Client *,
-                                       const char *, ...) AFP(3,4);
-
-
 #define ALL_MEMBERS  0
 #define NON_CHANOPS  1
 #define ONLY_CHANOPS_VOICED  2
 #define ONLY_CHANOPS 3
 #define ONLY_SERVERS 4 /* for channel_mode.c */
 
-#define L_ALL 	0
-#define L_OPER 	1
-#define L_ADMIN	2
+#define L_ALL   0
+#define L_OPER  1
+#define L_ADMIN 2
+
+#define SEND_NOTICE 1
+#define SEND_GLOBAL 2
+#define SEND_LOCOPS 3
 
 #define NOCAPS          0               /* no caps */
 #define NOFLAGS         0               /* no flags */
@@ -96,4 +48,47 @@ extern void kill_client_ll_serv_butone(struct Client *, struct Client *,
 #define MATCH_SERVER  1
 #define MATCH_HOST    2
 
+
+/*
+ * struct decls
+ */
+struct Channel;
+struct Client;
+
+/* send.c prototypes */
+extern void sendq_unblocked(fde_t *, struct Client *);
+extern void send_queued_write(struct Client *);
+extern void send_queued_all(void);
+extern void sendto_one(struct Client *, const char *, ...) AFP(2,3);
+extern void sendto_channel_butone(struct Client *, struct Client *,
+                                  struct Channel *, unsigned int,
+                                  const char *, ...) AFP(5,6);
+extern void sendto_common_channels_local(struct Client *, int, unsigned int,
+                                         const char *, ...) AFP(4,5);
+extern void sendto_channel_local(unsigned int, int, struct Channel *,
+                                 const char *, ...) AFP(4,5);
+extern void sendto_channel_local_butone(struct Client *, unsigned int, unsigned int, struct Channel *,
+                                        const char *, ...) AFP(5,6);
+extern void sendto_channel_remote(struct Client *, struct Client *, unsigned int,
+                                  const unsigned int, const unsigned int,
+                                  struct Channel *, const char *, ...) AFP(7,8);
+extern void sendto_server(struct Client *,
+                          const unsigned int,
+                          const unsigned int, const char *, ...) AFP(4,5);
+extern void sendto_match_butone(struct Client *, struct Client *, 
+                                char *, int, const char *, ...) AFP(5,6);
+extern void sendto_match_servs(struct Client *, const char *, unsigned int,
+                               const char *, ...) AFP(4,5);
+extern void sendto_realops_flags(unsigned int, int, int,
+                                 const char *, ...) AFP(4,5);
+extern void sendto_wallops_flags(unsigned int, struct Client *,
+                                 const char *, ...) AFP(3,4);
+extern void ts_warn(const char *, ...) AFP(1,2);
+
+extern void sendto_anywhere(struct Client *, struct Client *, 
+                            const char *, ...) AFP(3,4);
+extern void kill_client(struct Client *, struct Client *,
+                        const char *, ... ) AFP(3,4);
+extern void kill_client_serv_butone(struct Client *, struct Client *,
+                                    const char *, ...) AFP(3,4);
 #endif /* INCLUDED_send_h */
