@@ -22,7 +22,7 @@
 
 /*! \file watch.h
  * \brief Header including structures and prototypes for WATCH support
- * \version $Id: watch.h 1431 2012-06-09 19:15:10Z michael $
+ * \version $Id: watch.h 1857 2013-04-24 20:47:21Z michael $
  */
 
 #ifndef INCLUDED_watch_h
@@ -41,7 +41,7 @@ struct Watch
 extern void watch_init(void);
 extern void watch_add_to_hash_table(const char *, struct Client *);
 extern void watch_del_from_hash_table(const char *, struct Client *);
-extern void watch_check_hash(struct Client *, int);
+extern void watch_check_hash(struct Client *, const unsigned int);
 extern void watch_del_watch_list(struct Client *);
 extern void watch_count_memory(unsigned int *const, uint64_t *const);
 extern struct Watch *watch_find_hash(const char *);
