@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: whowas.h 1431 2012-06-09 19:15:10Z michael $
+ *  $Id: whowas.h 2747 2014-01-05 19:19:06Z michael $
  */
 
 #ifndef INCLUDED_whowas_h
@@ -33,6 +33,7 @@
 struct Whowas
 {
   int hashv;
+  int shide;
   time_t logoff;
   char name[NICKLEN + 1];
   char username[USERLEN + 1];
@@ -50,33 +51,33 @@ struct Whowas
 extern void whowas_init(void);
 
 /*
-** add_history
+** whowas_add_history
 **      Add the currently defined name of the client to history.
 **      usually called before changing to a new name (nick).
 **      Client must be a fully registered user.
 */
-extern void add_history(struct Client *, int);
+extern void whowas_add_history(struct Client *, const int);
 
 /*
-** off_history
+** whowas_off_history
 **      This must be called when the client structure is about to
 **      be released. History mechanism keeps pointers to client
 **      structures and it must know when they cease to exist. This
 **      also implicitly calls AddHistory.
 */
-extern void off_history(struct Client *);
+extern void whowas_off_history(struct Client *);
 
 /*
-** get_history
+** whowas_get_history
 **      Return the current client that was using the given
 **      nickname within the timelimit. Returns NULL, if no
 **      one found...
 */
-extern struct Client *get_history(const char *, time_t);
+extern struct Client *whowas_get_history(const char *, time_t);
 
 /*
 ** for debugging...counts related structures stored in whowas array.
 */
-extern void count_whowas_memory(unsigned int *, uint64_t *);
+extern void whowas_count_memory(unsigned int *const, uint64_t *const);
 extern dlink_list WHOWASHASH[];
 #endif /* INCLUDED_whowas_h */
