@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_join.c 1871 2013-04-26 08:53:04Z michael $
+ *  $Id: m_join.c 2685 2013-12-16 17:37:03Z michael $
  */
 
 #include "stdinc.h"
@@ -29,7 +29,6 @@
 #include "client.h"
 #include "hash.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
@@ -98,7 +97,7 @@ m_join(struct Client *client_p, struct Client *source_p,
   char *chan_list = NULL;
   char *chan = NULL;
   struct Channel *chptr = NULL;
-  struct ResvChannel *resv_cp = NULL;
+  struct MaskItem *conf = NULL;
   int i = 0;
   unsigned int flags = 0;
 
@@ -136,14 +135,13 @@ m_join(struct Client *client_p, struct Client *source_p,
 
     if (!IsExemptResv(source_p) &&
         !(HasUMode(source_p, UMODE_OPER) && ConfigFileEntry.oper_pass_resv) &&
-        (!(resv_cp = hash_find_resv(chan)) == ConfigChannel.restrict_channels))
+        ((conf = match_find_resv(chan)) && !resv_find_exempt(source_p, conf)))
     {
-      if (resv_cp)
-        ++resv_cp->count;
-      sendto_one(source_p, form_str(ERR_BADCHANNAME),
-                 me.name, source_p->name, chan);
-      sendto_realops_flags(UMODE_SPY, L_ALL,
-                           "Forbidding reserved channel [%s] from user %s",
+      ++conf->count;
+      sendto_one(source_p, form_str(ERR_CHANBANREASON), me.name, source_p->name,
+                 chan, conf->reason ? conf->reason : "Reserved channel");
+      sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
+                           "Forbidding reserved channel %s from user %s",
                            chan, get_client_name(source_p, HIDE_IP));
       continue;
     }
@@ -234,6 +232,11 @@ m_join(struct Client *client_p, struct Client *source_p,
                            source_p->host, chptr->chname);
       sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s +nt",
                            me.name, chptr->chname);
+      if (source_p->away[0])
+        sendto_channel_local_butone(source_p, 0, CAP_AWAY_NOTIFY, chptr,
+                                    ":%s!%s@%s AWAY :%s",
+                                    source_p->name, source_p->username,
+                                    source_p->host, source_p->away);
     }
     else
     {
@@ -249,6 +252,12 @@ m_join(struct Client *client_p, struct Client *source_p,
       sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s JOIN :%s",
                            source_p->name, source_p->username,
                            source_p->host, chptr->chname);
+
+      if (source_p->away[0])
+        sendto_channel_local_butone(source_p, 0, CAP_AWAY_NOTIFY, chptr,
+                                    ":%s!%s@%s AWAY :%s",
+                                    source_p->name, source_p->username,
+                                    source_p->host, source_p->away);
     }
 
     del_invite(chptr, source_p);
@@ -306,7 +315,7 @@ ms_join(struct Client *client_p, struct Client *source_p,
 
   if (!check_channel_name(parv[2], 0))
   {
-    sendto_realops_flags(UMODE_DEBUG, L_ALL,
+    sendto_realops_flags(UMODE_DEBUG, L_ALL, SEND_NOTICE,
                          "*** Too long or invalid channel name from %s: %s",
                          client_p->name, parv[2]);
     return;
@@ -330,7 +339,7 @@ ms_join(struct Client *client_p, struct Client *source_p,
   {
     if (newts < 800000000)
     {
-      sendto_realops_flags(UMODE_DEBUG, L_ALL,
+      sendto_realops_flags(UMODE_DEBUG, L_ALL, SEND_NOTICE,
                            "*** Bogus TS %lu on %s ignored from %s",
                            (unsigned long)newts, chptr->chname,
                            client_p->name);
@@ -345,7 +354,7 @@ ms_join(struct Client *client_p, struct Client *source_p,
       sendto_channel_local(ALL_MEMBERS, 0, chptr,
                            ":%s NOTICE %s :*** Notice -- TS for %s changed from %lu to 0",
                            me.name, chptr->chname, chptr->chname, (unsigned long)oldts);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "Server %s changing TS on %s from %lu to 0",
                            source_p->name, chptr->chname, (unsigned long)oldts);
     }
@@ -416,6 +425,11 @@ ms_join(struct Client *client_p, struct Client *source_p,
     sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s JOIN :%s",
                          source_p->name, source_p->username,
                          source_p->host, chptr->chname);
+    if (source_p->away[0])
+      sendto_channel_local_butone(source_p, 0, CAP_AWAY_NOTIFY, chptr,
+                                  ":%s!%s@%s AWAY :%s",
+                                  source_p->name, source_p->username,
+                                  source_p->host, source_p->away);
   }
 
   sendto_server(client_p, CAP_TS6, NOCAPS,
@@ -526,7 +540,7 @@ set_final_mode(struct Mode *mode, struct Mode *oldmode)
       what = -1;
     }
     *mbuf++ = 'k';
-    len = ircsprintf(pbuf, "%s ", oldmode->key);
+    len = sprintf(pbuf, "%s ", oldmode->key);
     pbuf += len;
   }
 
@@ -538,7 +552,7 @@ set_final_mode(struct Mode *mode, struct Mode *oldmode)
       what = 1;
     }
     *mbuf++ = 'l';
-    len = ircsprintf(pbuf, "%d ", mode->limit);
+    len = sprintf(pbuf, "%d ", mode->limit);
     pbuf += len;
   }
 
@@ -550,7 +564,7 @@ set_final_mode(struct Mode *mode, struct Mode *oldmode)
       what = 1;
     }
     *mbuf++ = 'k';
-    len = ircsprintf(pbuf, "%s ", mode->key);
+    len = sprintf(pbuf, "%s ", mode->key);
     pbuf += len;
   }
   *mbuf = '\0';
@@ -676,7 +690,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1871 $",
+  .version = "$Revision: 2685 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
