@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_kick.c 1894 2013-04-27 14:53:18Z michael $
+ *  $Id: m_kick.c 2476 2013-10-13 18:50:07Z michael $
  */
 
 #include "stdinc.h"
@@ -85,9 +85,6 @@ m_kick(struct Client *client_p, struct Client *source_p,
     comment[KICKLEN] = '\0';
 
   name = parv[1];
-  while (*name == ',')
-    name++;
-
   if ((p = strchr(name,',')) != NULL)
     *p = '\0';
   if (*name == '\0')
@@ -155,17 +152,13 @@ m_kick(struct Client *client_p, struct Client *source_p,
   }
 
   user = parv[2];
-
-  while (*user == ',')
-    user++;
-
   if ((p = strchr(user, ',')) != NULL)
     *p = '\0';
 
   if (*user == '\0')
     return;
 
-  if ((who = find_chasing(client_p, source_p, user, &chasing)) == NULL)
+  if ((who = find_chasing(source_p, user, &chasing)) == NULL)
     return;
 
   if ((ms_target = find_channel_link(who, chptr)) != NULL)
@@ -233,7 +226,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1894 $",
+  .version = "$Revision: 2476 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
