@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_kill.c 1705 2012-12-23 14:50:38Z michael $
+ *  $Id: m_kill.c 2299 2013-06-19 12:09:18Z michael $
  */
 
 #include "stdinc.h"
@@ -34,7 +34,6 @@
 #include "send.h"
 #include "whowas.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "parse.h"
 #include "modules.h"
 
@@ -93,13 +92,6 @@ mo_kill(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  if (!HasOFlag(source_p, OPER_FLAG_GLOBAL_KILL|OPER_FLAG_K))
-  {
-    sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
-               me.name, source_p->name);
-    return;
-  }
-
   if (!EmptyString(reason))
   {
     if (strlen(reason) > (size_t)KILLLEN)
@@ -115,7 +107,7 @@ mo_kill(struct Client *client_p, struct Client *source_p,
      * rewrite the KILL for this new nickname--this keeps
      * servers in synch when nick change and kill collide
      */
-    if ((target_p = get_history(user, 
+    if ((target_p = whowas_get_history(user, 
                                 (time_t)ConfigFileEntry.kill_chase_time_limit))
                                 == NULL)
     {
@@ -128,17 +120,18 @@ mo_kill(struct Client *client_p, struct Client *source_p,
                me.name, source_p->name, user, target_p->name);
   }
 
-  if (IsServer(target_p) || IsMe(target_p))
+  if ((!MyConnect(target_p) && !HasOFlag(source_p, OPER_FLAG_KILL_REMOTE)) ||
+       (MyConnect(target_p) && !HasOFlag(source_p, OPER_FLAG_KILL)))
   {
-    sendto_one(source_p, form_str(ERR_CANTKILLSERVER),
+    sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
                me.name, source_p->name);
     return;
   }
 
-  if (!MyConnect(target_p) && !HasOFlag(source_p, OPER_FLAG_GLOBAL_KILL))
+  if (IsServer(target_p) || IsMe(target_p))
   {
-    sendto_one(source_p, ":%s NOTICE %s :Nick %s isnt on your server",
-               me.name, source_p->name, target_p->name);
+    sendto_one(source_p, form_str(ERR_CANTKILLSERVER),
+               me.name, source_p->name);
     return;
   }
 
@@ -151,9 +144,12 @@ mo_kill(struct Client *client_p, struct Client *source_p,
    * Do not change the format of this message. There's no point in changing messages
    * that have been around for ever, for no reason..
    */
-  sendto_realops_flags(UMODE_ALL, L_ALL,
-                       "Received KILL message for %s. From %s Path: %s (%s)", 
-                       target_p->name, source_p->name, me.name, reason);
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                       "Received KILL message for %s!%s@%s[%s/%s]. From %s Path: %s (%s)", 
+                       target_p->name, target_p->username, target_p->host,
+                       target_p->servptr->name,
+                       target_p->servptr->id[0] ? target_p->servptr->id : "<>",
+                       source_p->name, me.name, reason);
 
   ilog(LOG_TYPE_KILL, "KILL From %s For %s Path %s (%s)",
        source_p->name, target_p->name, me.name, reason);
@@ -231,7 +227,7 @@ ms_kill(struct Client *client_p, struct Client *source_p,
      */
     if (IsDigit(*user))   /* Somehow an uid was not found in the hash ! */
       return;
-    if ((target_p = get_history(user,
+    if ((target_p = whowas_get_history(user,
                                 (time_t)ConfigFileEntry.kill_chase_time_limit))
        == NULL)
     {
@@ -278,14 +274,20 @@ ms_kill(struct Client *client_p, struct Client *source_p,
    * local --fl
    */
   if (HasUMode(source_p, UMODE_OPER)) /* send it normally */
-    sendto_realops_flags(UMODE_ALL, L_ALL,
-                         "Received KILL message for %s. From %s Path: %s!%s!%s!%s %s",
-                         target_p->name, source_p->name, source_p->servptr->name, 
-                         source_p->host, source_p->username, source_p->name, reason);
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "Received KILL message for %s!%s@%s[%s/%s]. From %s Path: %s!%s!%s!%s %s",
+                         target_p->name, target_p->username, target_p->host,
+                         target_p->servptr->name,
+                         target_p->servptr->id[0] ? target_p->servptr->id : "<>", source_p->name,
+                         source_p->servptr->name, source_p->host, source_p->username,
+                         source_p->name, reason);
   else
-    sendto_realops_flags(UMODE_SKILL, L_ALL,
-                         "Received KILL message for %s. From %s %s",
-                         target_p->name, source_p->name, reason);
+    sendto_realops_flags(UMODE_SKILL, L_ALL, SEND_NOTICE,
+                         "Received KILL message for %s!%s@%s[%s/%s]. From %s %s",
+                         target_p->name, target_p->username, target_p->host,
+                         target_p->servptr->name,
+                         target_p->servptr->id[0] ? target_p->servptr->id : "<>",
+                         source_p->name, reason);
 
   ilog(LOG_TYPE_KILL, "KILL From %s For %s Path %s %s",
        source_p->name, target_p->name, source_p->name, reason);
@@ -323,7 +325,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1705 $",
+  .version = "$Revision: 2299 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
