@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_message.c 1717 2012-12-27 18:52:00Z michael $
+ *  $Id: m_message.c 2787 2014-01-06 22:34:10Z michael $
  */
 
 #include "stdinc.h"
@@ -39,12 +39,15 @@
 #include "packet.h"
 
 
-struct entity
+static struct entity
 {
   void *ptr;
   int type;
   int flags;
-};
+} targets[IRCD_BUFSIZE];
+
+static int ntargets = 0;
+
 
 static int build_target_list(int p_or_n, const char *command,
                              struct Client *client_p,
@@ -62,10 +65,7 @@ static struct Client* find_userhost (char *, char *, int *);
 #define ENTITY_CHANOPS_ON_CHANNEL 2
 #define ENTITY_CLIENT  3
 
-static struct entity targets[512];
-static int ntargets = 0;
-
-static int duplicate_ptr(void *);
+static int duplicate_ptr(const void *);
 
 static void m_message(int, const char *, struct Client *,
                       struct Client *, int, char **);
@@ -141,7 +141,7 @@ static void
 m_message(int p_or_n, const char *command, struct Client *client_p,
           struct Client *source_p, int parc, char *parv[])
 {
-  int i;
+  int i = 0;
 
   if (parc < 2 || EmptyString(parv[1]))
   {
@@ -169,24 +169,21 @@ m_message(int p_or_n, const char *command, struct Client *client_p,
                         parv[2]) < 0)
     return;
 
-  for (i = 0; i < ntargets; i++)
+  for (i = 0; i < ntargets; ++i)
   {
     switch (targets[i].type)
     {
       case ENTITY_CHANNEL:
-        msg_channel(p_or_n, command, client_p, source_p,
-                    (struct Channel *)targets[i].ptr, parv[2]);
+        msg_channel(p_or_n, command, client_p, source_p, targets[i].ptr, parv[2]);
         break;
 
       case ENTITY_CHANOPS_ON_CHANNEL:
         msg_channel_flags(p_or_n, command, client_p, source_p,
-                          (struct Channel *)targets[i].ptr,
-                          targets[i].flags, parv[2]);
+                          targets[i].ptr, targets[i].flags, parv[2]);
         break;
 
       case ENTITY_CLIENT:
-        msg_client(p_or_n, command, source_p,
-                   (struct Client *)targets[i].ptr, parv[2]);
+        msg_client(p_or_n, command, source_p, targets[i].ptr, parv[2]);
         break;
     }
   }
@@ -244,7 +241,7 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
 		       ConfigFileEntry.max_targets);
             return (1);
           }
-          targets[ntargets].ptr = (void *)chptr;
+          targets[ntargets].ptr = chptr;
           targets[ntargets++].type = ENTITY_CHANNEL;
         }
       }
@@ -271,7 +268,7 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
 		       ConfigFileEntry.max_targets);
 	    return (1);
 	  }
-        targets[ntargets].ptr = (void *)target_p;
+        targets[ntargets].ptr = target_p;
         targets[ntargets].type = ENTITY_CLIENT;
         targets[ntargets++].flags = 0;
       }
@@ -337,7 +334,7 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
 		       ConfigFileEntry.max_targets);
             return(1);
           }
-          targets[ntargets].ptr = (void *)chptr;
+          targets[ntargets].ptr = chptr;
           targets[ntargets].type = ENTITY_CHANOPS_ON_CHANNEL;
           targets[ntargets++].flags = type;
         }
@@ -382,17 +379,15 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
  * side effects	- NONE
  */
 static int
-duplicate_ptr(void *ptr)
+duplicate_ptr(const void *ptr)
 {
-  int i;
+  int i = 0;
 
-  for (i = 0; i < ntargets; i++)
-  {
+  for (; i < ntargets; ++i)
     if (targets[i].ptr == ptr)
-      return(1);
-  }
+      return 1;
 
-  return(0);
+  return 0;
 }
 
 /* msg_channel()
@@ -409,7 +404,7 @@ static void
 msg_channel(int p_or_n, const char *command, struct Client *client_p,
             struct Client *source_p, struct Channel *chptr, char *text)
 {
-  int result;
+  int result = 0;
 
   if (MyClient(source_p))
   {
@@ -419,7 +414,7 @@ msg_channel(int p_or_n, const char *command, struct Client *client_p,
   }
 
   /* chanops and voiced can flood their own channel with impunity */
-  if ((result = can_send(chptr, source_p, NULL)) < 0)
+  if ((result = can_send(chptr, source_p, NULL, text)) < 0)
   {
     if (result == CAN_SEND_OPV ||
         !flood_attack_channel(p_or_n, source_p, chptr))
@@ -429,9 +424,20 @@ msg_channel(int p_or_n, const char *command, struct Client *client_p,
   else
   {
     if (p_or_n != NOTICE)
-      sendto_one(source_p, form_str(ERR_CANNOTSENDTOCHAN),
-                 ID_or_name(&me, client_p),
-                 ID_or_name(source_p, client_p), chptr->chname);
+    {
+      if (result == ERR_NOCTRLSONCHAN)
+        sendto_one(source_p, form_str(ERR_NOCTRLSONCHAN),
+                   ID_or_name(&me, client_p),
+                   ID_or_name(source_p, client_p), chptr->chname, text);
+      else if (result == ERR_NEEDREGGEDNICK)
+        sendto_one(source_p, form_str(ERR_NEEDREGGEDNICK),
+                   ID_or_name(&me, client_p),
+                   ID_or_name(source_p, client_p), chptr->chname);
+      else
+        sendto_one(source_p, form_str(ERR_CANNOTSENDTOCHAN),
+                   ID_or_name(&me, client_p),
+                   ID_or_name(source_p, client_p), chptr->chname);
+    }
   }
 }
 
@@ -534,11 +540,16 @@ msg_client(int p_or_n, const char *command, struct Client *source_p,
       }
       else
       {
+        int callerid = !!HasUMode(target_p, UMODE_CALLERID);
+
         /* check for accept, flag recipient incoming message */
         if (p_or_n != NOTICE)
 	  sendto_one(source_p, form_str(RPL_TARGUMODEG),
 		     ID_or_name(&me, source_p->from),
-		     ID_or_name(source_p, source_p->from), target_p->name);
+		     ID_or_name(source_p, source_p->from), target_p->name,
+                     callerid ? "+g" : "+G",
+                     callerid ? "server side ignore" :
+                                "server side ignore with the exception of common channels");
 
         if ((target_p->localClient->last_caller_id_time +
              ConfigFileEntry.caller_id_wait) < CurrentTime)
@@ -550,7 +561,8 @@ msg_client(int p_or_n, const char *command, struct Client *source_p,
 
           sendto_one(target_p, form_str(RPL_UMODEGMSG),
                      me.name, target_p->name,
-                     get_client_name(source_p, HIDE_IP));
+                     get_client_name(source_p, HIDE_IP),
+                     callerid ? "+g" : "+G");
 
           target_p->localClient->last_caller_id_time = CurrentTime;
 
@@ -620,7 +632,7 @@ flood_attack_client(int p_or_n, struct Client *source_p,
     {
       if (!HasFlag(target_p, FLAGS_FLOOD_NOTICED))
       {
-        sendto_realops_flags(UMODE_BOTS, L_ALL,
+        sendto_realops_flags(UMODE_BOTS, L_ALL, SEND_NOTICE,
                              "Possible Flooder %s on %s target: %s",
                              get_client_name(source_p, HIDE_IP),
                              source_p->servptr->name, target_p->name);
@@ -676,7 +688,7 @@ flood_attack_channel(int p_or_n, struct Client *source_p,
     {
       if (!IsSetFloodNoticed(chptr))
       {
-        sendto_realops_flags(UMODE_BOTS, L_ALL,
+        sendto_realops_flags(UMODE_BOTS, L_ALL, SEND_NOTICE,
                              "Possible Flooder %s on %s target: %s",
                              get_client_name(source_p, HIDE_IP),
                              source_p->servptr->name, chptr->chname);
@@ -888,7 +900,7 @@ find_userhost(char *user, char *host, int *count)
       if (!IsClient(c2ptr)) /* something other than a client */
         continue;
 
-      if ((!host || match(host, c2ptr->host)) &&
+      if ((!host || !match(host, c2ptr->host)) &&
           irccmp(user, c2ptr->username) == 0)
       {
         (*count)++;
@@ -927,7 +939,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1717 $",
+  .version = "$Revision: 2787 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
