@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_mode.c 1474 2012-07-22 14:44:07Z michael $
+ *  $Id: m_mode.c 2643 2013-12-10 19:19:08Z michael $
  */
 
 #include "stdinc.h"
@@ -29,7 +29,6 @@
 #include "client.h"
 #include "hash.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "s_user.h"
@@ -51,9 +50,9 @@ m_mode(struct Client *client_p, struct Client *source_p,
        int parc, char *parv[])
 {
   struct Channel *chptr = NULL;
-  struct Membership *member;
-  static char modebuf[MODEBUFLEN];
-  static char parabuf[MODEBUFLEN];
+  struct Membership *member = NULL;
+  char modebuf[MODEBUFLEN];
+  char parabuf[MODEBUFLEN];
 
   if (EmptyString(parv[1]))
   {
@@ -87,18 +86,17 @@ m_mode(struct Client *client_p, struct Client *source_p,
                me.name, source_p->name, chptr->chname, modebuf, parabuf);
     sendto_one(source_p, form_str(RPL_CREATIONTIME),
                me.name, source_p->name, chptr->chname, chptr->channelts);
+    return;
   }
-  /* bounce all modes from people we deop on sjoin
+
+  /*
+   * bounce all modes from people we deop on sjoin
    * servers have always gotten away with murder,
    * including telnet servers *g* - Dianora
-   *
-   * XXX Is it worth the bother to make an ms_mode() ? - Dianora
    */
-  else if (IsServer(source_p))
-  {
+  if (IsServer(source_p) || HasFlag(source_p, FLAGS_SERVICE))
     set_channel_mode(client_p, source_p, chptr, NULL, parc - 2, parv + 2,
                      chptr->chname);
-  }
   else
   {
     member = find_channel_link(source_p, chptr);
@@ -107,10 +105,8 @@ m_mode(struct Client *client_p, struct Client *source_p,
     {
       /* Finish the flood grace period... */
       if (MyClient(source_p) && !IsFloodDone(source_p))
-      {
         if (!((parc == 3) && (parv[2][0] == 'b') && (parv[2][1] == '\0')))
           flood_endgrace(source_p);
-      }
 
       set_channel_mode(client_p, source_p, chptr, member, parc - 2, parv + 2,
                        chptr->chname);
@@ -142,7 +138,7 @@ ms_tmode(struct Client *client_p, struct Client *source_p, int parc, char *parv[
   if (atol(parv[1]) > chptr->channelts)
     return;
 
-  if (IsServer(source_p))
+  if (IsServer(source_p) || HasFlag(source_p, FLAGS_SERVICE))
     set_channel_mode(client_p, source_p, chptr, NULL, parc - 3, parv + 3, chptr->chname);
   else
   {
@@ -215,8 +211,9 @@ ms_bmask(struct Client *client_p, struct Client *source_p, int parc, char *parv[
   strlcpy(s, parv[4], sizeof(banbuf));
 
   /* only need to construct one buffer, for non-ts6 servers */
-  mlen = ircsprintf(modebuf, ":%s MODE %s +",
-                    source_p->name, chptr->chname);
+  mlen = snprintf(modebuf, sizeof(modebuf), ":%s MODE %s +",
+                  (IsHidden(source_p) || ConfigServerHide.hide_servers) ? me.name : source_p->name,
+                  chptr->chname);
   mbuf = modebuf + mlen;
   pbuf = parabuf;
 
@@ -249,7 +246,7 @@ ms_bmask(struct Client *client_p, struct Client *source_p, int parc, char *parv[
       }
 
       *mbuf++ = parv[3][0];
-      pbuf += ircsprintf(pbuf, "%s ", s);
+      pbuf += sprintf(pbuf, "%s ", s);
       modecount++;
     }
 
@@ -305,7 +302,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1474 $",
+  .version = "$Revision: 2643 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
