@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_part.c 1474 2012-07-22 14:44:07Z michael $
+ *  $Id: m_part.c 2006 2013-05-12 12:37:49Z michael $
  */
 
 #include "stdinc.h"
@@ -48,8 +48,7 @@
  * side effects	- remove ONE client given the channel name 
  */
 static void
-part_one_client(struct Client *client_p, struct Client *source_p,
-                const char *name, const char *reason)
+part_one_client(struct Client *source_p, const char *name, const char *reason)
 {
   struct Channel *chptr = NULL;
   struct Membership *ms = NULL;
@@ -72,18 +71,18 @@ part_one_client(struct Client *client_p, struct Client *source_p,
     check_spambot_warning(source_p, NULL);
 
   /*
-   *  Remove user from the old channel (if any)
-   *  only allow /part reasons in -m chans
+   * Remove user from the old channel (if any)
+   * only allow /part reasons in -m chans
    */
-  if (reason[0] && (!MyConnect(source_p) ||
-      ((can_send(chptr, source_p, ms) &&
+  if (*reason && (!MyConnect(source_p) ||
+      ((can_send(chptr, source_p, ms, reason) &&
        (source_p->localClient->firsttime + ConfigFileEntry.anti_spam_exit_message_time)
         < CurrentTime))))
   {
-    sendto_server(client_p, CAP_TS6, NOCAPS,
+    sendto_server(source_p->from, CAP_TS6, NOCAPS,
                   ":%s PART %s :%s", ID(source_p), chptr->chname,
                   reason);
-    sendto_server(client_p, NOCAPS, CAP_TS6,
+    sendto_server(source_p->from, NOCAPS, CAP_TS6,
                   ":%s PART %s :%s", source_p->name, chptr->chname,
                   reason);
     sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s PART %s :%s",
@@ -92,9 +91,9 @@ part_one_client(struct Client *client_p, struct Client *source_p,
   }
   else
   {
-    sendto_server(client_p, CAP_TS6, NOCAPS,
+    sendto_server(source_p->from, CAP_TS6, NOCAPS,
                   ":%s PART %s", ID(source_p), chptr->chname);
-    sendto_server(client_p, NOCAPS, CAP_TS6,
+    sendto_server(source_p->from, NOCAPS, CAP_TS6,
                   ":%s PART %s", source_p->name, chptr->chname);
     sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s PART %s",
                          source_p->name, source_p->username,
@@ -127,7 +126,7 @@ m_part(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  if (parc > 2)
+  if (parc > 2 && !EmptyString(parv[2]))
     strlcpy(reason, parv[2], sizeof(reason));
 
   /* Finish the flood grace period... */
@@ -136,7 +135,7 @@ m_part(struct Client *client_p, struct Client *source_p,
 
   for (name = strtoken(&p, parv[1], ","); name;
        name = strtoken(&p,    NULL, ","))
-    part_one_client(client_p, source_p, name, reason);
+    part_one_client(source_p, name, reason);
 }
 
 static struct Message part_msgtab = {
@@ -159,7 +158,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1474 $",
+  .version = "$Revision: 2006 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
