@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_server.c 1569 2012-10-16 18:46:53Z michael $
+ *  $Id: m_server.c 2434 2013-08-02 18:43:30Z michael $
  */
 
 #include "stdinc.h"
@@ -30,7 +30,7 @@
 #include "irc_string.h" 
 #include "ircd.h"        /* me */
 #include "numeric.h"     /* ERR_xxx */
-#include "conf.h"      /* struct AccessItem */
+#include "conf.h"      /* struct MaskItem */
 #include "log.h"       /* log level defines */
 #include "s_serv.h"      /* server_estab, check_server */
 #include "s_user.h"
@@ -39,7 +39,29 @@
 #include "modules.h"
 
 
-static void set_server_gecos(struct Client *, const char *);
+/* set_server_gecos()
+ *
+ * input        - pointer to client
+ * output       - NONE
+ * side effects - servers gecos field is set
+ */
+static void
+set_server_gecos(struct Client *client_p, const char *info)
+{
+  const char *s = info;
+
+  /* check for (H) which is a hidden server */
+  if (!strncmp(s, "(H) ", 4))
+  {
+    SetHidden(client_p);
+    s = s + 4;
+  }
+
+  if (!EmptyString(s))
+    strlcpy(client_p->info, s, sizeof(client_p->info));
+  else
+    strlcpy(client_p->info, "(Unknown Location)", sizeof(client_p->info));
+}
 
 /* mr_server()
  *  parv[0] = sender prefix
@@ -70,10 +92,10 @@ mr_server(struct Client *client_p, struct Client *source_p,
    */
   if (!DoesTS(client_p))
   {
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
           "Unauthorized server connection attempt from %s: Non-TS server "
           "for server %s", get_client_name(client_p, HIDE_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
           "Unauthorized server connection attempt from %s: Non-TS server "
           "for server %s", get_client_name(client_p, MASK_IP), name);
     exit_client(client_p, client_p, "Non-TS server");
@@ -82,10 +104,10 @@ mr_server(struct Client *client_p, struct Client *source_p,
 
   if (!valid_servname(name))
   {
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
           "Unauthorized server connection attempt from %s: Bogus server name "
           "for server %s", get_client_name(client_p, HIDE_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
           "Unauthorized server connection attempt from %s: Bogus server name "
           "for server %s", get_client_name(client_p, MASK_IP), name);
     exit_client(client_p, client_p, "Bogus server name");
@@ -100,11 +122,11 @@ mr_server(struct Client *client_p, struct Client *source_p,
     case -1:
       if (ConfigFileEntry.warn_no_nline)
       {
-        sendto_realops_flags(UMODE_ALL, L_ADMIN,
+        sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: No entry for "
            "servername %s", get_client_name(client_p, HIDE_IP), name);
 
-        sendto_realops_flags(UMODE_ALL, L_OPER,
+        sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: No entry for "
            "servername %s", get_client_name(client_p, MASK_IP), name);
       }
@@ -115,11 +137,11 @@ mr_server(struct Client *client_p, struct Client *source_p,
       break;
 
     case -2:
-      sendto_realops_flags(UMODE_ALL, L_ADMIN,
+      sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: Bad password "
            "for server %s", get_client_name(client_p, HIDE_IP), name);
 
-      sendto_realops_flags(UMODE_ALL, L_OPER,
+      sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: Bad password "
            "for server %s", get_client_name(client_p, MASK_IP), name);
 
@@ -129,16 +151,27 @@ mr_server(struct Client *client_p, struct Client *source_p,
       break;
 
     case -3:
-      sendto_realops_flags(UMODE_ALL, L_ADMIN,
+      sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: Invalid host "
            "for server %s", get_client_name(client_p, HIDE_IP), name);
 
-      sendto_realops_flags(UMODE_ALL, L_OPER,
+      sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: Invalid host "
            "for server %s", get_client_name(client_p, MASK_IP), name);
 
       exit_client(client_p, client_p, "Invalid host.");
       return;
+    case -4:
+      sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
+           "Unauthorized server connection attempt from %s: Invalid certificate fingerprint "
+           "for server %s", get_client_name(client_p, HIDE_IP), name);
+
+      sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
+           "Unauthorized server connection attempt from %s: Invalid certificate fingerprint "
+           "for server %s", get_client_name(client_p, MASK_IP), name);
+
+      exit_client(client_p, client_p, "Invalid certificate fingerprint.");
+      return;
       /* NOT REACHED */
       break;
   }
@@ -156,11 +189,11 @@ mr_server(struct Client *client_p, struct Client *source_p,
      * Definitely don't do that here. This is from an unregistered
      * connect - A1kmm.
      */
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
 			 "Attempt to re-introduce server %s SID %s from %s",
                          name, client_p->id, 
 			 get_client_name(client_p, HIDE_IP));
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
 			 "Attempt to re-introduce server %s SID %s from %s",
                          name, client_p->id,
 			 get_client_name(client_p, MASK_IP));
@@ -198,7 +231,7 @@ ms_server(struct Client *client_p, struct Client *source_p,
 {
   char *name;
   struct Client *target_p;
-  struct AccessItem *aconf;
+  struct MaskItem *conf = NULL;
   int hop;
   int hlined = 0;
   int llined = 0;
@@ -219,10 +252,10 @@ ms_server(struct Client *client_p, struct Client *source_p,
 
   if (!valid_servname(name))
   {
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
                          "Link %s introduced server with bogus server name %s",
                          get_client_name(client_p, SHOW_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                          "Link %s introduced server with bogus server name %s",
                          get_client_name(client_p, MASK_IP), name);
     sendto_one(client_p, "ERROR :Bogus server name introduced");
@@ -250,10 +283,10 @@ ms_server(struct Client *client_p, struct Client *source_p,
      * solution.. --fl_ 
      */
     sendto_one(client_p, "ERROR :Server %s already exists", name);
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
 			 "Link %s cancelled, server %s already exists",
                          get_client_name(client_p, SHOW_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
 			 "Link %s cancelled, server %s already exists",
                          client_p->name, name);
     exit_client(client_p, &me, "Server Exists");
@@ -268,20 +301,20 @@ ms_server(struct Client *client_p, struct Client *source_p,
     if (target_p != client_p)
       exit_client(target_p, &me, "Overridden");
 
-  aconf = map_to_conf(client_p->localClient->confs.head->data);
+  conf = client_p->localClient->confs.head->data;
 
   /* See if the newly found server is behind a guaranteed
    * leaf. If so, close the link.
    */
-  DLINK_FOREACH(ptr, aconf->leaf_list.head)
-    if (match(ptr->data, name))
+  DLINK_FOREACH(ptr, conf->leaf_list.head)
+    if (!match(ptr->data, name))
     {
       llined = 1;
       break;
     }
 
-  DLINK_FOREACH(ptr, aconf->hub_list.head)
-    if (match(ptr->data, name))
+  DLINK_FOREACH(ptr, conf->hub_list.head)
+    if (!match(ptr->data, name))
     {
       hlined = 1;
       break;
@@ -314,9 +347,11 @@ ms_server(struct Client *client_p, struct Client *source_p,
   if (!hlined)
   {
     /* OOOPs nope can't HUB */
-    sendto_realops_flags(UMODE_ALL, L_ADMIN, "Non-Hub link %s introduced %s.",
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
+                         "Non-Hub link %s introduced %s.",
                          get_client_name(client_p, HIDE_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER,  "Non-Hub link %s introduced %s.",
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
+                         "Non-Hub link %s introduced %s.",
                          get_client_name(client_p, MASK_IP), name);
     exit_client(source_p, &me, "No matching hub_mask.");
     return;
@@ -326,10 +361,10 @@ ms_server(struct Client *client_p, struct Client *source_p,
   if (llined)
   {
     /* OOOPs nope can't HUB this leaf */
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
 			 "Link %s introduced leafed server %s.",
                          get_client_name(client_p, HIDE_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
 			 "Link %s introduced leafed server %s.",
                          get_client_name(client_p, MASK_IP), name);
     /* If it is new, we are probably misconfigured, so split the
@@ -353,7 +388,7 @@ ms_server(struct Client *client_p, struct Client *source_p,
   set_server_gecos(target_p, parv[3]);
   SetServer(target_p);
 
-  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(SERVICE_TYPE, target_p->name, NULL, NULL, 0))
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(CONF_SERVICE, target_p->name, NULL, NULL, 0))
     AddFlag(target_p, FLAGS_SERVICE);
 
   dlinkAdd(target_p, &target_p->node, &global_client_list);
@@ -366,7 +401,7 @@ ms_server(struct Client *client_p, struct Client *source_p,
                 source_p->name, target_p->name, hop + 1,
                 IsHidden(target_p) ? "(H) " : "", target_p->info);
 
-  sendto_realops_flags(UMODE_EXTERNAL, L_ALL,
+  sendto_realops_flags(UMODE_EXTERNAL, L_ALL, SEND_NOTICE,
                        "Server %s being introduced by %s",
                        target_p->name, source_p->name);
 }
@@ -383,7 +418,7 @@ ms_sid(struct Client *client_p, struct Client *source_p,
        int parc, char *parv[])
 {
   struct Client *target_p;
-  struct AccessItem *aconf = NULL;
+  struct MaskItem *conf = NULL;
   int hlined = 0;
   int llined = 0;
   dlink_node *ptr = NULL;
@@ -403,10 +438,10 @@ ms_sid(struct Client *client_p, struct Client *source_p,
 
   if (!valid_servname(parv[1]))
   {
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
                          "Link %s introduced server with bogus server name %s",
                          get_client_name(client_p, SHOW_IP), parv[1]);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                          "Link %s introduced server with bogus server name %s",
                          get_client_name(client_p, MASK_IP), parv[1]);
     sendto_one(client_p, "ERROR :Bogus server name introduced");
@@ -416,10 +451,10 @@ ms_sid(struct Client *client_p, struct Client *source_p,
 
   if (!valid_sid(parv[3]))
   {
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
                          "Link %s introduced server with bogus server ID %s",
                          get_client_name(client_p, SHOW_IP), parv[3]);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                          "Link %s introduced server with bogus server ID %s",
                          get_client_name(client_p, MASK_IP), parv[3]);
     sendto_one(client_p, "ERROR :Bogus server ID introduced");
@@ -431,10 +466,10 @@ ms_sid(struct Client *client_p, struct Client *source_p,
   if ((target_p = hash_find_id(parv[3])))
   {
     sendto_one(client_p, "ERROR :SID %s already exists", parv[3]);
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
 			 "Link %s cancelled, SID %s already exists",
                          get_client_name(client_p, SHOW_IP), parv[3]);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
 			 "Link %s cancelled, SID %s already exists",
                          client_p->name, parv[3]);
     exit_client(client_p, &me, "SID Exists");
@@ -445,10 +480,10 @@ ms_sid(struct Client *client_p, struct Client *source_p,
   if ((target_p = hash_find_server(parv[1])))
   {
     sendto_one(client_p, "ERROR :Server %s already exists", parv[1]);
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
                          "Link %s cancelled, server %s already exists",   
                          get_client_name(client_p, SHOW_IP), parv[1]);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                          "Link %s cancelled, server %s already exists",   
                          client_p->name, parv[1]);
     exit_client(client_p, &me, "Server Exists");
@@ -463,20 +498,20 @@ ms_sid(struct Client *client_p, struct Client *source_p,
     if (target_p != client_p)
       exit_client(target_p, &me, "Overridden");
 
-  aconf = map_to_conf(client_p->localClient->confs.head->data);
+  conf = client_p->localClient->confs.head->data;
 
   /* See if the newly found server is behind a guaranteed
    * leaf. If so, close the link.
    */
-  DLINK_FOREACH(ptr, aconf->leaf_list.head)
-    if (match(ptr->data, parv[1]))
+  DLINK_FOREACH(ptr, conf->leaf_list.head)
+    if (!match(ptr->data, parv[1]))
     {
       llined = 1;
       break;
     }
 
-  DLINK_FOREACH(ptr, aconf->hub_list.head)
-    if (match(ptr->data, parv[1]))
+  DLINK_FOREACH(ptr, conf->hub_list.head)
+    if (!match(ptr->data, parv[1]))
     {
       hlined = 1;
       break;
@@ -510,9 +545,11 @@ ms_sid(struct Client *client_p, struct Client *source_p,
   if (!hlined)
   {
     /* OOOPs nope can't HUB */
-    sendto_realops_flags(UMODE_ALL, L_ADMIN, "Non-Hub link %s introduced %s.",
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
+                         "Non-Hub link %s introduced %s.",
                          get_client_name(client_p, SHOW_IP), parv[1]);
-    sendto_realops_flags(UMODE_ALL, L_OPER,  "Non-Hub link %s introduced %s.",
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
+                         "Non-Hub link %s introduced %s.",
                          get_client_name(client_p, MASK_IP), parv[1]);
     exit_client(source_p, &me, "No matching hub_mask.");
     return;
@@ -522,10 +559,10 @@ ms_sid(struct Client *client_p, struct Client *source_p,
   if (llined)
   {
     /* OOOPs nope can't HUB this leaf */
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
 			 "Link %s introduced leafed server %s.",
                          get_client_name(client_p, SHOW_IP), parv[1]);
-    sendto_realops_flags(UMODE_ALL, L_OPER,  
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
 			 "Link %s introduced leafed server %s.",
                          get_client_name(client_p, MASK_IP), parv[1]);
     exit_client(client_p, &me, "Leafed Server.");
@@ -543,7 +580,7 @@ ms_sid(struct Client *client_p, struct Client *source_p,
   set_server_gecos(target_p, parv[4]);
   SetServer(target_p);
 
-  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(SERVICE_TYPE, target_p->name, NULL, NULL, 0))
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(CONF_SERVICE, target_p->name, NULL, NULL, 0))
     AddFlag(target_p, FLAGS_SERVICE);
 
   dlinkAdd(target_p, &target_p->node, &global_client_list);
@@ -560,35 +597,11 @@ ms_sid(struct Client *client_p, struct Client *source_p,
                 source_p->name, target_p->name, hop + 1,
                 IsHidden(target_p) ? "(H) " : "", target_p->info);
 
-  sendto_realops_flags(UMODE_EXTERNAL, L_ALL, 
+  sendto_realops_flags(UMODE_EXTERNAL, L_ALL, SEND_NOTICE,
                        "Server %s being introduced by %s",
                        target_p->name, source_p->name);
 }
 
-/* set_server_gecos()
- *
- * input	- pointer to client
- * output	- NONE
- * side effects - servers gecos field is set
- */
-static void
-set_server_gecos(struct Client *client_p, const char *info)
-{
-  const char *s = info;
-
-  /* check for (H) which is a hidden server */
-  if (!strncmp(s, "(H) ", 4))
-  {
-    SetHidden(client_p);
-    s = s + 4;
-  }
-
-  if (!EmptyString(s))
-    strlcpy(client_p->info, s, sizeof(client_p->info));
-  else
-    strlcpy(client_p->info, "(Unknown Location)", sizeof(client_p->info));
-}
-
 static struct Message server_msgtab = {
   "SERVER", 0, 0, 4, MAXPARA, MFLG_SLOW, 0,
   {mr_server, m_registered, ms_server, m_ignore, m_registered, m_ignore}
@@ -596,7 +609,7 @@ static struct Message server_msgtab = {
 
 static struct Message sid_msgtab = {
   "SID", 0, 0, 5, MAXPARA, MFLG_SLOW, 0,
-  {rfc1459_command_send_error, m_ignore, ms_sid, m_ignore, m_ignore, m_ignore}
+  {m_ignore, m_ignore, ms_sid, m_ignore, m_ignore, m_ignore}
 };
 
 static void
@@ -616,7 +629,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1569 $",
+  .version = "$Revision: 2434 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
