@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_sjoin.c 1750 2013-01-16 14:07:08Z michael $
+ *  $Id: m_sjoin.c 2476 2013-10-13 18:50:07Z michael $
  */
 
 #include "stdinc.h"
@@ -29,7 +29,6 @@
 #include "client.h"
 #include "hash.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
@@ -104,7 +103,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
 
   if (!check_channel_name(parv[2], 0))
   {
-    sendto_realops_flags(UMODE_DEBUG, L_ALL,
+    sendto_realops_flags(UMODE_DEBUG, L_ALL, SEND_NOTICE,
                          "*** Too long or invalid channel name from %s: %s",
                          client_p->name, parv[2]);
     return;
@@ -123,6 +122,9 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   {
     switch (*s)
     {
+      case 'c':
+        mode.mode |= MODE_NOCTRL;
+        break;
       case 't':
         mode.mode |= MODE_TOPICLIMIT;
         break;
@@ -144,6 +146,9 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
       case 'r':
         mode.mode |= MODE_REGISTERED;
         break;
+      case 'M':
+        mode.mode |= MODE_MODREG;
+        break;
       case 'O':
         mode.mode |= MODE_OPERONLY;
         break;
@@ -181,7 +186,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   {
     if (newts < 800000000)
     {
-      sendto_realops_flags(UMODE_DEBUG, L_ALL,
+      sendto_realops_flags(UMODE_DEBUG, L_ALL, SEND_NOTICE,
 			   "*** Bogus TS %lu on %s ignored from %s",
 			   (unsigned long)newts, chptr->chname,
 			   client_p->name);
@@ -196,7 +201,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
       sendto_channel_local(ALL_MEMBERS, 0, chptr,
  		  	   ":%s NOTICE %s :*** Notice -- TS for %s changed from %lu to 0",
 		  	   me.name, chptr->chname, chptr->chname, (unsigned long)oldts);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
 		           "Server %s changing TS on %s from %lu to 0",
 			   source_p->name, chptr->chname, (unsigned long)oldts);
     }
@@ -270,14 +275,14 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
     modebuf[1] = '\0';
   }
 
-  buflen = ircsprintf(nick_buf, ":%s SJOIN %lu %s %s %s:",
-		      source_p->name, (unsigned long)tstosend,
-		      chptr->chname, modebuf, parabuf);
+  buflen = snprintf(nick_buf, sizeof(nick_buf), ":%s SJOIN %lu %s %s %s:",
+                    source_p->name, (unsigned long)tstosend,
+                    chptr->chname, modebuf, parabuf);
   nick_ptr = nick_buf + buflen;
 
-  buflen = ircsprintf(uid_buf, ":%s SJOIN %lu %s %s %s:",
-                      ID(source_p), (unsigned long)tstosend,
-                      chptr->chname, modebuf, parabuf);
+  buflen = snprintf(uid_buf, sizeof(uid_buf), ":%s SJOIN %lu %s %s %s:",
+                    ID(source_p), (unsigned long)tstosend,
+                    chptr->chname, modebuf, parabuf);
   uid_ptr = uid_buf + buflen;
 
   /* check we can fit a nick on the end, as well as \r\n and a prefix "
@@ -285,7 +290,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
    */
   if (buflen >= (IRCD_BUFSIZE - IRCD_MAX(NICKLEN, IDLEN) - 2 - 3 - 1))
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
 			 "Long SJOIN from server: %s(via %s) (ignored)",
 			 source_p->name, client_p->name);
     return;
@@ -337,7 +342,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
       }
     } while (valid_mode);
 
-    target_p = find_chasing(client_p, source_p, s, NULL);
+    target_p = find_chasing(source_p, s, NULL);
 
     /*
      * if the client doesnt exist, or if its fake direction/server, skip.
@@ -397,25 +402,25 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
     {
       sendto_server(client_p, 0, CAP_TS6, "%s", nick_buf);
       
-      buflen = ircsprintf(nick_buf, ":%s SJOIN %lu %s %s %s:",
-                          source_p->name, (unsigned long)tstosend,
-                          chptr->chname, modebuf, parabuf);
+      buflen = snprintf(nick_buf, sizeof(nick_buf), ":%s SJOIN %lu %s %s %s:",
+                        source_p->name, (unsigned long)tstosend,
+                        chptr->chname, modebuf, parabuf);
       nick_ptr = nick_buf + buflen;
     }
 
-    nick_ptr += ircsprintf(nick_ptr, "%s%s ", nick_prefix, target_p->name);
+    nick_ptr += sprintf(nick_ptr, "%s%s ", nick_prefix, target_p->name);
     
     if ((uid_ptr - uid_buf + len_uid) > (IRCD_BUFSIZE - 2))
     {
       sendto_server(client_p, CAP_TS6, 0, "%s", uid_buf);
       
-      buflen = ircsprintf(uid_buf, ":%s SJOIN %lu %s %s %s:",
-                          ID(source_p), (unsigned long)tstosend,
-                          chptr->chname, modebuf, parabuf);
+      buflen = snprintf(uid_buf, sizeof(uid_buf), ":%s SJOIN %lu %s %s %s:",
+                        ID(source_p), (unsigned long)tstosend,
+                        chptr->chname, modebuf, parabuf);
       uid_ptr = uid_buf + buflen;
     }
 
-    uid_ptr  += ircsprintf(uid_ptr,  "%s%s ", uid_prefix, ID(target_p));
+    uid_ptr += sprintf(uid_ptr,  "%s%s ", uid_prefix, ID(target_p));
 	
     if (!IsMember(target_p, chptr))
     {
@@ -423,6 +428,11 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
       sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s JOIN :%s",
                            target_p->name, target_p->username,
                            target_p->host, chptr->chname);
+      if (target_p->away[0])
+        sendto_channel_local_butone(target_p, 0, CAP_AWAY_NOTIFY, chptr,
+                                    ":%s!%s@%s AWAY :%s",
+                                    target_p->name, target_p->username,
+                                    target_p->host, target_p->away);
     }
 
     if (fl & CHFL_CHANOP)
@@ -445,7 +455,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
         *mbuf = '\0';
         for(lcount = 0; lcount < MAXMODEPARAMS; lcount++)
         {
-          slen = ircsprintf(sptr, " %s", para[lcount]);	/* see? */
+          slen = sprintf(sptr, " %s", para[lcount]);	/* see? */
 	  sptr += slen;					/* ready for next */
         }
         sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
@@ -469,7 +479,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
         *mbuf = '\0';
         for(lcount = 0; lcount < MAXMODEPARAMS; lcount++)
         {
-          slen = ircsprintf(sptr, " %s", para[lcount]);
+          slen = sprintf(sptr, " %s", para[lcount]);
           sptr += slen;
         }
         sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
@@ -494,7 +504,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
         *mbuf = '\0';
         for (lcount = 0; lcount < MAXMODEPARAMS; lcount++)
         {
-          slen = ircsprintf(sptr, " %s", para[lcount]);
+          slen = sprintf(sptr, " %s", para[lcount]);
 	  sptr += slen;
         }
         sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
@@ -539,7 +549,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
 
     for (lcount = 0; lcount < pargs; lcount++)
     {
-      slen = ircsprintf(sptr, " %s", para[lcount]);
+      slen = sprintf(sptr, " %s", para[lcount]);
       sptr += slen;
     }
 
@@ -626,7 +636,7 @@ set_final_mode(struct Mode *mode, struct Mode *oldmode)
   if (oldmode->key[0] && !mode->key[0])
   {
     *mbuf++ = 'k';
-    len = ircsprintf(pbuf, "%s ", oldmode->key);
+    len = sprintf(pbuf, "%s ", oldmode->key);
     pbuf += len;
     pargs++;
   }
@@ -646,7 +656,7 @@ set_final_mode(struct Mode *mode, struct Mode *oldmode)
   if (mode->limit != 0 && oldmode->limit != mode->limit)
   {
     *mbuf++ = 'l';
-    len = ircsprintf(pbuf, "%d ", mode->limit);
+    len = sprintf(pbuf, "%d ", mode->limit);
     pbuf += len;
     pargs++;
   }
@@ -654,7 +664,7 @@ set_final_mode(struct Mode *mode, struct Mode *oldmode)
   if (mode->key[0] && strcmp(oldmode->key, mode->key))
   {
     *mbuf++ = 'k';
-    len = ircsprintf(pbuf, "%s ", mode->key);
+    len = sprintf(pbuf, "%s ", mode->key);
     pbuf += len;
     pargs++;
   }
@@ -725,7 +735,7 @@ remove_a_mode(struct Channel *chptr, struct Client *source_p,
     {
       for(i = 0; i < MAXMODEPARAMS; i++)
       {
-        l = ircsprintf(sp, " %s", lpara[i]);
+        l = sprintf(sp, " %s", lpara[i]);
 	sp += l;
       }
 
@@ -749,7 +759,7 @@ remove_a_mode(struct Channel *chptr, struct Client *source_p,
     *mbuf = '\0';
     for(i = 0; i < count; i++)
     {
-      l = ircsprintf(sp, " %s", lpara[i]);
+      l = sprintf(sp, " %s", lpara[i]);
       sp += l;
     }
     sendto_channel_local(ALL_MEMBERS, 0, chptr,
@@ -782,8 +792,8 @@ remove_ban_list(struct Channel *chptr, struct Client *source_p,
 
   pbuf = lparabuf;
   
-  cur_len = mlen = ircsprintf(lmodebuf, ":%s MODE %s -",
-                              source_p->name, chptr->chname);
+  cur_len = mlen = sprintf(lmodebuf, ":%s MODE %s -",
+                           source_p->name, chptr->chname);
   mbuf = lmodebuf + mlen;
 
   DLINK_FOREACH_SAFE(ptr, next_ptr, list->head)
@@ -809,8 +819,8 @@ remove_ban_list(struct Channel *chptr, struct Client *source_p,
 
     *mbuf++ = c;
     cur_len += plen;
-    pbuf += ircsprintf(pbuf, "%s!%s@%s ", banptr->name, banptr->username,
-		       banptr->host);
+    pbuf += sprintf(pbuf, "%s!%s@%s ", banptr->name, banptr->user,
+                    banptr->host);
     ++count;
 
     remove_ban(banptr, list);
@@ -842,7 +852,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1750 $",
+  .version = "$Revision: 2476 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
