@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_squit.c 1705 2012-12-23 14:50:38Z michael $
+ *  $Id: m_squit.c 2013 2013-05-12 14:51:41Z michael $
  */
 
 #include "stdinc.h"
@@ -71,7 +71,7 @@ mo_squit(struct Client *client_p, struct Client *source_p,
 
     if (IsServer(p) || IsMe(p))
     {
-      if (match(server, p->name))
+      if (!match(server, p->name))
       {
         target_p = p;
         break;
@@ -86,7 +86,8 @@ mo_squit(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  if (!MyConnect(target_p) && !HasOFlag(source_p, OPER_FLAG_REMOTE))
+  if ((!MyConnect(target_p) && !HasOFlag(source_p, OPER_FLAG_SQUIT_REMOTE)) ||
+      (MyConnect(target_p) && !HasOFlag(source_p, OPER_FLAG_SQUIT)))
   {
     sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
                me.name, source_p->name);
@@ -100,7 +101,8 @@ mo_squit(struct Client *client_p, struct Client *source_p,
 
   if (MyConnect(target_p))
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL, "Received SQUIT %s from %s (%s)",
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "Received SQUIT %s from %s (%s)",
                          target_p->name, get_client_name(source_p, HIDE_IP), comment);
     ilog(LOG_TYPE_IRCD, "Received SQUIT %s from %s (%s)",
          target_p->name, get_client_name(source_p, HIDE_IP), comment);
@@ -176,7 +178,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1705 $",
+  .version = "$Revision: 2013 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
