@@ -21,13 +21,12 @@
 
 /*! \file m_accept.c
  * \brief Includes required functions for processing the ACCEPT command.
- * \version $Id: m_accept.c 1309 2012-03-25 11:24:18Z michael $
+ * \version $Id: m_accept.c 2364 2013-07-04 21:29:43Z michael $
  */
 
 #include "stdinc.h"
 #include "client.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
 #include "list.h"
 #include "numeric.h"
@@ -36,6 +35,7 @@
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
+#include "memory.h"
 
 
 /*! \brief Creates and sends a list of nick!user\@host masks a Client
@@ -68,9 +68,9 @@ list_accepts(struct Client *source_p)
       t = nicks;
     }
 
-    t += ircsprintf(t, "%s!%s@%s ",
-                    accept_p->nickptr,
-                    accept_p->userptr, accept_p->hostptr);
+    t += sprintf(t, "%s!%s@%s ",
+                 accept_p->nickptr,
+                 accept_p->userptr, accept_p->hostptr);
   }
 
   if (nicks[0] != '\0')
@@ -95,9 +95,9 @@ add_accept(const struct split_nuh_item *nuh, struct Client *source_p)
 {
   struct split_nuh_item *accept_p = MyMalloc(sizeof(*accept_p));
 
-  DupString(accept_p->nickptr, nuh->nickptr);
-  DupString(accept_p->userptr, nuh->userptr);
-  DupString(accept_p->hostptr, nuh->hostptr);
+  accept_p->nickptr = xstrdup(nuh->nickptr);
+  accept_p->userptr = xstrdup(nuh->userptr);
+  accept_p->hostptr = xstrdup(nuh->hostptr);
 
   dlinkAdd(accept_p, &accept_p->node, &source_p->localClient->acceptlist);
 
@@ -151,7 +151,7 @@ m_accept(struct Client *client_p, struct Client *source_p,
 
       split_nuh(&nuh);
 
-      if ((accept_p = find_accept(nick, user, host, source_p, 0)) == NULL)
+      if ((accept_p = find_accept(nick, user, host, source_p, irccmp)) == NULL)
       {
         sendto_one(source_p, form_str(ERR_ACCEPTNOT),
                    me.name, source_p->name, nick, user, host);
@@ -181,7 +181,7 @@ m_accept(struct Client *client_p, struct Client *source_p,
 
       split_nuh(&nuh);
 
-      if ((accept_p = find_accept(nick, user, host, source_p, 0)) != NULL)
+      if ((accept_p = find_accept(nick, user, host, source_p, irccmp)) != NULL)
       {
         sendto_one(source_p, form_str(ERR_ACCEPTEXIST),
                    me.name, source_p->name, nick, user, host);
@@ -213,7 +213,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1309 $",
+  .version = "$Revision: 2364 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
