@@ -21,7 +21,7 @@
 
 /*! \file m_admin.c
  * \brief Includes required functions for processing the ADMIN command.
- * \version $Id: m_admin.c 1569 2012-10-16 18:46:53Z michael $
+ * \version $Id: m_admin.c 2342 2013-07-03 13:59:41Z michael $
  */
 
 #include "stdinc.h"
@@ -47,7 +47,7 @@ do_admin(struct Client *source_p)
   const char *me_name = ID_or_name(&me, source_p->from);
   const char *nick = ID_or_name(source_p, source_p->from);
 
-  sendto_realops_flags(UMODE_SPY, L_ALL,
+  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
                        "ADMIN requested by %s (%s@%s) [%s]",
                        source_p->name, source_p->username,
                        source_p->host, source_p->servptr->name);
@@ -95,7 +95,7 @@ m_admin(struct Client *client_p, struct Client *source_p,
 
   last_used = CurrentTime;
 
-  if (!ConfigFileEntry.disable_remote)
+  if (!ConfigServerHide.disable_remote_commands)
     if (hunt_server(client_p, source_p, ":%s ADMIN :%s", 1,
                     parc, parv) != HUNTED_ISME)
       return;
@@ -125,8 +125,7 @@ ms_admin(struct Client *client_p, struct Client *source_p,
                   parc, parv) != HUNTED_ISME)
     return;
 
-  if (IsClient(source_p))
-    do_admin(source_p);
+  do_admin(source_p);
 }
 
 static struct Message admin_msgtab = {
@@ -149,7 +148,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1569 $",
+  .version = "$Revision: 2342 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
