@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_challenge.c 1638 2012-11-04 17:30:49Z michael $
+ *  $Id: m_challenge.c 2249 2013-06-16 11:30:30Z michael $
  */
 
 #include "stdinc.h"
@@ -34,6 +34,7 @@
 #include "irc_string.h"
 #include "log.h"
 #include "s_user.h"
+#include "memory.h"
 
 
 #ifdef HAVE_LIBCRYPTO
@@ -50,7 +51,8 @@ failed_challenge_notice(struct Client *source_p, const char *name,
                         const char *reason)
 {
   if (ConfigFileEntry.failed_oper_notice)
-    sendto_realops_flags(UMODE_ALL, L_ALL, "Failed CHALLENGE attempt as %s "
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "Failed CHALLENGE attempt as %s "
                          "by %s (%s@%s) - %s", name, source_p->name,
                          source_p->username, source_p->host, reason);
 
@@ -70,8 +72,7 @@ m_challenge(struct Client *client_p, struct Client *source_p,
             int parc, char *parv[])
 {
   char *challenge = NULL;
-  struct ConfItem *conf = NULL;
-  struct AccessItem *aconf = NULL;
+  struct MaskItem *conf = NULL;
 
   if (*parv[1] == '+')
   {
@@ -88,28 +89,29 @@ m_challenge(struct Client *client_p, struct Client *source_p,
       return;
     }
     
-    conf = find_exact_name_conf(OPER_TYPE, source_p,
+    conf = find_exact_name_conf(CONF_OPER, source_p,
                                 source_p->localClient->auth_oper, NULL, NULL);
     if (conf == NULL)
     {
       /* XXX: logging */
-      sendto_one (source_p, form_str(ERR_NOOPERHOST), me.name, source_p->name);
+      sendto_one(source_p, form_str(ERR_NOOPERHOST), me.name, source_p->name);
       return;
     }
 
     if (attach_conf(source_p, conf) != 0)
     {
       sendto_one(source_p,":%s NOTICE %s :Can't attach conf!",
-		 me.name, source_p->name);   
+                 me.name, source_p->name);   
       failed_challenge_notice(source_p, conf->name, "can't attach conf!");
       return;
     }
 
+    ++conf->count;
     oper_up(source_p);
 
     ilog(LOG_TYPE_OPER, "OPER %s by %s!%s@%s",
-	 source_p->localClient->auth_oper, source_p->name, source_p->username,
-	 source_p->host);
+         source_p->localClient->auth_oper, source_p->name, source_p->username,
+         source_p->host);
 
     MyFree(source_p->localClient->response);
     MyFree(source_p->localClient->auth_oper);
@@ -123,34 +125,48 @@ m_challenge(struct Client *client_p, struct Client *source_p,
   source_p->localClient->response  = NULL;
   source_p->localClient->auth_oper = NULL;
 
+  conf = find_exact_name_conf(CONF_OPER, source_p, parv[1], NULL, NULL);
 
-  conf = find_exact_name_conf(OPER_TYPE, source_p, parv[1], NULL, NULL);
-  if (conf)
-    aconf = map_to_conf(conf);
-
-  if (aconf == NULL)
+  if (!conf)
   {
-    sendto_one (source_p, form_str(ERR_NOOPERHOST), me.name, source_p->name);
-    conf = find_exact_name_conf(OPER_TYPE, NULL, parv[1], NULL, NULL);
+    sendto_one(source_p, form_str(ERR_NOOPERHOST), me.name, source_p->name);
+    conf = find_exact_name_conf(CONF_OPER, NULL, parv[1], NULL, NULL);
     failed_challenge_notice(source_p, parv[1], (conf != NULL)
                             ? "host mismatch" : "no oper {} block");
     return;
   }
 
-  if (aconf->rsa_public_key == NULL)
+  if (conf->rsa_public_key == NULL)
   {
-    sendto_one (source_p, ":%s NOTICE %s :I'm sorry, PK authentication "
-		"is not enabled for your oper{} block.", me.name,
-		source_p->name);
+    sendto_one(source_p, ":%s NOTICE %s :I'm sorry, PK authentication "
+               "is not enabled for your oper{} block.", me.name,
+               source_p->name);
     return;
   }
 
+  if (IsConfSSL(conf) && !HasUMode(source_p, UMODE_SSL))
+  {
+    sendto_one(source_p, form_str(ERR_NOOPERHOST), me.name, source_p->name);
+    failed_challenge_notice(source_p, conf->name, "requires SSL/TLS");
+    return;
+  }
+
+  if (!EmptyString(conf->certfp))
+  {
+    if (EmptyString(source_p->certfp) || strcasecmp(source_p->certfp, conf->certfp))
+    {
+      sendto_one(source_p, form_str(ERR_NOOPERHOST), me.name, source_p->name);
+      failed_challenge_notice(source_p, conf->name, "client certificate fingerprint mismatch");
+      return;
+    }
+  }
+
   if (!generate_challenge(&challenge, &(source_p->localClient->response),
-                          aconf->rsa_public_key))
+                          conf->rsa_public_key))
     sendto_one(source_p, form_str(RPL_RSACHALLENGE),
                me.name, source_p->name, challenge);
 
-  DupString(source_p->localClient->auth_oper, conf->name);
+  source_p->localClient->auth_oper = xstrdup(conf->name);
   MyFree(challenge);
 }
 
@@ -195,7 +211,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1638 $",
+  .version = "$Revision: 2249 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
