@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_connect.c 1474 2012-07-22 14:44:07Z michael $
+ *  $Id: m_connect.c 2343 2013-07-03 18:13:17Z michael $
  */
 
 #include "stdinc.h"
@@ -55,8 +55,7 @@ mo_connect(struct Client *client_p, struct Client *source_p,
 {
   int port;
   int tmpport;
-  struct ConfItem *conf = NULL;
-  struct AccessItem *aconf = NULL;
+  struct MaskItem *conf = NULL;
   const struct Client *target_p = NULL;
 
   if (EmptyString(parv[1]))
@@ -68,7 +67,7 @@ mo_connect(struct Client *client_p, struct Client *source_p,
 
   if (parc > 3)
   {
-    if (!HasOFlag(source_p, OPER_FLAG_REMOTE))
+    if (!HasOFlag(source_p, OPER_FLAG_CONNECT_REMOTE))
     {
       sendto_one(source_p, form_str(ERR_NOPRIVS),
                  me.name, source_p->name, "connect");
@@ -79,11 +78,17 @@ mo_connect(struct Client *client_p, struct Client *source_p,
                     parc, parv) != HUNTED_ISME)
       return;
   }
+  else if (!HasOFlag(source_p, OPER_FLAG_CONNECT))
+  {
+    sendto_one(source_p, form_str(ERR_NOPRIVS),
+               me.name, source_p->name, "connect");
+    return;
+  }
 
   if ((target_p = hash_find_server(parv[1])))
   {
     sendto_one(source_p,
-	       ":%s NOTICE %s :Connect: Server %s already exists from %s.",
+               ":%s NOTICE %s :Connect: Server %s already exists from %s.",
                me.name, source_p->name, parv[1], target_p->from->name);
     return;
   }
@@ -91,26 +96,23 @@ mo_connect(struct Client *client_p, struct Client *source_p,
   /*
    * try to find the name, then host, if both fail notify ops and bail
    */
-  if ((conf = find_matching_name_conf(SERVER_TYPE,
-				      parv[1], NULL, NULL, 0)) != NULL)
-    aconf = (struct AccessItem *)map_to_conf(conf);
-  else if ((conf = find_matching_name_conf(SERVER_TYPE,
-					   NULL, NULL, parv[1], 0)) != NULL)
-    aconf = (struct AccessItem *)map_to_conf(conf);
-  
-  if (conf == NULL || aconf == NULL)
+  if (!(conf = find_matching_name_conf(CONF_SERVER, parv[1], NULL, NULL, 0)))
   {
-    sendto_one(source_p,
-	       ":%s NOTICE %s :Connect: Host %s not listed in ircd.conf",
-	       me.name, source_p->name, parv[1]);
-    return;
+    if  (!(conf = find_matching_name_conf(CONF_SERVER,  NULL, NULL, parv[1], 0)))
+    {
+      sendto_one(source_p,
+                 ":%s NOTICE %s :Connect: Host %s not listed in ircd.conf",
+                 me.name, source_p->name, parv[1]);
+      return;
+    }
   }
 
-  /* Get port number from user, if given. If not specified,
+  /*
+   * Get port number from user, if given. If not specified,
    * use the default form configuration structure. If missing
    * from there, then use the precompiled default.
    */
-  tmpport = port = aconf->port;
+  tmpport = port = conf->port;
 
   if (parc > 2 && !EmptyString(parv[2]))
   {
@@ -141,31 +143,33 @@ mo_connect(struct Client *client_p, struct Client *source_p,
   ilog(LOG_TYPE_IRCD, "CONNECT From %s : %s %s", 
        source_p->name, parv[1], parv[2] ? parv[2] : "");
 
-  aconf->port = port;
+  conf->port = port;
 
-  /* at this point we should be calling connect_server with a valid
+  /*
+   * at this point we should be calling connect_server with a valid
    * C:line and a valid port in the C:line
    */
-  if (serv_connect(aconf, source_p))
+  if (serv_connect(conf, source_p))
   {
     if (!ConfigServerHide.hide_server_ips && HasUMode(source_p, UMODE_ADMIN))
       sendto_one(source_p, ":%s NOTICE %s :*** Connecting to %s[%s].%d",
-                 me.name, source_p->name, aconf->host,
-                 conf->name, aconf->port);
+                 me.name, source_p->name, conf->host,
+                 conf->name, conf->port);
     else
       sendto_one(source_p, ":%s NOTICE %s :*** Connecting to %s.%d",
-                 me.name, source_p->name, conf->name, aconf->port);
+                 me.name, source_p->name, conf->name, conf->port);
   }
   else
   {
     sendto_one(source_p, ":%s NOTICE %s :*** Couldn't connect to %s.%d",
-               me.name, source_p->name, conf->name, aconf->port);
+               me.name, source_p->name, conf->name, conf->port);
   }
 
-  /* client is either connecting with all the data it needs or has been
+  /*
+   * Client is either connecting with all the data it needs or has been
    * destroyed
    */
-  aconf->port = tmpport;
+  conf->port = tmpport;
 }
 
 /*
@@ -185,12 +189,11 @@ ms_connect(struct Client *client_p, struct Client *source_p,
 {
   int port;
   int tmpport;
-  struct ConfItem *conf = NULL;
-  struct AccessItem *aconf = NULL;
+  struct MaskItem *conf = NULL;
   const struct Client *target_p = NULL;
 
-  if (hunt_server(client_p, source_p,
-                  ":%s CONNECT %s %s :%s", 3, parc, parv) != HUNTED_ISME)
+  if (hunt_server(client_p, source_p, ":%s CONNECT %s %s :%s",
+                  3, parc, parv) != HUNTED_ISME)
     return;
 
   if (EmptyString(parv[1]))
@@ -203,7 +206,7 @@ ms_connect(struct Client *client_p, struct Client *source_p,
   if ((target_p = hash_find_server(parv[1])))
   {
     sendto_one(source_p,
-	       ":%s NOTICE %s :Connect: Server %s already exists from %s.",
+               ":%s NOTICE %s :Connect: Server %s already exists from %s.",
                me.name, source_p->name, parv[1], target_p->from->name);
     return;
   }
@@ -211,34 +214,33 @@ ms_connect(struct Client *client_p, struct Client *source_p,
   /*
    * try to find the name, then host, if both fail notify ops and bail
    */
-  if ((conf = find_matching_name_conf(SERVER_TYPE,
-				      parv[1], NULL, NULL, 0)) != NULL)
-    aconf = map_to_conf(conf);
-  else if ((conf = find_matching_name_conf(SERVER_TYPE,
-					   NULL, NULL, parv[1], 0)) != NULL)
-    aconf = map_to_conf(conf);
-
-  if (conf == NULL || aconf == NULL)
-  {
-    sendto_one(source_p,
-	       ":%s NOTICE %s :Connect: Host %s not listed in ircd.conf",
-	       me.name, source_p->name, parv[1]);
-    return;
+  if (!(conf = find_matching_name_conf(CONF_SERVER, parv[1], NULL, NULL, 0)))
+  { 
+    if  (!(conf = find_matching_name_conf(CONF_SERVER,  NULL, NULL, parv[1], 0)))
+    {
+      sendto_one(source_p,
+                 ":%s NOTICE %s :Connect: Host %s not listed in ircd.conf",
+                 me.name, source_p->name, parv[1]);
+      return;
+    }
   }
 
-  /* Get port number from user, if given. If not specified,
+  /*
+   * Get port number from user, if given. If not specified,
    * use the default form configuration structure. If missing
    * from there, then use the precompiled default.
    */
-  tmpport = port = aconf->port;
+  tmpport = port = conf->port;
 
   if (parc > 2 && !EmptyString(parv[2]))
   {
     port = atoi(parv[2]);
 
-    /* if someone sends port 0, and we have a config port.. use it */
-    if (port == 0 && aconf->port)
-      port = aconf->port;
+    /*
+     * if someone sends port 0, and we have a config port.. use it
+     */
+    if (port == 0 && conf->port)
+      port = conf->port;
     else if (port <= 0)
     {
       sendto_one(source_p, ":%s NOTICE %s :Connect: Illegal port number",
@@ -275,23 +277,23 @@ ms_connect(struct Client *client_p, struct Client *source_p,
   ilog(LOG_TYPE_IRCD, "CONNECT From %s : %s %d", 
        source_p->name, parv[1], port);
 
-  aconf->port = port;
+  conf->port = port;
 
   /*
    * At this point we should be calling connect_server with a valid
    * C:line and a valid port in the C:line
    */
-  if (serv_connect(aconf, source_p))
+  if (serv_connect(conf, source_p))
     sendto_one(source_p, ":%s NOTICE %s :*** Connecting to %s.%d",
-               me.name, source_p->name, conf->name, aconf->port);
+               me.name, source_p->name, conf->name, conf->port);
   else
     sendto_one(source_p, ":%s NOTICE %s :*** Couldn't connect to %s.%d",
-               me.name, source_p->name, conf->name, aconf->port);
+               me.name, source_p->name, conf->name, conf->port);
   /*
    * Client is either connecting with all the data it needs or has been
    * destroyed
    */
-  aconf->port = tmpport;
+  conf->port = tmpport;
 }
 
 static struct Message connect_msgtab = {
@@ -314,7 +316,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1474 $",
+  .version = "$Revision: 2343 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
