@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_dline.c 1871 2013-04-26 08:53:04Z michael $
+ *  $Id: m_dline.c 2476 2013-10-13 18:50:07Z michael $
  */
 
 #include "stdinc.h"
@@ -27,21 +27,21 @@
 #include "channel.h"
 #include "client.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
+#include "conf.h"
 #include "ircd.h"
 #include "hostmask.h"
 #include "numeric.h"
 #include "fdlist.h"
 #include "s_bsd.h"
-#include "conf.h"
 #include "log.h"
 #include "s_misc.h"
 #include "send.h"
 #include "hash.h"
 #include "s_serv.h"
-#include "s_gline.h"
 #include "parse.h"
 #include "modules.h"
+#include "conf_db.h"
+#include "memory.h"
 
 
 /* apply_tdline()
@@ -51,28 +51,38 @@
  * side effects	- tkline as given is placed
  */
 static void
-apply_tdline(struct Client *source_p, struct ConfItem *conf,
-	     const char *current_date, int tkline_time)
+apply_dline(struct Client *source_p, struct MaskItem *conf,
+            time_t tkline_time)
 {
-  struct AccessItem *aconf;
+  if (tkline_time)
+  {
+    conf->until = CurrentTime + tkline_time;
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "%s added temporary %d min. D-Line for [%s] [%s]",
+                         get_oper_name(source_p), tkline_time/60,
+                         conf->host, conf->reason);
+    sendto_one(source_p, ":%s NOTICE %s :Added temporary %d min. D-Line [%s]",
+               MyConnect(source_p) ? me.name : ID_or_name(&me, source_p->from),
+               source_p->name, tkline_time/60, conf->host);
+    ilog(LOG_TYPE_DLINE, "%s added temporary %d min. D-Line for [%s] [%s]",
+         get_oper_name(source_p), tkline_time/60, conf->host, conf->reason);
+  }
+  else
+  {
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "%s added D-Line for [%s] [%s]",
+                         get_oper_name(source_p), conf->host, conf->reason);
+    sendto_one(source_p, ":%s NOTICE %s :Added D-Line [%s]",
+               MyConnect(source_p) ? me.name : ID_or_name(&me, source_p->from),
+               source_p->name, conf->host);
+    ilog(LOG_TYPE_DLINE, "%s added D-Line for [%s] [%s]",
+         get_oper_name(source_p), conf->host, conf->reason);
 
-  aconf = map_to_conf(conf);
-  aconf->hold = CurrentTime + tkline_time;
-  SetConfTemporary(aconf);
-  add_conf_by_address(CONF_DLINE, aconf);
-
-
-  sendto_realops_flags(UMODE_ALL, L_ALL,
-		       "%s added temporary %d min. D-Line for [%s] [%s]",
-		       get_oper_name(source_p), tkline_time/60,
-		       aconf->host, aconf->reason);
-
-  sendto_one(source_p, ":%s NOTICE %s :Added temporary %d min. D-Line [%s]",
-	     MyConnect(source_p) ? me.name : ID_or_name(&me, source_p->from),
-             source_p->name, tkline_time/60, aconf->host);
-  ilog(LOG_TYPE_DLINE, "%s added temporary %d min. D-Line for [%s] [%s]",
-       source_p->name, tkline_time/60, aconf->host, aconf->reason);
+  }
 
+  SetConfDatabase(conf);
+  conf->setat = CurrentTime;
+  add_conf_by_address(CONF_DLINE, conf);
   rehashed_klines = 1;
 }
 
@@ -82,33 +92,31 @@ apply_tdline(struct Client *source_p, struct ConfItem *conf,
  * Side effects: Any matching tdlines are removed.
  */
 static int
-remove_tdline_match(const char *host)
+remove_dline_match(const char *host)
 {
   struct irc_ssaddr iphost, *piphost;
-  struct AccessItem *aconf;
-  int t;
+  struct MaskItem *conf;
+  int t = 0;
+  int aftype = 0;
 
   if ((t = parse_netmask(host, &iphost, NULL)) != HM_HOST)
   {
 #ifdef IPV6
     if (t == HM_IPV6)
-      t = AF_INET6;
+      aftype = AF_INET6;
     else
 #endif
-      t = AF_INET;
+      aftype = AF_INET;
     piphost = &iphost;
   }
   else
-  {
-    t = 0;
     piphost = NULL;
-  }
 
-  if ((aconf = find_conf_by_address(host, piphost, CONF_DLINE | 1, t, NULL, NULL, 0)))
+  if ((conf = find_conf_by_address(host, piphost, CONF_DLINE | 1, aftype, NULL, NULL, 0)))
   {
-    if (IsConfTemporary(aconf))
+    if (IsConfDatabase(conf))
     {
-      delete_one_address_conf(host, aconf);
+      delete_one_address_conf(host, conf);
       return 1;
     }
   }
@@ -131,15 +139,14 @@ mo_dline(struct Client *client_p, struct Client *source_p,
          int parc, char *parv[])
 {
   char def_reason[] = CONF_NOREASON;
-  char *dlhost = NULL, *oper_reason = NULL, *reason = NULL;
+  char *dlhost = NULL, *reason = NULL;
   char *target_server = NULL;
   const char *creason;
   const struct Client *target_p = NULL;
   struct irc_ssaddr daddr;
-  struct ConfItem *conf=NULL;
-  struct AccessItem *aconf=NULL;
+  struct MaskItem *conf=NULL;
   time_t tkline_time=0;
-  int bits, t;
+  int bits = 0, aftype = 0, t = 0;
   const char *current_date = NULL;
   time_t cur_time;
   char hostip[HOSTIPLEN + 1];
@@ -176,7 +183,7 @@ mo_dline(struct Client *client_p, struct Client *source_p,
                     dlhost, reason);
 
     /* Allow ON to apply local kline as well if it matches */
-    if (!match(target_server, me.name))
+    if (match(target_server, me.name))
       return;
   }
   else
@@ -185,7 +192,7 @@ mo_dline(struct Client *client_p, struct Client *source_p,
 
   if ((t = parse_netmask(dlhost, NULL, &bits)) == HM_HOST)
   {
-    if ((target_p = find_chasing(client_p, source_p, dlhost, NULL)) == NULL)
+    if ((target_p = find_chasing(source_p, dlhost, NULL)) == NULL)
       return;
 
     if (!MyConnect(target_p))
@@ -222,60 +229,44 @@ mo_dline(struct Client *client_p, struct Client *source_p,
 
 #ifdef IPV6
   if (t == HM_IPV6)
-    t = AF_INET6;
+    aftype = AF_INET6;
   else
 #endif
-    t = AF_INET;
+    aftype = AF_INET;
 
   parse_netmask(dlhost, &daddr, NULL);
 
-  if ((aconf = find_dline_conf(&daddr, t)) != NULL)
+  if ((conf = find_dline_conf(&daddr, aftype)) != NULL)
   {
-    creason = aconf->reason ? aconf->reason : def_reason;
-    if (IsConfExemptKline(aconf))
+    creason = conf->reason ? conf->reason : def_reason;
+    if (IsConfExemptKline(conf))
       sendto_one(source_p,
 		 ":%s NOTICE %s :[%s] is (E)d-lined by [%s] - %s",
-		 me.name, source_p->name, dlhost, aconf->host, creason);
+		 me.name, source_p->name, dlhost, conf->host, creason);
     else
       sendto_one(source_p,
 		 ":%s NOTICE %s :[%s] already D-lined by [%s] - %s",
-		 me.name, source_p->name, dlhost, aconf->host, creason);
+		 me.name, source_p->name, dlhost, conf->host, creason);
     return;
   }
 
   cur_time = CurrentTime;
   current_date = smalldate(cur_time);
 
-  /* Look for an oper reason */
-  if ((oper_reason = strchr(reason, '|')) != NULL)
-    *oper_reason++ = '\0';
-
   if (!valid_comment(source_p, reason, 1))
     return;
 
-  conf = make_conf_item(DLINE_TYPE);
-  aconf = map_to_conf(conf);
-  DupString(aconf->host, dlhost);
+  conf = conf_make(CONF_DLINE);
+  conf->host = xstrdup(dlhost);
 
   if (tkline_time != 0)
-  {
     snprintf(buffer, sizeof(buffer), "Temporary D-line %d min. - %s (%s)",
              (int)(tkline_time/60), reason, current_date);
-    DupString(aconf->reason, buffer);
-    if (oper_reason != NULL)
-      DupString(aconf->oper_reason, oper_reason);
-    apply_tdline(source_p, conf, current_date, tkline_time);
-  }
   else
-  {
     snprintf(buffer, sizeof(buffer), "%s (%s)", reason, current_date);
-    DupString(aconf->reason, buffer);
-    if (oper_reason != NULL)
-      DupString(aconf->oper_reason, oper_reason);
-    add_conf_by_address(CONF_DLINE, aconf);
-    write_conf_line(source_p, conf, current_date, cur_time);
-  }
 
+  conf->reason = xstrdup(buffer);
+  apply_dline(source_p, conf, tkline_time);
   rehashed_klines = 1;
 }
 
@@ -284,14 +275,13 @@ ms_dline(struct Client *client_p, struct Client *source_p,
          int parc, char *parv[])
 {
   char def_reason[] = CONF_NOREASON;
-  char *dlhost, *oper_reason, *reason;
+  char *dlhost, *reason;
   const char *creason;
   const struct Client *target_p = NULL;
   struct irc_ssaddr daddr;
-  struct ConfItem *conf=NULL;
-  struct AccessItem *aconf=NULL;
+  struct MaskItem *conf=NULL;
   time_t tkline_time=0;
-  int bits, t;
+  int bits = 0, aftype = 0, t = 0;
   const char *current_date = NULL;
   time_t cur_time;
   char hostip[HOSTIPLEN + 1];
@@ -306,14 +296,14 @@ ms_dline(struct Client *client_p, struct Client *source_p,
                      "DLINE %s %s %s :%s",
                      parv[1], parv[2], parv[3], parv[4]);
 
-  if (!match(parv[1], me.name))
+  if (match(parv[1], me.name))
     return;
 
   tkline_time = valid_tkline(parv[2], TK_SECONDS);
   dlhost = parv[3];
   reason = parv[4];
 
-  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(ULINE_TYPE, source_p->servptr->name,
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(CONF_ULINE, source_p->servptr->name,
                               source_p->username, source_p->host,
                               SHARED_DLINE))
   {
@@ -321,7 +311,7 @@ ms_dline(struct Client *client_p, struct Client *source_p,
       return;
     if ((t = parse_netmask(dlhost, NULL, &bits)) == HM_HOST)
     {
-      if ((target_p = find_chasing(client_p, source_p, dlhost, NULL)) == NULL)
+      if ((target_p = find_chasing(source_p, dlhost, NULL)) == NULL)
         return;
 
       if (!MyConnect(target_p))
@@ -358,60 +348,44 @@ ms_dline(struct Client *client_p, struct Client *source_p,
 
 #ifdef IPV6
     if (t == HM_IPV6)
-      t = AF_INET6;
+      aftype= AF_INET6;
     else
 #endif
-      t = AF_INET;
+      aftype = AF_INET;
 
     parse_netmask(dlhost, &daddr, NULL);
 
-    if ((aconf = find_dline_conf(&daddr, t)) != NULL)
+    if ((conf = find_dline_conf(&daddr, aftype)) != NULL)
     {
-      creason = aconf->reason ? aconf->reason : def_reason;
-      if (IsConfExemptKline(aconf))
+      creason = conf->reason ? conf->reason : def_reason;
+      if (IsConfExemptKline(conf))
         sendto_one(source_p,
                    ":%s NOTICE %s :[%s] is (E)d-lined by [%s] - %s",
-                   me.name, source_p->name, dlhost, aconf->host, creason);
+                   me.name, source_p->name, dlhost, conf->host, creason);
       else
         sendto_one(source_p,
                    ":%s NOTICE %s :[%s] already D-lined by [%s] - %s",
-                   me.name, source_p->name, dlhost, aconf->host, creason);
+                   me.name, source_p->name, dlhost, conf->host, creason);
       return;
     }
 
     cur_time = CurrentTime;
     current_date = smalldate(cur_time);
 
-    /* Look for an oper reason */
-    if ((oper_reason = strchr(reason, '|')) != NULL)
-      *oper_reason++ = '\0';
-
     if (!valid_comment(source_p, reason, 1))
       return;
 
-    conf = make_conf_item(DLINE_TYPE);
-    aconf = map_to_conf(conf);
-    DupString(aconf->host, dlhost);
+    conf = conf_make(CONF_DLINE);
+    conf->host = xstrdup(dlhost);
 
     if (tkline_time != 0)
-    {
       snprintf(buffer, sizeof(buffer), "Temporary D-line %d min. - %s (%s)",
                (int)(tkline_time/60), reason, current_date);
-      DupString(aconf->reason, buffer);
-      if (oper_reason != NULL)
-        DupString(aconf->oper_reason, oper_reason);
-      apply_tdline(source_p, conf, current_date, tkline_time);
-    }
     else
-    {
       snprintf(buffer, sizeof(buffer), "%s (%s)", reason, current_date);
-      DupString(aconf->reason, buffer);
-      if (oper_reason != NULL)
-        DupString(aconf->oper_reason, oper_reason);
-      add_conf_by_address(CONF_DLINE, aconf);
-      write_conf_line(source_p, conf, current_date, cur_time);
-    }
 
+    conf->reason = xstrdup(buffer);
+    apply_dline(source_p, conf, tkline_time);
     rehashed_klines = 1;
   }
 }
@@ -456,33 +430,21 @@ mo_undline(struct Client *client_p, struct Client *source_p,
                        "UNDLINE %s %s", target_server, addr);
 
     /* Allow ON to apply local unkline as well if it matches */
-    if (!match(target_server, me.name))
+    if (match(target_server, me.name))
       return;
   }
   else
     cluster_a_line(source_p, "UNDLINE", CAP_UNDLN, SHARED_UNDLINE,
                    "%s", addr);
 
-  if (remove_tdline_match(addr))
+  if (remove_dline_match(addr))
   {
     sendto_one(source_p,
-               ":%s NOTICE %s :Un-Dlined [%s] from temporary D-Lines",
+               ":%s NOTICE %s :D-Line for [%s] is removed",
                me.name, source_p->name, addr);
-    sendto_realops_flags(UMODE_ALL, L_ALL,
-                         "%s has removed the temporary D-Line for: [%s]",
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "%s has removed the D-Line for: [%s]",
                          get_oper_name(source_p), addr);
-    ilog(LOG_TYPE_DLINE, "%s removed temporary D-Line for [%s]",
-         source_p->name, addr);
-    return;
-  }
-
-  if (remove_conf_line(DLINE_TYPE, source_p, addr, NULL) > 0)
-  {
-    sendto_one(source_p, ":%s NOTICE %s :D-Line for [%s] is removed",
-               me.name, source_p->name, addr);
-    sendto_realops_flags(UMODE_ALL, L_ALL,
-			 "%s has removed the D-Line for: [%s]",
-			 get_oper_name(source_p), addr);
     ilog(LOG_TYPE_DLINE, "%s removed D-Line for [%s]",
          get_oper_name(source_p), addr);
   }
@@ -502,32 +464,20 @@ me_undline(struct Client *client_p, struct Client *source_p,
 
   addr = parv[2];
 
-  if (!IsClient(source_p) || !match(parv[1], me.name))
+  if (!IsClient(source_p) || match(parv[1], me.name))
     return;
 
-  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(ULINE_TYPE,
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(CONF_ULINE,
                                    source_p->servptr->name,
                                    source_p->username, source_p->host,
                                    SHARED_UNDLINE))
   {
-    if (remove_tdline_match(addr))
+    if (remove_dline_match(addr))
     {
       sendto_one(source_p,
-                 ":%s NOTICE %s :Un-Dlined [%s] from temporary D-Lines",
+                 ":%s NOTICE %s :D-Line for [%s] is removed",
                  me.name, source_p->name, addr);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
-                           "%s has removed the temporary D-Line for: [%s]",
-                           get_oper_name(source_p), addr);
-      ilog(LOG_TYPE_DLINE, "%s removed temporary D-Line for [%s]",
-           source_p->name, addr);
-      return;
-    }
-
-    if (remove_conf_line(DLINE_TYPE, source_p, addr, NULL) > 0)
-    {
-      sendto_one(source_p, ":%s NOTICE %s :D-Line for [%s] is removed",
-                 me.name, source_p->name, addr);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "%s has removed the D-Line for: [%s]",
                            get_oper_name(source_p), addr);
       ilog(LOG_TYPE_DLINE, "%s removed D-Line for [%s]",
@@ -584,7 +534,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1871 $",
+  .version = "$Revision: 2476 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
