@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_eob.c 1569 2012-10-16 18:46:53Z michael $
+ *  $Id: m_eob.c 1985 2013-05-09 12:20:30Z michael $
  */
 
 #include "stdinc.h"
@@ -28,6 +28,7 @@
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
+#include "s_serv.h"
 
 
 /*
@@ -40,12 +41,19 @@ ms_eob(struct Client *client_p, struct Client *source_p,
        int parc, char *parv[])
 {
   assert(IsServer(source_p));
-  assert(client_p == source_p);
 
-  sendto_realops_flags(UMODE_ALL, L_ALL, "End of burst from %s (%u seconds)",
-                       source_p->name,
-                       (unsigned int)(CurrentTime - source_p->localClient->firsttime));
+  if (client_p == source_p)
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "End of burst from %s (%u seconds)",
+                         source_p->name,
+                         (unsigned int)(CurrentTime - source_p->localClient->firsttime));
+
   AddFlag(source_p, FLAGS_EOB);
+
+  sendto_server(client_p, CAP_TS6, NOCAPS,
+                ":%s EOB", ID(source_p));
+  sendto_server(client_p, NOCAPS, CAP_TS6,
+                ":%s EOB", source_p->name);
 }
 
 static struct Message eob_msgtab = {
@@ -68,7 +76,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1569 $",
+  .version = "$Revision: 1985 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
