@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_etrace.c 1400 2012-05-07 20:08:32Z michael $
+ *  $Id: m_etrace.c 2188 2013-06-04 12:44:37Z michael $
  */
 
 #include "stdinc.h"
@@ -36,9 +36,10 @@
 #include "parse.h"
 #include "modules.h"
 #include "conf.h"
+#include "conf_class.h"
 
 
-static void report_this_status(struct Client *, struct Client *, int);
+static void report_this_status(struct Client *, struct Client *);
 
 /*
  * do_etrace()
@@ -50,24 +51,13 @@ do_etrace(struct Client *source_p, int parc, char *parv[])
   struct Client *target_p = NULL;
   int wilds = 0;
   int do_all = 0;
-  int full_etrace = 0;
   dlink_node *ptr;
 
-  sendto_realops_flags(UMODE_SPY, L_ALL,
+  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
                        "ETRACE requested by %s (%s@%s) [%s]",
                        source_p->name, source_p->username,
                        source_p->host, source_p->servptr->name);
 
-  if (parc > 1)
-  {
-    if (irccmp(parv[1], "-full") == 0)
-    {
-      ++parv;
-      --parc;
-      full_etrace = 1;
-    }
-  }
-
   if (parc > 1)
   {
     tname = parv[1];
@@ -83,15 +73,12 @@ do_etrace(struct Client *source_p, int parc, char *parv[])
     tname = "*";
   }
 
-  if (HasUMode(source_p, UMODE_CCONN_FULL))
-    full_etrace = 1;
-
   if (!wilds && !do_all)
   {
     target_p = hash_find_client(tname);
 
     if (target_p && MyClient(target_p))
-      report_this_status(source_p, target_p, full_etrace);
+      report_this_status(source_p, target_p);
       
     sendto_one(source_p, form_str(RPL_ENDOFTRACE), me.name, 
                source_p->name, tname);
@@ -104,11 +91,11 @@ do_etrace(struct Client *source_p, int parc, char *parv[])
 
     if (wilds)
     {
-      if (match(tname, target_p->name))
-        report_this_status(source_p, target_p, full_etrace);
+      if (!match(tname, target_p->name))
+        report_this_status(source_p, target_p);
     }
     else
-      report_this_status(source_p, target_p, full_etrace);
+      report_this_status(source_p, target_p);
   }
 
   sendto_one(source_p, form_str(RPL_ENDOFTRACE), me.name,
@@ -135,73 +122,32 @@ mo_etrace(struct Client *client_p, struct Client *source_p,
  * side effects - NONE
  */
 static void
-report_this_status(struct Client *source_p, struct Client *target_p,
-                   int full_etrace)
+report_this_status(struct Client *source_p, struct Client *target_p)
 {
-  const char *name;
-  const char *class_name;
-
-  name = get_client_name(target_p, HIDE_IP);
-  class_name = get_client_class(target_p);
-
-  set_time();
-
   if (target_p->status == STAT_CLIENT)
   {
-    if (full_etrace)
-    {
-      if (ConfigFileEntry.hide_spoof_ips)
-	sendto_one(source_p, form_str(RPL_ETRACE_FULL),
+    if (ConfigFileEntry.hide_spoof_ips)
+	sendto_one(source_p, form_str(RPL_ETRACE),
 		   me.name,
 		   source_p->name,
 		   HasUMode(target_p, UMODE_OPER) ? "Oper" : "User",
-		   class_name,
+		   get_client_class(&target_p->localClient->confs),
 		   target_p->name,
 		   target_p->username,
 		   target_p->host,
 		   IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
-		   IsIPSpoof(target_p) ? "<hidden>" : target_p->localClient->client_host,
-		   IsIPSpoof(target_p) ? "<hidden>" : target_p->localClient->client_server,
 		   target_p->info);
-      else
-        sendto_one(source_p, form_str(RPL_ETRACE_FULL),
-		   me.name,
-		   source_p->name, 
-		   HasUMode(target_p, UMODE_OPER) ? "Oper" : "User", 
-		   class_name,
-		   target_p->name,
-		   target_p->username,
-		   target_p->host,
-		   target_p->sockhost,
-		   target_p->localClient->client_host,
-		   target_p->localClient->client_server,
-		   target_p->info);
-    }
     else
-    {
-      if (ConfigFileEntry.hide_spoof_ips)
-	sendto_one(source_p, form_str(RPL_ETRACE),
-		   me.name,
-		   source_p->name,
-		   HasUMode(target_p, UMODE_OPER) ? "Oper" : "User",
-		   class_name,
-		   target_p->name,
-		   target_p->username,
-		   target_p->host,
-		   IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
-		   target_p->info);
-      else
 	sendto_one(source_p, form_str(RPL_ETRACE),
 		   me.name,
 		   source_p->name, 
 		   HasUMode(target_p, UMODE_OPER) ? "Oper" : "User", 
-		   class_name,
+		   get_client_class(&target_p->localClient->confs),
 		   target_p->name,
 		   target_p->username,
 		   target_p->host,
 		   target_p->sockhost,
 		   target_p->info);
-    }
   }
 }
 
@@ -225,7 +171,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1400 $",
+  .version = "$Revision: 2188 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
