@@ -21,7 +21,7 @@
 
 /*! \file m_gline.c
  * \brief Includes required functions for processing the GLINE command.
- * \version $Id: m_gline.c 1871 2013-04-26 08:53:04Z michael $
+ * \version $Id: m_gline.c 2339 2013-07-03 13:24:43Z michael $
  */
 
 #include "stdinc.h"
@@ -30,12 +30,11 @@
 #include "channel.h"
 #include "client.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
+#include "conf.h"
 #include "hostmask.h"
 #include "numeric.h"
 #include "s_bsd.h"
-#include "conf.h"
 #include "s_misc.h"
 #include "send.h"
 #include "s_serv.h"
@@ -43,6 +42,8 @@
 #include "parse.h"
 #include "modules.h"
 #include "log.h"
+#include "conf_db.h"
+#include "memory.h"
 
 #define GLINE_NOT_PLACED     0
 #define GLINE_ALREADY_VOTED -1
@@ -61,30 +62,25 @@ set_local_gline(const struct Client *source_p, const char *user,
                 const char *host, const char *reason)
 {
   char buffer[IRCD_BUFSIZE];
-  struct ConfItem *conf;
-  struct AccessItem *aconf;
-
-
-  conf = make_conf_item(GLINE_TYPE);
-  aconf = map_to_conf(conf);
+  struct MaskItem *conf = conf_make(CONF_GLINE);
 
   snprintf(buffer, sizeof(buffer), "%s (%s)", reason, smalldate(CurrentTime));
-  DupString(aconf->reason, buffer);
-  DupString(aconf->user, user);
-  DupString(aconf->host, host);
+  conf->reason = xstrdup(buffer);
+  conf->user = xstrdup(user);
+  conf->host = xstrdup(host);
 
-  aconf->hold = CurrentTime + ConfigFileEntry.gline_time;
-  SetConfTemporary(aconf);
-  add_conf_by_address(CONF_GLINE, aconf);
+  conf->setat = CurrentTime;
+  conf->until = CurrentTime + ConfigFileEntry.gline_time;
+  SetConfDatabase(conf);
 
-  sendto_realops_flags(UMODE_ALL, L_ALL,
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                        "%s added G-Line for [%s@%s] [%s]",
                        get_oper_name(source_p),
-                       aconf->user, aconf->host, aconf->reason);
+                       conf->user, conf->host, conf->reason);
   ilog(LOG_TYPE_GLINE, "%s added G-Line for [%s@%s] [%s]",
-       get_oper_name(source_p), aconf->user, aconf->host, aconf->reason);
+       get_oper_name(source_p), conf->user, conf->host, conf->reason);
 
-  /* Now, activate gline against current online clients */
+  add_conf_by_address(CONF_GLINE, conf);
   rehashed_klines = 1;
 }
 
@@ -97,30 +93,28 @@ static int
 remove_gline_match(const char *user, const char *host)
 {
   struct irc_ssaddr iphost, *piphost;
-  struct AccessItem *aconf;
-  int t;
+  struct MaskItem *conf;
+  int t = 0;
+  int aftype = 0;
 
   if ((t = parse_netmask(host, &iphost, NULL)) != HM_HOST)
   {
 #ifdef IPV6
     if (t == HM_IPV6)
-      t = AF_INET6;
+      aftype = AF_INET6;
     else
 #endif
-      t = AF_INET;
+      aftype = AF_INET;
     piphost = &iphost;
   }
   else
-  {
-    t = 0;
     piphost = NULL;
-  }
 
-  if ((aconf = find_conf_by_address(host, piphost, CONF_GLINE, t, user, NULL, 0)))
+  if ((conf = find_conf_by_address(host, piphost, CONF_GLINE, aftype, user, NULL, 0)))
   {
-    if (IsConfTemporary(aconf))
+    if (IsConfDatabase(conf))
     {
-      delete_one_address_conf(host, aconf);
+      delete_one_address_conf(host, conf);
       return 1;
     }
   }
@@ -215,7 +209,7 @@ check_majority(const struct Client *source_p, const char *user,
       {
         if (remove_gline_match(user, host))
         {
-          sendto_realops_flags(UMODE_ALL, L_ALL,
+          sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                                "%s has removed the G-Line for: [%s@%s]",
                                get_oper_name(source_p), user, host);
           ilog(LOG_TYPE_GLINE, "%s removed G-Line for [%s@%s]",
@@ -296,10 +290,10 @@ do_sgline(struct Client *source_p, int parc, char *parv[], int prop)
 
         if (bitlen < min_bitlen)
         {
-          sendto_realops_flags(UMODE_ALL, L_ALL, "%s!%s@%s on %s is requesting "
-                               "a GLINE with a CIDR mask < %d for [%s@%s] [%s]",
-                               source_p->name, source_p->username, source_p->host,
-                               source_p->servptr->name, min_bitlen, user, host, reason);
+          sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                               "%s is requesting a GLINE with a CIDR mask < %d for [%s@%s] [%s]",
+                               get_oper_name(source_p), min_bitlen,
+                               user, host, reason);
           return;
         }
       }
@@ -309,15 +303,16 @@ do_sgline(struct Client *source_p, int parc, char *parv[], int prop)
      if (check_majority(source_p, user, host, reason, GLINE_PENDING_ADD_TYPE) ==
          GLINE_ALREADY_VOTED)
      {
-       sendto_realops_flags(UMODE_ALL, L_ALL, "oper or server has already voted");
+       sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                            "oper or server has already voted");
        return;
      }
 
-     sendto_realops_flags(UMODE_ALL, L_ALL,
+     sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                           "%s requesting G-Line for [%s@%s] [%s]",
                           get_oper_name(source_p),
                           user, host, reason);
-     ilog(LOG_TYPE_GLINE, "#gline for %s@%s [%s] requested by %s",
+     ilog(LOG_TYPE_GLINE, "G-Line for [%s@%s] [%s] requested by %s",
           user, host, reason, get_oper_name(source_p));
   }
 }
@@ -391,13 +386,12 @@ mo_gline(struct Client *client_p, struct Client *source_p,
    * call these two functions first so the 'requesting' notice always comes
    * before the 'has triggered' notice.  -bill
    */
-  sendto_realops_flags(UMODE_ALL, L_ALL,
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                        "%s requesting G-Line for [%s@%s] [%s]",
                        get_oper_name(source_p),
                        user, host, reason);
-  ilog(LOG_TYPE_GLINE, "#gline for %s@%s [%s] requested by %s!%s@%s",
-       user, host, reason, source_p->name, source_p->username,
-       source_p->host);
+  ilog(LOG_TYPE_GLINE, "G-Line for [%s@%s] [%s] requested by %s",
+       user, host, reason, get_oper_name(source_p));
 
   /* 4 param version for hyb-7 servers */
   sendto_server(NULL, CAP_GLN|CAP_TS6, NOCAPS,
@@ -444,16 +438,17 @@ do_sungline(struct Client *source_p, const char *user,
 {
   assert(source_p->servptr != NULL);
 
-  sendto_realops_flags(UMODE_ALL, L_ALL,
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                        "%s requesting UNG-Line for [%s@%s] [%s]",
                        get_oper_name(source_p), user, host, reason);
-  ilog(LOG_TYPE_GLINE, "#ungline for %s@%s [%s] requested by %s",
+  ilog(LOG_TYPE_GLINE, "UNG-Line for [%s@%s] [%s] requested by %s",
        user, host, reason, get_oper_name(source_p));
 
   /* If at least 3 opers agree this user should be un G lined then do it */
   if (check_majority(source_p, user, host, reason, GLINE_PENDING_DEL_TYPE) ==
       GLINE_ALREADY_VOTED)
-    sendto_realops_flags(UMODE_ALL, L_ALL, "oper or server has already voted");
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "oper or server has already voted");
 
   if (prop)
   {
@@ -567,7 +562,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1871 $",
+  .version = "$Revision: 2339 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
