@@ -18,7 +18,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_globops.c 1474 2012-07-22 14:44:07Z michael $
+ *  $Id: m_globops.c 2273 2013-06-18 16:00:15Z michael $
  */
 
 #include "stdinc.h"
@@ -27,7 +27,6 @@
 #include "irc_string.h"
 #include "numeric.h"
 #include "send.h"
-#include "s_user.h"
 #include "s_serv.h"
 #include "parse.h"
 #include "modules.h"
@@ -64,7 +63,7 @@ mo_globops(struct Client *client_p, struct Client *source_p,
   sendto_server(NULL, NOCAPS, CAP_TS6,
                 ":%s GLOBOPS :%s", source_p->name, message);
 
-  sendto_globops_flags(UMODE_ALL, L_ALL, "from: %s: %s",
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_GLOBAL, "from: %s: %s",
                        source_p->name, message);
 }
 
@@ -80,7 +79,7 @@ ms_globops(struct Client *client_p, struct Client *source_p,
   sendto_server(client_p, NOCAPS, CAP_TS6, ":%s GLOBOPS :%s",
                 source_p->name, parv[1]);
 
-  sendto_globops_flags(UMODE_ALL, L_ALL, "from: %s: %s",
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_GLOBAL, "from: %s: %s",
                        source_p->name, parv[1]);
 }
 
@@ -104,7 +103,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1474 $",
+  .version = "$Revision: 2273 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
