@@ -18,7 +18,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_hash.c 1243 2011-09-30 10:47:53Z michael $
+ *  $Id: m_hash.c 2541 2013-11-13 16:56:44Z michael $
  */
 
 #include "stdinc.h"
@@ -32,7 +32,7 @@
 #include "parse.h"
 #include "modules.h"
 #include "s_user.h"
-#include "resv.h"
+#include "conf.h"
 #include "userhost.h"
 
 
@@ -40,21 +40,20 @@ static void
 mo_hash(struct Client *client_p, struct Client *source_p,
         int parc, char *parv[])
 {
-  int i;
-  int max_chain = 0;
-  int buckets   = 0;
-  int count     = 0;
-  struct Client *cl;
-  struct Client *icl;
-  struct Channel *ch;
-  struct UserHost *ush;
-  struct ResvChannel *rch;
+  unsigned int i = 0;
+  unsigned int max_chain = 0;
+  unsigned int buckets   = 0;
+  unsigned int count     = 0;
+  const struct Client *cl = NULL;
+  const struct Client *icl = NULL;
+  const struct Channel *ch = NULL;
+  const struct UserHost *ush = NULL;
 
   for (i = 0; i < HASHSIZE; ++i)
   {
     if ((cl = hash_get_bucket(HASH_TYPE_CLIENT, i)) != NULL)
     {
-      int len = 0;
+      unsigned int len = 0;
 
       ++buckets;
       for (; cl != NULL; cl = cl->hnext)
@@ -65,8 +64,8 @@ mo_hash(struct Client *client_p, struct Client *source_p,
     }
   }
 
-  sendto_one(source_p, ":%s NOTICE %s :Client: entries: %d buckets: %d "
-             "max chain: %d", me.name, source_p->name, count, buckets,
+  sendto_one(source_p, ":%s NOTICE %s :Client: entries: %u buckets: %u "
+             "max chain: %u", me.name, source_p->name, count, buckets,
              max_chain);
 
   count     = 0;
@@ -77,7 +76,7 @@ mo_hash(struct Client *client_p, struct Client *source_p,
   {
     if ((ch = hash_get_bucket(HASH_TYPE_CHANNEL, i)) != NULL)
     {
-      int len = 0;
+      unsigned int len = 0;
 
       ++buckets;
       for (; ch != NULL; ch = ch->hnextch)
@@ -88,31 +87,8 @@ mo_hash(struct Client *client_p, struct Client *source_p,
     }
   }
 
-  sendto_one(source_p, ":%s NOTICE %s :Channel: entries: %d buckets: %d "
-             "max chain: %d", me.name, source_p->name, count, buckets,
-             max_chain);
-
-  count     = 0;
-  buckets   = 0;
-  max_chain = 0;
-
-  for (i = 0; i < HASHSIZE; ++i)
-  {
-    if ((rch = hash_get_bucket(HASH_TYPE_RESERVED, i)) != NULL)
-    {
-      int len = 0;
-
-      ++buckets;
-      for (; rch != NULL; rch = rch->hnext)
-        ++len;
-      if (len > max_chain)
-        max_chain = len;
-      count += len;
-    }
-  }
-
-  sendto_one(source_p, ":%s NOTICE %s :Resv: entries: %d buckets: %d "
-             "max chain: %d", me.name, source_p->name, count, buckets,
+  sendto_one(source_p, ":%s NOTICE %s :Channel: entries: %u buckets: %u "
+             "max chain: %u", me.name, source_p->name, count, buckets,
              max_chain);
 
   count     = 0;
@@ -123,7 +99,7 @@ mo_hash(struct Client *client_p, struct Client *source_p,
   {
     if ((icl = hash_get_bucket(HASH_TYPE_ID, i)) != NULL)
     {
-      int len = 0;
+      unsigned int len = 0;
 
       ++buckets;
       for (; icl != NULL; icl = icl->idhnext)
@@ -134,8 +110,8 @@ mo_hash(struct Client *client_p, struct Client *source_p,
     }
   }
 
-  sendto_one(source_p, ":%s NOTICE %s :Id: entries: %d buckets: %d "
-             "max chain: %d", me.name, source_p->name, count, buckets,
+  sendto_one(source_p, ":%s NOTICE %s :Id: entries: %u buckets: %u "
+             "max chain: %u", me.name, source_p->name, count, buckets,
              max_chain);
 
   count     = 0;
@@ -146,7 +122,7 @@ mo_hash(struct Client *client_p, struct Client *source_p,
   {
     if ((ush = hash_get_bucket(HASH_TYPE_USERHOST, i)) != NULL)
     {
-      int len = 0;
+      unsigned int len = 0;
 
       ++buckets;
       for (; ush != NULL; ush = ush->next)
@@ -157,8 +133,8 @@ mo_hash(struct Client *client_p, struct Client *source_p,
     }
   }
 
-  sendto_one(source_p, ":%s NOTICE %s :UserHost: entries: %d buckets: %d "
-             "max chain: %d", me.name, source_p->name, count, buckets,
+  sendto_one(source_p, ":%s NOTICE %s :UserHost: entries: %u buckets: %u "
+             "max chain: %u", me.name, source_p->name, count, buckets,
              max_chain);
 }
 
@@ -182,7 +158,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1243 $",
+  .version = "$Revision: 2541 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
