@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_help.c 1738 2013-01-14 17:38:34Z michael $
+ *  $Id: m_help.c 2656 2013-12-12 19:04:40Z michael $
  */
 
 #include "stdinc.h"
@@ -34,9 +34,91 @@
 
 #define HELPLEN 400
 
-static void dohelp(struct Client *, const char *, char *);
-static void sendhelpfile(struct Client *, const char *, const char *);
 
+static void
+sendhelpfile(struct Client *source_p, const char *path, const char *topic)
+{
+  FILE *file = NULL;
+  char line[HELPLEN] = { '\0' };
+
+  if ((file = fopen(path, "r")) == NULL)
+  {
+    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
+               me.name, source_p->name, topic);
+    return;
+  }
+
+  if (fgets(line, sizeof(line), file) == NULL)
+  {
+    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
+               me.name, source_p->name, topic);
+    fclose(file);
+    return;
+  }
+
+  line[strlen(line) - 1] = '\0';
+  sendto_one(source_p, form_str(RPL_HELPSTART),
+             me.name, source_p->name, topic, line);
+
+  while (fgets(line, sizeof(line), file))
+  {
+    line[strlen(line) - 1] = '\0';
+
+    sendto_one(source_p, form_str(RPL_HELPTXT),
+               me.name, source_p->name, topic, line);
+  }
+
+  fclose(file);
+  sendto_one(source_p, form_str(RPL_ENDOFHELP),
+             me.name, source_p->name, topic);
+}
+
+static void
+dohelp(struct Client *source_p, char *topic)
+{
+  char *p = topic;
+  char h_index[] = "index";
+  char path[HYB_PATH_MAX + 1];
+  struct stat sb;
+
+  if (EmptyString(topic))
+    topic = h_index;
+  else
+    for (; *p; ++p)
+      *p = ToLower(*p);
+
+  if (strpbrk(topic, "/\\"))
+  {
+    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
+               me.name, source_p->name, topic);
+    return;
+  }
+
+  if (strlen(HPATH) + strlen(topic) + 1 > HYB_PATH_MAX)
+  {
+    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
+               me.name, source_p->name, topic);
+    return;
+  }
+
+  snprintf(path, sizeof(path), "%s/%s", HPATH, topic);
+
+  if (stat(path, &sb) < 0)
+  {
+    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
+               me.name, source_p->name, topic);
+    return;
+  }
+
+  if (!S_ISREG(sb.st_mode))
+  {
+    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
+               me.name, source_p->name, topic);
+    return;
+  }
+
+  sendhelpfile(source_p, path, topic);
+}
 
 /*
  * m_help - HELP message handler
@@ -59,7 +141,7 @@ m_help(struct Client *client_p, struct Client *source_p,
 
   last_used = CurrentTime;
 
-  dohelp(source_p, UHPATH, parv[1]);
+  dohelp(source_p, parv[1]);
 }
 
 /*
@@ -70,159 +152,30 @@ static void
 mo_help(struct Client *client_p, struct Client *source_p,
         int parc, char *parv[])
 {
-  dohelp(source_p, HPATH, parv[1]);
-}
-
-/*
- * mo_uhelp - HELP message handler
- * This is used so that opers can view the user help file without deopering
- *      parv[0] = sender prefix
- */
-static void
-mo_uhelp(struct Client *client_p, struct Client *source_p,
-            int parc, char *parv[])
-{
-  dohelp(source_p, UHPATH, parv[1]);
-}
-
-static void
-dohelp(struct Client *source_p, const char *hpath, char *topic)
-{
-  char h_index[] = "index";
-  char path[HYB_PATH_MAX + 1];
-  struct stat sb;
-  int i;
-
-  if (topic != NULL)
-  {
-    if (*topic == '\0')
-      topic = h_index;
-    else
-    {
-      /* convert to lower case */
-      for (i = 0; topic[i] != '\0'; ++i)
-        topic[i] = ToLower(topic[i]);
-    }
-  }
-  else
-    topic = h_index;    /* list available help topics */
-
-  if (strpbrk(topic, "/\\"))
-  {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
-    return;
-  }
-
-  if (strlen(hpath) + strlen(topic) + 1 > HYB_PATH_MAX)
-  {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
-    return;
-  }
-
-  snprintf(path, sizeof(path), "%s/%s", hpath, topic);
-
-  if (stat(path, &sb) < 0)
-  {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
-    return;
-  }
-
-  if (!S_ISREG(sb.st_mode))
-  {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
-    return;
-  }
-
-  sendhelpfile(source_p, path, topic);
-}
-
-static void 
-sendhelpfile(struct Client *source_p, const char *path, const char *topic)
-{
-  FILE *file;
-  char line[HELPLEN];
-  char started = 0;
-  int type;
-
-  if ((file = fopen(path, "r")) == NULL)
-  {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
-    return;
-  }
-
-  if (fgets(line, sizeof(line), file) == NULL)
-  {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
-    return;
-  }
-
-  else if (line[0] != '#')
-  {
-    line[strlen(line) - 1] = '\0';	  
-    sendto_one(source_p, form_str(RPL_HELPSTART),
-             me.name, source_p->name, topic, line);
-    started = 1;
-  }
-
-  while (fgets(line, sizeof(line), file))
-  {
-    line[strlen(line) - 1] = '\0';
-    if (line[0] != '#')
-    {
-      if (!started)
-      {
-        type = RPL_HELPSTART;
-        started = 1;
-      }
-      else
-        type = RPL_HELPTXT;
-      
-      sendto_one(source_p, form_str(RPL_HELPTXT),
-                 me.name, source_p->name, topic, line);
-    }
-  }
-
-  fclose(file);
-  sendto_one(source_p, form_str(RPL_HELPTXT),
-             me.name, source_p->name, topic, "");
-  sendto_one(source_p, form_str(RPL_ENDOFHELP),
-             me.name, source_p->name, topic);
+  dohelp(source_p, parv[1]);
 }
 
 static struct Message help_msgtab = {
-  "HELP", 0, 0, 0, 0, MFLG_SLOW, 0,
+  "HELP", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   {m_unregistered, m_help, m_ignore, m_ignore, mo_help, m_ignore}
 };
 
-static struct Message uhelp_msgtab = {
-  "UHELP", 0, 0, 0, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_help, m_ignore, m_ignore, mo_uhelp, m_ignore}
-};
-
 static void
 module_init(void)
 {
   mod_add_cmd(&help_msgtab);
-  mod_add_cmd(&uhelp_msgtab);
 }
 
 static void
 module_exit(void)
 {
   mod_del_cmd(&help_msgtab);
-  mod_del_cmd(&uhelp_msgtab);
 }
 
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1738 $",
+  .version = "$Revision: 2656 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
