@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_info.c 1766 2013-01-19 18:22:39Z michael $
+ *  $Id: m_info.c 2342 2013-07-03 13:59:41Z michael $
  */
 
 #include "stdinc.h"
@@ -37,10 +37,6 @@
 #include "modules.h"
 
 
-static void send_conf_options(struct Client *);
-static void send_birthdate_online_time(struct Client *);
-static void send_info_text(struct Client *);
-
 /*
  * jdc -- Structure for our configuration value table
  */
@@ -53,12 +49,12 @@ struct InfoStruct
 };
 
 /* Types for output_type in InfoStruct */
-#define OUTPUT_STRING     0x0001 /* Output option as %s w/ dereference  */
-#define OUTPUT_STRING_PTR 0x0002 /* Output option as %s w/out deference */
-#define OUTPUT_DECIMAL    0x0004 /* Output option as decimal (%d)       */
-#define OUTPUT_BOOLEAN    0x0008 /* Output option as "ON" or "OFF"      */
-#define OUTPUT_BOOLEAN_YN 0x0010 /* Output option as "YES" or "NO"      */
-#define OUTPUT_BOOLEAN2   0x0020 /* Output option as "YES/NO/MASKED"    */
+#define OUTPUT_STRING     0x0001  /* Output option as %s w/ dereference  */
+#define OUTPUT_STRING_PTR 0x0002  /* Output option as %s w/out deference */
+#define OUTPUT_DECIMAL    0x0004  /* Output option as decimal (%d)       */
+#define OUTPUT_BOOLEAN    0x0008  /* Output option as "ON" or "OFF"      */
+#define OUTPUT_BOOLEAN_YN 0x0010  /* Output option as "YES" or "NO"      */
+#define OUTPUT_BOOLEAN2   0x0020  /* Output option as "YES/NO/MASKED"    */
 
 static const struct InfoStruct info_table[] =
 {
@@ -68,25 +64,43 @@ static const struct InfoStruct info_table[] =
     "CPATH",
     OUTPUT_STRING,
     &ConfigFileEntry.configfile,
-    "Path to Main Configuration File"
+    "Path to main configuration file"
   },
   {
     "DPATH",
     OUTPUT_STRING,
     &ConfigFileEntry.dpath,
-    "Directory Containing Configuration Files"
+    "Root directory of installation"
   },
   {
     "DLPATH",
     OUTPUT_STRING,
     &ConfigFileEntry.dlinefile,
-    "Path to D-line File"
+    "Path to D-line database file"
   },
   {
     "KPATH",
     OUTPUT_STRING,
     &ConfigFileEntry.klinefile,
-    "Path to K-line File"
+    "Path to K-line database file"
+  },
+  {
+    "GPATH",
+    OUTPUT_STRING,
+    &ConfigFileEntry.glinefile,
+    "Path to G-line database file"
+  },
+  {
+    "XPATH",
+    OUTPUT_STRING,
+    &ConfigFileEntry.xlinefile,
+    "Path to X-line database file"
+  },
+  {
+    "RESVPATH",
+    OUTPUT_STRING,
+    &ConfigFileEntry.resvfile,
+    "Path to resv database file"
   },
   {
     "network_name",
@@ -136,12 +150,6 @@ static const struct InfoStruct info_table[] =
     &ConfigChannel.disable_fake_channels,
     "Forbids channels with special ASCII characters in their name"
   },
-  {
-    "restrict_channels",
-    OUTPUT_BOOLEAN_YN,
-    &ConfigChannel.restrict_channels,
-    "Only reserved channels are allowed"
-  },
   {
     "knock_delay",
     OUTPUT_DECIMAL,
@@ -166,12 +174,6 @@ static const struct InfoStruct info_table[] =
     &ConfigChannel.max_chans_per_oper,
     "Maximum number of channels an oper can join"
   },
-  {
-    "quiet_on_ban",
-    OUTPUT_BOOLEAN_YN,
-    &ConfigChannel.quiet_on_ban,
-    "Banned users may not send text to a channel"
-  },
   {
     "max_bans",
     OUTPUT_DECIMAL,
@@ -226,6 +228,12 @@ static const struct InfoStruct info_table[] =
     &ConfigServerHide.hide_servers,
     "Hide servernames from users"
   },
+  {
+    "hide_services",
+    OUTPUT_BOOLEAN_YN,
+    &ConfigServerHide.hide_services,
+    "Hides the location of services server"
+  },
   {
     "hidden_name",
     OUTPUT_STRING,
@@ -236,7 +244,7 @@ static const struct InfoStruct info_table[] =
     "hide_server_ips",
     OUTPUT_BOOLEAN_YN,
     &ConfigServerHide.hide_server_ips,
-    "Prevent people from seeing server IPs"
+    "Prevent people from seeing server IP addresses"
   },
   {
     "gline_min_cidr",
@@ -266,7 +274,7 @@ static const struct InfoStruct info_table[] =
     "hide_spoof_ips",
     OUTPUT_BOOLEAN_YN,
     &ConfigFileEntry.hide_spoof_ips,
-    "Hide spoofed IP's"
+    "Hide spoofed IP addresses"
   },
   {
     "ignore_bogus_ts",
@@ -274,6 +282,12 @@ static const struct InfoStruct info_table[] =
     &ConfigFileEntry.ignore_bogus_ts,
     "Ignore bogus timestamps from other servers"
   },
+  {
+    "cycle_on_host_change",
+    OUTPUT_BOOLEAN_YN,
+    &ConfigFileEntry.cycle_on_host_change,
+    "Send a fake QUIT/JOIN combination on host change"
+  },
   {
     "disable_auth",
     OUTPUT_BOOLEAN_YN,
@@ -283,7 +297,7 @@ static const struct InfoStruct info_table[] =
   {
     "disable_remote_commands",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.disable_remote,
+    &ConfigServerHide.disable_remote_commands,
     "Prevent users issuing commands on remote servers"
   },
   {
@@ -302,7 +316,7 @@ static const struct InfoStruct info_table[] =
     "failed_oper_notice",
     OUTPUT_BOOLEAN,
     &ConfigFileEntry.failed_oper_notice,
-    "Inform opers if someone /oper's with the wrong password"
+    "Inform opers if someone tries to /oper with the wrong password"
   },
   {
     "dots_in_ident",
@@ -388,6 +402,12 @@ static const struct InfoStruct info_table[] =
     &ConfigFileEntry.stats_P_oper_only,
     "STATS P is only shown to operators"
   },
+  {
+    "stats_u_oper_only",
+    OUTPUT_BOOLEAN_YN,
+    &ConfigFileEntry.stats_u_oper_only,
+    "STATS u is only shown to operators"
+  },
   {
     "stats_i_oper_only",
     OUTPUT_BOOLEAN2,
@@ -478,7 +498,6 @@ static const struct InfoStruct info_table[] =
     &ConfigFileEntry.gline_time,
     "Expiry time for G-lines"
   },
-
   {
     "gline_request_duration",
     OUTPUT_DECIMAL,
@@ -495,113 +514,6 @@ static const struct InfoStruct info_table[] =
   }
 };
 
-/*
-** m_info()
-**  parv[0] = sender prefix
-**  parv[1] = servername
-*/
-static void
-m_info(struct Client *client_p, struct Client *source_p,
-       int parc, char *parv[])
-{
-  static time_t last_used = 0;
-
-  if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime)
-  {
-    /* safe enough to give this on a local connect only */
-    sendto_one(source_p, form_str(RPL_LOAD2HI),
-               me.name, source_p->name);
-    return;
-  }
-
-  last_used = CurrentTime;
-
-  if (!ConfigFileEntry.disable_remote)
-    if (hunt_server(client_p,source_p, ":%s INFO :%s", 1,
-                    parc, parv) != HUNTED_ISME)
-      return;
-
-  send_info_text(source_p);
-}
-
-/*
-** mo_info()
-**  parv[0] = sender prefix
-**  parv[1] = servername
-*/
-static void
-mo_info(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
-{
-  if (hunt_server(client_p, source_p, ":%s INFO :%s", 1,
-                  parc, parv) != HUNTED_ISME)
-    return;
-
-  send_info_text(source_p);
-}
-
-/*
-** ms_info()
-**  parv[0] = sender prefix
-**  parv[1] = servername
-*/
-static void
-ms_info(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
-{
-  if (!IsClient(source_p))
-      return;
-
-  if (hunt_server(client_p, source_p, ":%s INFO :%s", 1,
-                  parc, parv) != HUNTED_ISME)
-    return;
-
-  send_info_text(source_p);
-}
-
-/* send_info_text()
- *
- * inputs	- client pointer to send info text to
- * output	- NONE
- * side effects	- info text is sent to client
- */
-static void
-send_info_text(struct Client *source_p)
-{
-  const char **text = infotext;
-  char *source, *target;
-
-  sendto_realops_flags(UMODE_SPY, L_ALL,
-                       "INFO requested by %s (%s@%s) [%s]",
-                       source_p->name, source_p->username,
-                       source_p->host, source_p->servptr->name);
-
-  if (!MyClient(source_p) && IsCapable(source_p->from, CAP_TS6) &&
-      HasID(source_p))
-    source = me.id, target = source_p->id;
-  else
-    source = me.name, target = source_p->name;
-
-  while (*text)
-  {
-    const char *line = *text++;
-
-    if (*line == '\0')
-      line = " ";
-
-    sendto_one(source_p, form_str(RPL_INFO),
-               source, target, line);
-  }
-
-  if (HasUMode(source_p, UMODE_OPER))
-    send_conf_options(source_p);
-
-  send_birthdate_online_time(source_p);
-
-  sendto_one(source_p, form_str(RPL_ENDOFINFO),
-             me.name, source_p->name);
-}
-
 /* send_birthdate_online_time()
  *
  * inputs	- client pointer to send to
@@ -612,17 +524,13 @@ static void
 send_birthdate_online_time(struct Client *source_p)
 {
   if (!MyClient(source_p) && IsCapable(source_p->from, CAP_TS6) && HasID(source_p))
-  {
     sendto_one(source_p, ":%s %d %s :On-line since %s",
                me.id, RPL_INFO, source_p->id,
                myctime(me.localClient->firsttime));
-  }
   else
-  {
     sendto_one(source_p, ":%s %d %s :On-line since %s",
                me.name, RPL_INFO, source_p->name,
                myctime(me.localClient->firsttime));
-  }
 }
 
 /* send_conf_options()
@@ -709,7 +617,7 @@ send_conf_options(struct Client *source_p)
       /* Output info_table[i].option as "YES" or "NO" */
       case OUTPUT_BOOLEAN_YN:
       {
-        int option = *((int *)iptr->option);
+        const int option = *((int *)iptr->option);
 
         sendto_one(source_p, ":%s %d %s :%-30s %-5s [%-30s]",
                    from, RPL_INFO, to,
@@ -720,7 +628,7 @@ send_conf_options(struct Client *source_p)
 
       case OUTPUT_BOOLEAN2:
       {
-        int option = *((int *)iptr->option);
+        const int option = *((int *)iptr->option);
 
         sendto_one(source_p, ":%s %d %s :%-30s %-5s [%-30s]",
                    from, RPL_INFO, to,
@@ -735,9 +643,97 @@ send_conf_options(struct Client *source_p)
              from, to, "");
 }
 
+/* send_info_text()
+ *
+ * inputs       - client pointer to send info text to
+ * output       - NONE
+ * side effects - info text is sent to client
+ */
+static void
+send_info_text(struct Client *source_p)
+{
+  const char **text = infotext;
+  char *source, *target;
+
+  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
+                       "INFO requested by %s (%s@%s) [%s]",
+                       source_p->name, source_p->username,
+                       source_p->host, source_p->servptr->name);
+
+  if (!MyClient(source_p) && IsCapable(source_p->from, CAP_TS6) &&
+      HasID(source_p))
+    source = me.id, target = source_p->id;
+  else
+    source = me.name, target = source_p->name;
+
+  while (*text)
+  {
+    const char *line = *text++;
+
+    if (*line == '\0')
+      line = " ";
+
+    sendto_one(source_p, form_str(RPL_INFO),
+               source, target, line);
+  }
+
+  if (HasUMode(source_p, UMODE_OPER))
+    send_conf_options(source_p);
+
+  send_birthdate_online_time(source_p);
+
+  sendto_one(source_p, form_str(RPL_ENDOFINFO),
+             me.name, source_p->name);
+}
+
+/*
+** m_info()
+**  parv[0] = sender prefix
+**  parv[1] = servername
+*/
+static void
+m_info(struct Client *client_p, struct Client *source_p,
+       int parc, char *parv[])
+{
+  static time_t last_used = 0;
+
+  if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime)
+  {
+    /* safe enough to give this on a local connect only */
+    sendto_one(source_p, form_str(RPL_LOAD2HI),
+               me.name, source_p->name);
+    return;
+  }
+
+  last_used = CurrentTime;
+
+  if (!ConfigServerHide.disable_remote_commands)
+    if (hunt_server(client_p,source_p, ":%s INFO :%s", 1,
+                    parc, parv) != HUNTED_ISME)
+      return;
+
+  send_info_text(source_p);
+}
+
+/*
+** ms_info()
+**  parv[0] = sender prefix
+**  parv[1] = servername
+*/
+static void
+ms_info(struct Client *client_p, struct Client *source_p,
+        int parc, char *parv[])
+{
+  if (hunt_server(client_p, source_p, ":%s INFO :%s", 1,
+                  parc, parv) != HUNTED_ISME)
+    return;
+
+  send_info_text(source_p);
+}
+
 static struct Message info_msgtab = {
   "INFO", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
-  { m_unregistered, m_info, ms_info, m_ignore, mo_info, m_ignore }
+  { m_unregistered, m_info, ms_info, m_ignore, ms_info, m_ignore }
 };
 
 static void
@@ -755,7 +751,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1766 $",
+  .version = "$Revision: 2342 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
