@@ -19,13 +19,12 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_ison.c 1723 2013-01-03 00:02:43Z michael $
+ *  $Id: m_ison.c 2100 2013-05-19 11:00:50Z michael $
  */
 
 #include "stdinc.h"
 #include "client.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
@@ -33,9 +32,18 @@
 #include "modules.h"
 
 
+/*
+ * m_ison added by Darren Reed 13/8/91 to act as an efficent user indicator
+ * with respect to cpu/bandwidth used. Implemented for NOTIFY feature in
+ * clients. Designed to reduce number of whois requests. Can process
+ * nicknames in batches as long as the maximum buffer length.
+ *
+ * format:
+ * ISON :nicklist
+ */
 static void
-do_ison(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
+m_ison(struct Client *client_p, struct Client *source_p,
+       int parc, char *parv[])
 {
   struct Client *target_p = NULL;
   char *nick;
@@ -90,22 +98,6 @@ do_ison(struct Client *client_p, struct Client *source_p,
   sendto_one(source_p, "%s", buf);
 }
 
-/*
- * m_ison added by Darren Reed 13/8/91 to act as an efficent user indicator
- * with respect to cpu/bandwidth used. Implemented for NOTIFY feature in
- * clients. Designed to reduce number of whois requests. Can process
- * nicknames in batches as long as the maximum buffer length.
- *
- * format:
- * ISON :nicklist
- */
-static void
-m_ison(struct Client *client_p, struct Client *source_p,
-       int parc, char *parv[])
-{
-  do_ison(client_p, source_p, parc, parv);
-}
-
 static struct Message ison_msgtab = {
   "ISON", 0, 0, 2, 1, MFLG_SLOW, 0,
   {m_unregistered, m_ison, m_ignore, m_ignore, m_ison, m_ignore}
@@ -126,7 +118,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1723 $",
+  .version = "$Revision: 2100 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
