@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_kline.c 1871 2013-04-26 08:53:04Z michael $
+ *  $Id: m_kline.c 2339 2013-07-03 13:24:43Z michael $
  */
 
 #include "stdinc.h"
@@ -27,13 +27,12 @@
 #include "channel.h"
 #include "client.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
+#include "conf.h"
 #include "hostmask.h"
 #include "numeric.h"
 #include "fdlist.h"
 #include "s_bsd.h"
-#include "conf.h"
 #include "log.h"
 #include "s_misc.h"
 #include "send.h"
@@ -42,14 +41,14 @@
 #include "s_gline.h"
 #include "parse.h"
 #include "modules.h"
-
+#include "conf_db.h"
+#include "memory.h"
 
 static int already_placed_kline(struct Client *, const char *, const char *, int);
-static void apply_kline(struct Client *, struct ConfItem *, const char *, time_t);
-static void apply_tkline(struct Client *, struct ConfItem *, int);
+static void m_kline_add_kline(struct Client *, struct MaskItem *, time_t);
 
 static char buffer[IRCD_BUFSIZE];
-static int remove_tkline_match(const char *, const char *);
+static int remove_kline_match(const char *, const char *);
 
 
 /* mo_kline()
@@ -66,13 +65,11 @@ mo_kline(struct Client *client_p, struct Client *source_p,
          int parc, char *parv[])
 {
   char *reason = NULL;
-  char *oper_reason;
   char *user = NULL;
   char *host = NULL;
   const char *current_date;
   char *target_server = NULL;
-  struct ConfItem *conf;
-  struct AccessItem *aconf;
+  struct MaskItem *conf;
   time_t tkline_time = 0;
   time_t cur_time;
 
@@ -107,7 +104,7 @@ mo_kline(struct Client *client_p, struct Client *source_p,
                     user, host, reason);
 
     /* Allow ON to apply local kline as well if it matches */
-    if (!match(target_server, me.name))
+    if (match(target_server, me.name))
       return;
   }
   else
@@ -117,37 +114,21 @@ mo_kline(struct Client *client_p, struct Client *source_p,
   if (already_placed_kline(source_p, user, host, 1))
     return;
 
-  /* Look for an oper reason */
-  if ((oper_reason = strchr(reason, '|')) != NULL)
-    *oper_reason++ = '\0';
-
   cur_time = CurrentTime;
   current_date = smalldate(cur_time);
-  conf = make_conf_item(KLINE_TYPE);
-  aconf = map_to_conf(conf);
+  conf = conf_make(CONF_KLINE);
 
-  DupString(aconf->host, host);
-  DupString(aconf->user, user);
+  conf->host = xstrdup(host);
+  conf->user = xstrdup(user);
 
   if (tkline_time != 0)
-  {
     snprintf(buffer, sizeof(buffer), "Temporary K-line %d min. - %s (%s)",
              (int)(tkline_time/60), reason, current_date);
-    DupString(aconf->reason, buffer);
-
-    if (oper_reason != NULL)
-      DupString(aconf->oper_reason, oper_reason);
-    apply_tkline(source_p, conf, tkline_time);
-  }
   else
-  {
     snprintf(buffer, sizeof(buffer), "%s (%s)", reason, current_date);
-    DupString(aconf->reason, buffer);
 
-    if (oper_reason != NULL)
-      DupString(aconf->oper_reason, oper_reason);
-    apply_kline(source_p, conf, current_date, cur_time);
-  }
+  conf->reason = xstrdup(buffer);
+  m_kline_add_kline(source_p, conf, tkline_time);
 }
 
 /* me_kline - handle remote kline. no propagation */
@@ -155,17 +136,16 @@ static void
 me_kline(struct Client *client_p, struct Client *source_p,
          int parc, char *parv[])
 {
-  struct ConfItem *conf=NULL;
-  struct AccessItem *aconf=NULL;
-  int tkline_time;
+  struct MaskItem *conf = NULL;
+  int tkline_time = 0;
   const char* current_date;
   time_t cur_time;
-  char *kuser, *khost, *kreason, *oper_reason;
+  char *kuser, *khost, *kreason;
 
   if (parc != 6 || EmptyString(parv[5]))
     return;
 
-  if (!match(parv[1], me.name))
+  if (match(parv[1], me.name))
     return;
 
   tkline_time = valid_tkline(parv[2], TK_SECONDS);
@@ -173,13 +153,10 @@ me_kline(struct Client *client_p, struct Client *source_p,
   khost = parv[4];
   kreason = parv[5];
 
-  if ((oper_reason = strchr(kreason, '|')) != NULL)
-    *oper_reason++ = '\0';
-
   cur_time = CurrentTime;
   current_date = smalldate(cur_time);
 
-  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(ULINE_TYPE, source_p->servptr->name,
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(CONF_ULINE, source_p->servptr->name,
                               source_p->username, source_p->host,
                               SHARED_KLINE))
   {
@@ -187,30 +164,18 @@ me_kline(struct Client *client_p, struct Client *source_p,
         already_placed_kline(source_p, kuser, khost, 1))
       return;
 
-    conf = make_conf_item(KLINE_TYPE);
-    aconf = map_to_conf(conf);
-    DupString(aconf->host, khost);
-    DupString(aconf->user, kuser);
+    conf = conf_make(CONF_KLINE);
+    conf->host = xstrdup(khost);
+    conf->user = xstrdup(kuser);
 
     if (tkline_time != 0)
-    {
       snprintf(buffer, sizeof(buffer), "Temporary K-line %d min. - %s (%s)",
                (int)(tkline_time/60), kreason, current_date);
-      DupString(aconf->reason, buffer);
-
-      if (oper_reason != NULL)
-        DupString(aconf->oper_reason, oper_reason);
-      apply_tkline(source_p, conf, tkline_time);
-    }
     else
-    {
       snprintf(buffer, sizeof(buffer), "%s (%s)", kreason, current_date);
-      DupString(aconf->reason, buffer);
 
-      if (oper_reason != NULL)
-        DupString(aconf->oper_reason, oper_reason);
-      apply_kline(source_p, conf, current_date, cur_time);
-    }
+    conf->reason = xstrdup(buffer);
+    m_kline_add_kline(source_p, conf, tkline_time);
   }
 }
 
@@ -230,25 +195,6 @@ ms_kline(struct Client *client_p, struct Client *source_p,
   me_kline(client_p, source_p, parc, parv);
 }
 
-/* apply_kline()
- *
- * inputs	-
- * output	- NONE
- * side effects	- kline as given, is added to the hashtable
- *		  and conf file
- */
-static void 
-apply_kline(struct Client *source_p, struct ConfItem *conf,
-            const char *current_date, time_t cur_time)
-{
-  struct AccessItem *aconf = map_to_conf(conf);
-
-  add_conf_by_address(CONF_KLINE, aconf);
-  write_conf_line(source_p, conf, current_date, cur_time);
-  /* Now, activate kline against current online clients */
-  rehashed_klines = 1;
-}
-
 /* apply_tkline()
  *
  * inputs	-
@@ -256,28 +202,41 @@ apply_kline(struct Client *source_p, struct ConfItem *conf,
  * side effects	- tkline as given is placed
  */
 static void
-apply_tkline(struct Client *source_p, struct ConfItem *conf,
-             int tkline_time)
+m_kline_add_kline(struct Client *source_p, struct MaskItem *conf,
+                  time_t tkline_time)
 {
-  struct AccessItem *aconf;
+  if (tkline_time)
+  {
+    conf->until = CurrentTime + tkline_time;
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "%s added temporary %d min. K-Line for [%s@%s] [%s]",
+                         get_oper_name(source_p), tkline_time/60,
+                         conf->user, conf->host,
+                         conf->reason);
+    sendto_one(source_p, ":%s NOTICE %s :Added temporary %d min. K-Line [%s@%s]",
+               MyConnect(source_p) ? me.name : ID_or_name(&me, source_p->from),
+               source_p->name, tkline_time/60, conf->user, conf->host);
+    ilog(LOG_TYPE_KLINE, "%s added temporary %d min. K-Line for [%s@%s] [%s]",
+         get_oper_name(source_p), tkline_time/60,
+         conf->user, conf->host, conf->reason);
+  }
+  else
+  {
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "%s added K-Line for [%s@%s] [%s]",
+                         get_oper_name(source_p),
+                         conf->user, conf->host, conf->reason);
+    sendto_one(source_p, ":%s NOTICE %s :Added K-Line [%s@%s]",
+               MyConnect(source_p) ? me.name : ID_or_name(&me, source_p->from),
+                 source_p->name, conf->user, conf->host);
+    ilog(LOG_TYPE_KLINE, "%s added K-Line for [%s@%s] [%s]",
+         get_oper_name(source_p), conf->user, conf->host, conf->reason);
+  }
 
-  aconf = (struct AccessItem *)map_to_conf(conf);
-  aconf->hold = CurrentTime + tkline_time;
-  SetConfTemporary(aconf);
-  add_conf_by_address(CONF_KLINE, aconf);
-
-  sendto_realops_flags(UMODE_ALL, L_ALL,
-		       "%s added temporary %d min. K-Line for [%s@%s] [%s]",
-		       get_oper_name(source_p), tkline_time/60,
-		       aconf->user, aconf->host,
-		       aconf->reason);
-  sendto_one(source_p, ":%s NOTICE %s :Added temporary %d min. K-Line [%s@%s]",
-	     MyConnect(source_p) ? me.name : ID_or_name(&me, source_p->from),
-	     source_p->name, tkline_time/60, aconf->user, aconf->host);
-  ilog(LOG_TYPE_KLINE, "%s added temporary %d min. K-Line for [%s@%s] [%s]",
-       source_p->name, tkline_time/60,
-       aconf->user, aconf->host, aconf->reason);
+  conf->setat = CurrentTime;
+  SetConfDatabase(conf);
 
+  add_conf_by_address(CONF_KLINE, conf);
   rehashed_klines = 1;
 }
 
@@ -296,34 +255,32 @@ already_placed_kline(struct Client *source_p, const char *luser, const char *lho
 {
   const char *reason;
   struct irc_ssaddr iphost, *piphost;
-  struct AccessItem *aconf;
-  int t;
+  struct MaskItem *conf = NULL;
+  int t = 0;
+  int aftype = 0;
 
-  if ((t = parse_netmask(lhost, &iphost, &t)) != HM_HOST)
+  if ((t = parse_netmask(lhost, &iphost, NULL)) != HM_HOST)
   {
 #ifdef IPV6
     if (t == HM_IPV6)
-      t = AF_INET6;
+      aftype = AF_INET6;
     else
 #endif
-      t = AF_INET;
+      aftype = AF_INET;
     piphost = &iphost;
   }
   else
-  {
-    t = 0;
     piphost = NULL;
-  }
 
-  if ((aconf = find_conf_by_address(lhost, piphost, CONF_KLINE, t, luser, NULL, 0)))
+  if ((conf = find_conf_by_address(lhost, piphost, CONF_KLINE, aftype, luser, NULL, 0)))
   {
     if (warn)
     {
-      reason = aconf->reason ? aconf->reason : CONF_NOREASON;
+      reason = conf->reason ? conf->reason : CONF_NOREASON;
       sendto_one(source_p,
                  ":%s NOTICE %s :[%s@%s] already K-Lined by [%s@%s] - %s",
-                 me.name, source_p->name, luser, lhost, aconf->user,
-                 aconf->host, reason);
+                 me.name, source_p->name, luser, lhost, conf->user,
+                 conf->host, reason);
     }
 
     return 1;
@@ -356,7 +313,7 @@ mo_unkline(struct Client *client_p,struct Client *source_p,
     return;
   }
 
-  if (parc < 2 || EmptyString(parv[1]))
+  if (EmptyString(parv[1]))
   {
     sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
                me.name, source_p->name, "UNKLINE");
@@ -374,35 +331,23 @@ mo_unkline(struct Client *client_p,struct Client *source_p,
                         target_server, user, host);
 
     /* Allow ON to apply local unkline as well if it matches */
-    if (!match(target_server, me.name))
+    if (match(target_server, me.name))
       return;
   }
   else
     cluster_a_line(source_p, "UNKLINE", CAP_UNKLN, SHARED_UNKLINE,
                    "%s %s", user, host);
 
-  if (remove_tkline_match(host, user))
+  if (remove_kline_match(host, user))
   {
     sendto_one(source_p,
-               ":%s NOTICE %s :Un-klined [%s@%s] from temporary K-Lines",
+               ":%s NOTICE %s :K-Line for [%s@%s] is removed",
                me.name, source_p->name, user, host);
-    sendto_realops_flags(UMODE_ALL, L_ALL,
-                         "%s has removed the temporary K-Line for: [%s@%s]",
-                         get_oper_name(source_p), user, host);
-    ilog(LOG_TYPE_KLINE, "%s removed temporary K-Line for [%s@%s]",
-         source_p->name, user, host);
-    return;
-  }
-
-  if (remove_conf_line(KLINE_TYPE, source_p, user, host) > 0)
-  {
-    sendto_one(source_p, ":%s NOTICE %s :K-Line for [%s@%s] is removed", 
-               me.name, source_p->name, user,host);
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "%s has removed the K-Line for: [%s@%s]",
                          get_oper_name(source_p), user, host);
     ilog(LOG_TYPE_KLINE, "%s removed K-Line for [%s@%s]",
-         source_p->name, user, host);
+         get_oper_name(source_p), user, host);
   }
   else
     sendto_one(source_p, ":%s NOTICE %s :No K-Line for [%s@%s] found", 
@@ -431,37 +376,24 @@ me_unkline(struct Client *client_p, struct Client *source_p,
   kuser = parv[2];
   khost = parv[3];
 
-  if (!IsClient(source_p) || !match(parv[1], me.name))
+  if (!IsClient(source_p) || match(parv[1], me.name))
     return;
 
-  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(ULINE_TYPE,
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(CONF_ULINE,
 				   source_p->servptr->name,
 				   source_p->username, source_p->host,
 				   SHARED_UNKLINE))
   {
-    if (remove_tkline_match(khost, kuser))
+    if (remove_kline_match(khost, kuser))
     {
       sendto_one(source_p,
-                 ":%s NOTICE %s :Un-klined [%s@%s] from temporary K-Lines",
+                 ":%s NOTICE %s :K-Line for [%s@%s] is removed",
                  me.name, source_p->name, kuser, khost);
-      sendto_realops_flags(UMODE_ALL, L_ALL,  
-                           "%s has removed the temporary K-Line for: [%s@%s]",
-                           get_oper_name(source_p), kuser, khost);
-      ilog(LOG_TYPE_KLINE, "%s removed temporary K-Line for [%s@%s]",
-           source_p->name, kuser, khost);
-      return;
-    }
-
-    if (remove_conf_line(KLINE_TYPE, source_p, kuser, khost) > 0)
-    {
-      sendto_one(source_p, ":%s NOTICE %s :K-Line for [%s@%s] is removed",
-                 me.name, source_p->name, kuser, khost);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "%s has removed the K-Line for: [%s@%s]",
-                         get_oper_name(source_p), kuser, khost);
-
+                           get_oper_name(source_p), kuser, khost);
       ilog(LOG_TYPE_KLINE, "%s removed K-Line for [%s@%s]",
-           source_p->name, kuser, khost);
+           get_oper_name(source_p), kuser, khost);
     }
     else
       sendto_one(source_p, ":%s NOTICE %s :No K-Line for [%s@%s] found",
@@ -490,33 +422,31 @@ ms_unkline(struct Client *client_p, struct Client *source_p,
  * Side effects: Any matching tklines are removed.
  */
 static int
-remove_tkline_match(const char *host, const char *user)
+remove_kline_match(const char *host, const char *user)
 {
   struct irc_ssaddr iphost, *piphost;
-  struct AccessItem *aconf;
-  int t;
+  struct MaskItem *conf;
+  int t = 0;
+  int aftype = 0;
 
   if ((t = parse_netmask(host, &iphost, NULL)) != HM_HOST)
   {
 #ifdef IPV6
     if (t == HM_IPV6)
-      t = AF_INET6;
+      aftype = AF_INET6;
     else
 #endif
-      t = AF_INET;
+      aftype = AF_INET;
     piphost = &iphost;
   }
   else
-  {
-    t = 0;
     piphost = NULL;
-  }
 
-  if ((aconf = find_conf_by_address(host, piphost, CONF_KLINE, t, user, NULL, 0)))
+  if ((conf = find_conf_by_address(host, piphost, CONF_KLINE, aftype, user, NULL, 0)))
   {
-    if (IsConfTemporary(aconf))
+    if (IsConfDatabase(conf))
     {
-      delete_one_address_conf(host, aconf);
+      delete_one_address_conf(host, conf);
       return 1;
     }
   }
@@ -555,7 +485,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1871 $",
+  .version = "$Revision: 2339 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
