@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_links.c 1546 2012-09-30 15:24:00Z michael $
+ *  $Id: m_links.c 2342 2013-07-03 13:59:41Z michael $
  */
 
 #include "stdinc.h"
@@ -38,7 +38,9 @@
 static void
 do_links(struct Client *source_p, int parc, char *parv[])
 {
-  sendto_realops_flags(UMODE_SPY, L_ALL,
+  dlink_node *ptr = NULL;
+
+  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
                        "LINKS requested by %s (%s@%s) [%s]",
                        source_p->name,
                        source_p->username, source_p->host,
@@ -48,7 +50,6 @@ do_links(struct Client *source_p, int parc, char *parv[])
   {
     const char *mask = (parc > 2 ? parv[2] : parv[1]);
     const char *me_name, *nick;
-    dlink_node *ptr;
 
     me_name = ID_or_name(&me, source_p->from);
     nick = ID_or_name(source_p, source_p->from);
@@ -62,7 +63,11 @@ do_links(struct Client *source_p, int parc, char *parv[])
         if (!HasUMode(source_p, UMODE_OPER))
           continue;
 
-      if (!EmptyString(mask) && !match(mask, target_p->name))
+      if (HasFlag(target_p, FLAGS_SERVICE) && ConfigServerHide.hide_services)
+        if (!HasUMode(source_p, UMODE_OPER))
+          continue;
+
+      if (!EmptyString(mask) && match(mask, target_p->name))
         continue;
 
       /*
@@ -89,7 +94,12 @@ do_links(struct Client *source_p, int parc, char *parv[])
                ID_or_name(&me, source_p->from),
                ID_or_name(source_p, source_p->from),
                me.name, me.name, 0, me.info);
-    send_message_file(source_p, &ConfigFileEntry.linksfile);
+
+    DLINK_FOREACH(ptr, flatten_links.head)
+      sendto_one(source_p, ":%s %d %s %s",
+                 ID_or_name(&me, source_p->from), RPL_LINKS,
+                 ID_or_name(source_p, source_p->from),
+                 ptr->data);
     sendto_one(source_p, form_str(RPL_ENDOFLINKS),
                ID_or_name(&me, source_p->from),
                ID_or_name(source_p, source_p->from), "*");
@@ -101,7 +111,7 @@ mo_links(struct Client *client_p, struct Client *source_p,
          int parc, char *parv[])
 {
   if (parc > 2)
-    if (!ConfigFileEntry.disable_remote || HasUMode(source_p, UMODE_OPER))
+    if (!ConfigServerHide.disable_remote_commands || HasUMode(source_p, UMODE_OPER))
       if (hunt_server(client_p, source_p, ":%s LINKS %s :%s", 1,
                       parc, parv) != HUNTED_ISME)
         return;
@@ -159,8 +169,7 @@ ms_links(struct Client *client_p, struct Client *source_p,
                   parc, parv) != HUNTED_ISME)
     return;
 
-  if (IsClient(source_p))
-    m_links(client_p, source_p, parc, parv);
+  m_links(client_p, source_p, parc, parv);
 }
 
 static struct Message links_msgtab = {
@@ -183,7 +192,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1546 $",
+  .version = "$Revision: 2342 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
