@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_locops.c 1474 2012-07-22 14:44:07Z michael $
+ *  $Id: m_locops.c 2639 2013-12-10 18:14:27Z michael $
  */
 
 #include "stdinc.h"
@@ -29,9 +29,7 @@
 #include "numeric.h"
 #include "send.h"
 #include "conf.h"
-#include "s_user.h"
 #include "s_serv.h"
-#include "hash.h"
 #include "parse.h"
 #include "modules.h"
 
@@ -48,6 +46,13 @@ mo_locops(struct Client *client_p, struct Client *source_p,
 {
   const char *message = parv[1];
 
+  if (!HasOFlag(source_p, OPER_FLAG_LOCOPS))
+  {
+    sendto_one(source_p, form_str(ERR_NOPRIVS),
+               me.name, source_p->name, "locops");
+    return;
+  }
+
   if (EmptyString(message))
   {
     sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
@@ -55,9 +60,8 @@ mo_locops(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  sendto_wallops_flags(UMODE_LOCOPS, source_p, "LOCOPS - %s",
-                       message);
-
+  sendto_realops_flags(UMODE_LOCOPS, L_ALL, SEND_LOCOPS, "from: %s: %s",
+                       source_p->name, message);
   cluster_a_line(source_p, "LOCOPS", 0, SHARED_LOCOPS, message);
 }
 
@@ -68,15 +72,18 @@ ms_locops(struct Client *client_p, struct Client *source_p,
   if (parc != 3 || EmptyString(parv[2]))
     return;
 
-  sendto_server(client_p, CAP_CLUSTER, 0, "LOCOPS %s :%s",
-                parv[1], parv[2]);
+  sendto_server(client_p, CAP_TS6|CAP_CLUSTER, NOCAPS, ":%s LOCOPS %s :%s",
+                ID(source_p), parv[1], parv[2]);
+  sendto_server(client_p, CAP_CLUSTER, CAP_TS6, ":%s LOCOPS %s :%s",
+                source_p->name, parv[1], parv[2]);
 
-  if (!IsClient(source_p) || !match(parv[1], me.name))
+  if (!IsClient(source_p) || match(parv[1], me.name))
     return;
 
-  if (find_matching_name_conf(ULINE_TYPE, source_p->servptr->name,
+  if (find_matching_name_conf(CONF_ULINE, source_p->servptr->name,
                               "*", "*", SHARED_LOCOPS))
-    sendto_wallops_flags(UMODE_LOCOPS, source_p, "SLOCOPS - %s", parv[2]);
+    sendto_realops_flags(UMODE_LOCOPS, L_ALL, SEND_LOCOPS, "from: %s: %s",
+                         source_p->name, parv[2]);
 }
 
 static struct Message locops_msgtab = {
@@ -99,7 +106,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1474 $",
+  .version = "$Revision: 2639 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
