@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_lusers.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: m_lusers.c 2342 2013-07-03 13:59:41Z michael $
  */
 
 #include "stdinc.h"
@@ -59,7 +59,7 @@ m_lusers(struct Client *client_p, struct Client *source_p,
 
   last_used = CurrentTime;
 
-  if (parc > 2 && !ConfigFileEntry.disable_remote)
+  if (parc > 2 && !ConfigServerHide.disable_remote_commands)
     if (hunt_server(client_p, source_p, ":%s LUSERS %s :%s", 2,
                     parc, parv) != HUNTED_ISME)
       return;
@@ -81,8 +81,7 @@ ms_lusers(struct Client *client_p, struct Client *source_p,
                     parc, parv) != HUNTED_ISME)
         return;
 
-  if (IsClient(source_p))
-    show_lusers(source_p);
+  show_lusers(source_p);
 }
 
 static struct Message lusers_msgtab = {
@@ -105,7 +104,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1309 $",
+  .version = "$Revision: 2342 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
