@@ -21,7 +21,7 @@
 
 /*! \file m_module.c
  * \brief Includes required functions for processing the MODULE command.
- * \version $Id: m_module.c 1566 2012-10-15 18:31:05Z michael $
+ * \version $Id: m_module.c 1834 2013-04-19 19:50:27Z michael $
  */
 
 #include "stdinc.h"
@@ -165,7 +165,7 @@ mo_module(struct Client *client_p, struct Client *source_p,
       load_conf_modules();
       load_core_modules(0);
 
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "Module Restart: %u modules unloaded, %u modules loaded",
                            modnum, dlink_list_length(&modules_list));
       ilog(LOG_TYPE_IRCD, "Module Restart: %u modules unloaded, %u modules loaded",
@@ -199,7 +199,8 @@ mo_module(struct Client *client_p, struct Client *source_p,
 
     if ((load_one_module(parv[2]) == -1) && check_core)
     {
-      sendto_realops_flags(UMODE_ALL, L_ALL, "Error reloading core "
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                           "Error reloading core "
                            "module: %s: terminating ircd", parv[2]);
       ilog(LOG_TYPE_IRCD, "Error loading core module %s: terminating ircd", parv[2]);
       exit(0);
@@ -216,7 +217,7 @@ mo_module(struct Client *client_p, struct Client *source_p,
     {
       modp = ptr->data;
 
-      if (parc > 2 && !match(parv[2], modp->name))
+      if (!EmptyString(parv[2]) && match(parv[2], modp->name))
         continue;
 
       sendto_one(source_p, form_str(RPL_MODLIST), me.name, source_p->name,
@@ -255,7 +256,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1566 $",
+  .version = "$Revision: 1834 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
