@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_motd.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: m_motd.c 2342 2013-07-03 13:59:41Z michael $
  */
 
 #include "stdinc.h"
@@ -38,11 +38,11 @@
 static void
 do_motd(struct Client *source_p)
 {
-  sendto_realops_flags(UMODE_SPY, L_ALL,
+  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
                        "MOTD requested by %s (%s@%s) [%s]",
                        source_p->name, source_p->username,
                        source_p->host, source_p->servptr->name);
-  send_message_file(source_p, &ConfigFileEntry.motd);
+  motd_send(source_p);
 }
 
 /*
@@ -67,7 +67,7 @@ m_motd(struct Client *client_p, struct Client *source_p,
   last_used = CurrentTime;
 
   /* This is safe enough to use during non hidden server mode */
-  if (!ConfigFileEntry.disable_remote && !ConfigServerHide.hide_servers)
+  if (!ConfigServerHide.disable_remote_commands && !ConfigServerHide.hide_servers)
     if (hunt_server(client_p, source_p, ":%s MOTD :%s", 1,
                     parc, parv) != HUNTED_ISME)
       return;
@@ -92,9 +92,6 @@ static void
 mo_motd(struct Client *client_p, struct Client *source_p,
         int parc, char *parv[])
 {
-  if (!IsClient(source_p))
-    return;
-
   if (hunt_server(client_p, source_p, ":%s MOTD :%s", 1,
                   parc, parv) != HUNTED_ISME)
     return;
@@ -122,7 +119,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1309 $",
+  .version = "$Revision: 2342 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
