@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_names.c 1243 2011-09-30 10:47:53Z michael $
+ *  $Id: m_names.c 1887 2013-04-27 13:36:32Z michael $
  */
 
 #include "stdinc.h"
@@ -29,7 +29,6 @@
 #include "client.h"
 #include "hash.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
@@ -144,9 +143,6 @@ m_names(struct Client *client_p, struct Client *source_p,
 
   if (!EmptyString(para))
   {
-    while (*para == ',')
-      ++para;
-
     if ((s = strchr(para, ',')) != NULL)
       *s = '\0';
 
@@ -188,7 +184,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1243 $",
+  .version = "$Revision: 1887 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
