@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_oper.c 1446 2012-06-25 20:05:12Z michael $
+ *  $Id: m_oper.c 2249 2013-06-16 11:30:30Z michael $
  */
 
 #include "stdinc.h"
@@ -51,13 +51,14 @@ failed_oper_notice(struct Client *source_p, const char *name,
                    const char *reason)
 {
   if (ConfigFileEntry.failed_oper_notice)
-    sendto_realops_flags(UMODE_ALL, L_ALL, "Failed OPER attempt as %s "
-                         "by %s (%s@%s) - %s", name, source_p->name,
-                         source_p->username, source_p->host, reason);
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "Failed OPER attempt as %s by %s (%s@%s) - %s",
+                         name, source_p->name, source_p->username,
+                         source_p->host, reason);
 
-  ilog(LOG_TYPE_OPER, "Failed OPER attempt as %s "
-       "by %s (%s@%s) - %s", name, source_p->name,
-       source_p->username, source_p->host, reason);
+  ilog(LOG_TYPE_OPER, "Failed OPER attempt as %s by %s (%s@%s) - %s",
+       name, source_p->name, source_p->username,
+       source_p->host, reason);
 }
 
 /*
@@ -70,8 +71,7 @@ static void
 m_oper(struct Client *client_p, struct Client *source_p,
        int parc, char *parv[])
 {
-  struct ConfItem *conf;
-  struct AccessItem *aconf=NULL;
+  struct MaskItem *conf = NULL;
   const char *name = parv[1];
   const char *password = parv[2];
 
@@ -86,18 +86,33 @@ m_oper(struct Client *client_p, struct Client *source_p,
   if (!IsFloodDone(source_p))
     flood_endgrace(source_p);
 
-  if ((conf = find_exact_name_conf(OPER_TYPE, source_p, name, NULL, NULL)) == NULL)
+  if ((conf = find_exact_name_conf(CONF_OPER, source_p, name, NULL, NULL)) == NULL)
   {
     sendto_one(source_p, form_str(ERR_NOOPERHOST), me.name, source_p->name);
-    conf = find_exact_name_conf(OPER_TYPE, NULL, name, NULL, NULL);
+    conf = find_exact_name_conf(CONF_OPER, NULL, name, NULL, NULL);
     failed_oper_notice(source_p, name, (conf != NULL) ?
                        "host mismatch" : "no oper {} block");
     return;
   }
 
-  aconf = map_to_conf(conf);
+  if (IsConfSSL(conf) && !HasUMode(source_p, UMODE_SSL))
+  {
+    sendto_one(source_p, form_str(ERR_NOOPERHOST), me.name, source_p->name);
+    failed_oper_notice(source_p, name, "requires SSL/TLS");
+    return;
+  }
 
-  if (match_conf_password(password, aconf))
+  if (!EmptyString(conf->certfp))
+  {
+    if (EmptyString(source_p->certfp) || strcasecmp(source_p->certfp, conf->certfp))
+    {
+      sendto_one(source_p, form_str(ERR_NOOPERHOST), me.name, source_p->name);
+      failed_oper_notice(source_p, name, "client certificate fingerprint mismatch");
+      return;
+    }
+  }
+
+  if (match_conf_password(password, conf))
   {
     if (attach_conf(source_p, conf) != 0)
     {
@@ -107,6 +122,7 @@ m_oper(struct Client *client_p, struct Client *source_p,
       return;
     }
 
+    ++conf->count;
     oper_up(source_p);
 
     ilog(LOG_TYPE_OPER, "OPER %s by %s!%s@%s",
@@ -153,7 +169,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1446 $",
+  .version = "$Revision: 2249 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
