@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_pass.c 1569 2012-10-16 18:46:53Z michael $
+ *  $Id: m_pass.c 1834 2013-04-19 19:50:27Z michael $
  */
 
 #include "stdinc.h"
@@ -32,6 +32,8 @@
 #include "modules.h"
 #include "s_serv.h"
 #include "s_user.h"
+#include "s_misc.h"
+#include "memory.h"
 
 
 /*
@@ -47,11 +49,9 @@ static void
 mr_pass(struct Client *client_p, struct Client *source_p,
         int parc, char *parv[])
 {
-  char *password = parv[1];
-
   assert(client_p == source_p);
 
-  if (EmptyString(password))
+  if (EmptyString(parv[1]))
   {
     sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name,
                source_p->name[0] ? source_p->name : "*", "PASS");
@@ -59,9 +59,7 @@ mr_pass(struct Client *client_p, struct Client *source_p,
   }
 
   MyFree(source_p->localClient->passwd);
-  if (strlen(password) > PASSWDLEN)
-    password[PASSWDLEN] = '\0';
-  DupString(source_p->localClient->passwd, password);
+  source_p->localClient->passwd = xstrndup(parv[1], IRCD_MIN(strlen(parv[1]), PASSWDLEN));
 
   if (parc > 2)
   {
@@ -107,7 +105,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1569 $",
+  .version = "$Revision: 1834 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
