@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_ping.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: m_ping.c 2225 2013-06-12 18:11:29Z michael $
  */
 
 #include "stdinc.h"
@@ -58,7 +58,7 @@ m_ping(struct Client *client_p, struct Client *source_p,
   origin = parv[1];
   destination = parv[2]; /* Will get NULL or pointer (parc >= 2!!) */
 
-  if (ConfigFileEntry.disable_remote && !HasUMode(source_p, UMODE_OPER))
+  if (ConfigServerHide.disable_remote_commands && !HasUMode(source_p, UMODE_OPER))
   {
     sendto_one(source_p, ":%s PONG %s :%s", me.name,
               (destination) ? destination : me.name, origin);
@@ -108,7 +108,7 @@ ms_ping(struct Client *client_p, struct Client *source_p,
   {
     if ((target_p = hash_find_server(destination)))
       sendto_one(target_p, ":%s PING %s :%s", source_p->name,
-		 origin, destination);
+                 origin, destination);
     else
     {
       sendto_one(source_p, form_str(ERR_NOSUCHSERVER),
@@ -141,7 +141,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1309 $",
+  .version = "$Revision: 2225 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
