@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_resv.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: m_resv.c 2339 2013-07-03 13:24:43Z michael $
  */
 
 #include "stdinc.h"
@@ -74,7 +74,7 @@ mo_resv(struct Client *client_p, struct Client *source_p,
 			 "RESV %s %s :%s",
 			 target_server, resv, reason);
     /* Allow ON to apply local resv as well if it matches */
-    if (!match(target_server, me.name))
+    if (match(target_server, me.name))
       return;
   }
   else
@@ -137,10 +137,10 @@ ms_resv(struct Client *client_p, struct Client *source_p,
                      "RESV %s %s :%s",
                      parv[1], parv[2], parv[3]);
 
-  if (!IsClient(source_p) || !match(parv[1], me.name))
+  if (!IsClient(source_p) || match(parv[1], me.name))
     return;
 
-  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(ULINE_TYPE, source_p->servptr->name,
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(CONF_ULINE, source_p->servptr->name,
                               source_p->username, source_p->host,
                               SHARED_RESV))
     parse_resv(source_p, parv[2], 0, parv[3]);
@@ -171,7 +171,7 @@ mo_unresv(struct Client *client_p, struct Client *source_p,
                        target_server, resv);
 
     /* Allow ON to apply local unresv as well if it matches */
-    if (!match(target_server, me.name))
+    if (match(target_server, me.name))
       return;
   }
   else 
@@ -196,10 +196,10 @@ ms_unresv(struct Client *client_p, struct Client *source_p,
                      "UNRESV %s %s",
                      parv[1], parv[2]);
 
-  if (!IsClient(source_p) || !match(parv[1], me.name))
+  if (!IsClient(source_p) || match(parv[1], me.name))
     return;
 
-  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(ULINE_TYPE, source_p->servptr->name,
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(CONF_ULINE, source_p->servptr->name,
                               source_p->username, source_p->host,
                               SHARED_UNRESV))
     remove_resv(source_p, parv[2]);
@@ -217,13 +217,11 @@ ms_unresv(struct Client *client_p, struct Client *source_p,
 static void
 parse_resv(struct Client *source_p, char *name, int tkline_time, char *reason)
 {
-  struct ConfItem *conf = NULL;
-
   if (IsChanPrefix(*name))
   {
-    struct ResvChannel *resv_p;
+    struct MaskItem *conf = NULL;
 
-    if ((conf = create_channel_resv(name, reason, 0)) == NULL)
+    if ((conf = create_resv(name, reason, NULL)) == NULL)
     {
       sendto_one(source_p,
 	   ":%s NOTICE %s :A RESV has already been placed on channel: %s",
@@ -231,7 +229,8 @@ parse_resv(struct Client *source_p, char *name, int tkline_time, char *reason)
       return;
     }
 
-    resv_p = map_to_conf(conf);
+    conf->setat = CurrentTime;
+    SetConfDatabase(conf);
 
     if (tkline_time != 0)
     {
@@ -240,17 +239,16 @@ parse_resv(struct Client *source_p, char *name, int tkline_time, char *reason)
                  me.name, source_p->name,
 		 tkline_time/60,
                  (MyClient(source_p) ? "local" : "remote"), name);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
 			   "%s has placed a %d minute %s RESV on channel: %s [%s]",
 			   get_oper_name(source_p),
 			   tkline_time/60,
 			   (MyClient(source_p) ? "local" : "remote"),
-			   resv_p->name, resv_p->reason);
-      ilog(LOG_TYPE_IRCD, "%s added temporary %d min. RESV for [%s] [%s]",
+			   conf->name, conf->reason);
+      ilog(LOG_TYPE_RESV, "%s added temporary %d min. RESV for [%s] [%s]",
 	   source_p->name, (int)tkline_time/60,
-	   conf->name, resv_p->reason);
-      resv_p->hold = CurrentTime + tkline_time;
-      add_temp_line(conf);
+	   conf->name, conf->reason);
+      conf->until = CurrentTime + tkline_time;
     }
     else
     {
@@ -258,17 +256,16 @@ parse_resv(struct Client *source_p, char *name, int tkline_time, char *reason)
 		 ":%s NOTICE %s :A %s RESV has been placed on channel %s",
                  me.name, source_p->name,
                  (MyClient(source_p) ? "local" : "remote"), name);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
 			   "%s has placed a %s RESV on channel %s : [%s]",
 			   get_oper_name(source_p),
 			   (MyClient(source_p) ? "local" : "remote"),
-			   resv_p->name, resv_p->reason);
-      write_conf_line(source_p, conf, NULL /* not used */, 0 /* not used */);
+			   conf->name, conf->reason);
     }
   }
   else
   {
-    struct MatchItem *resv_p = NULL;
+    struct MaskItem *conf = NULL;
 
     if (!valid_wild_card_simple(name))
     {
@@ -277,14 +274,14 @@ parse_resv(struct Client *source_p, char *name, int tkline_time, char *reason)
       return;
     }
 
-    if (!HasUMode(source_p, UMODE_ADMIN) && strpbrk(name, "*?#"))
+    if (!HasUMode(source_p, UMODE_ADMIN) && has_wildcards(name))
     {
       sendto_one(source_p, ":%s NOTICE %s :You must be an admin to perform a "
                  "wildcard RESV", me.name, source_p->name);
       return;
     }
 
-    if ((conf = create_nick_resv(name, reason, 0)) == NULL)
+    if ((conf = create_resv(name, reason, NULL)) == NULL)
     {
       sendto_one(source_p,
                  ":%s NOTICE %s :A RESV has already been placed on nick %s",
@@ -292,7 +289,8 @@ parse_resv(struct Client *source_p, char *name, int tkline_time, char *reason)
       return;
     }
 
-    resv_p = map_to_conf(conf);
+    conf->setat = CurrentTime;
+    SetConfDatabase(conf);
 
     if (tkline_time != 0)
     {
@@ -301,18 +299,17 @@ parse_resv(struct Client *source_p, char *name, int tkline_time, char *reason)
                  me.name, source_p->name,
 		 tkline_time/60,
                  (MyClient(source_p) ? "local" : "remote"),
-                 conf->name, resv_p->reason);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+                 conf->name, conf->reason);
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
 			   "%s has placed a %d minute %s RESV on nick %s : [%s]",
 			   get_oper_name(source_p),
 			   tkline_time/60,
 			   (MyClient(source_p) ? "local" : "remote"),
-			   conf->name, resv_p->reason);
-      ilog(LOG_TYPE_IRCD, "%s added temporary %d min. RESV for [%s] [%s]",
+			   conf->name, conf->reason);
+      ilog(LOG_TYPE_RESV, "%s added temporary %d min. RESV for [%s] [%s]",
 	   source_p->name, (int)tkline_time/60,
-	   conf->name, resv_p->reason);
-      resv_p->hold = CurrentTime + tkline_time;
-      add_temp_line(conf);
+	   conf->name, conf->reason);
+      conf->until = CurrentTime + tkline_time;
     }
     else
     {
@@ -320,13 +317,12 @@ parse_resv(struct Client *source_p, char *name, int tkline_time, char *reason)
 		 ":%s NOTICE %s :A %s RESV has been placed on nick %s : [%s]",
                  me.name, source_p->name,
                  (MyClient(source_p) ? "local" : "remote"),
-                 conf->name, resv_p->reason);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+                 conf->name, conf->reason);
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
 			   "%s has placed a %s RESV on nick %s : [%s]",
 			   get_oper_name(source_p),
 			   (MyClient(source_p) ? "local" : "remote"),
-			   conf->name, resv_p->reason);
-      write_conf_line(source_p, conf, NULL /* not used */, 0 /* not used */);
+			   conf->name, conf->reason);
     }
   }
 }
@@ -334,14 +330,11 @@ parse_resv(struct Client *source_p, char *name, int tkline_time, char *reason)
 static void
 remove_resv(struct Client *source_p, const char *name)
 {
-  struct ConfItem *conf = NULL;
+  struct MaskItem *conf = NULL;
 
   if (IsChanPrefix(*name))
   {
-    struct ResvChannel *resv_p;
-
-    if (resv_channel_list.head == NULL ||
-        !(resv_p = hash_find_resv(name)))
+    if ((conf = find_exact_name_conf(CONF_CRESV, NULL, name, NULL, NULL)) == NULL)
     {
       sendto_one(source_p,
                  ":%s NOTICE %s :A RESV does not exist for channel: %s",
@@ -349,7 +342,7 @@ remove_resv(struct Client *source_p, const char *name)
       return;
     }
 
-    if (resv_p->conf)
+    if (!IsConfDatabase(conf))
     {
       sendto_one(source_p,
                  ":%s NOTICE %s :The RESV for channel: %s is in ircd.conf and must be removed by hand.",
@@ -357,30 +350,26 @@ remove_resv(struct Client *source_p, const char *name)
       return;
     }
 
-    delete_channel_resv(resv_p);
-    remove_conf_line(CRESV_TYPE, source_p, name, NULL);
-
+    conf_free(conf);
     sendto_one(source_p,
                ":%s NOTICE %s :The RESV has been removed on channel: %s",
                me.name, source_p->name, name);
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "%s has removed the RESV for channel: %s",
                          get_oper_name(source_p), name);
+    ilog(LOG_TYPE_RESV, "%s removed RESV for [%s]",
+         get_oper_name(source_p), name);
   }
   else
   {
-    struct MatchItem *resv_p = NULL;
-
-    if ((conf = find_exact_name_conf(NRESV_TYPE, NULL, name, NULL, NULL)) == NULL)
+    if ((conf = find_exact_name_conf(CONF_NRESV, NULL, name, NULL, NULL)) == NULL)
     {
       sendto_one(source_p, ":%s NOTICE %s :A RESV does not exist for nick: %s",
                  me.name, source_p->name, name);
       return;
     }
 
-    resv_p = map_to_conf(conf);
-
-    if (resv_p->action)
+    if (!IsConfDatabase(conf))
     {
       sendto_one(source_p,
                  ":%s NOTICE %s :The RESV for nick: %s is in ircd.conf and must be removed by hand.",
@@ -388,14 +377,14 @@ remove_resv(struct Client *source_p, const char *name)
       return;
     }
 
-    delete_conf_item(conf);
-    remove_conf_line(NRESV_TYPE, source_p, name, NULL);
-
+    conf_free(conf);
     sendto_one(source_p, ":%s NOTICE %s :The RESV has been removed on nick: %s",
                me.name, source_p->name, name);
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "%s has removed the RESV for nick: %s",
                          get_oper_name(source_p), name);
+    ilog(LOG_TYPE_RESV, "%s removed RESV for [%s]",
+         get_oper_name(source_p), name);
   }
 }
 
@@ -426,7 +415,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1309 $",
+  .version = "$Revision: 2339 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
