@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_set.c 1732 2013-01-11 12:57:19Z michael $
+ *  $Id: m_set.c 2434 2013-08-02 18:43:30Z michael $
  */
 
 /* rewritten by jdc */
@@ -42,117 +42,22 @@
 #include "s_misc.h"
 
 
-/* Structure used for the SET table itself */
-struct SetStruct
-{
-  const char *name;
-  void (*handler)();
-  const int wants_char; /* 1 if it expects (char *, [int]) */
-  const int wants_int;  /* 1 if it expects ([char *], int) */
-  /* eg:  0, 1 == only an int arg
-   * eg:  1, 1 == char and int args */
-};
-
-static void quote_autoconn(struct Client *, const char *, int);
-static void quote_autoconnall(struct Client *, int);
-static void quote_floodcount(struct Client *, int);
-static void quote_identtimeout(struct Client *, int);
-static void quote_max(struct Client *, int);
-static void quote_msglocale(struct Client *, char *);
-static void quote_spamnum(struct Client *, int);
-static void quote_spamtime(struct Client *, int);
-static void quote_splitmode(struct Client *, char *);
-static void quote_splitnum(struct Client *, int);
-static void quote_splitusers(struct Client *, int);
-static void list_quote_commands(struct Client *);
-static void quote_jfloodtime(struct Client *, int);
-static void quote_jfloodcount(struct Client *, int);
-
-/* 
- * If this ever needs to be expanded to more than one arg of each
- * type, want_char/want_int could be the count of the arguments,
- * instead of just a boolean flag...
- *
- * -davidt
- */
-
-static const struct SetStruct set_cmd_table[] =
-{
-  /* name		function        string arg  int arg */
-  /* -------------------------------------------------------- */
-  { "AUTOCONN",		quote_autoconn,		1,	1 },
-  { "AUTOCONNALL",	quote_autoconnall,	0,	1 },
-  { "FLOODCOUNT",	quote_floodcount,	0,	1 },
-  { "IDENTTIMEOUT",	quote_identtimeout,	0,	1 },
-  { "MAX",		quote_max,		0,	1 },
-  { "MSGLOCALE",	quote_msglocale,	1,	0 },
-  { "SPAMNUM",		quote_spamnum,		0,	1 },
-  { "SPAMTIME",		quote_spamtime,		0,	1 },
-  { "SPLITMODE",	quote_splitmode,	1,	0 },
-  { "SPLITNUM",		quote_splitnum,		0,	1 },
-  { "SPLITUSERS",	quote_splitusers,	0,	1 },
-  { "JFLOODTIME",	quote_jfloodtime,	0,	1 },
-  { "JFLOODCOUNT",	quote_jfloodcount,	0,	1 },
-  /* -------------------------------------------------------- */
-  { NULL,               NULL,                   0,      0 }
-};
-
-/*
- * list_quote_commands() sends the client all the available commands.
- * Four to a line for now.
- */
-static void
-list_quote_commands(struct Client *source_p)
-{
-  int j = 0;
-  const struct SetStruct *tab = set_cmd_table;
-  const char *names[4] = { "", "", "", "" };
-
-  sendto_one(source_p, ":%s NOTICE %s :Available QUOTE SET commands:",
-             me.name, source_p->name);
-
-  for (; tab->handler; ++tab)
-  {
-    names[j++] = tab->name;
-
-    if (j > 3)
-    {
-      sendto_one(source_p, ":%s NOTICE %s :%s %s %s %s",
-                 me.name, source_p->name,
-                 names[0], names[1], 
-                 names[2], names[3]);
-      j = 0;
-      names[0] = names[1] = names[2] = names[3] = "";
-    }
-
-  }
-
-  if (j)
-    sendto_one(source_p, ":%s NOTICE %s :%s %s %s %s",
-               me.name, source_p->name,
-               names[0], names[1], 
-               names[2], names[3]);
-}
-
 /* SET AUTOCONN */
 static void
 quote_autoconn(struct Client *source_p, const char *arg, int newval)
 {
-  struct AccessItem *aconf;
-
   if (arg != NULL)
   {
-    struct ConfItem *conf = find_exact_name_conf(SERVER_TYPE, NULL, arg, NULL, NULL);
+    struct MaskItem *conf = find_exact_name_conf(CONF_SERVER, NULL, arg, NULL, NULL);
 
     if (conf != NULL)
     {
-      aconf = map_to_conf(conf);
       if (newval)
-        SetConfAllowAutoConn(aconf);
+        SetConfAllowAutoConn(conf);
       else
-        ClearConfAllowAutoConn(aconf);
+        ClearConfAllowAutoConn(conf);
 
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "%s has changed AUTOCONN for %s to %i",
                            get_oper_name(source_p), arg, newval);
       sendto_one(source_p,
@@ -178,7 +83,8 @@ quote_autoconnall(struct Client *source_p, int newval)
 {
   if (newval >= 0)
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL, "%s has changed AUTOCONNALL to %i",
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "%s has changed AUTOCONNALL to %i",
                          get_oper_name(source_p), newval);
 
     GlobalSetOptions.autoconn = newval;
@@ -195,7 +101,7 @@ quote_floodcount(struct Client *source_p, int newval)
   if (newval >= 0)
   {
     GlobalSetOptions.floodcount = newval;
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "%s has changed FLOODCOUNT to %i",
                          get_oper_name(source_p), GlobalSetOptions.floodcount);
   }
@@ -217,7 +123,7 @@ quote_identtimeout(struct Client *source_p, int newval)
 
   if (newval > 0)
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "%s has changed IDENTTIMEOUT to %d",
                          get_oper_name(source_p), newval);
     GlobalSetOptions.ident_timeout = newval;
@@ -251,7 +157,7 @@ quote_max(struct Client *source_p, int newval)
 
     ServerInfo.max_clients = newval;
 
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
         "%s set new MAXCLIENTS to %d (%d current)",
         get_oper_name(source_p), ServerInfo.max_clients, Count.local);
   }
@@ -260,22 +166,6 @@ quote_max(struct Client *source_p, int newval)
                me.name, source_p->name, ServerInfo.max_clients, Count.local);
 }
 
-/* SET MSGLOCALE */
-static void
-quote_msglocale(struct Client *source_p, char *locale)
-{
-  if (locale != NULL)
-  {
-    set_locale(locale);
-    rebuild_isupport_message_line();
-    sendto_one(source_p, ":%s NOTICE %s :Set MSGLOCALE to '%s'",
-               me.name, source_p->name, get_locale());
-  }
-  else
-    sendto_one(source_p, ":%s NOTICE %s :MSGLOCALE is currently '%s'",
-               me.name, source_p->name, get_locale());
-}
-
 /* SET SPAMNUM */
 static void
 quote_spamnum(struct Client *source_p, int newval)
@@ -284,14 +174,15 @@ quote_spamnum(struct Client *source_p, int newval)
   {
     if (newval == 0)
     {
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "%s has disabled ANTI_SPAMBOT", source_p->name);
       GlobalSetOptions.spam_num = newval;
       return;
     }
 
     GlobalSetOptions.spam_num = IRCD_MAX(newval, MIN_SPAM_NUM);
-    sendto_realops_flags(UMODE_ALL, L_ALL, "%s has changed SPAMNUM to %i",
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "%s has changed SPAMNUM to %i",
                          get_oper_name(source_p), GlobalSetOptions.spam_num);
   }
   else
@@ -306,7 +197,8 @@ quote_spamtime(struct Client *source_p, int newval)
   if (newval > 0)
   {
     GlobalSetOptions.spam_time = IRCD_MAX(newval, MIN_SPAM_TIME);
-    sendto_realops_flags(UMODE_ALL, L_ALL, "%s has changed SPAMTIME to %i",
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "%s has changed SPAMTIME to %i",
                          get_oper_name(source_p), GlobalSetOptions.spam_time);
   }
   else
@@ -348,7 +240,7 @@ quote_splitmode(struct Client *source_p, char *charval)
     /* OFF */
     if (newval == 0)
     {
-      sendto_realops_flags(UMODE_ALL, L_ALL, 
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "%s is disabling splitmode",
                            get_oper_name(source_p));
 
@@ -360,7 +252,7 @@ quote_splitmode(struct Client *source_p, char *charval)
     /* ON */
     else if (newval == 1)
     {
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "%s is enabling and activating splitmode",
                            get_oper_name(source_p));
 
@@ -373,7 +265,7 @@ quote_splitmode(struct Client *source_p, char *charval)
     /* AUTO */
     else if (newval == 2)
     {
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "%s is enabling automatic splitmode",
                            get_oper_name(source_p));
 
@@ -397,7 +289,7 @@ quote_splitnum(struct Client *source_p, int newval)
 {
   if (newval >= 0)
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "%s has changed SPLITNUM to %i", 
                          get_oper_name(source_p), newval);
     split_servers = newval;
@@ -416,7 +308,7 @@ quote_splitusers(struct Client *source_p, int newval)
 {
   if (newval >= 0)
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "%s has changed SPLITUSERS to %i", 
                          get_oper_name(source_p), newval);
     split_users = newval;
@@ -435,7 +327,7 @@ quote_jfloodtime(struct Client *source_p, int newval)
 {
   if (newval >= 0)
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "%s has changed JFLOODTIME to %i", 
                          get_oper_name(source_p), newval);
     GlobalSetOptions.joinfloodtime = newval;
@@ -451,7 +343,7 @@ quote_jfloodcount(struct Client *source_p, int newval)
 {
   if (newval >= 0)
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "%s has changed JFLOODCOUNT to %i", 
                          get_oper_name(source_p), newval);
     GlobalSetOptions.joinfloodcount = newval;
@@ -461,6 +353,81 @@ quote_jfloodcount(struct Client *source_p, int newval)
                me.name, source_p->name, GlobalSetOptions.joinfloodcount);
 }
 
+/* Structure used for the SET table itself */
+struct SetStruct
+{
+  const char *name;
+  void (*handler)();
+  const int wants_char; /* 1 if it expects (char *, [int]) */
+  const int wants_int;  /* 1 if it expects ([char *], int) */
+  /* eg:  0, 1 == only an int arg
+   * eg:  1, 1 == char and int args */
+};
+
+/*
+ * If this ever needs to be expanded to more than one arg of each
+ * type, want_char/want_int could be the count of the arguments,
+ * instead of just a boolean flag...
+ *
+ * -davidt
+ */
+static const struct SetStruct set_cmd_table[] =
+{
+  /* name               function        string arg  int arg */
+  /* -------------------------------------------------------- */
+  { "AUTOCONN",         quote_autoconn,         1,      1 },
+  { "AUTOCONNALL",      quote_autoconnall,      0,      1 },
+  { "FLOODCOUNT",       quote_floodcount,       0,      1 },
+  { "IDENTTIMEOUT",     quote_identtimeout,     0,      1 },
+  { "MAX",              quote_max,              0,      1 },
+  { "SPAMNUM",          quote_spamnum,          0,      1 },
+  { "SPAMTIME",         quote_spamtime,         0,      1 },
+  { "SPLITMODE",        quote_splitmode,        1,      0 },
+  { "SPLITNUM",         quote_splitnum,         0,      1 },
+  { "SPLITUSERS",       quote_splitusers,       0,      1 },
+  { "JFLOODTIME",       quote_jfloodtime,       0,      1 },
+  { "JFLOODCOUNT",      quote_jfloodcount,      0,      1 },
+  /* -------------------------------------------------------- */
+  { NULL,               NULL,                   0,      0 }
+};
+
+/*
+ * list_quote_commands() sends the client all the available commands.
+ * Four to a line for now.
+ */
+static void
+list_quote_commands(struct Client *source_p)
+{
+  int j = 0;
+  const struct SetStruct *tab = set_cmd_table;
+  const char *names[4] = { "", "", "", "" };
+
+  sendto_one(source_p, ":%s NOTICE %s :Available QUOTE SET commands:",
+             me.name, source_p->name);
+
+  for (; tab->handler; ++tab)
+  {
+    names[j++] = tab->name;
+
+    if (j > 3)
+    {
+      sendto_one(source_p, ":%s NOTICE %s :%s %s %s %s",
+                 me.name, source_p->name,
+                 names[0], names[1],
+                 names[2], names[3]);
+      j = 0;
+      names[0] = names[1] = names[2] = names[3] = "";
+    }
+
+  }
+
+  if (j)
+    sendto_one(source_p, ":%s NOTICE %s :%s %s %s %s",
+               me.name, source_p->name,
+               names[0], names[1],
+               names[2], names[3]);
+}
+
 /*
  * mo_set - SET command handler
  * set options while running
@@ -588,7 +555,7 @@ mo_set(struct Client *client_p, struct Client *source_p,
 
 static struct Message set_msgtab = {
   "SET", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
-  {m_unregistered, m_not_oper, rfc1459_command_send_error, m_ignore, mo_set, m_ignore}
+  {m_unregistered, m_not_oper, m_ignore, m_ignore, mo_set, m_ignore}
 };
 
 static void
@@ -606,7 +573,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1732 $",
+  .version = "$Revision: 2434 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
