@@ -19,23 +19,23 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_stats.c 1879 2013-04-27 10:41:48Z michael $
+ *  $Id: m_stats.c 2690 2013-12-17 18:55:43Z michael $
  */
 
 #include "stdinc.h"
 #include "list.h"	 /* dlink_node/dlink_list */
-#include "balloc.h"
 #include "client.h"      /* Client */
 #include "irc_string.h"  
 #include "ircd.h"        /* me */
 #include "listener.h"    /* show_ports */
 #include "s_gline.h"
+#include "conf.h"
+#include "conf_class.h"
 #include "hostmask.h"
 #include "numeric.h"     /* ERR_xxx */
 #include "send.h"        /* sendto_one */
 #include "fdlist.h"      /* PF and friends */
 #include "s_bsd.h"       /* highest_fd */
-#include "conf.h"      /* AccessItem, report_configured_links */
 #include "s_misc.h"      /* serv_info */
 #include "s_serv.h"      /* hunt_server */
 #include "s_user.h"      /* show_opers */
@@ -48,10 +48,196 @@
 #include "whowas.h"
 #include "watch.h"
 #include "irc_res.h"
+#include "motd.h"
 
 
 const char *from, *to;
 
+static const struct shared_flags
+{
+  const unsigned int type;
+  const unsigned char letter;
+} flag_table[] = {
+  { SHARED_KLINE,   'K' },
+  { SHARED_UNKLINE, 'U' },
+  { SHARED_XLINE,   'X' },
+  { SHARED_UNXLINE, 'Y' },
+  { SHARED_RESV,    'Q' },
+  { SHARED_UNRESV,  'R' },
+  { SHARED_LOCOPS,  'L' },
+  { SHARED_DLINE,   'D' },
+  { SHARED_UNDLINE, 'E' },
+  { 0, '\0' }
+};
+
+/*
+ * inputs	- pointer to client requesting confitem report
+ *		- ConfType to report
+ * output	- none
+ * side effects	-
+ */
+static void
+report_confitem_types(struct Client *source_p, enum maskitem_type type)
+{
+  dlink_node *ptr = NULL;
+  struct MaskItem *conf = NULL;
+  const struct shared_flags *shared = NULL;
+  char buf[IRCD_BUFSIZE];
+  char *p = NULL;
+
+  switch (type)
+  {
+  case CONF_XLINE:
+    DLINK_FOREACH(ptr, xconf_items.head)
+    {
+      conf = ptr->data;
+
+      sendto_one(source_p, form_str(RPL_STATSXLINE),
+		 me.name, source_p->name, 
+		 conf->until ? 'x': 'X', conf->count,
+		 conf->name, conf->reason);
+    }
+    break;
+
+  case CONF_ULINE:
+    shared = flag_table;
+    DLINK_FOREACH(ptr, uconf_items.head)
+    {
+      conf = ptr->data;
+
+      p = buf;
+
+      *p++ = 'c';
+      for (; shared->type; ++shared)
+        if (shared->type & conf->flags)
+          *p++ = shared->letter;
+        else
+          *p++ = ToLower(shared->letter);
+
+      sendto_one(source_p, form_str(RPL_STATSULINE),
+		 me.name, source_p->name, conf->name,
+                 conf->user?conf->user: "*",
+		 conf->host?conf->host: "*", buf);
+    }
+
+    shared = flag_table;
+    DLINK_FOREACH(ptr, cluster_items.head)
+    {
+      conf = ptr->data;
+
+      p = buf;
+
+      *p++ = 'C';
+      for (; shared->type; ++shared)
+        if (shared->type & conf->flags)
+          *p++ = shared->letter;
+        else
+          *p++ = ToLower(shared->letter);
+
+      sendto_one(source_p, form_str(RPL_STATSULINE),
+                 me.name, source_p->name, conf->name,
+                 "*", "*", buf);
+    }
+
+    break;
+
+  case CONF_OPER:
+    DLINK_FOREACH(ptr, oconf_items.head)
+    {
+      conf = ptr->data;
+
+      /* Don't allow non opers to see oper privs */
+      if (HasUMode(source_p, UMODE_OPER))
+	sendto_one(source_p, form_str(RPL_STATSOLINE),
+		   me.name, source_p->name, 'O', conf->count, conf->user, conf->host,
+		   conf->name, oper_privs_as_string(conf->port),
+		   conf->class ? conf->class->name : "<default>");
+      else
+	sendto_one(source_p, form_str(RPL_STATSOLINE),
+		   me.name, source_p->name, 'O', conf->count, conf->user, conf->host,
+                   conf->name, "0",
+		   conf->class ? conf->class->name : "<default>");
+    }
+    break;
+
+  case CONF_SERVICE:
+    DLINK_FOREACH(ptr, service_items.head)
+    {
+      conf = ptr->data;
+      sendto_one(source_p, form_str(RPL_STATSSERVICE),
+                 me.name, source_p->name, 'S', "*", conf->name, 0, 0);
+    }
+    break;
+
+  case CONF_SERVER:
+    DLINK_FOREACH(ptr, server_items.head)
+    {
+      p = buf;
+      conf = ptr->data;
+
+      buf[0] = '\0';
+
+      if (IsConfAllowAutoConn(conf))
+	*p++ = 'A';
+      if (IsConfSSL(conf))
+        *p++ = 'S';
+      if (buf[0] == '\0')
+	*p++ = '*';
+
+      *p = '\0';
+
+      /*
+       * Allow admins to see actual ips unless hide_server_ips is enabled
+       */
+      if (!ConfigServerHide.hide_server_ips && HasUMode(source_p, UMODE_ADMIN))
+	sendto_one(source_p, form_str(RPL_STATSCLINE),
+		   me.name, source_p->name, 'C', conf->host,
+		   buf, conf->name, conf->port,
+		   conf->class ? conf->class->name : "<default>");
+        else
+          sendto_one(source_p, form_str(RPL_STATSCLINE),
+                     me.name, source_p->name, 'C',
+		     "*@127.0.0.1", buf, conf->name, conf->port,
+		     conf->class ? conf->class->name : "<default>");
+    }
+    break;
+
+  default:
+    break;
+  }
+}
+
+/* report_resv()
+ *
+ * inputs       - pointer to client pointer to report to.
+ * output       - NONE
+ * side effects - report all resvs to client.
+ */
+static void
+report_resv(struct Client *source_p)
+{
+  dlink_node *ptr = NULL;
+  struct MaskItem *conf = NULL;
+
+  DLINK_FOREACH(ptr, cresv_items.head)
+  {
+    conf = ptr->data;
+    sendto_one(source_p, form_str(RPL_STATSQLINE),
+               me.name, source_p->name,
+               conf->until ? 'q' : 'Q', conf->count,
+               conf->name, conf->reason);
+  }
+
+  DLINK_FOREACH(ptr, nresv_items.head)
+  {
+    conf = ptr->data;
+    sendto_one(source_p, form_str(RPL_STATSQLINE),
+               me.name, source_p->name,
+               conf->until ? 'q' : 'Q', conf->count,
+               conf->name, conf->reason);
+  }
+}
+
 /*
  * This is part of the STATS replies. There is no offical numeric for this
  * since this isnt an official command, in much the same way as HASH isnt.
@@ -93,7 +279,7 @@ stats_usage(struct Client *source_p, int parc, char *parv[])
     rup = 1;
 
   sendto_one(source_p,
-             ":%s %d %s R :CPU Secs %d:%d User %d:%d System %d:%d",
+             ":%s %d %s R :CPU Secs %d:%02d User %d:%02d System %d:%02d",
              me.name, RPL_STATSDEBUG, source_p->name, (int)(secs/60), (int)(secs%60),
              (int)(rus.ru_utime.tv_sec/60), (int)(rus.ru_utime.tv_sec%60),
              (int)(rus.ru_stime.tv_sec/60), (int)(rus.ru_stime.tv_sec%60));
@@ -265,12 +451,12 @@ stats_memory(struct Client *source_p, int parc, char *parv[])
   }
 #endif
   /* count up all classes */
-  class_count = dlink_list_length(&class_items);
+  class_count = dlink_list_length(class_get_list());
 
-  count_whowas_memory(&wwu, &wwm);
+  whowas_count_memory(&wwu, &wwm);
   watch_count_memory(&watch_list_headers, &watch_list_memory);
 
-  sendto_one(source_p, ":%s %d %s z :WATCH headers %u(%u) entries %d(%u)",
+  sendto_one(source_p, ":%s %d %s z :WATCH headers %u(%llu) entries %d(%u)",
              me.name, RPL_STATSDEBUG, source_p->name, watch_list_headers,
              watch_list_memory, watch_list_entries,
              watch_list_entries * sizeof(dlink_node) * 2);
@@ -290,10 +476,10 @@ stats_memory(struct Client *source_p, int parc, char *parv[])
 
   sendto_one(source_p, ":%s %d %s z :Resv channels %u(%lu) nicks %u(%lu)",
              me.name, RPL_STATSDEBUG, source_p->name,
-             dlink_list_length(&resv_channel_list),
-             dlink_list_length(&resv_channel_list) * sizeof(struct ResvChannel),
+             dlink_list_length(&cresv_items),
+             dlink_list_length(&cresv_items) * sizeof(struct MaskItem),
              dlink_list_length(&nresv_items),
-             dlink_list_length(&nresv_items) * sizeof(struct MatchItem));
+             dlink_list_length(&nresv_items) * sizeof(struct MaskItem));
 
   sendto_one(source_p, ":%s %d %s z :Classes %u(%llu)",
              me.name, RPL_STATSDEBUG, source_p->name,
@@ -341,6 +527,7 @@ stats_memory(struct Client *source_p, int parc, char *parv[])
 
   totww = wwu * sizeof(struct Client) + wwm;
 
+  motd_memory_count(source_p);
   count_ip_hash(&number_ips_stored,&mem_ips_stored);
   sendto_one(source_p, ":%s %d %s z :iphash %u(%llu)",
              me.name, RPL_STATSDEBUG, source_p->name,
@@ -364,8 +551,6 @@ stats_memory(struct Client *source_p, int parc, char *parv[])
              me.name, RPL_STATSDEBUG, source_p->name, remote_client_count,
              remote_client_memory_used);
 
-  block_heap_report_stats(source_p);
-
   sendto_one(source_p,
              ":%s %d %s z :TOTAL: %llu",
              me.name, RPL_STATSDEBUG, source_p->name,
@@ -381,7 +566,7 @@ stats_dns_servers(struct Client *source_p)
 static void
 stats_connect(struct Client *source_p, int parc, char *parv[])
 {
-  report_confitem_types(source_p, SERVER_TYPE);
+  report_confitem_types(source_p, CONF_SERVER);
 }
 
 /* stats_deny()
@@ -393,7 +578,7 @@ stats_connect(struct Client *source_p, int parc, char *parv[])
 static void
 stats_deny(struct Client *source_p, int parc, char *parv[])
 {
-  struct AccessItem *aconf;
+  struct MaskItem *conf;
   dlink_node *ptr = NULL;
   unsigned int i = 0;
 
@@ -404,18 +589,17 @@ stats_deny(struct Client *source_p, int parc, char *parv[])
     {
       struct AddressRec *arec = ptr->data;
 
-      if (arec->type == CONF_DLINE)
-      {
-        aconf = arec->aconf;
+      if (arec->type != CONF_DLINE)
+        continue;
 
-        /* dont report a tdline as a dline */
-        if (aconf->flags & CONF_FLAGS_TEMPORARY)
-          continue;
+      conf = arec->conf;
 
-        sendto_one(source_p, form_str(RPL_STATSDLINE),
-                   from, to, 'D', aconf->host, aconf->reason,
-		   aconf->oper_reason ? aconf->oper_reason : "");
-      }
+      /* dont report a tdline as a dline */
+      if (conf->until)
+        continue;
+
+      sendto_one(source_p, form_str(RPL_STATSDLINE),
+                 from, to, 'D', conf->host, conf->reason);
     }
   }
 }
@@ -429,7 +613,7 @@ stats_deny(struct Client *source_p, int parc, char *parv[])
 static void
 stats_tdeny(struct Client *source_p, int parc, char *parv[])
 {
-  struct AccessItem *aconf;
+  struct MaskItem *conf = NULL;
   dlink_node *ptr = NULL;
   unsigned int i = 0;
 
@@ -440,18 +624,17 @@ stats_tdeny(struct Client *source_p, int parc, char *parv[])
     {
       struct AddressRec *arec = ptr->data;
 
-      if (arec->type == CONF_DLINE)
-      {
-        aconf = arec->aconf;
+      if (arec->type != CONF_DLINE)
+        continue;
 
-        /* dont report a permanent dline as a tdline */
-        if (!(aconf->flags & CONF_FLAGS_TEMPORARY))
-          continue;
+      conf = arec->conf;
 
-        sendto_one(source_p, form_str(RPL_STATSDLINE),
-                   from, to, 'd', aconf->host, aconf->reason,
-		   aconf->oper_reason ? aconf->oper_reason : "");
-      }
+      /* dont report a permanent dline as a tdline */
+      if (!conf->until)
+        continue;
+
+      sendto_one(source_p, form_str(RPL_STATSDLINE),
+                 from, to, 'd', conf->host, conf->reason);
     }
   }
 }
@@ -465,7 +648,7 @@ stats_tdeny(struct Client *source_p, int parc, char *parv[])
 static void
 stats_exempt(struct Client *source_p, int parc, char *parv[])
 {
-  struct AccessItem *aconf;
+  struct MaskItem *conf;
   dlink_node *ptr = NULL;
   unsigned int i = 0;
 
@@ -483,14 +666,13 @@ stats_exempt(struct Client *source_p, int parc, char *parv[])
     {
       struct AddressRec *arec = ptr->data;
 
-      if (arec->type == CONF_EXEMPTDLINE)
-      {
-        aconf = arec->aconf;
+      if (arec->type != CONF_EXEMPT)
+        continue;
 
-        sendto_one(source_p, form_str(RPL_STATSDLINE),
-                   from, to, 'e', aconf->host, 
-		   aconf->reason, aconf->oper_reason ? aconf->oper_reason : "");
-      }
+      conf = arec->conf;
+
+      sendto_one(source_p, form_str(RPL_STATSDLINE), from, to, 'e',
+                 conf->host, "");
     }
   }
 }
@@ -617,13 +799,13 @@ stats_glines(struct Client *source_p, int parc, char *parv[])
 
       if (arec->type == CONF_GLINE)
       {
-        const struct AccessItem *aconf = arec->aconf;
+        const struct MaskItem *conf = arec->conf;
 
         sendto_one(source_p, form_str(RPL_STATSKLINE),
-                   from, to, "G",
-                   aconf->host ? aconf->host : "*",
-                   aconf->user ? aconf->user : "*",
-                   aconf->reason ? aconf->reason : CONF_NOREASON, "" );
+                   from, to, 'G',
+                   conf->host ? conf->host : "*",
+                   conf->user ? conf->user : "*",
+                   conf->reason ? conf->reason : CONF_NOREASON);
       }
     }
   }
@@ -632,49 +814,68 @@ stats_glines(struct Client *source_p, int parc, char *parv[])
 static void
 stats_hubleaf(struct Client *source_p, int parc, char *parv[])
 {
-  report_confitem_types(source_p, HUB_TYPE);
-  report_confitem_types(source_p, LEAF_TYPE);
+  const dlink_node *ptr = NULL, *dptr = NULL;
+
+  DLINK_FOREACH(ptr, server_items.head)
+  {
+    const struct MaskItem *conf = ptr->data;
+
+    DLINK_FOREACH(dptr, conf->hub_list.head)
+      sendto_one(source_p, form_str(RPL_STATSHLINE), me.name,
+                 source_p->name, 'H', dptr->data, conf->name, 0, "*");
+  }
+
+  DLINK_FOREACH(ptr, server_items.head)
+  {
+    const struct MaskItem *conf = ptr->data;
+
+    DLINK_FOREACH(dptr, conf->leaf_list.head)
+      sendto_one(source_p, form_str(RPL_STATSLLINE), me.name,
+                 source_p->name, 'L', dptr->data, conf->name, 0, "*");
+  }
 }
 
 /*
  * show_iline_prefix()
  *
  * inputs       - pointer to struct Client requesting output
- *              - pointer to struct AccessItem 
+ *              - pointer to struct MaskItem 
  *              - name to which iline prefix will be prefixed to
  * output       - pointer to static string with prefixes listed in ascii form
  * side effects - NONE
  */
 static const char *
-show_iline_prefix(struct Client *sptr, struct AccessItem *aconf, const char *name)
+show_iline_prefix(const struct Client *sptr, const struct MaskItem *conf)
 {
-  static char prefix_of_host[USERLEN + 14];
+  static char prefix_of_host[USERLEN + 15];
   char *prefix_ptr = prefix_of_host;
 
-  if (IsNoTilde(aconf))
+  if (IsConfWebIRC(conf))
+    *prefix_ptr++ = '<';
+  if (IsNoTilde(conf))
     *prefix_ptr++ = '-';
-  if (IsLimitIp(aconf))
+  if (IsLimitIp(conf))
     *prefix_ptr++ = '!';
-  if (IsNeedIdentd(aconf))
+  if (IsNeedIdentd(conf))
     *prefix_ptr++ = '+';
-  if (!IsNeedPassword(aconf))
+  if (!IsNeedPassword(conf))
     *prefix_ptr++ = '&';
-  if (IsConfExemptResv(aconf))
+  if (IsConfExemptResv(conf))
     *prefix_ptr++ = '$';
-  if (IsNoMatchIp(aconf))
+  if (IsNoMatchIp(conf))
     *prefix_ptr++ = '%';
-  if (IsConfDoSpoofIp(aconf))
+  if (IsConfDoSpoofIp(conf))
     *prefix_ptr++ = '=';
-  if (MyOper(sptr) && IsConfExemptKline(aconf))
+  if (MyOper(sptr) && IsConfExemptKline(conf))
     *prefix_ptr++ = '^';
-  if (MyOper(sptr) && IsConfExemptGline(aconf))
+  if (MyOper(sptr) && IsConfExemptGline(conf))
     *prefix_ptr++ = '_';
-  if (MyOper(sptr) && IsConfExemptLimits(aconf))
+  if (MyOper(sptr) && IsConfExemptLimits(conf))
     *prefix_ptr++ = '>';
-  if (IsConfCanFlood(aconf))
+  if (IsConfCanFlood(conf))
     *prefix_ptr++ = '|';
 
-  strlcpy(prefix_ptr, name, USERLEN+1);
+  strlcpy(prefix_ptr, conf->user, USERLEN+1);
 
   return prefix_of_host;
 }
@@ -682,8 +883,7 @@ show_iline_prefix(struct Client *sptr, struct AccessItem *aconf, const char *nam
 static void
 report_auth(struct Client *client_p, int parc, char *parv[])
 {
-  struct ConfItem *conf;
-  struct AccessItem *aconf;
+  struct MaskItem *conf = NULL;
   dlink_node *ptr = NULL;
   unsigned int i;
 
@@ -694,35 +894,33 @@ report_auth(struct Client *client_p, int parc, char *parv[])
     {
       struct AddressRec *arec = ptr->data;
 
-      if (arec->type == CONF_CLIENT)
-      {
-        aconf = arec->aconf;
+      if (arec->type != CONF_CLIENT)
+        continue;
 
-        if (!MyOper(client_p) && IsConfDoSpoofIp(aconf))
-          continue;
+      conf = arec->conf;
 
-        conf = unmap_conf_item(aconf);
+      if (!MyOper(client_p) && IsConfDoSpoofIp(conf))
+        continue;
 
-        /* We are doing a partial list, based on what matches the u@h of the
-         * sender, so prepare the strings for comparing --fl_
-         */
-        if (ConfigFileEntry.hide_spoof_ips)
-          sendto_one(client_p, form_str(RPL_STATSILINE), me.name,
-                     client_p->name, 'I',
-                     conf->name == NULL ? "*" : conf->name,
-                     show_iline_prefix(client_p, aconf, aconf->user),
-                     IsConfDoSpoofIp(aconf) ? "255.255.255.255" :
-                     aconf->host, aconf->port,
-                     aconf->class_ptr ? aconf->class_ptr->name : "<default>");
+      /* We are doing a partial list, based on what matches the u@h of the
+       * sender, so prepare the strings for comparing --fl_
+       */
+      if (ConfigFileEntry.hide_spoof_ips)
+        sendto_one(client_p, form_str(RPL_STATSILINE), me.name,
+                   client_p->name, 'I',
+                   conf->name == NULL ? "*" : conf->name,
+                   show_iline_prefix(client_p, conf),
+                   IsConfDoSpoofIp(conf) ? "255.255.255.255" :
+                   conf->host, conf->port,
+                   conf->class ? conf->class->name : "<default>");
 
-        else
-          sendto_one(client_p, form_str(RPL_STATSILINE), me.name,
-                     client_p->name, 'I',
-                     conf->name == NULL ? "*" : conf->name,
-                     show_iline_prefix(client_p, aconf, aconf->user),
-                     aconf->host, aconf->port,
-                     aconf->class_ptr ? aconf->class_ptr->name : "<default>");
-      }
+      else
+        sendto_one(client_p, form_str(RPL_STATSILINE), me.name,
+                   client_p->name, 'I',
+                   conf->name == NULL ? "*" : conf->name,
+                   show_iline_prefix(client_p, conf),
+                   conf->host, conf->port,
+                   conf->class ? conf->class->name : "<default>");
     }
   }
 }
@@ -738,30 +936,27 @@ stats_auth(struct Client *source_p, int parc, char *parv[])
   /* If unopered, Only return matching auth blocks */
   else if ((ConfigFileEntry.stats_i_oper_only == 1) && !HasUMode(source_p, UMODE_OPER))
   {
-    struct ConfItem *conf;
-    struct AccessItem *aconf;
+    struct MaskItem *conf;
 
     if (MyConnect(source_p))
-      aconf = find_conf_by_address(source_p->host,
+      conf = find_conf_by_address(source_p->host,
                                    &source_p->localClient->ip,
 				   CONF_CLIENT,
 				   source_p->localClient->aftype,
 				   source_p->username,
                                    source_p->localClient->passwd, 1);
     else
-      aconf = find_conf_by_address(source_p->host, NULL, CONF_CLIENT,
+      conf = find_conf_by_address(source_p->host, NULL, CONF_CLIENT,
                                    0, source_p->username, NULL, 1);
 
-    if (aconf == NULL)
+    if (conf == NULL)
       return;
 
-    conf = unmap_conf_item(aconf);
-
     sendto_one(source_p, form_str(RPL_STATSILINE), from,
                to, 'I',
-	       "*", show_iline_prefix(source_p, aconf, aconf->user), 
-	       aconf->host, aconf->port,
-	       aconf->class_ptr ? aconf->class_ptr->name : "<default>");
+	       "*", show_iline_prefix(source_p, conf),
+	       conf->host, conf->port,
+	       conf->class ? conf->class->name : "<default>");
   }
   /* They are opered, or allowed to see all auth blocks */
   else
@@ -778,15 +973,15 @@ stats_auth(struct Client *source_p, int parc, char *parv[])
 static void
 report_Klines(struct Client *client_p, int tkline)
 {
-  struct AccessItem *aconf = NULL;
+  struct MaskItem *conf = NULL;
   unsigned int i = 0;
-  const char *p = NULL;
+  char c = '\0';
   dlink_node *ptr = NULL;
 
   if (tkline)
-    p = "k";
+    c = 'k';
   else
-    p = "K";
+    c = 'K';
 
   for (i = 0; i < ATABLE_SIZE; ++i)
   {
@@ -794,21 +989,23 @@ report_Klines(struct Client *client_p, int tkline)
     {
       struct AddressRec *arec = ptr->data;
 
-      if (arec->type == CONF_KLINE)
-      {
-        if ((tkline && !((aconf = arec->aconf)->flags & CONF_FLAGS_TEMPORARY)) ||
-            (!tkline && ((aconf = arec->aconf)->flags & CONF_FLAGS_TEMPORARY)))
-          continue;
+      if (arec->type != CONF_KLINE)
+        continue;
 
-        if (HasUMode(client_p, UMODE_OPER))
-          sendto_one(client_p, form_str(RPL_STATSKLINE), me.name,
-                     client_p->name, p, aconf->host, aconf->user,
-                     aconf->reason, aconf->oper_reason ? aconf->oper_reason : "");
-        else
-          sendto_one(client_p, form_str(RPL_STATSKLINE), me.name,
-                     client_p->name, p, aconf->host, aconf->user,
-                     aconf->reason, "");
-      }
+      conf = arec->conf;
+
+      if ((!tkline && conf->until) ||
+          (tkline && !conf->until))
+        continue;
+
+      if (HasUMode(client_p, UMODE_OPER))
+        sendto_one(client_p, form_str(RPL_STATSKLINE), me.name,
+                   client_p->name, c, conf->host, conf->user,
+                   conf->reason);
+      else
+        sendto_one(client_p, form_str(RPL_STATSKLINE), me.name,
+                   client_p->name, c, conf->host, conf->user,
+                   conf->reason);
     }
   }
 }
@@ -824,27 +1021,27 @@ stats_tklines(struct Client *source_p, int parc, char *parv[])
   /* If unopered, Only return matching klines */
   else if ((ConfigFileEntry.stats_k_oper_only == 1) && !HasUMode(source_p, UMODE_OPER))
   {
-    struct AccessItem *aconf = NULL;
+    struct MaskItem *conf = NULL;
 
     if (MyConnect(source_p))
-      aconf = find_conf_by_address(source_p->host,
-                                   &source_p->localClient->ip,
+      conf = find_conf_by_address(source_p->host,
+                                  &source_p->localClient->ip,
 				   CONF_KLINE,
 				   source_p->localClient->aftype,
 				   source_p->username, NULL, 1);
     else
-      aconf = find_conf_by_address(source_p->host, NULL, CONF_KLINE,
+      conf = find_conf_by_address(source_p->host, NULL, CONF_KLINE,
                                    0, source_p->username, NULL, 1);
 
-    if (aconf == NULL)
+    if (!conf)
       return;
 
     /* dont report a permanent kline as a tkline */
-    if (!(aconf->flags & CONF_FLAGS_TEMPORARY))
+    if (!conf->until)
       return;
 
     sendto_one(source_p, form_str(RPL_STATSKLINE), from,
-               to, "k", aconf->host, aconf->user, aconf->reason, "");
+               to, 'k', conf->host, conf->user, conf->reason);
   }
   /* Theyre opered, or allowed to see all klines */
   else {
@@ -863,34 +1060,32 @@ stats_klines(struct Client *source_p, int parc, char *parv[])
   /* If unopered, Only return matching klines */
   else if ((ConfigFileEntry.stats_k_oper_only == 1) && !HasUMode(source_p, UMODE_OPER))
   {
-    struct AccessItem *aconf = NULL;
+    struct MaskItem *conf = NULL;
 
     /* search for a kline */
     if (MyConnect(source_p))
-      aconf = find_conf_by_address(source_p->host,
+      conf = find_conf_by_address(source_p->host,
                                    &source_p->localClient->ip,
 				   CONF_KLINE,
 				   source_p->localClient->aftype,
 				   source_p->username, NULL, 0);
     else
-      aconf = find_conf_by_address(source_p->host, NULL, CONF_KLINE,
+      conf = find_conf_by_address(source_p->host, NULL, CONF_KLINE,
                                    0, source_p->username, NULL, 0);
 
-    if (aconf == NULL)
+    if (!conf)
       return;
 
     /* dont report a tkline as a kline */
-    if (aconf->flags & CONF_FLAGS_TEMPORARY)
+    if (conf->until)
       return;
       
     sendto_one(source_p, form_str(RPL_STATSKLINE), from,
-               to, "K", aconf->host, aconf->user, aconf->reason, "");
+               to, 'K', conf->host, conf->user, conf->reason);
   }
   /* Theyre opered, or allowed to see all klines */
-  else {
+  else
     report_Klines(source_p, 0);
-    report_confitem_types(source_p, RKLINE_TYPE);
-  }
 }
 
 static void
@@ -906,7 +1101,7 @@ stats_oper(struct Client *source_p, int parc, char *parv[])
     sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
                from, to);
   else
-    report_confitem_types(source_p, OPER_TYPE);
+    report_confitem_types(source_p, CONF_OPER);
 }
 
 /* stats_operedup()
@@ -922,7 +1117,7 @@ stats_operedup(struct Client *source_p, int parc, char *parv[])
 
   DLINK_FOREACH(ptr, oper_list.head)
   {
-    struct Client *target_p = ptr->data;
+    const struct Client *target_p = ptr->data;
 
     if (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER))
       continue;
@@ -965,7 +1160,7 @@ stats_resv(struct Client *source_p, int parc, char *parv[])
 static void
 stats_service(struct Client *source_p, int parc, char *parv[])
 {
-  report_confitem_types(source_p, SERVICE_TYPE);
+  report_confitem_types(source_p, CONF_SERVICE);
 }
 
 static void
@@ -973,10 +1168,9 @@ stats_tstats(struct Client *source_p, int parc, char *parv[])
 {
   const struct Client *target_p = NULL;
   const dlink_node *ptr = NULL;
-  struct ServerStatistics *sp;
   struct ServerStatistics tmp;
+  struct ServerStatistics *sp = &tmp;
 
-  sp = &tmp;
   memcpy(sp, &ServerStats, sizeof(struct ServerStatistics));
 
   /*
@@ -1042,20 +1236,26 @@ stats_tstats(struct Client *source_p, int parc, char *parv[])
 static void
 stats_uptime(struct Client *source_p, int parc, char *parv[])
 {
-  time_t now = CurrentTime - me.localClient->since;
+  if (!HasUMode(source_p, UMODE_OPER) && ConfigFileEntry.stats_u_oper_only)
+    sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
+               from, to);
+  else
+  {
+    time_t now = CurrentTime - me.localClient->since;
 
-  sendto_one(source_p, form_str(RPL_STATSUPTIME), from, to,
-             now / 86400, (now / 3600) % 24, (now / 60) % 60, now % 60);
+    sendto_one(source_p, form_str(RPL_STATSUPTIME), from, to,
+               now / 86400, (now / 3600) % 24, (now / 60) % 60, now % 60);
 
-  if (!ConfigFileEntry.disable_remote || HasUMode(source_p, UMODE_OPER))
-     sendto_one(source_p, form_str(RPL_STATSCONN), from, to,
-                Count.max_loc_con, Count.max_loc_cli, Count.totalrestartcount);
+    if (!ConfigServerHide.disable_remote_commands || HasUMode(source_p, UMODE_OPER))
+       sendto_one(source_p, form_str(RPL_STATSCONN), from, to,
+                  Count.max_loc_con, Count.max_loc_cli, Count.totalrestartcount);
+  }
 }
 
 static void
 stats_shared(struct Client *source_p, int parc, char *parv[])
 {
-  report_confitem_types(source_p, ULINE_TYPE);
+  report_confitem_types(source_p, CONF_ULINE);
 }
 
 /* stats_servers()
@@ -1086,14 +1286,29 @@ stats_servers(struct Client *source_p, int parc, char *parv[])
 static void
 stats_gecos(struct Client *source_p, int parc, char *parv[])
 {
-  report_confitem_types(source_p, XLINE_TYPE);
-  report_confitem_types(source_p, RXLINE_TYPE);
+  report_confitem_types(source_p, CONF_XLINE);
 }
 
 static void
 stats_class(struct Client *source_p, int parc, char *parv[])
 {
-  report_confitem_types(source_p, CLASS_TYPE);
+  const dlink_node *ptr = NULL;
+
+  DLINK_FOREACH(ptr, class_get_list()->head)
+  {
+    const struct ClassItem *class = ptr->data;
+
+    sendto_one(source_p, form_str(RPL_STATSYLINE),
+               me.name, source_p->name, 'Y',
+               class->name, class->ping_freq,
+               class->con_freq,
+               class->max_total, class->max_sendq,
+               class->max_recvq,
+               class->ref_count,
+               class->number_per_cidr, class->cidr_bitlen_ipv4,
+               class->number_per_cidr, class->cidr_bitlen_ipv6,
+               class->active ? "active" : "disabled");
+  }
 }
 
 static void
@@ -1114,6 +1329,10 @@ stats_servlinks(struct Client *source_p, int parc, char *parv[])
   {
     struct Client *target_p = ptr->data;
 
+    if (HasFlag(target_p, FLAGS_SERVICE) && ConfigServerHide.hide_services)
+      if (!HasUMode(source_p, UMODE_OPER))
+        continue;
+
     sendB += target_p->localClient->send.bytes;
     recvB += target_p->localClient->recv.bytes;
 
@@ -1181,7 +1400,7 @@ parse_stats_args(int parc, char *parv[], int *doall, int *wilds)
 
     if (!irccmp(name, from))
       *doall = 2;
-    else if (match(name, from))
+    else if (!match(name, from))
       *doall = 1;
 
     *wilds = has_wildcards(name);
@@ -1213,7 +1432,7 @@ stats_L_list(struct Client *source_p,char *name, int doall, int wilds,
         !(MyConnect(source_p) && HasUMode(source_p, UMODE_OPER)) &&
         !HasUMode(target_p, UMODE_OPER) && (target_p != source_p))
       continue;
-    if (!doall && wilds && !match(name, target_p->name))
+    if (!doall && wilds && match(name, target_p->name))
       continue;
     if (!(doall || wilds) && irccmp(name, target_p->name))
       continue;
@@ -1351,7 +1570,7 @@ static const struct StatsStruct
   { 's',        stats_service,          1,      0       },
   { 'S',        stats_service,          1,      0       },
   { 't',        stats_tstats,           1,      0       },
-  { 'T',        stats_tstats,           1,      0       },
+  { 'T',        motd_report,            1,      0       },
   { 'u',        stats_uptime,           0,      0       },
   { 'U',        stats_shared,           1,      0       },
   { 'v',        stats_servers,          1,      0       },
@@ -1390,7 +1609,7 @@ do_stats(struct Client *source_p, int parc, char *parv[])
         break;
       }
 
-      sendto_realops_flags(UMODE_SPY, L_ALL,
+      sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
                            "STATS %c requested by %s (%s@%s) [%s]",
                            statchar, source_p->name, source_p->username,
                            source_p->host, source_p->servptr->name);
@@ -1418,12 +1637,6 @@ m_stats(struct Client *client_p, struct Client *source_p,
 {
   static time_t last_used = 0;
 
-  /* Is the stats meant for us? */
-  if (!ConfigFileEntry.disable_remote)
-    if (hunt_server(client_p, source_p, ":%s STATS %s :%s", 2,
-                    parc, parv) != HUNTED_ISME)
-      return;
-
   if (!MyClient(source_p) && IsCapable(source_p->from, CAP_TS6) && HasID(source_p))
   {
     from = me.id;
@@ -1445,6 +1658,12 @@ m_stats(struct Client *client_p, struct Client *source_p,
 
   last_used = CurrentTime;
 
+  /* Is the stats meant for us? */
+  if (!ConfigServerHide.disable_remote_commands)
+    if (hunt_server(client_p, source_p, ":%s STATS %s :%s", 2,
+                    parc, parv) != HUNTED_ISME)
+      return;
+
   do_stats(source_p, parc, parv);
 }
 
@@ -1479,27 +1698,9 @@ mo_stats(struct Client *client_p, struct Client *source_p,
   do_stats(source_p, parc, parv);
 }
 
-/*
- * ms_stats - STATS message handler
- *      parv[0] = sender prefix
- *      parv[1] = statistics selector (defaults to Message frequency)
- *      parv[2] = server name (current server defaulted, if omitted)
- */
-static void
-ms_stats(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
-{
-  if (hunt_server(client_p, source_p, ":%s STATS %s :%s", 2,
-                  parc, parv) != HUNTED_ISME)
-    return;
-
-  if (IsClient(source_p))
-    mo_stats(client_p, source_p, parc, parv);
-}
-
 static struct Message stats_msgtab = {
   "STATS", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
-  { m_unregistered, m_stats, ms_stats, m_ignore, mo_stats, m_ignore }
+  { m_unregistered, m_stats, mo_stats, m_ignore, mo_stats, m_ignore }
 };
 
 static void
@@ -1517,7 +1718,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1879 $",
+  .version = "$Revision: 2690 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
