@@ -19,14 +19,13 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_svinfo.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: m_svinfo.c 2273 2013-06-18 16:00:15Z michael $
  */
 
 #include "stdinc.h"
 #include "client.h"
 #include "irc_string.h"
 #include "ircd.h"
-#include "numeric.h"
 #include "send.h"
 #include "conf.h"
 #include "log.h"
@@ -65,10 +64,10 @@ ms_svinfo(struct Client *client_p, struct Client *source_p,
      * TS_ONLY we can't fall back to the non-TS protocol so
      * we drop the link  -orabidoo
      */
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
             "Link %s dropped, wrong TS protocol version (%s,%s)",
             get_client_name(source_p, SHOW_IP), parv[1], parv[2]);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                  "Link %s dropped, wrong TS protocol version (%s,%s)",
                  get_client_name(source_p, MASK_IP), parv[1], parv[2]);
     exit_client(source_p, source_p, "Incompatible TS version");
@@ -84,13 +83,13 @@ ms_svinfo(struct Client *client_p, struct Client *source_p,
 
   if (deltat > ConfigFileEntry.ts_max_delta)
   {
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
           "Link %s dropped, excessive TS delta (my TS=%lu, their TS=%lu, delta=%d)",
           get_client_name(source_p, SHOW_IP),
           (unsigned long) CurrentTime,
           (unsigned long) theirtime,
           (int) deltat);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
           "Link %s dropped, excessive TS delta (my TS=%lu, their TS=%lu, delta=%d)",
            get_client_name(source_p, MASK_IP),
            (unsigned long) CurrentTime,
@@ -107,7 +106,7 @@ ms_svinfo(struct Client *client_p, struct Client *source_p,
   }
 
   if (deltat > ConfigFileEntry.ts_warn_delta)
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                 "Link %s notable TS delta (my TS=%lu, their TS=%lu, delta=%d)",
                 source_p->name,
                 (unsigned long) CurrentTime,
@@ -135,7 +134,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1309 $",
+  .version = "$Revision: 2273 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
