@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_time.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: m_time.c 2197 2013-06-04 18:33:25Z michael $
  */
 
 #include "stdinc.h"
@@ -49,7 +49,7 @@ m_time(struct Client *client_p, struct Client *source_p,
     flood_endgrace(source_p);
 
   /* This is safe enough to use during non hidden server mode */
-  if (!ConfigFileEntry.disable_remote)
+  if (!ConfigServerHide.disable_remote_commands)
     if (hunt_server(client_p, source_p, ":%s TIME :%s", 1, parc, parv) != HUNTED_ISME)
       return;
 
@@ -91,7 +91,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1309 $",
+  .version = "$Revision: 2197 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
