@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_trace.c 1776 2013-01-24 17:26:55Z michael $
+ *  $Id: m_trace.c 2745 2014-01-05 17:43:16Z michael $
  */
 
 #include "stdinc.h"
@@ -35,14 +35,15 @@
 #include "parse.h"
 #include "modules.h"
 #include "conf.h"
+#include "conf_class.h"
 
 
 static void do_actual_trace(struct Client *, int, char *[]);
 static void report_this_status(struct Client *, struct Client *, int);
 
 static void
-trace_get_dependent(int *const server,
-                    int *const client, const struct Client *target_p)
+trace_get_dependent(unsigned int *const server,
+                    unsigned int *const client, const struct Client *target_p)
 {
   const dlink_node *ptr = NULL;
 
@@ -63,7 +64,7 @@ static void
 m_trace(struct Client *client_p, struct Client *source_p,
         int parc, char *parv[])
 {
-  const char *tname;
+  const char *tname = NULL;
 
   if (parc > 1)
     tname = parv[1];
@@ -74,7 +75,6 @@ m_trace(struct Client *client_p, struct Client *source_p,
              me.name, source_p->name, tname);
 }
 
-
 /* mo_trace()
  *      parv[0] = sender prefix
  *      parv[1] = servername
@@ -88,7 +88,7 @@ mo_trace(struct Client *client_p, struct Client *source_p,
   const char *from, *to;
 
   if (parc > 2)
-    if (hunt_server(client_p, source_p, ":%s TRACE %s :%s", 2, parc, parv))
+    if (hunt_server(client_p, source_p, ":%s TRACE %s :%s", 2, parc, parv) != HUNTED_ISME)
       return;
 
   if (parc > 1)
@@ -119,7 +119,7 @@ mo_trace(struct Client *client_p, struct Client *source_p,
         {
           ac2ptr = ptr->data;
 
-          if (match(tname, ac2ptr->name))
+          if (!match(tname, ac2ptr->name))
             break;
           else
             ac2ptr = NULL;
@@ -163,8 +163,6 @@ static void
 do_actual_trace(struct Client *source_p, int parc, char *parv[])
 {
   struct Client *target_p = NULL;
-  struct ConfItem *conf;
-  struct ClassItem *cltmp;
   int doall = 0;
   int wilds, dow;
   dlink_node *ptr;
@@ -186,12 +184,12 @@ do_actual_trace(struct Client *source_p, int parc, char *parv[])
     to = source_p->name;
   }
 
-  sendto_realops_flags(UMODE_SPY, L_ALL,
+  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
                        "TRACE requested by %s (%s@%s) [%s]",
                        source_p->name, source_p->username,
                        source_p->host, source_p->servptr->name);
 
-  if (match(tname, me.name))
+  if (!match(tname, me.name))
     doall = 1;
   else if (!MyClient(source_p) && !strcmp(tname, me.id))
   {
@@ -207,19 +205,16 @@ do_actual_trace(struct Client *source_p, int parc, char *parv[])
                               /* lets also do this for opers tracing nicks */
   {
     const char *name;
-    const char *class_name;
-
     target_p = hash_find_client(tname);
-      
+
     if (target_p && IsClient(target_p)) 
     {
       name = get_client_name(target_p, HIDE_IP);
-      class_name = get_client_class(target_p);
 
       if (HasUMode(target_p, UMODE_OPER))
       {
         sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
-                   from, to, class_name, name, 
+                   from, to, get_client_class(&target_p->localClient->confs), name,
                    IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
                    CurrentTime - target_p->localClient->lasttime,
                    idle_time_get(source_p, target_p));
@@ -227,13 +222,13 @@ do_actual_trace(struct Client *source_p, int parc, char *parv[])
       else
       {
         sendto_one(source_p,form_str(RPL_TRACEUSER),
-                   from, to, class_name, name, 
-		   IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
+                   from, to, get_client_class(&target_p->localClient->confs), name,
+                   IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
                    CurrentTime - target_p->localClient->lasttime,
                    idle_time_get(source_p, target_p));
       }
     }
-      
+
     sendto_one(source_p, form_str(RPL_ENDOFTRACE),
                from, to, tname);
     return;
@@ -245,10 +240,10 @@ do_actual_trace(struct Client *source_p, int parc, char *parv[])
     target_p = ptr->data;
 
     if (HasUMode(target_p, UMODE_INVISIBLE) && dow &&
-	!(MyConnect(source_p) && HasUMode(source_p, UMODE_OPER)) &&
-	!HasUMode(target_p, UMODE_OPER) && (target_p != source_p))
+        !(MyConnect(source_p) && HasUMode(source_p, UMODE_OPER)) &&
+        !HasUMode(target_p, UMODE_OPER) && (target_p != source_p))
       continue;
-    if (!doall && wilds && !match(tname, target_p->name))
+    if (!doall && wilds && match(tname, target_p->name))
       continue;
     if (!dow && irccmp(tname, target_p->name))
       continue;
@@ -260,7 +255,7 @@ do_actual_trace(struct Client *source_p, int parc, char *parv[])
   {
     target_p = ptr->data;
 
-    if (!doall && wilds && !match(tname, target_p->name))
+    if (!doall && wilds && match(tname, target_p->name))
       continue;
     if (!dow && irccmp(tname, target_p->name))
       continue;
@@ -273,7 +268,7 @@ do_actual_trace(struct Client *source_p, int parc, char *parv[])
   {
     target_p = ptr->data;
 
-    if (!doall && wilds && !match(tname, target_p->name))
+    if (!doall && wilds && match(tname, target_p->name))
       continue;
     if (!dow && irccmp(tname, target_p->name))
       continue;
@@ -281,14 +276,13 @@ do_actual_trace(struct Client *source_p, int parc, char *parv[])
     report_this_status(source_p, target_p, dow);
   }
 
-  DLINK_FOREACH(ptr, class_items.head)
+  DLINK_FOREACH(ptr, class_get_list()->head)
   {
-    conf = ptr->data;
-    cltmp = map_to_conf(conf);
+    const struct ClassItem *class = ptr->data;
 
-    if (cltmp->curr_user_count > 0)
+    if (class->ref_count > 0)
       sendto_one(source_p, form_str(RPL_TRACECLASS),
-                 from, to, conf->name, cltmp->curr_user_count);
+                 from, to, class->name, class->ref_count);
   }
 
   sendto_one(source_p, form_str(RPL_ENDOFTRACE), from, to, tname);
@@ -320,7 +314,7 @@ report_this_status(struct Client *source_p, struct Client *target_p, int dow)
   }
 
   name = get_client_name(target_p, HIDE_IP);
-  class_name = get_client_class(target_p);
+  class_name = get_client_class(&target_p->localClient->confs);
 
   set_time();
 
@@ -328,12 +322,12 @@ report_this_status(struct Client *source_p, struct Client *target_p, int dow)
   {
     case STAT_CONNECTING:
       sendto_one(source_p, form_str(RPL_TRACECONNECTING),
-                 from, to, class_name, 
-		 HasUMode(source_p, UMODE_ADMIN) ? name : target_p->name);
+                 from, to, class_name,
+                 HasUMode(source_p, UMODE_ADMIN) ? name : target_p->name);
       break;
     case STAT_HANDSHAKE:
       sendto_one(source_p, form_str(RPL_TRACEHANDSHAKE),
-                 from, to, class_name, 
+                 from, to, class_name,
                  HasUMode(source_p, UMODE_ADMIN) ? name : target_p->name);
       break;
     case STAT_ME:
@@ -341,9 +335,8 @@ report_this_status(struct Client *source_p, struct Client *target_p, int dow)
     case STAT_UNKNOWN:
       /* added time -Taner */
       sendto_one(source_p, form_str(RPL_TRACEUNKNOWN),
-		 from, to, class_name, name, target_p->sockhost,
-		 target_p->localClient->firsttime ? /* TBD: can't be 0 */
-                 CurrentTime - target_p->localClient->firsttime : -1);
+                 from, to, class_name, name, target_p->sockhost,
+                 CurrentTime - target_p->localClient->firsttime);
       break;
     case STAT_CLIENT:
       /*
@@ -351,54 +344,54 @@ report_this_status(struct Client *source_p, struct Client *target_p, int dow)
        * but anyone can see all the opers.
        */
       if ((HasUMode(source_p, UMODE_OPER) &&
-	   (MyClient(source_p) || !(dow && HasUMode(target_p, UMODE_INVISIBLE))))
-	  || !dow || HasUMode(target_p, UMODE_OPER))
-	{
+           (MyClient(source_p) || !(dow && HasUMode(target_p, UMODE_INVISIBLE))))
+          || !dow || HasUMode(target_p, UMODE_OPER))
+        {
           if (HasUMode(target_p, UMODE_ADMIN) && !ConfigFileEntry.hide_spoof_ips)
-	    sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
+            sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                        from, to, class_name, name,
                        HasUMode(source_p, UMODE_ADMIN) ? target_p->sockhost : "255.255.255.255",
                        CurrentTime - target_p->localClient->lasttime,
                        idle_time_get(source_p, target_p));
-		       
-	  else if (HasUMode(target_p, UMODE_OPER))
+
+          else if (HasUMode(target_p, UMODE_OPER))
           {
-	    if (ConfigFileEntry.hide_spoof_ips)
-	      sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
-		         from, to, class_name, name, 
-		         IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
-		         CurrentTime - target_p->localClient->lasttime,
-		         idle_time_get(source_p, target_p));
-	    else
+            if (ConfigFileEntry.hide_spoof_ips)
+              sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
+                         from, to, class_name, name, 
+                         IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
+                         CurrentTime - target_p->localClient->lasttime,
+                         idle_time_get(source_p, target_p));
+            else
               sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                          from, to, class_name, name,
                          MyOper(source_p) ? target_p->sockhost :
                          (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
                          CurrentTime - target_p->localClient->lasttime,
                          idle_time_get(source_p, target_p));
-	  }		       
-	  else
+          }
+          else
           {
             if (ConfigFileEntry.hide_spoof_ips)
-	      sendto_one(source_p, form_str(RPL_TRACEUSER),
-		         from, to, class_name, name,
+              sendto_one(source_p, form_str(RPL_TRACEUSER),
+                         from, to, class_name, name,
                          IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
-		         CurrentTime - target_p->localClient->lasttime,
-		         idle_time_get(source_p, target_p));
-	    else
+                         CurrentTime - target_p->localClient->lasttime,
+                         idle_time_get(source_p, target_p));
+            else
               sendto_one(source_p, form_str(RPL_TRACEUSER),
                          from, to, class_name, name,
                          MyOper(source_p) ? target_p->sockhost :
                          (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
                          CurrentTime - target_p->localClient->lasttime,
                          idle_time_get(source_p, target_p));
-	  }
-	}
+          }
+        }
       break;
     case STAT_SERVER:
     {
-      int clients = 0;
-      int servers = 0;
+      unsigned int clients = 0;
+      unsigned int servers = 0;
 
       trace_get_dependent(&servers, &clients, target_p);
 
@@ -406,14 +399,14 @@ report_this_status(struct Client *source_p, struct Client *target_p, int dow)
         name = get_client_name(target_p, MASK_IP);
 
       sendto_one(source_p, form_str(RPL_TRACESERVER),
-		 from, to, class_name, servers,
-		 clients, name, *(target_p->serv->by) ?
-		 target_p->serv->by : "*", "*",
-		 me.name, CurrentTime - target_p->localClient->lasttime);
+                 from, to, class_name, servers,
+                 clients, name, *(target_p->serv->by) ?
+                 target_p->serv->by : "*", "*",
+                 me.name, CurrentTime - target_p->localClient->lasttime);
       break;
     }
       
-    default: /* ...we actually shouldn't come here... --msa */
+    default:  /* ...we actually shouldn't come here... --msa */
       sendto_one(source_p, form_str(RPL_TRACENEWTYPE),
                  from, to, name);
       break;
@@ -440,7 +433,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1776 $",
+  .version = "$Revision: 2745 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
