@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_user.c 1243 2011-09-30 10:47:53Z michael $
+ *  $Id: m_user.c 2188 2013-06-04 12:44:37Z michael $
  */
 
 #include "stdinc.h"
@@ -58,10 +58,6 @@ do_local_user(struct Client *source_p,
 
   strlcpy(source_p->info, realname, sizeof(source_p->info));
 
-  /* stash for later */
-  strlcpy(source_p->localClient->client_host, host, sizeof(source_p->localClient->client_host));
-  strlcpy(source_p->localClient->client_server, server, sizeof(source_p->localClient->client_server));
-
   if (!IsGotId(source_p))
     strlcpy(source_p->username, username, sizeof(source_p->username));
 
@@ -126,7 +122,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1243 $",
+  .version = "$Revision: 2188 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
