@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_userhost.c 1723 2013-01-03 00:02:43Z michael $
+ *  $Id: m_userhost.c 1834 2013-04-19 19:50:27Z michael $
  */
 
 #include "stdinc.h"
@@ -29,7 +29,6 @@
 #include "s_serv.h"
 #include "send.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "parse.h"
 #include "modules.h"
 
@@ -67,27 +66,27 @@ m_userhost(struct Client *client_p, struct Client *source_p,
        */
       if (MyClient(target_p) && (target_p == source_p))
       {
-        rl = ircsprintf(response, "%s%s=%c%s@%s ",
-                        target_p->name,
-                        HasUMode(target_p, UMODE_OPER) ? "*" : "",
-                        (target_p->away[0]) ? '-' : '+',
-                        target_p->username,
-                        target_p->sockhost);
+        rl = sprintf(response, "%s%s=%c%s@%s ",
+                     target_p->name,
+                     HasUMode(target_p, UMODE_OPER) ? "*" : "",
+                     (target_p->away[0]) ? '-' : '+',
+                     target_p->username,
+                     target_p->sockhost);
       }
       else
       {
-        rl = ircsprintf(response, "%s%s=%c%s@%s ",
-                        target_p->name, (HasUMode(target_p, UMODE_OPER) &&
-                                         (!HasUMode(target_p, UMODE_HIDDEN) ||
-                                           HasUMode(source_p, UMODE_OPER))) ? "*" : "",
-                        (target_p->away[0]) ? '-' : '+',
-                        target_p->username,
-                        target_p->host);
+        rl = sprintf(response, "%s%s=%c%s@%s ",
+                     target_p->name, (HasUMode(target_p, UMODE_OPER) &&
+                                      (!HasUMode(target_p, UMODE_HIDDEN) ||
+                                        HasUMode(source_p, UMODE_OPER))) ? "*" : "",
+                     (target_p->away[0]) ? '-' : '+',
+                     target_p->username,
+                     target_p->host);
       }
 
       if ((rl + cur_len) < (IRCD_BUFSIZE - 10))
       {
-        ircsprintf(t, "%s", response);
+        sprintf(t, "%s", response);
         t += rl;
         cur_len += rl;
       }
@@ -119,7 +118,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1723 $",
+  .version = "$Revision: 1834 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
