@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_wallops.c 1474 2012-07-22 14:44:07Z michael $
+ *  $Id: m_wallops.c 2039 2013-05-14 19:25:54Z michael $
  */
 
 #include "stdinc.h"
@@ -41,10 +41,17 @@
  */
 static void
 mo_wallops(struct Client *client_p, struct Client *source_p,
-	   int parc, char *parv[])
+           int parc, char *parv[])
 { 
   const char *message = parv[1];
 
+  if (!HasOFlag(source_p, OPER_FLAG_WALLOPS))
+  {
+    sendto_one(source_p, form_str(ERR_NOPRIVS),
+               me.name, source_p->name, "wallops");
+    return;
+  }
+
   if (EmptyString(message))
   {
     sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
@@ -104,7 +111,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1474 $",
+  .version = "$Revision: 2039 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
