@@ -21,66 +21,21 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_webirc.c 1871 2013-04-26 08:53:04Z michael $
+ *  $Id: m_webirc.c 2711 2013-12-24 18:37:50Z michael $
  */
 
 #include "stdinc.h"
 #include "list.h"
 #include "client.h"
 #include "ircd.h"
-#include "numeric.h"
 #include "send.h"
 #include "irc_string.h"
-#include "hash.h"
 #include "parse.h"
 #include "modules.h"
 #include "conf.h"
 #include "hostmask.h"
+#include "s_user.h"
 
-/*
- * Usage:
- *
- * auth {
- *   user = "webirc@<cgiirc ip>"; # if identd used, put ident username instead
- *   password = "<password>"; # encryption possible
- *   spoof = "webirc."
- *   class = "users";
- *   encrypted = yes; # [Using encryption is highly recommended]
- * };
- *
- * Possible flags:
- *   kline_exempt - k/g lines on the cgiirc ip are ignored
- *   gline_exempt - glines on the cgiirc ip are ignored
- *
- * dlines are checked on the cgiirc ip (of course).
- * k/d/g/x lines, auth blocks, user limits, etc are checked using the
- * real host/ip.
- *
- * The password should be specified unencrypted in webirc_password in
- * cgiirc.config
- */
-
-static int
-invalid_hostname(const char *hostname)
-{
-  const char *p = hostname;
-  unsigned int has_sep = 0;
-
-  assert(p != NULL);
-
-  if (*p == '.' || *p == ':')
-    return 1;
-
-  for (; *p; ++p)
-  {
-    if (!IsHostChar(*p))
-      return 1;
-    if (*p == '.' || *p == ':')
-      ++has_sep;
-  }
-
-  return !has_sep;
-}
 
 /*
  * mr_webirc
@@ -93,43 +48,40 @@ invalid_hostname(const char *hostname)
 static void
 mr_webirc(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
 {
-  struct AccessItem *aconf = NULL;
-  struct ConfItem *conf = NULL;
+  struct MaskItem *conf = NULL;
   struct addrinfo hints, *res;
 
   assert(source_p == client_p);
 
-  if (invalid_hostname(parv[4]))
+  if (!valid_hostname(parv[3]))
   {
-    sendto_one(source_p, ":%s NOTICE %s :CGI:IRC: Invalid IP", me.name,
+    sendto_one(source_p, ":%s NOTICE %s :CGI:IRC: Invalid hostname", me.name,
                source_p->name[0] ? source_p->name : "*");
     return;
   }
 
-  aconf = find_address_conf(source_p->host,
-                            IsGotId(source_p) ? source_p->username : "webirc",
-                            &source_p->localClient->ip,
-                            source_p->localClient->aftype, parv[1]);
-  if (aconf == NULL || !IsConfClient(aconf))
+  conf = find_address_conf(source_p->host,
+                           IsGotId(source_p) ? source_p->username : "webirc",
+                           &source_p->localClient->ip,
+                           source_p->localClient->aftype, parv[1]);
+  if (conf == NULL || !IsConfClient(conf))
     return;
 
-  conf = unmap_conf_item(aconf);
-
-  if (!IsConfDoSpoofIp(aconf) || irccmp(conf->name, "webirc."))
+  if (!IsConfWebIRC(conf))
   {
     sendto_one(source_p, ":%s NOTICE %s :Not a CGI:IRC auth block", me.name,
                source_p->name[0] ? source_p->name : "*");
     return;
   }
 
-  if (EmptyString(aconf->passwd))
+  if (EmptyString(conf->passwd))
   {
     sendto_one(source_p, ":%s NOTICE %s :CGI:IRC auth blocks must have a password",
                me.name, source_p->name[0] ? source_p->name : "*");
     return;
   }
 
-  if (!match_conf_password(parv[1], aconf))
+  if (!match_conf_password(parv[1], conf))
   {
     sendto_one(source_p, ":%s NOTICE %s :CGI:IRC password incorrect",
                me.name, source_p->name[0] ? source_p->name : "*");
@@ -159,30 +111,27 @@ mr_webirc(struct Client *client_p, struct Client *source_p, int parc, char *parv
   freeaddrinfo(res);
 
   strlcpy(source_p->sockhost, parv[4], sizeof(source_p->sockhost));
-
-  if (strlen(parv[3]) <= HOSTLEN)
-    strlcpy(source_p->host, parv[3], sizeof(source_p->host));
-  else
-    strlcpy(source_p->host, source_p->sockhost, sizeof(source_p->host));
+  strlcpy(source_p->host, parv[3], sizeof(source_p->host));
 
   /* Check dlines now, k/glines will be checked on registration */
-  if ((aconf = find_dline_conf(&client_p->localClient->ip,
-                                client_p->localClient->aftype)))
+  if ((conf = find_dline_conf(&client_p->localClient->ip,
+                               client_p->localClient->aftype)))
   {
-    if (!(aconf->status & CONF_EXEMPTDLINE))
+    if (!(conf->type == CONF_EXEMPT))
     {
       exit_client(client_p, &me, "D-lined");
       return;
     }
   }
 
+  AddUMode(source_p, UMODE_WEBIRC);
   sendto_one(source_p, ":%s NOTICE %s :CGI:IRC host/IP set to %s %s", me.name,
              source_p->name[0] ? source_p->name : "*", parv[3], parv[4]);
 }
 
 static struct Message webirc_msgtab = {
   "WEBIRC", 0, 0, 5, MAXPARA, MFLG_SLOW, 0,
-  { mr_webirc, m_ignore, m_ignore, m_ignore, m_ignore, m_ignore }
+  { mr_webirc, m_registered, m_ignore, m_ignore, m_registered, m_ignore }
 };
 
 static void
@@ -200,7 +149,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1871 $",
+  .version = "$Revision: 2711 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
