@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_who.c 1549 2012-10-01 18:11:11Z michael $
+ *  $Id: m_who.c 2755 2014-01-05 22:50:23Z michael $
  */
 
 #include "stdinc.h"
@@ -33,16 +33,207 @@
 #include "s_serv.h"
 #include "send.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "conf.h"
 #include "parse.h"
 #include "modules.h"
 
-static void who_global(struct Client *, char *, int);
-static void do_who(struct Client *, struct Client *,
-                   const char *, const char *);
-static void do_who_on_channel(struct Client *, struct Channel *,
-                              const char *, int, int);
+
+/* do_who()
+ *
+ * inputs       - pointer to client requesting who
+ *              - pointer to client to do who on
+ *              - The reported name
+ *              - channel flags
+ * output       - NONE
+ * side effects - do a who on given person
+ */
+static void
+do_who(struct Client *source_p, struct Client *target_p,
+       const char *chname, const char *op_flags)
+{
+  char status[IRCD_BUFSIZE];
+
+  if (HasUMode(source_p, UMODE_OPER))
+    snprintf(status, sizeof(status), "%c%s%s%s", target_p->away[0] ? 'G' : 'H',
+             HasUMode(target_p, UMODE_REGISTERED) ? "r" : "",
+             HasUMode(target_p, UMODE_OPER) ? "*" : "", op_flags);
+  else
+    snprintf(status, sizeof(status), "%c%s%s%s", target_p->away[0] ? 'G' : 'H',
+             HasUMode(target_p, UMODE_REGISTERED) ? "r" : "",
+             HasUMode(target_p, UMODE_OPER) &&
+             !HasUMode(target_p, UMODE_HIDDEN) ? "*" : "", op_flags);
+
+  if (ConfigServerHide.hide_servers || IsHidden(target_p->servptr))
+    sendto_one(source_p, form_str(RPL_WHOREPLY), me.name, source_p->name,
+               (chname) ? (chname) : "*",
+               target_p->username, target_p->host,
+               HasUMode(source_p, UMODE_OPER) ? target_p->servptr->name : "*",
+               target_p->name, status,
+               HasUMode(source_p, UMODE_OPER) ? target_p->hopcount : 0, target_p->info);
+  else
+    sendto_one(source_p, form_str(RPL_WHOREPLY), me.name, source_p->name,
+               (chname) ? (chname) : "*", target_p->username,
+               target_p->host, target_p->servptr->name, target_p->name,
+               status, target_p->hopcount, target_p->info);
+}
+
+/* who_common_channel
+ * inputs	- pointer to client requesting who
+ * 		- pointer to channel member chain.
+ *		- char * mask to match
+ *		- int if oper on a server or not
+ *		- pointer to int maxmatches
+ * output	- NONE
+ * side effects - lists matching clients on specified channel,
+ * 		  marks matched clients.
+ *
+ */
+static void
+who_common_channel(struct Client *source_p, struct Channel *chptr,
+                   char *mask, int server_oper, int *maxmatches)
+{
+  dlink_node *ptr = NULL;
+
+  DLINK_FOREACH(ptr, chptr->members.head)
+  {
+    struct Client *target_p = ((struct Membership *)ptr->data)->client_p;
+
+    if (!HasUMode(target_p, UMODE_INVISIBLE) || HasFlag(target_p, FLAGS_MARK))
+      continue;
+
+    if (server_oper)
+      if (!HasUMode(target_p, UMODE_OPER) ||
+          (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER)))
+        continue;
+
+    AddFlag(target_p, FLAGS_MARK);
+
+    assert(target_p->servptr != NULL);
+
+    if ((mask == NULL) ||
+      !match(mask, target_p->name) || !match(mask, target_p->username) ||
+      !match(mask, target_p->host) || 
+      ((!ConfigServerHide.hide_servers || HasUMode(source_p, UMODE_OPER)) &&
+       !match(mask, target_p->servptr->name)) ||
+      !match(mask, target_p->info))
+    {
+      do_who(source_p, target_p, NULL, "");
+
+      if (*maxmatches > 0)
+      {
+        if (--(*maxmatches) == 0)
+          return;
+      }
+    }
+  }
+}
+
+/* who_global()
+ *
+ * inputs	- pointer to client requesting who
+ *		- char * mask to match
+ *		- int if oper on a server or not
+ * output	- NONE
+ * side effects - do a global scan of all clients looking for match
+ *		  this is slightly expensive on EFnet ...
+ */
+static void
+who_global(struct Client *source_p, char *mask, int server_oper)
+{
+  struct Channel *chptr;
+  struct Client *target_p;
+  dlink_node *lp = NULL, *gcptr = NULL;
+  int maxmatches = 500;
+  static time_t last_used = 0;
+
+  if (!HasUMode(source_p, UMODE_OPER))
+  {
+    if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime)
+    {
+      /* safe enough to give this on a local connect only */
+      sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, source_p->name);
+      return;
+    }
+
+    last_used = CurrentTime;
+  }
+
+  /* first, list all matching invisible clients on common channels */
+  DLINK_FOREACH(lp, source_p->channel.head)
+  {
+    chptr = ((struct Membership *)lp->data)->chptr;
+    who_common_channel(source_p, chptr, mask, server_oper, &maxmatches);
+  }
+
+  /* second, list all matching visible clients */
+  DLINK_FOREACH(gcptr, global_client_list.head)
+  {
+    target_p = gcptr->data;
+
+    if (!IsClient(target_p))
+      continue;
+
+    if (HasUMode(target_p, UMODE_INVISIBLE))
+    {
+      DelFlag(target_p, FLAGS_MARK);
+      continue;
+    }
+
+    if (server_oper)
+      if (!HasUMode(target_p, UMODE_OPER) ||
+          (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER)))
+        continue;
+
+    assert(target_p->servptr != NULL);
+
+    if (!mask ||
+        !match(mask, target_p->name) || !match(mask, target_p->username) ||
+        !match(mask, target_p->host) || !match(mask, target_p->servptr->name) ||
+        !match(mask, target_p->info))
+    {
+      do_who(source_p, target_p, NULL, "");
+
+      if (maxmatches > 0)
+      {
+        if (--maxmatches == 0)
+          return;
+      }
+    }
+  }
+}
+
+/* do_who_on_channel()
+ *
+ * inputs	- pointer to client requesting who
+ *		- pointer to channel to do who on
+ *		- The "real name" of this channel
+ *		- int if source_p is a server oper or not
+ *		- int if client is member or not
+ *		- int server_op flag
+ * output	- NONE
+ * side effects - do a who on given channel
+ */
+static void
+do_who_on_channel(struct Client *source_p, struct Channel *chptr,
+                  const char *chname, int member, int server_oper)
+{
+  dlink_node *ptr = NULL;
+
+  DLINK_FOREACH(ptr, chptr->members.head)
+  {
+    struct Membership *ms = ptr->data;
+    struct Client *target_p = ms->client_p;
+
+    if (member || !HasUMode(target_p, UMODE_INVISIBLE))
+    {
+      if (server_oper)
+        if (!HasUMode(target_p, UMODE_OPER) ||
+            (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER)))
+          continue;
+      do_who(source_p, target_p, chname, get_member_status(ms, !!HasCap(source_p, CAP_MULTI_PREFIX)));
+    }
+  }
+}
 
 /*
 ** m_who
@@ -54,11 +245,11 @@ static void
 m_who(struct Client *client_p, struct Client *source_p,
       int parc, char *parv[])
 {
-  struct Client *target_p;
+  struct Client *target_p = NULL;
+  struct Channel *chptr = NULL;
   char *mask = parv[1];
-  dlink_node *lp;
+  dlink_node *lp = NULL;
   int server_oper = parc > 2 ? (*parv[2] == 'o') : 0; /* Show OPERS only */
-  struct Channel *chptr;
 
   /* See if mask is there, collapse it or return if not there */
   if (EmptyString(mask))
@@ -72,21 +263,6 @@ m_who(struct Client *client_p, struct Client *source_p,
   /* mask isn't NULL at this point. repeat after me... -db */
   collapse(mask);
 
-  /* '/who *' */
-  if (!strcmp(mask, "*"))
-  {
-    if ((lp = source_p->channel.head) != NULL)
-    {
-      struct Channel *mychannel = ((struct Membership *)lp->data)->chptr;
-      do_who_on_channel(source_p, mychannel, mychannel->chname, 1,
-                        server_oper);
-    }
-
-    sendto_one(source_p, form_str(RPL_ENDOFWHO),
-               me.name, source_p->name, "*");
-    return;
-  }
-
   /* '/who #some_channel' */
   if (IsChanPrefix(*mask))
   {
@@ -126,6 +302,21 @@ m_who(struct Client *client_p, struct Client *source_p,
     return;
   }
 
+  /* '/who *' */
+  if (!strcmp(mask, "*"))
+  {
+    if ((lp = source_p->channel.head) != NULL)
+    {
+      struct Channel *mychannel = ((struct Membership *)lp->data)->chptr;
+      do_who_on_channel(source_p, mychannel, mychannel->chname, 1,
+                        server_oper);
+    }
+
+    sendto_one(source_p, form_str(RPL_ENDOFWHO),
+               me.name, source_p->name, "*");
+    return;
+  }
+
   /* '/who 0' */
   if (!strcmp(mask, "0"))
     who_global(source_p, NULL, server_oper);
@@ -137,209 +328,6 @@ m_who(struct Client *client_p, struct Client *source_p,
              me.name, source_p->name, mask);
 }
 
-/* who_common_channel
- * inputs	- pointer to client requesting who
- * 		- pointer to channel member chain.
- *		- char * mask to match
- *		- int if oper on a server or not
- *		- pointer to int maxmatches
- * output	- NONE
- * side effects - lists matching clients on specified channel,
- * 		  marks matched clients.
- *
- */
-static void
-who_common_channel(struct Client *source_p, struct Channel *chptr,
-                   char *mask, int server_oper, int *maxmatches)
-{
-  dlink_node *ptr = NULL;
-
-  DLINK_FOREACH(ptr, chptr->members.head)
-  {
-    struct Client *target_p = ((struct Membership *)ptr->data)->client_p;
-
-    if (!HasUMode(target_p, UMODE_INVISIBLE) || HasFlag(target_p, FLAGS_MARK))
-      continue;
-
-    if (server_oper)
-      if (!HasUMode(target_p, UMODE_OPER) ||
-          (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER)))
-        continue;
-
-    AddFlag(target_p, FLAGS_MARK);
-
-    assert(target_p->servptr != NULL);
-
-    if ((mask == NULL) ||
-      match(mask, target_p->name) || match(mask, target_p->username) ||
-      match(mask, target_p->host) || 
-      ((!ConfigServerHide.hide_servers || HasUMode(source_p, UMODE_OPER)) &&
-       match(mask, target_p->servptr->name)) ||
-      match(mask, target_p->info))
-    {
-      do_who(source_p, target_p, NULL, "");
-
-      if (*maxmatches > 0)
-      {
-        if (--(*maxmatches) == 0)
-          return;
-      }
-    }
-  }
-}
-
-/* who_global()
- *
- * inputs	- pointer to client requesting who
- *		- char * mask to match
- *		- int if oper on a server or not
- * output	- NONE
- * side effects - do a global scan of all clients looking for match
- *		  this is slightly expensive on EFnet ...
- */
-static void
-who_global(struct Client *source_p, char *mask, int server_oper)
-{
-  struct Channel *chptr;
-  struct Client *target_p;
-  dlink_node *lp;
-  dlink_node *lp_next;
-  dlink_node *gcptr;
-  dlink_node *gcptr_next;
-  int maxmatches = 500;
-  static time_t last_used = 0;
-
-  if (!HasUMode(source_p, UMODE_OPER))
-  {
-    if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime)
-    {
-      /* safe enough to give this on a local connect only */
-      sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, source_p->name);
-      return;
-    }
-
-    last_used = CurrentTime;
-  }
-
-  /* first, list all matching invisible clients on common channels */
-  DLINK_FOREACH_SAFE(lp, lp_next, source_p->channel.head)
-  {
-    chptr = ((struct Membership *)lp->data)->chptr;
-    who_common_channel(source_p, chptr, mask, server_oper, &maxmatches);
-  }
-
-  /* second, list all matching visible clients */
-  DLINK_FOREACH_SAFE(gcptr, gcptr_next, global_client_list.head)
-  {
-    target_p = gcptr->data;
-
-    if (!IsClient(target_p))
-      continue;
-
-    if (HasUMode(target_p, UMODE_INVISIBLE))
-    {
-      DelFlag(target_p, FLAGS_MARK);
-      continue;
-    }
-
-    if (server_oper)
-      if (!HasUMode(target_p, UMODE_OPER) ||
-          (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER)))
-        continue;
-
-    assert(target_p->servptr != NULL);
-
-    if (!mask ||
-        match(mask, target_p->name) || match(mask, target_p->username) ||
-        match(mask, target_p->host) || match(mask, target_p->servptr->name) ||
-        match(mask, target_p->info))
-    {
-      do_who(source_p, target_p, NULL, "");
-
-      if (maxmatches > 0)
-      {
-        if (--maxmatches == 0)
-          return;
-      }
-    }
-  }
-}
-
-/* do_who_on_channel()
- *
- * inputs	- pointer to client requesting who
- *		- pointer to channel to do who on
- *		- The "real name" of this channel
- *		- int if source_p is a server oper or not
- *		- int if client is member or not
- *		- int server_op flag
- * output	- NONE
- * side effects - do a who on given channel
- */
-static void
-do_who_on_channel(struct Client *source_p, struct Channel *chptr,
-                  const char *chname, int member, int server_oper)
-{
-  dlink_node *ptr = NULL, *ptr_next = NULL;
-  struct Client *target_p;
-  struct Membership *ms;
-
-  DLINK_FOREACH_SAFE(ptr, ptr_next, chptr->members.head)
-  {
-    ms = ptr->data;
-    target_p = ms->client_p;
-
-    if (member || !HasUMode(target_p, UMODE_INVISIBLE))
-    {
-      if (server_oper)
-        if (!HasUMode(target_p, UMODE_OPER) ||
-            (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER)))
-          continue;
-      do_who(source_p, target_p, chname, get_member_status(ms, !!HasCap(source_p, CAP_MULTI_PREFIX)));
-    }
-  }
-}
-
-/* do_who()
- *
- * inputs	- pointer to client requesting who
- *		- pointer to client to do who on
- *		- The reported name
- *		- channel flags
- * output	- NONE
- * side effects - do a who on given person
- */
-static void
-do_who(struct Client *source_p, struct Client *target_p,
-       const char *chname, const char *op_flags)
-{
-  char status[7]; /* G*@%+\0 */
-
-  if (HasUMode(source_p, UMODE_OPER))
-    snprintf(status, sizeof(status), "%c%s%s", target_p->away[0] ? 'G' : 'H',
-             HasUMode(target_p, UMODE_OPER) ? "*" : "", op_flags);
-  else
-    snprintf(status, sizeof(status), "%c%s%s", target_p->away[0] ? 'G' : 'H',
-             HasUMode(target_p, UMODE_OPER) &&
-             !HasUMode(target_p, UMODE_HIDDEN) ? "*" : "", op_flags);
-
-  if (ConfigServerHide.hide_servers)
-  {
-    sendto_one(source_p, form_str(RPL_WHOREPLY), me.name, source_p->name,
-	       (chname) ? (chname) : "*",
-               target_p->username, target_p->host,
-	       HasUMode(source_p, UMODE_OPER) ? target_p->servptr->name : "*",
-	       target_p->name, status, 0, target_p->info);
-  }
-  else
-  {
-    sendto_one(source_p, form_str(RPL_WHOREPLY), me.name, source_p->name,
-	       (chname) ? (chname) : "*", target_p->username,
-	       target_p->host, target_p->servptr->name, target_p->name,
-	       status, target_p->hopcount, target_p->info);
-  }
-}
-
 static struct Message who_msgtab = {
   "WHO", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
   {m_unregistered, m_who, m_ignore, m_ignore, m_who, m_ignore}
@@ -360,7 +348,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1549 $",
+  .version = "$Revision: 2755 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
