@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_whowas.c 1358 2012-04-22 13:49:23Z michael $
+ *  $Id: m_whowas.c 2747 2014-01-05 19:19:06Z michael $
  */
 
 #include "stdinc.h"
@@ -41,52 +41,39 @@
 
 static void
 whowas_do(struct Client *client_p, struct Client *source_p,
-          int parc, char *parv[])
+          const int parc, char *parv[])
 {
   int cur = 0;
   int max = -1;
-  char *p = NULL, *nick = NULL;
   const dlink_node *ptr = NULL;
 
-  if (parc > 2)
-  {
-    max = atoi(parv[2]);
-
-    if (!MyConnect(source_p) && max > 20)
-      max = 20;
-  }
-
   if (parc > 3)
     if (hunt_server(client_p, source_p, ":%s WHOWAS %s %s :%s", 3,
                     parc, parv) != HUNTED_ISME)
       return;
 
-  nick = parv[1];
-  while (*nick == ',')
-    nick++;
-  if ((p = strchr(nick,',')) != NULL)
-    *p = '\0';
-  if (*nick == '\0')
-    return;
+  if (parc > 2 && !EmptyString(parv[2]))
+    if ((max = atoi(parv[2])) > 20 && !MyConnect(source_p))
+      max = 20;
 
-  DLINK_FOREACH(ptr, WHOWASHASH[strhash(nick)].head)
+  DLINK_FOREACH(ptr, WHOWASHASH[strhash(parv[1])].head)
   {
     const struct Whowas *temp = ptr->data;
 
-    if (!irccmp(nick, temp->name))
+    if (!irccmp(parv[1], temp->name))
     {
       sendto_one(source_p, form_str(RPL_WHOWASUSER),
                  me.name, source_p->name, temp->name,
                  temp->username, temp->hostname,
                  temp->realname);
 
-      if (ConfigServerHide.hide_servers && !HasUMode(source_p, UMODE_OPER))
-        sendto_one(source_p, form_str(RPL_WHOISSERVER),
-                   me.name, source_p->name, temp->name,
+      if ((temp->shide || ConfigServerHide.hide_servers) && !HasUMode(source_p, UMODE_OPER))
+        sendto_one(source_p, form_str(RPL_WHOISSERVER), me.name,
+                   source_p->name, temp->name,
                    ServerInfo.network_name, myctime(temp->logoff));
       else
-        sendto_one(source_p, form_str(RPL_WHOISSERVER),
-                   me.name, source_p->name, temp->name,
+        sendto_one(source_p, form_str(RPL_WHOISSERVER), me.name,
+                   source_p->name, temp->name,
                    temp->servername, myctime(temp->logoff));
       ++cur;
     }
@@ -97,7 +84,7 @@ whowas_do(struct Client *client_p, struct Client *source_p,
 
   if (!cur)
     sendto_one(source_p, form_str(ERR_WASNOSUCHNICK),
-               me.name, source_p->name, nick);
+               me.name, source_p->name, parv[1]);
 
   sendto_one(source_p, form_str(RPL_ENDOFWHOWAS),
              me.name, source_p->name, parv[1]);
@@ -167,7 +154,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1358 $",
+  .version = "$Revision: 2747 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
