@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_xline.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: m_xline.c 2361 2013-07-04 21:15:16Z michael $
  */
 
 #include "stdinc.h"
@@ -27,13 +27,12 @@
 #include "channel.h"
 #include "client.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
+#include "conf.h"
 #include "hostmask.h"
 #include "numeric.h"
 #include "fdlist.h"
 #include "s_bsd.h"
-#include "conf.h"
 #include "log.h"
 #include "s_misc.h"
 #include "send.h"
@@ -42,12 +41,14 @@
 #include "parse.h"
 #include "modules.h"
 #include "resv.h"
+#include "conf_db.h"
+#include "memory.h"
 
 
 static int valid_xline(struct Client *, char *, char *, int);
 static void write_xline(struct Client *, char *, char *, time_t);
 static void remove_xline(struct Client *, char *);
-static int remove_txline_match(const char *);
+static int remove_xline_match(const char *);
 
 static void relay_xline(struct Client *, char *[]);
 
@@ -67,8 +68,7 @@ mo_xline(struct Client *client_p, struct Client *source_p,
 {
   char *reason = NULL;
   char *gecos = NULL;
-  struct ConfItem *conf = NULL;
-  struct MatchItem *match_item = NULL;
+  struct MaskItem *conf = NULL;
   char *target_server = NULL;
   time_t tkline_time = 0;
 
@@ -100,7 +100,7 @@ mo_xline(struct Client *client_p, struct Client *source_p,
 			 target_server, gecos, (int)tkline_time, reason);
 
     /* Allow ON to apply local xline as well if it matches */
-    if (!match(target_server, me.name))
+    if (match(target_server, me.name))
       return;
   }
   else 
@@ -116,14 +116,12 @@ mo_xline(struct Client *client_p, struct Client *source_p,
   if (!valid_xline(source_p, gecos, reason, 0))
     return;
 
-  if ((conf = find_matching_name_conf(XLINE_TYPE, gecos,
+  if ((conf = find_matching_name_conf(CONF_XLINE, gecos,
                                       NULL, NULL, 0)) != NULL)
   {
-    match_item = map_to_conf(conf);
-
     sendto_one(source_p, ":%s NOTICE %s :[%s] already X-Lined by [%s] - %s",
                me.name, source_p->name, gecos,
-               conf->name, match_item->reason);
+               conf->name, conf->reason);
     return;
   }
 
@@ -133,7 +131,6 @@ mo_xline(struct Client *client_p, struct Client *source_p,
 /* ms_xline()
  *
  * inputs	- oper, target server, xline, {type}, reason
- *		  deprecate {type} reserve for temp xlines later? XXX
  *
  * outputs	- none
  * side effects	- propagates xline, applies it if we are a target
@@ -183,34 +180,29 @@ me_xline(struct Client *client_p, struct Client *source_p,
 static void
 relay_xline(struct Client *source_p, char *parv[])
 {
-  struct ConfItem *conf;
-  struct MatchItem *match_item;
+  struct MaskItem *conf = NULL;
   int t_sec;
 
   t_sec = atoi(parv[3]);
-  /* XXX kludge! */
-  if (t_sec < 3)
-    t_sec = 0;
 
   sendto_match_servs(source_p, parv[1], CAP_CLUSTER,
                      "XLINE %s %s %s :%s",
                      parv[1], parv[2], parv[3], parv[4]);
 
-  if (!match(parv[1], me.name))
+  if (match(parv[1], me.name))
     return;
 
-  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(ULINE_TYPE, source_p->servptr->name,
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(CONF_ULINE, source_p->servptr->name,
                               source_p->username, source_p->host,
                               SHARED_XLINE))
   {
-    if ((conf = find_matching_name_conf(XLINE_TYPE, parv[2],
+    if ((conf = find_matching_name_conf(CONF_XLINE, parv[2],
 					NULL, NULL, 0)) != NULL)
     {
-      match_item = map_to_conf(conf);
       sendto_one(source_p, ":%s NOTICE %s :[%s] already X-Lined by [%s] - %s",
                  ID_or_name(&me, source_p->from),
                  ID_or_name(source_p, source_p->from),
-                 parv[2], conf->name, match_item->reason);
+                 parv[2], conf->name, conf->reason);
       return;
     }
 
@@ -252,7 +244,7 @@ mo_unxline(struct Client *client_p, struct Client *source_p,
                        "UNXLINE %s %s", target_server, gecos);
 
     /* Allow ON to apply local unxline as well if it matches */
-    if (!match(target_server, me.name))
+    if (match(target_server, me.name))
       return;
   }
   else
@@ -281,10 +273,10 @@ ms_unxline(struct Client *client_p, struct Client *source_p,
   sendto_match_servs(source_p, parv[1], CAP_CLUSTER,
                      "UNXLINE %s %s", parv[1], parv[2]);
 
-  if (!match(parv[1], me.name))
+  if (match(parv[1], me.name))
     return;
 
-  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(ULINE_TYPE, source_p->servptr->name,
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(CONF_ULINE, source_p->servptr->name,
                               source_p->username, source_p->host,
                               SHARED_UNXLINE))
     remove_xline(source_p, parv[2]);
@@ -307,13 +299,6 @@ valid_xline(struct Client *source_p, char *gecos, char *reason, int warn)
     return 0;
   }
 
-  if (strchr(gecos, '"'))
-  {
-    sendto_one(source_p, ":%s NOTICE %s :Invalid character '\"'",
-               me.name, source_p->name);
-    return 0;
-  }
-
   if (!valid_wild_card_simple(gecos))
   {
     if (warn)
@@ -336,66 +321,58 @@ static void
 write_xline(struct Client *source_p, char *gecos, char *reason,
 	    time_t tkline_time)
 {
-  struct ConfItem *conf;
-  struct MatchItem *match_item;
-  const char *current_date;
-  time_t cur_time;
-
-  conf = make_conf_item(XLINE_TYPE);
-  match_item = map_to_conf(conf);
+  struct MaskItem *conf = conf_make(CONF_XLINE);
 
   collapse(gecos);
-  DupString(conf->name, gecos);
-  DupString(match_item->reason, reason);
-  DupString(match_item->oper_reason, "");	/* XXX */
-  cur_time = CurrentTime;
-  current_date = smalldate(cur_time);
+  conf->name = xstrdup(gecos);
+  conf->reason = xstrdup(reason);
+  conf->setat = CurrentTime;
+
+  SetConfDatabase(conf);
 
   if (tkline_time != 0)
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
 			 "%s added temporary %d min. X-Line for [%s] [%s]",
 			 get_oper_name(source_p), (int)tkline_time/60,
-			 conf->name, match_item->reason);
+			 conf->name, conf->reason);
     sendto_one(source_p, ":%s NOTICE %s :Added temporary %d min. X-Line [%s]",
 	       MyConnect(source_p) ? me.name : ID_or_name(&me, source_p->from),
 	       source_p->name, (int)tkline_time/60, conf->name);
-    ilog(LOG_TYPE_KLINE, "%s added temporary %d min. X-Line for [%s] [%s]",
+    ilog(LOG_TYPE_XLINE, "%s added temporary %d min. X-Line for [%s] [%s]",
 	 source_p->name, (int)tkline_time/60,
-	 conf->name, match_item->reason);
-    match_item->hold = CurrentTime + tkline_time;
-    add_temp_line(conf);
+	 conf->name, conf->reason);
+    conf->until = CurrentTime + tkline_time;
   }
   else
-    write_conf_line(source_p, conf, current_date, cur_time);
+  {
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "%s added X-Line for [%s] [%s]",
+                         get_oper_name(source_p), conf->name,
+                         conf->reason);
+    sendto_one(source_p,
+               ":%s NOTICE %s :Added X-Line [%s] [%s]",
+               MyConnect(source_p) ? me.name : ID_or_name(&me, source_p->from),
+               source_p->name, conf->name, conf->reason);
+    ilog(LOG_TYPE_XLINE, "%s added X-Line for [%s] [%s]",
+         get_oper_name(source_p), conf->name, conf->reason);
+  }
+
   rehashed_klines = 1;
 }
 
 static void
 remove_xline(struct Client *source_p, char *gecos)
 {
-  /* XXX use common temporary un function later */
-  if (remove_txline_match(gecos))
+  if (remove_xline_match(gecos))
   {
     sendto_one(source_p,
-               ":%s NOTICE %s :Un-xlined [%s] from temporary X-Lines",
+               ":%s NOTICE %s :X-Line for [%s] is removed",
                me.name, source_p->name, gecos);
-    sendto_realops_flags(UMODE_ALL, L_ALL,
-                         "%s has removed the temporary X-Line for: [%s]",
-                         get_oper_name(source_p), gecos);
-    ilog(LOG_TYPE_KLINE, "%s removed temporary X-Line for [%s]",
-         source_p->name, gecos);
-    return;
-  }
-
-  if (remove_conf_line(XLINE_TYPE, source_p, gecos, NULL) > 0)
-  {
-    sendto_one(source_p, ":%s NOTICE %s :X-Line for [%s] is removed",
-               me.name, source_p->name, gecos);
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "%s has removed the X-Line for: [%s]",
                          get_oper_name(source_p), gecos);
-    ilog(LOG_TYPE_KLINE, "%s removed X-Line for [%s]",
+    ilog(LOG_TYPE_XLINE, "%s removed X-Line for [%s]",
          get_oper_name(source_p), gecos);
   }
   else
@@ -410,21 +387,20 @@ remove_xline(struct Client *source_p, char *gecos)
  * Side effects: Any matching tklines are removed.
  */
 static int
-remove_txline_match(const char *gecos)
+remove_xline_match(const char *gecos)
 {
   dlink_node *ptr = NULL, *next_ptr = NULL;
-  struct ConfItem *conf = NULL;
 
-  DLINK_FOREACH_SAFE(ptr, next_ptr, temporary_xlines.head)
+  DLINK_FOREACH_SAFE(ptr, next_ptr, xconf_items.head)
   {
-    conf = ptr->data;
+    struct MaskItem *conf = ptr->data;
 
-    if (irccmp(gecos, conf->name) == 0)
+    if (!IsConfDatabase(conf))
+      continue;
+
+    if (!irccmp(gecos, conf->name))
     {
-      dlinkDelete(ptr, &temporary_xlines);
-      free_dlink_node(ptr);
-      delete_conf_item(conf);
-
+      conf_free(conf);
       return 1;
     }
   }
@@ -459,7 +435,7 @@ module_exit(void)
 struct module module_entry = {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 1309 $",
+  .version = "$Revision: 2361 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
