@@ -21,7 +21,7 @@
 
 /*! \file channel.c
  * \brief Responsible for managing channels, members, bans and topics
- * \version $Id: channel.c 1913 2013-04-28 11:46:26Z michael $
+ * \version $Id: channel.c 2568 2013-11-17 20:24:04Z michael $
  */
 
 #include "stdinc.h"
@@ -30,27 +30,26 @@
 #include "channel_mode.h"
 #include "client.h"
 #include "hash.h"
+#include "conf.h"
 #include "hostmask.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "s_serv.h"             /* captab */
 #include "s_user.h"
 #include "send.h"
-#include "conf.h"             /* ConfigFileEntry, ConfigChannel */
 #include "event.h"
 #include "memory.h"
-#include "balloc.h"
+#include "mempool.h"
 #include "s_misc.h"
-
+#include "resv.h"
 
 struct config_channel_entry ConfigChannel;
 dlink_list global_channel_list = { NULL, NULL, 0 };
-BlockHeap *ban_heap;    /*! \todo ban_heap shouldn't be a global var */
+mp_pool_t *ban_pool;    /*! \todo ban_pool shouldn't be a global var */
 
-static BlockHeap *member_heap = NULL;
-static BlockHeap *channel_heap = NULL;
+static mp_pool_t *member_pool = NULL;
+static mp_pool_t *channel_pool = NULL;
 
 static char buf[IRCD_BUFSIZE];
 static char modebuf[MODEBUFLEN];
@@ -60,15 +59,15 @@ static char parabuf[MODEBUFLEN];
 /*! \brief Initializes the channel blockheap, adds known channel CAPAB
  */
 void
-init_channels(void)
+channel_init(void)
 {
   add_capability("EX", CAP_EX, 1);
   add_capability("IE", CAP_IE, 1);
   add_capability("CHW", CAP_CHW, 1);
 
-  channel_heap = BlockHeapCreate("channel", sizeof(struct Channel), CHANNEL_HEAP_SIZE);
-  ban_heap = BlockHeapCreate("ban", sizeof(struct Ban), BAN_HEAP_SIZE);
-  member_heap = BlockHeapCreate("member", sizeof(struct Membership), CHANNEL_HEAP_SIZE*2);
+  channel_pool = mp_pool_new(sizeof(struct Channel), MP_CHUNK_SIZE_CHANNEL);
+  ban_pool = mp_pool_new(sizeof(struct Ban), MP_CHUNK_SIZE_BAN);
+  member_pool = mp_pool_new(sizeof(struct Membership), MP_CHUNK_SIZE_MEMBER);
 }
 
 /*! \brief adds a user to a channel by adding another link to the
@@ -105,7 +104,7 @@ add_user_to_channel(struct Channel *chptr, struct Client *who,
       if (!IsSetJoinFloodNoticed(chptr))
       {
         SetJoinFloodNoticed(chptr);
-        sendto_realops_flags(UMODE_BOTS, L_ALL,
+        sendto_realops_flags(UMODE_BOTS, L_ALL, SEND_NOTICE,
                              "Possible Join Flooder %s on %s target: %s",
                              get_client_name(who, HIDE_IP),
                              who->servptr->name, chptr->chname);
@@ -115,7 +114,9 @@ add_user_to_channel(struct Channel *chptr, struct Client *who,
     chptr->last_join_time = CurrentTime;
   }
 
-  ms = BlockHeapAlloc(member_heap);
+  ms = mp_pool_get(member_pool);
+  memset(ms, 0, sizeof(*ms));
+
   ms->client_p = who;
   ms->chptr = chptr;
   ms->flags = flags;
@@ -137,7 +138,7 @@ remove_user_from_channel(struct Membership *member)
   dlinkDelete(&member->channode, &chptr->members);
   dlinkDelete(&member->usernode, &client_p->channel);
 
-  BlockHeapFree(member_heap, member);
+  mp_pool_release(member);
 
   if (chptr->members.head == NULL)
     destroy_channel(chptr);
@@ -153,19 +154,18 @@ static void
 send_members(struct Client *client_p, struct Channel *chptr,
              char *lmodebuf, char *lparabuf)
 {
-  struct Membership *ms;
-  dlink_node *ptr;
+  const dlink_node *ptr = NULL;
   int tlen;              /* length of text to append */
   char *t, *start;       /* temp char pointer */
 
-  start = t = buf + ircsprintf(buf, ":%s SJOIN %lu %s %s %s:",
-                               ID_or_name(&me, client_p),
-                               (unsigned long)chptr->channelts,
-                               chptr->chname, lmodebuf, lparabuf);
+  start = t = buf + snprintf(buf, sizeof(buf), ":%s SJOIN %lu %s %s %s:",
+                             ID_or_name(&me, client_p),
+                             (unsigned long)chptr->channelts,
+                             chptr->chname, lmodebuf, lparabuf);
 
   DLINK_FOREACH(ptr, chptr->members.head)
   {
-    ms = ptr->data;
+    const struct Membership *ms = ptr->data;
 
     tlen = strlen(IsCapable(client_p, CAP_TS6) ?
       ID(ms->client_p) : ms->client_p->name) + 1;  /* nick + space */
@@ -218,11 +218,10 @@ send_members(struct Client *client_p, struct Channel *chptr,
  */
 static void
 send_mode_list(struct Client *client_p, struct Channel *chptr,
-               dlink_list *top, char flag)
+               const dlink_list *top, char flag)
 {
   int ts5 = !IsCapable(client_p, CAP_TS6);
-  dlink_node *lp;
-  struct Ban *banptr;
+  const dlink_node *lp = NULL;
   char pbuf[IRCD_BUFSIZE];
   int tlen, mlen, cur_len, count = 0;
   char *mp = NULL, *pp = pbuf;
@@ -231,10 +230,10 @@ send_mode_list(struct Client *client_p, struct Channel *chptr,
     return;
 
   if (ts5)
-    mlen = ircsprintf(buf, ":%s MODE %s +", me.name, chptr->chname);
+    mlen = snprintf(buf, sizeof(buf), ":%s MODE %s +", me.name, chptr->chname);
   else
-    mlen = ircsprintf(buf, ":%s BMASK %lu %s %c :", me.id,
-                      (unsigned long)chptr->channelts, chptr->chname, flag);
+    mlen = snprintf(buf, sizeof(buf), ":%s BMASK %lu %s %c :", me.id,
+                   (unsigned long)chptr->channelts, chptr->chname, flag);
 
   /* MODE needs additional one byte for space between buf and pbuf */
   cur_len = mlen + ts5;
@@ -242,7 +241,7 @@ send_mode_list(struct Client *client_p, struct Channel *chptr,
 
   DLINK_FOREACH(lp, top->head)
   {
-    banptr = lp->data;
+    const struct Ban *banptr = lp->data;
 
     /* must add another b/e/I letter if we use MODE */
     tlen = banptr->len + 3 + ts5;
@@ -272,8 +271,8 @@ send_mode_list(struct Client *client_p, struct Channel *chptr,
       *mp = '\0';
     }
 
-    pp += ircsprintf(pp, "%s!%s@%s ", banptr->name, banptr->username,
-                     banptr->host);
+    pp += sprintf(pp, "%s!%s@%s ", banptr->name, banptr->user,
+                  banptr->host);
     cur_len += tlen;
   }
 
@@ -303,7 +302,7 @@ send_channel_modes(struct Client *client_p, struct Channel *chptr)
  * \return 0 if invalid, 1 otherwise
  */
 int
-check_channel_name(const char *name, int local)
+check_channel_name(const char *name, const int local)
 {
   const char *p = name;
   const int max_length = local ? LOCAL_CHANNELLEN : CHANNELLEN;
@@ -334,11 +333,11 @@ remove_ban(struct Ban *bptr, dlink_list *list)
   dlinkDelete(&bptr->node, list);
 
   MyFree(bptr->name);
-  MyFree(bptr->username);
+  MyFree(bptr->user);
   MyFree(bptr->host);
   MyFree(bptr->who);
 
-  BlockHeapFree(ban_heap, bptr);
+  mp_pool_release(bptr);
 }
 
 /* free_channel_list()
@@ -370,7 +369,9 @@ make_channel(const char *chname)
 
   assert(!EmptyString(chname));
 
-  chptr = BlockHeapAlloc(channel_heap);
+  chptr = mp_pool_get(channel_pool);
+
+  memset(chptr, 0, sizeof(*chptr));
 
   /* doesn't hurt to set it here */
   chptr->channelts = CurrentTime;
@@ -403,7 +404,7 @@ destroy_channel(struct Channel *chptr)
   dlinkDelete(&chptr->node, &global_channel_list);
   hash_del_channel(chptr);
 
-  BlockHeapFree(channel_heap, chptr);
+  mp_pool_release(chptr);
 }
 
 /*!
@@ -430,9 +431,7 @@ void
 channel_member_names(struct Client *source_p, struct Channel *chptr,
                      int show_eon)
 {
-  struct Client *target_p = NULL;
-  struct Membership *ms = NULL;
-  dlink_node *ptr = NULL;
+  const dlink_node *ptr = NULL;
   char lbuf[IRCD_BUFSIZE + 1];
   char *t = NULL, *start = NULL;
   int tlen = 0;
@@ -441,21 +440,19 @@ channel_member_names(struct Client *source_p, struct Channel *chptr,
 
   if (PubChannel(chptr) || is_member)
   {
-    t = lbuf + ircsprintf(lbuf, form_str(RPL_NAMREPLY),
-                          me.name, source_p->name,
-                          channel_pub_or_secret(chptr),
-                          chptr->chname);
+    t = lbuf + snprintf(lbuf, sizeof(lbuf), form_str(RPL_NAMREPLY),
+                        me.name, source_p->name,
+                        channel_pub_or_secret(chptr), chptr->chname);
     start = t;
 
     DLINK_FOREACH(ptr, chptr->members.head)
     {
-      ms       = ptr->data;
-      target_p = ms->client_p;
+      const struct Membership *ms = ptr->data;
 
-      if (HasUMode(target_p, UMODE_INVISIBLE) && !is_member)
+      if (HasUMode(ms->client_p, UMODE_INVISIBLE) && !is_member)
         continue;
 
-      tlen = strlen(target_p->name) + 1;  /* nick + space */
+      tlen = strlen(ms->client_p->name) + 1;  /* nick + space */
 
       if (!multi_prefix)
       {
@@ -479,8 +476,8 @@ channel_member_names(struct Client *source_p, struct Channel *chptr,
         t = start;
       }
 
-      t += ircsprintf(t, "%s%s ", get_member_status(ms, multi_prefix),
-                      target_p->name);
+      t += sprintf(t, "%s%s ", get_member_status(ms, multi_prefix),
+                   ms->client_p->name);
     }
 
     if (tlen != 0)
@@ -547,14 +544,10 @@ del_invite(struct Channel *chptr, struct Client *who)
  * (like in get_client_name)
  */
 const char *
-get_member_status(const struct Membership *ms, int combine)
+get_member_status(const struct Membership *ms, const int combine)
 {
   static char buffer[4];
-  char *p = NULL;
-
-  if (ms == NULL)
-    return "";
-  p = buffer;
+  char *p = buffer;
 
   if (ms->flags & CHFL_CHANOP)
   {
@@ -594,12 +587,12 @@ find_bmask(const struct Client *who, const dlink_list *const list)
   {
     const struct Ban *bp = ptr->data;
 
-    if (match(bp->name, who->name) && match(bp->username, who->username))
+    if (!match(bp->name, who->name) && !match(bp->user, who->username))
     {
       switch (bp->type)
       {
         case HM_HOST:
-          if (match(bp->host, who->host) || match(bp->host, who->sockhost))
+          if (!match(bp->host, who->host) || !match(bp->host, who->sockhost))
             return 1;
           break;
         case HM_IPV4:
@@ -640,7 +633,7 @@ is_banned(const struct Channel *chptr, const struct Client *who)
 
 /*!
  * \param source_p pointer to client attempting to join
- * \param chptr    pointer to channel 
+ * \param chptr    pointer to channel
  * \param key      key sent by client attempting to join if present
  * \return ERR_BANNEDFROMCHAN, ERR_INVITEONLYCHAN, ERR_CHANNELISFULL
  *         or 0 if allowed to join.
@@ -648,13 +641,8 @@ is_banned(const struct Channel *chptr, const struct Client *who)
 int
 can_join(struct Client *source_p, struct Channel *chptr, const char *key)
 {
-  if (is_banned(chptr, source_p))
-    return ERR_BANNEDFROMCHAN;
-
-#ifdef HAVE_LIBCRYPTO
-  if ((chptr->mode.mode & MODE_SSLONLY) && !source_p->localClient->fd.ssl)
+  if ((chptr->mode.mode & MODE_SSLONLY) && !HasUMode(source_p, UMODE_SSL))
     return ERR_SSLONLYCHAN;
-#endif
 
   if ((chptr->mode.mode & MODE_REGONLY) && !HasUMode(source_p, UMODE_REGISTERED))
     return ERR_NEEDREGGEDNICK;
@@ -674,11 +662,14 @@ can_join(struct Client *source_p, struct Channel *chptr, const char *key)
       chptr->mode.limit)
     return ERR_CHANNELISFULL;
 
+  if (is_banned(chptr, source_p))
+    return ERR_BANNEDFROMCHAN;
+
   return 0;
 }
 
 int
-has_member_flags(struct Membership *ms, unsigned int flags)
+has_member_flags(const struct Membership *ms, const unsigned int flags)
 {
   if (ms != NULL)
     return ms->flags & flags;
@@ -693,13 +684,51 @@ find_channel_link(struct Client *client_p, struct Channel *chptr)
   if (!IsClient(client_p))
     return NULL;
 
-  DLINK_FOREACH(ptr, client_p->channel.head)
-    if (((struct Membership *)ptr->data)->chptr == chptr)
-      return ptr->data;
+  if (dlink_list_length(&chptr->members) < dlink_list_length(&client_p->channel))
+  {
+    DLINK_FOREACH(ptr, chptr->members.head)
+      if (((struct Membership *)ptr->data)->client_p == client_p)
+        return ptr->data;
+  }
+  else
+  {
+    DLINK_FOREACH(ptr, client_p->channel.head)
+      if (((struct Membership *)ptr->data)->chptr == chptr)
+        return ptr->data;
+  }
 
   return NULL;
 }
 
+/*
+ * Basically the same functionality as in bahamut
+ */
+static int
+msg_has_ctrls(const char *message)
+{
+  const unsigned char *p = (const unsigned char *)message;
+
+  for (; *p; ++p)
+  {
+    if (*p > 31 || *p == 1)
+      continue;
+
+    if (*p == 27)
+    {
+      if (*(p + 1) == '$' ||
+          *(p + 1) == '(')
+      {
+        ++p;
+        continue;
+      }
+    }
+
+    return 1;
+  }
+
+  return 0;
+}
+
 /*!
  * \param chptr    pointer to Channel struct
  * \param source_p pointer to Client struct
@@ -709,23 +738,35 @@ find_channel_link(struct Client *client_p, struct Channel *chptr)
  *         ERR_CANNOTSENDTOCHAN or ERR_NEEDREGGEDNICK if they cannot send to channel\n
  */
 int
-can_send(struct Channel *chptr, struct Client *source_p, struct Membership *ms)
+can_send(struct Channel *chptr, struct Client *source_p,
+         struct Membership *ms, const char *message)
 {
+  struct MaskItem *conf = NULL;
+
   if (IsServer(source_p) || HasFlag(source_p, FLAGS_SERVICE))
     return CAN_SEND_OPV;
 
   if (MyClient(source_p) && !IsExemptResv(source_p))
     if (!(HasUMode(source_p, UMODE_OPER) && ConfigFileEntry.oper_pass_resv))
-      if (!hash_find_resv(chptr->chname) == ConfigChannel.restrict_channels)
+      if ((conf = match_find_resv(chptr->chname)) && !resv_find_exempt(source_p, conf))
         return ERR_CANNOTSENDTOCHAN;
 
-  if (ms != NULL || (ms = find_channel_link(source_p, chptr)))
-  {
+  if ((chptr->mode.mode & MODE_NOCTRL) && msg_has_ctrls(message))
+    return ERR_NOCTRLSONCHAN;
+  if (ms || (ms = find_channel_link(source_p, chptr)))
     if (ms->flags & (CHFL_CHANOP|CHFL_HALFOP|CHFL_VOICE))
       return CAN_SEND_OPV;
+  if (!ms && (chptr->mode.mode & MODE_NOPRIVMSGS))
+    return ERR_CANNOTSENDTOCHAN;
+  if (chptr->mode.mode & MODE_MODERATED)
+    return ERR_CANNOTSENDTOCHAN;
+  if ((chptr->mode.mode & MODE_MODREG) && !HasUMode(source_p, UMODE_REGISTERED))
+    return ERR_NEEDREGGEDNICK;
 
-    /* cache can send if quiet_on_ban and banned */
-    if (ConfigChannel.quiet_on_ban && MyClient(source_p))
+  /* cache can send if banned */
+  if (MyClient(source_p))
+  {
+    if (ms)
     {
       if (ms->flags & CHFL_BAN_SILENCED)
         return ERR_CANNOTSENDTOCHAN;
@@ -741,15 +782,9 @@ can_send(struct Channel *chptr, struct Client *source_p, struct Membership *ms)
         ms->flags |= CHFL_BAN_CHECKED;
       }
     }
+    else if (is_banned(chptr, source_p))
+      return ERR_CANNOTSENDTOCHAN;
   }
-  else if (chptr->mode.mode & MODE_NOPRIVMSGS)
-    return ERR_CANNOTSENDTOCHAN;
-
-  if (chptr->mode.mode & MODE_MODERATED)
-    return ERR_CANNOTSENDTOCHAN;
-
-  if ((chptr->mode.mode & MODE_REGONLY) && !HasUMode(source_p, UMODE_REGISTERED))
-    return ERR_NEEDREGGEDNICK;
 
   return CAN_SEND_NONOP;
 }
@@ -779,12 +814,12 @@ check_spambot_warning(struct Client *source_p, const char *name)
     {
       /* Its already known as a possible spambot */
       if (name != NULL)
-        sendto_realops_flags(UMODE_BOTS, L_ALL,
+        sendto_realops_flags(UMODE_BOTS, L_ALL, SEND_NOTICE,
                              "User %s (%s@%s) trying to join %s is a possible spambot",
                              source_p->name, source_p->username,
                              source_p->host, name);
       else
-        sendto_realops_flags(UMODE_BOTS, L_ALL,
+        sendto_realops_flags(UMODE_BOTS, L_ALL, SEND_NOTICE,
                              "User %s (%s@%s) is a possible spambot",
                              source_p->name, source_p->username,
                              source_p->host);
@@ -835,7 +870,7 @@ check_splitmode(void *unused)
     {
       splitmode = 1;
 
-      sendto_realops_flags(UMODE_ALL,L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "Network split, activating splitmode");
       eventAddIsh("check_splitmode", check_splitmode, NULL, 10);
     }
@@ -843,7 +878,7 @@ check_splitmode(void *unused)
     {
       splitmode = 0;
 
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "Network rejoined, deactivating splitmode");
       eventDelete(check_splitmode, NULL);
     }
@@ -866,5 +901,5 @@ set_channel_topic(struct Channel *chptr, const char *topic,
     strlcpy(chptr->topic, topic, sizeof(chptr->topic));
 
   strlcpy(chptr->topic_info, topic_info, sizeof(chptr->topic_info));
-  chptr->topic_time = topicts; 
+  chptr->topic_time = topicts;
 }
