@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: channel_mode.c 1771 2013-01-19 23:11:22Z michael $
+ *  $Id: channel_mode.c 2696 2013-12-18 12:12:12Z michael $
  */
 
 #include "stdinc.h"
@@ -28,19 +28,18 @@
 #include "channel_mode.h"
 #include "client.h"
 #include "hash.h"
+#include "conf.h"
 #include "hostmask.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "s_serv.h"             /* captab */
 #include "s_user.h"
 #include "send.h"
 #include "whowas.h"
-#include "conf.h"             /* ConfigFileEntry, ConfigChannel */
 #include "event.h"
 #include "memory.h"
-#include "balloc.h"
+#include "mempool.h"
 #include "log.h"
 #include "parse.h"
 
@@ -107,7 +106,7 @@ static int channel_capabs[] = { CAP_EX, CAP_IE, CAP_TS6, CAP_HOPS };
 static int channel_capabs[] = { CAP_EX, CAP_IE, CAP_TS6 };
 #endif
 static struct ChCapCombo chcap_combos[NCHCAP_COMBOS];
-extern BlockHeap *ban_heap;
+extern mp_pool_t *ban_pool;
 
 
 /* check_string()
@@ -192,7 +191,7 @@ add_id(struct Client *client_p, struct Channel *chptr, char *banid, int type)
    * Re-assemble a new n!u@h and print it back to banid for sending
    * the mode to the channel.
    */
-  len = ircsprintf(banid, "%s!%s@%s", name, user, host);
+  len = sprintf(banid, "%s!%s@%s", name, user, host);
 
   switch (type)
   {
@@ -216,19 +215,18 @@ add_id(struct Client *client_p, struct Channel *chptr, char *banid, int type)
   {
     ban_p = ban->data;
     if (!irccmp(ban_p->name, name) &&
-        !irccmp(ban_p->username, user) &&
+        !irccmp(ban_p->user, user) &&
         !irccmp(ban_p->host, host))
     {
       return 0;
     }
   }
 
-  ban_p = BlockHeapAlloc(ban_heap);
-
-  DupString(ban_p->name, name);
-  DupString(ban_p->username, user);
-  DupString(ban_p->host, host);
-
+  ban_p = mp_pool_get(ban_pool);
+  memset(ban_p, 0, sizeof(*ban_p));
+  ban_p->name = xstrdup(name);
+  ban_p->user = xstrdup(user);
+  ban_p->host = xstrdup(host);
   ban_p->when = CurrentTime;
   ban_p->len = len - 2; /* -2 for @ and ! */
   ban_p->type = parse_netmask(host, &ban_p->addr, &ban_p->bits);
@@ -238,11 +236,13 @@ add_id(struct Client *client_p, struct Channel *chptr, char *banid, int type)
     ban_p->who = MyMalloc(strlen(client_p->name) +
                           strlen(client_p->username) +
                           strlen(client_p->host) + 3);
-    ircsprintf(ban_p->who, "%s!%s@%s", client_p->name,
-               client_p->username, client_p->host);
+    sprintf(ban_p->who, "%s!%s@%s", client_p->name,
+            client_p->username, client_p->host);
   }
+  else if (IsHidden(client_p) || (IsServer(client_p) && ConfigServerHide.hide_servers))
+    ban_p->who = xstrdup(me.name);
   else
-    DupString(ban_p->who, client_p->name);
+    ban_p->who = xstrdup(client_p->name);
 
   dlinkAdd(ban_p, &ban_p->node, list);
 
@@ -285,7 +285,7 @@ del_id(struct Channel *chptr, char *banid, int type)
    * Re-assemble a new n!u@h and print it back to banid for sending
    * the mode to the channel.
    */
-  ircsprintf(banid, "%s!%s@%s", name, user, host);
+  sprintf(banid, "%s!%s@%s", name, user, host);
 
   switch (type)
   {
@@ -301,7 +301,7 @@ del_id(struct Channel *chptr, char *banid, int type)
       list = &chptr->invexlist;
       break;
     default:
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "del_id() called with unknown ban type %d!", type);
       return 0;
   }
@@ -311,7 +311,7 @@ del_id(struct Channel *chptr, char *banid, int type)
     banptr = ban->data;
 
     if (!irccmp(name, banptr->name) &&
-        !irccmp(user, banptr->username) &&
+        !irccmp(user, banptr->user) &&
         !irccmp(host, banptr->host))
     {
       remove_ban(banptr, list);
@@ -323,6 +323,7 @@ del_id(struct Channel *chptr, char *banid, int type)
 }
 
 const struct mode_letter chan_modes[] = {
+  { MODE_NOCTRL,     'c' },
   { MODE_INVITEONLY, 'i' },
   { MODE_MODERATED,  'm' },
   { MODE_NOPRIVMSGS, 'n' },
@@ -330,6 +331,7 @@ const struct mode_letter chan_modes[] = {
   { MODE_REGISTERED, 'r' },
   { MODE_SECRET,     's' },
   { MODE_TOPICLIMIT, 't' },
+  { MODE_MODREG,     'M' },
   { MODE_OPERONLY,   'O' },
   { MODE_REGONLY,    'R' },
   { MODE_SSLONLY,    'S' },
@@ -364,7 +366,7 @@ channel_modes(struct Channel *chptr, struct Client *client_p,
     *mbuf++ = 'l';
 
     if (IsServer(client_p) || HasFlag(client_p, FLAGS_SERVICE) || IsMember(client_p, chptr))
-      pbuf += ircsprintf(pbuf, "%d ", chptr->mode.limit);
+      pbuf += sprintf(pbuf, "%d ", chptr->mode.limit);
   }
 
   if (chptr->mode.key[0])
@@ -372,7 +374,7 @@ channel_modes(struct Channel *chptr, struct Client *client_p,
     *mbuf++ = 'k';
 
     if (IsServer(client_p) || HasFlag(client_p, FLAGS_SERVICE) || IsMember(client_p, chptr))
-      ircsprintf(pbuf, "%s ", chptr->mode.key);
+      sprintf(pbuf, "%s ", chptr->mode.key);
   }
 
   *mbuf = '\0';
@@ -586,7 +588,7 @@ chm_simple(struct Client *client_p, struct Client *source_p, struct Channel *chp
    * 
    * -Dianora 
    */ 
-  if ((dir == MODE_ADD)) /* && !(chptr->mode.mode & mode_type)) */
+  if (dir == MODE_ADD) /* && !(chptr->mode.mode & mode_type)) */
   {
     chptr->mode.mode |= mode_type;
 
@@ -598,7 +600,7 @@ chm_simple(struct Client *client_p, struct Client *source_p, struct Channel *chp
     mode_changes[mode_count].mems = ALL_MEMBERS;
     mode_changes[mode_count++].arg = NULL;
   }
-  else if ((dir == MODE_DEL)) /* && (chptr->mode.mode & mode_type)) */
+  else if (dir == MODE_DEL) /* && (chptr->mode.mode & mode_type)) */
   {
     /* setting - */
 
@@ -648,7 +650,7 @@ chm_registered(struct Client *client_p, struct Client *source_p, struct Channel
    * 
    * -Dianora 
    */
-  if ((dir == MODE_ADD)) /* && !(chptr->mode.mode & mode_type)) */
+  if (dir == MODE_ADD) /* && !(chptr->mode.mode & mode_type)) */
   {
     chptr->mode.mode |= mode_type;
 
@@ -660,7 +662,7 @@ chm_registered(struct Client *client_p, struct Client *source_p, struct Channel
     mode_changes[mode_count].mems = ALL_MEMBERS;
     mode_changes[mode_count++].arg = NULL;
   }
-  else if ((dir == MODE_DEL)) /* && (chptr->mode.mode & mode_type)) */
+  else if (dir == MODE_DEL) /* && (chptr->mode.mode & mode_type)) */
   {
     /* setting - */
 
@@ -717,7 +719,7 @@ chm_operonly(struct Client *client_p, struct Client *source_p, struct Channel *c
 
   simple_modes_mask |= mode_type;
 
-  if ((dir == MODE_ADD)) /* && !(chptr->mode.mode & mode_type)) */
+  if (dir == MODE_ADD) /* && !(chptr->mode.mode & mode_type)) */
   {
     chptr->mode.mode |= mode_type;
 
@@ -730,7 +732,7 @@ chm_operonly(struct Client *client_p, struct Client *source_p, struct Channel *c
     mode_changes[mode_count].mems = ALL_MEMBERS;
     mode_changes[mode_count++].arg = NULL;
   }
-  else if ((dir == MODE_DEL)) /* && (chptr->mode.mode & mode_type)) */
+  else if (dir == MODE_DEL) /* && (chptr->mode.mode & mode_type)) */
   {
     /* setting - */
 
@@ -768,7 +770,7 @@ chm_ban(struct Client *client_p, struct Client *source_p,
       const struct Ban *banptr = ptr->data;
       sendto_one(client_p, form_str(RPL_BANLIST),
                  me.name, client_p->name, chname,
-                 banptr->name, banptr->username, banptr->host,
+                 banptr->name, banptr->user, banptr->host,
 		 banptr->who, banptr->when);
     }
 
@@ -853,7 +855,7 @@ chm_except(struct Client *client_p, struct Client *source_p,
       const struct Ban *banptr = ptr->data;
       sendto_one(client_p, form_str(RPL_EXCEPTLIST),
                  me.name, client_p->name, chname,
-                 banptr->name, banptr->username, banptr->host,
+                 banptr->name, banptr->user, banptr->host,
 		 banptr->who, banptr->when);
     }
 
@@ -889,7 +891,7 @@ chm_except(struct Client *client_p, struct Client *source_p,
 
   mode_changes[mode_count].letter = c;
   mode_changes[mode_count].dir = dir;
-  mode_changes[mode_count].caps = CAP_EX;
+  mode_changes[mode_count].caps = 0;
   mode_changes[mode_count].nocaps = 0;
   mode_changes[mode_count].mems = ONLY_CHANOPS;
   mode_changes[mode_count].id = NULL;
@@ -928,7 +930,7 @@ chm_invex(struct Client *client_p, struct Client *source_p,
       const struct Ban *banptr = ptr->data;
       sendto_one(client_p, form_str(RPL_INVITELIST), me.name,
                  client_p->name, chname,
-		 banptr->name, banptr->username, banptr->host,
+		 banptr->name, banptr->user, banptr->host,
                  banptr->who, banptr->when);
     }
 
@@ -964,7 +966,7 @@ chm_invex(struct Client *client_p, struct Client *source_p,
 
   mode_changes[mode_count].letter = c;
   mode_changes[mode_count].dir = dir;
-  mode_changes[mode_count].caps = CAP_IE;
+  mode_changes[mode_count].caps = 0;
   mode_changes[mode_count].nocaps = 0;
   mode_changes[mode_count].mems = ONLY_CHANOPS;
   mode_changes[mode_count].id = NULL;
@@ -1028,7 +1030,7 @@ chm_op(struct Client *client_p, struct Client *source_p,
 
   opnick = parv[(*parn)++];
 
-  if ((targ_p = find_chasing(client_p, source_p, opnick, NULL)) == NULL)
+  if ((targ_p = find_chasing(source_p, opnick, NULL)) == NULL)
     return;
   if (!IsClient(targ_p))
     return;
@@ -1134,7 +1136,7 @@ chm_hop(struct Client *client_p, struct Client *source_p,
 
   opnick = parv[(*parn)++];
 
-  if ((targ_p = find_chasing(client_p, source_p, opnick, NULL)) == NULL)
+  if ((targ_p = find_chasing(source_p, opnick, NULL)) == NULL)
     return;
   if (!IsClient(targ_p))
     return;
@@ -1210,7 +1212,7 @@ chm_voice(struct Client *client_p, struct Client *source_p,
 
   opnick = parv[(*parn)++];
 
-  if ((targ_p = find_chasing(client_p, source_p, opnick, NULL)) == NULL)
+  if ((targ_p = find_chasing(source_p, opnick, NULL)) == NULL)
     return;
   if (!IsClient(targ_p))
     return;
@@ -1277,7 +1279,7 @@ chm_limit(struct Client *client_p, struct Client *source_p,
     if ((limit = atoi(lstr)) <= 0)
       return;
 
-    ircsprintf(lstr, "%d", limit);
+    sprintf(lstr, "%d", limit);
 
     /* if somebody sets MODE #channel +ll 1 2, accept latter --fl */
     for (i = 0; i < mode_count; i++)
@@ -1409,7 +1411,7 @@ static struct ChannelMode ModeTable[255] =
   {chm_nosuch, NULL},                             /* J */
   {chm_nosuch, NULL},                             /* K */
   {chm_nosuch, NULL},                             /* L */
-  {chm_nosuch, NULL},                             /* M */
+  {chm_simple, (void *)MODE_MODREG},              /* M */
   {chm_nosuch, NULL},                             /* N */
   {chm_operonly, (void *) MODE_OPERONLY},         /* O */
   {chm_nosuch, NULL},                             /* P */
@@ -1431,7 +1433,7 @@ static struct ChannelMode ModeTable[255] =
   {chm_nosuch, NULL},
   {chm_nosuch, NULL},				  /* a */
   {chm_ban, NULL},                                /* b */
-  {chm_nosuch, NULL},                             /* c */
+  {chm_simple, (void *) MODE_NOCTRL},             /* c */
   {chm_nosuch, NULL},                             /* d */
   {chm_except, NULL},                             /* e */
   {chm_nosuch, NULL},                             /* f */
@@ -1532,10 +1534,10 @@ send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
   parptr = parabuf;
 
   if ((cap & CAP_TS6) && source_p->id[0] != '\0')
-    mbl = ircsprintf(modebuf, ":%s TMODE %lu %s ", source_p->id,
+    mbl = sprintf(modebuf, ":%s TMODE %lu %s ", source_p->id,
                      (unsigned long)chptr->channelts, chptr->chname);
   else
-    mbl = ircsprintf(modebuf, ":%s MODE %s ", source_p->name,
+    mbl = sprintf(modebuf, ":%s MODE %s ", source_p->name,
                      chptr->chname);
 
   /* loop the list of - modes we have */
@@ -1581,11 +1583,11 @@ send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
       mc = 0;
 
       if ((cap & CAP_TS6) && source_p->id[0] != '\0')
-        mbl = ircsprintf(modebuf, ":%s MODE %s ", source_p->id,
-                         chptr->chname);
+        mbl = sprintf(modebuf, ":%s MODE %s ", source_p->id,
+                      chptr->chname);
       else
-        mbl = ircsprintf(modebuf, ":%s MODE %s ", source_p->name,
-                         chptr->chname);
+        mbl = sprintf(modebuf, ":%s MODE %s ", source_p->name,
+                      chptr->chname);
 
       pbl = 0;
       parabuf[0] = '\0';
@@ -1605,7 +1607,7 @@ send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
 
     if (arg != NULL)
     {
-      len = ircsprintf(parptr, "%s ", arg);
+      len = sprintf(parptr, "%s ", arg);
       pbl += len;
       parptr += len;
       mc++;
@@ -1646,12 +1648,12 @@ send_mode_changes(struct Client *client_p, struct Client *source_p,
     return;
 
   if (IsServer(source_p))
-    mbl = ircsprintf(modebuf, ":%s MODE %s ", (IsHidden(source_p) ||
-		     ConfigServerHide.hide_servers) ?
-		     me.name : source_p->name, chname);
+    mbl = sprintf(modebuf, ":%s MODE %s ", (IsHidden(source_p) ||
+                  ConfigServerHide.hide_servers) ?
+                  me.name : source_p->name, chname);
   else
-    mbl = ircsprintf(modebuf, ":%s!%s@%s MODE %s ", source_p->name,
-                     source_p->username, source_p->host, chname);
+    mbl = sprintf(modebuf, ":%s!%s@%s MODE %s ", source_p->name,
+                  source_p->username, source_p->host, chname);
 
   mc = 0;
   nc = 0;
@@ -1687,10 +1689,12 @@ send_mode_changes(struct Client *client_p, struct Client *source_p,
       mc = 0;
 
       if (IsServer(source_p))
-        mbl = ircsprintf(modebuf, ":%s MODE %s ", me.name, chname);
+        mbl = sprintf(modebuf, ":%s MODE %s ", (IsHidden(source_p) ||
+                      ConfigServerHide.hide_servers) ?
+                      me.name : source_p->name, chname);
       else
-        mbl = ircsprintf(modebuf, ":%s!%s@%s MODE %s ", source_p->name,
-                         source_p->username, source_p->host, chname);
+        mbl = sprintf(modebuf, ":%s!%s@%s MODE %s ", source_p->name,
+                      source_p->username, source_p->host, chname);
 
       pbl = 0;
       parabuf[0] = '\0';
@@ -1710,7 +1714,7 @@ send_mode_changes(struct Client *client_p, struct Client *source_p,
 
     if (arg != NULL)
     {
-      len = ircsprintf(parptr, "%s ", arg);
+      len = sprintf(parptr, "%s ", arg);
       pbl += len;
       parptr += len;
       mc++;
@@ -1763,10 +1767,6 @@ set_channel_mode(struct Client *client_p, struct Client *source_p, struct Channe
 
   for (; (c = *ml) != '\0'; ml++) 
   {
-#if 0
-    if(mode_count > 20)
-      break;
-#endif
     switch (c)
     {
       case '+':
