@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: client.c 1784 2013-01-26 22:40:14Z michael $
+ *  $Id: client.c 2740 2014-01-04 20:04:39Z michael $
  */
 
 #include "stdinc.h"
@@ -45,13 +45,14 @@
 #include "s_user.h"
 #include "dbuf.h"
 #include "memory.h"
+#include "mempool.h"
 #include "hostmask.h"
-#include "balloc.h"
 #include "listener.h"
 #include "irc_res.h"
 #include "userhost.h"
 #include "watch.h"
 #include "rng_mt.h"
+#include "parse.h"
 
 dlink_list listing_client_list = { NULL, NULL, 0 };
 /* Pointer to beginning of Client list */
@@ -65,8 +66,8 @@ dlink_list oper_list = {NULL, NULL, 0};
 
 static EVH check_pings;
 
-static BlockHeap *client_heap  = NULL;
-static BlockHeap *lclient_heap = NULL;
+static mp_pool_t *client_pool  = NULL;
+static mp_pool_t *lclient_pool = NULL;
 
 static dlink_list dead_list  = { NULL, NULL, 0};
 static dlink_list abort_list = { NULL, NULL, 0};
@@ -75,23 +76,23 @@ static dlink_node *eac_next;  /* next aborted client to exit */
 
 static void check_pings_list(dlink_list *);
 static void check_unknowns_list(void);
-static void ban_them(struct Client *, struct ConfItem *);
+static void ban_them(struct Client *, struct MaskItem *);
 
 
-/* init_client()
+/* client_init()
  *
  * inputs	- NONE
  * output	- NONE
  * side effects	- initialize client free memory
  */
 void
-init_client(void)
+client_init(void)
 {
   /* start off the check ping event ..  -- adrian
    * Every 30 seconds is plenty -- db
    */
-  client_heap = BlockHeapCreate("client", sizeof(struct Client), CLIENT_HEAP_SIZE);
-  lclient_heap = BlockHeapCreate("local client", sizeof(struct LocalUser), LCLIENT_HEAP_SIZE);
+  client_pool = mp_pool_new(sizeof(struct Client), MP_CHUNK_SIZE_CLIENT);
+  lclient_pool = mp_pool_new(sizeof(struct LocalUser), MP_CHUNK_SIZE_LCLIENT);
   eventAdd("check_pings", check_pings, NULL, 5);
 }
 
@@ -113,12 +114,17 @@ init_client(void)
 struct Client *
 make_client(struct Client *from)
 {
-  struct Client *client_p = BlockHeapAlloc(client_heap);
+  struct Client *client_p = mp_pool_get(client_pool);
+
+  memset(client_p, 0, sizeof(*client_p));
 
   if (from == NULL)
   {
     client_p->from                      = client_p; /* 'from' of local client is self! */
-    client_p->localClient               = BlockHeapAlloc(lclient_heap);
+    client_p->localClient               = mp_pool_get(lclient_pool);
+
+    memset(client_p->localClient, 0, sizeof(*client_p->localClient));
+
     client_p->localClient->since        = CurrentTime;
     client_p->localClient->lasttime     = CurrentTime;
     client_p->localClient->firsttime    = CurrentTime;
@@ -132,7 +138,7 @@ make_client(struct Client *from)
 
   client_p->idhnext = client_p;
   client_p->hnext  = client_p;
-  client_p->status = STAT_UNKNOWN;
+  SetUnknown(client_p);
   strcpy(client_p->username, "unknown");
   strcpy(client_p->svid, "0");
 
@@ -159,12 +165,13 @@ free_client(struct Client *client_p)
   assert(!IsServer(client_p) || (IsServer(client_p) && client_p->serv));
 
   MyFree(client_p->serv);
+  MyFree(client_p->certfp);
 
   if (MyConnect(client_p))
   {
     assert(client_p->localClient->invited.head == NULL);
     assert(dlink_list_length(&client_p->localClient->invited) == 0);
-    assert(dlink_list_length(&client_p->localClient->watches) == 0); 
+    assert(dlink_list_length(&client_p->localClient->watches) == 0);
     assert(IsClosing(client_p) && IsDead(client_p));
 
     MyFree(client_p->localClient->response);
@@ -177,17 +184,17 @@ free_client(struct Client *client_p)
     {
       assert(0 < client_p->localClient->listener->ref_count);
       if (0 == --client_p->localClient->listener->ref_count &&
-          !client_p->localClient->listener->active) 
+          !client_p->localClient->listener->active)
         free_listener(client_p->localClient->listener);
     }
 
     dbuf_clear(&client_p->localClient->buf_recvq);
     dbuf_clear(&client_p->localClient->buf_sendq);
 
-    BlockHeapFree(lclient_heap, client_p->localClient);
+    mp_pool_release(client_p->localClient);
   }
 
-  BlockHeapFree(client_heap, client_p);
+  mp_pool_release(client_p);
 }
 
 /*
@@ -196,7 +203,7 @@ free_client(struct Client *client_p)
  *
  * inputs       - NOT USED (from event)
  * output       - next time_t when check_pings() should be called again
- * side effects - 
+ * side effects -
  *
  *
  * A PING can be sent to clients as necessary.
@@ -218,7 +225,7 @@ free_client(struct Client *client_p)
 
 static void
 check_pings(void *notused)
-{               
+{
   check_pings_list(&local_client_list);
   check_pings_list(&serv_list);
   check_unknowns_list();
@@ -228,19 +235,18 @@ check_pings(void *notused)
  *
  * inputs	- pointer to list to check
  * output	- NONE
- * side effects	- 
+ * side effects	-
  */
 static void
 check_pings_list(dlink_list *list)
 {
-  char scratch[32];        /* way too generous but... */
-  struct Client *client_p; /* current local client_p being examined */
-  int ping, pingwarn;      /* ping time value from client */
-  dlink_node *ptr, *next_ptr;
+  char scratch[IRCD_BUFSIZE];
+  int ping = 0;      /* ping time value from client */
+  dlink_node *ptr = NULL, *next_ptr = NULL;
 
   DLINK_FOREACH_SAFE(ptr, next_ptr, list->head)
   {
-    client_p = ptr->data;
+    struct Client *client_p = ptr->data;
 
     /*
     ** Note: No need to notify opers here. It's
@@ -249,27 +255,26 @@ check_pings_list(dlink_list *list)
     if (IsDead(client_p))
     {
       /* Ignore it, its been exited already */
-      continue; 
+      continue;
     }
 
     if (!IsRegistered(client_p))
-      ping = CONNECTTIMEOUT, pingwarn = 0;
+      ping = CONNECTTIMEOUT;
     else
-      ping = get_client_ping(client_p, &pingwarn);
+      ping = get_client_ping(&client_p->localClient->confs);
 
     if (ping < CurrentTime - client_p->localClient->lasttime)
     {
       if (!IsPingSent(client_p))
       {
-	/*
-	 * if we havent PINGed the connection and we havent
-	 * heard from it in a while, PING it to make sure
-	 * it is still alive.
-	 */
-	SetPingSent(client_p);
-	ClearPingWarning(client_p);
-	client_p->localClient->lasttime = CurrentTime - ping;
-	sendto_one(client_p, "PING :%s", ID_or_name(&me, client_p));
+        /*
+         * if we havent PINGed the connection and we havent
+         * heard from it in a while, PING it to make sure
+         * it is still alive.
+         */
+        SetPingSent(client_p);
+        client_p->localClient->lasttime = CurrentTime - ping;
+        sendto_one(client_p, "PING :%s", ID_or_name(&me, client_p));
       }
       else
       {
@@ -280,39 +285,21 @@ check_pings_list(dlink_list *list)
            * and it has a ping time, then close its connection.
            */
           if (IsServer(client_p) || IsHandshake(client_p))
-	  {
-	    sendto_realops_flags(UMODE_ALL, L_ADMIN,
-				 "No response from %s, closing link",
-				 get_client_name(client_p, HIDE_IP));
-	    sendto_realops_flags(UMODE_ALL, L_OPER,
-				 "No response from %s, closing link",
-				 get_client_name(client_p, MASK_IP));
-	    ilog(LOG_TYPE_IRCD, "No response from %s, closing link",
-		 get_client_name(client_p, HIDE_IP));
-	  }
+          {
+            sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
+                                 "No response from %s, closing link",
+                                 get_client_name(client_p, HIDE_IP));
+            sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
+                                 "No response from %s, closing link",
+                                 get_client_name(client_p, MASK_IP));
+            ilog(LOG_TYPE_IRCD, "No response from %s, closing link",
+                 get_client_name(client_p, HIDE_IP));
+          }
 
           snprintf(scratch, sizeof(scratch), "Ping timeout: %d seconds",
                    (int)(CurrentTime - client_p->localClient->lasttime));
           exit_client(client_p, &me, scratch);
         }
-        else if (!IsPingWarning(client_p) && pingwarn > 0 &&
-	         (IsServer(client_p) || IsHandshake(client_p)) &&
-	         CurrentTime - client_p->localClient->lasttime >= ping + pingwarn)
-        {
-          /*
-           * If the server hasn't replied in pingwarn seconds after sending
-           * the PING, notify the opers so that they are aware of the problem.
-           */
-	  SetPingWarning(client_p);
-          sendto_realops_flags(UMODE_ALL, L_ADMIN,
-	                       "Warning, no response from %s in %d seconds",
-	                       get_client_name(client_p, HIDE_IP), pingwarn);
-	  sendto_realops_flags(UMODE_ALL, L_OPER,
-	                       "Warning, no response from %s in %d seconds",
-	                       get_client_name(client_p, MASK_IP), pingwarn);
-          ilog(LOG_TYPE_IRCD, "No response from %s in %d seconds",
-	       get_client_name(client_p, HIDE_IP), pingwarn);
-        }
       }
     }
   }
@@ -349,12 +336,11 @@ check_unknowns_list(void)
  * side effects - Check all connections for a pending kline against the
  * 		  client, exit the client if a kline matches.
  */
-void 
+void
 check_conf_klines(void)
-{               
+{
   struct Client *client_p = NULL;       /* current local client_p being examined */
-  struct AccessItem *aconf = NULL;
-  struct ConfItem *conf = NULL;
+  struct MaskItem *conf = NULL;
   dlink_node *ptr, *next_ptr;
 
   DLINK_FOREACH_SAFE(ptr, next_ptr, local_client_list.head)
@@ -366,57 +352,55 @@ check_conf_klines(void)
     if (IsDead(client_p) || !IsClient(client_p))
       continue;
 
-    /* if there is a returned struct ConfItem then kill it */
-    if ((aconf = find_dline_conf(&client_p->localClient->ip,
+    if ((conf = find_dline_conf(&client_p->localClient->ip,
                                   client_p->localClient->aftype)) != NULL)
     {
-      if (aconf->status & CONF_EXEMPTDLINE)
-	continue;
+      if (conf->type == CONF_EXEMPT)
+        continue;
 
-      conf = unmap_conf_item(aconf);
       ban_them(client_p, conf);
       continue; /* and go examine next fd/client_p */
     }
 
-    if (ConfigFileEntry.glines && (aconf = find_gline(client_p)))
+    if (ConfigFileEntry.glines)
     {
-      if (IsExemptKline(client_p) ||
-          IsExemptGline(client_p))
+      if ((conf = find_conf_by_address(client_p->host, &client_p->localClient->ip,
+                                       CONF_GLINE, client_p->localClient->aftype,
+                                       client_p->username, NULL, 1)))
       {
-        sendto_realops_flags(UMODE_ALL, L_ALL,
-                             "GLINE over-ruled for %s, client is %sline_exempt",
-                             get_client_name(client_p, HIDE_IP), IsExemptKline(client_p) ? "k" : "g");
+        if (IsExemptKline(client_p) ||
+            IsExemptGline(client_p))
+        {
+          sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                               "GLINE over-ruled for %s, client is %sline_exempt",
+                               get_client_name(client_p, HIDE_IP), IsExemptKline(client_p) ? "k" : "g");
+          continue;
+        }
+
+        ban_them(client_p, conf);
+        /* and go examine next fd/client_p */
         continue;
       }
+    }
 
-      conf = unmap_conf_item(aconf);
-      ban_them(client_p, conf);
-      /* and go examine next fd/client_p */    
-      continue;
-    } 
-
-    if ((aconf = find_kill(client_p)) != NULL) 
+    if ((conf = find_conf_by_address(client_p->host, &client_p->localClient->ip,
+                                     CONF_KLINE, client_p->localClient->aftype,
+                                     client_p->username, NULL, 1)))
     {
-
-      /* if there is a returned struct AccessItem.. then kill it */
       if (IsExemptKline(client_p))
       {
-        sendto_realops_flags(UMODE_ALL, L_ALL,
+        sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                              "KLINE over-ruled for %s, client is kline_exempt",
                              get_client_name(client_p, HIDE_IP));
         continue;
       }
 
-      conf = unmap_conf_item(aconf);
       ban_them(client_p, conf);
-      continue; 
+      continue;
     }
 
-    /* if there is a returned struct MatchItem then kill it */
-    if ((conf = find_matching_name_conf(XLINE_TYPE,  client_p->info,
-                                        NULL, NULL, 0)) != NULL ||
-        (conf = find_matching_name_conf(RXLINE_TYPE, client_p->info,
-                                        NULL, NULL, 0)) != NULL)
+    if ((conf = find_matching_name_conf(CONF_XLINE,  client_p->info,
+                                        NULL, NULL, 0)))
     {
       ban_them(client_p, conf);
       continue;
@@ -428,10 +412,10 @@ check_conf_klines(void)
   {
     client_p = ptr->data;
 
-    if ((aconf = find_dline_conf(&client_p->localClient->ip,
-                                  client_p->localClient->aftype)))
+    if ((conf = find_dline_conf(&client_p->localClient->ip,
+                                 client_p->localClient->aftype)))
     {
-      if (aconf->status & CONF_EXEMPTDLINE)
+      if (conf->type == CONF_EXEMPT)
         continue;
 
       exit_client(client_p, &me, "D-lined");
@@ -443,16 +427,14 @@ check_conf_klines(void)
  * ban_them
  *
  * inputs	- pointer to client to ban
- * 		- pointer to ConfItem
+ * 		- pointer to MaskItem
  * output	- NONE
  * side effects	- given client_p is banned
  */
 static void
-ban_them(struct Client *client_p, struct ConfItem *conf)
+ban_them(struct Client *client_p, struct MaskItem *conf)
 {
-  const char *user_reason = NULL;	/* What is sent to user */
-  struct AccessItem *aconf = NULL;
-  struct MatchItem *xconf = NULL;
+  const char *user_reason = NULL;  /* What is sent to user */
   const char *type_string = NULL;
   const char dline_string[] = "D-line";
   const char kline_string[] = "K-line";
@@ -461,41 +443,32 @@ ban_them(struct Client *client_p, struct ConfItem *conf)
 
   switch (conf->type)
   {
-    case RKLINE_TYPE:
-    case KLINE_TYPE:
+    case CONF_KLINE:
       type_string = kline_string;
-      aconf = map_to_conf(conf);
       break;
-    case DLINE_TYPE:
+    case CONF_DLINE:
       type_string = dline_string;
-      aconf = map_to_conf(conf);
       break;
-    case GLINE_TYPE:
+    case CONF_GLINE:
       type_string = gline_string;
-      aconf = map_to_conf(conf);
       break;
-    case RXLINE_TYPE:
-    case XLINE_TYPE:
+    case CONF_XLINE:
       type_string = xline_string;
-      xconf = map_to_conf(conf);
-      ++xconf->count;
+      ++conf->count;
       break;
     default:
       assert(0);
       break;
   }
 
-  if (aconf != NULL)
-    user_reason = aconf->reason ? aconf->reason : type_string;
-  if (xconf != NULL)
-    user_reason = xconf->reason ? xconf->reason : type_string;
+  user_reason = conf->reason ? conf->reason : type_string;
 
-  sendto_realops_flags(UMODE_ALL, L_ALL, "%s active for %s",
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE, "%s active for %s",
                        type_string, get_client_name(client_p, HIDE_IP));
 
   if (IsClient(client_p))
     sendto_one(client_p, form_str(ERR_YOUREBANNEDCREEP),
-	       me.name, client_p->name, user_reason);
+               me.name, client_p->name, user_reason);
 
   exit_client(client_p, &me, user_reason);
 }
@@ -504,7 +477,7 @@ ban_them(struct Client *client_p, struct ConfItem *conf)
  *
  * input	- pointer to client
  * output	- NONE
- * side effects	- 
+ * side effects	-
  */
 static void
 update_client_exit_stats(struct Client *client_p)
@@ -519,7 +492,8 @@ update_client_exit_stats(struct Client *client_p)
       --Count.invisi;
   }
   else if (IsServer(client_p))
-    sendto_realops_flags(UMODE_EXTERNAL, L_ALL, "Server %s split from %s",
+    sendto_realops_flags(UMODE_EXTERNAL, L_ALL, SEND_NOTICE,
+                         "Server %s split from %s",
                          client_p->name, client_p->servptr->name);
 
   if (splitchecking && !splitmode)
@@ -535,34 +509,29 @@ update_client_exit_stats(struct Client *client_p)
 struct Client *
 find_person(const struct Client *client_p, const char *name)
 {
-  struct Client *c2ptr = NULL;
+  struct Client *target_p = NULL;
 
   if (IsDigit(*name))
   {
-    if ((c2ptr = hash_find_id(name)) != NULL)
-    {
-      /* invisible users shall not be found by UID guessing */
-      if (HasUMode(c2ptr, UMODE_INVISIBLE))
-        if (!IsServer(client_p) && !HasFlag(client_p, FLAGS_SERVICE))
-          c2ptr = NULL;
-    }
+    if (IsServer(client_p) || HasFlag(client_p, FLAGS_SERVICE))
+      target_p = hash_find_id(name);
   }
   else
-    c2ptr = hash_find_client(name);
+    target_p = hash_find_client(name);
 
-  return ((c2ptr != NULL && IsClient(c2ptr)) ? c2ptr : NULL);
+  return (target_p && IsClient(target_p)) ? target_p : NULL;
 }
 
 /*
- * find_chasing - find the client structure for a nick name (user) 
- *      using history mechanism if necessary. If the client is not found, 
+ * find_chasing - find the client structure for a nick name (name)
+ *      using history mechanism if necessary. If the client is not found,
  *      an error message (NO SUCH NICK) is generated. If the client was found
  *      through the history, chasing will be 1 and otherwise 0.
  */
 struct Client *
-find_chasing(struct Client *client_p, struct Client *source_p, const char *user, int *chasing)
+find_chasing(struct Client *source_p, const char *name, int *const chasing)
 {
-  struct Client *who = find_person(client_p, user);
+  struct Client *who = find_person(source_p->from, name);
 
   if (chasing)
     *chasing = 0;
@@ -570,15 +539,15 @@ find_chasing(struct Client *client_p, struct Client *source_p, const char *user,
   if (who)
     return who;
 
-  if (IsDigit(*user))
+  if (IsDigit(*name))
     return NULL;
 
-  if ((who = get_history(user,
-			(time_t)ConfigFileEntry.kill_chase_time_limit))
-			 == NULL)
+  if ((who = whowas_get_history(name,
+                         (time_t)ConfigFileEntry.kill_chase_time_limit))
+                         == NULL)
   {
     sendto_one(source_p, form_str(ERR_NOSUCHNICK),
-               me.name, source_p->name, user);
+               me.name, source_p->name, name);
     return NULL;
   }
 
@@ -613,7 +582,7 @@ get_client_name(const struct Client *client, enum addr_mask_type type)
 
   assert(client != NULL);
 
-  if (!MyConnect(client)) 
+  if (!MyConnect(client))
     return client->name;
 
   if (IsServer(client) || IsConnecting(client) || IsHandshake(client))
@@ -657,7 +626,7 @@ void
 free_exited_clients(void)
 {
   dlink_node *ptr = NULL, *next = NULL;
-  
+
   DLINK_FOREACH_SAFE(ptr, next, dead_list.head)
   {
     free_client(ptr->data);
@@ -690,14 +659,14 @@ exit_one_client(struct Client *source_p, const char *quitmsg)
      * that the client can show the "**signoff" message).
      * (Note: The notice is to the local clients *only*)
      */
-    sendto_common_channels_local(source_p, 0, ":%s!%s@%s QUIT :%s",
+    sendto_common_channels_local(source_p, 0, 0, ":%s!%s@%s QUIT :%s",
                                  source_p->name, source_p->username,
                                  source_p->host, quitmsg);
     DLINK_FOREACH_SAFE(lp, next_lp, source_p->channel.head)
       remove_user_from_channel(lp->data);
 
-    add_history(source_p, 0);
-    off_history(source_p);
+    whowas_add_history(source_p, 0);
+    whowas_off_history(source_p);
 
     watch_check_hash(source_p, RPL_LOGOFF);
 
@@ -786,10 +755,10 @@ recurse_send_quits(struct Client *original_source_p, struct Client *source_p,
   }
 }
 
-/* 
+/*
  * Remove all clients that depend on source_p; assumes all (S)QUITs have
- * already been sent.  we make sure to exit a server's dependent clients 
- * and servers before the server itself; exit_one_client takes care of 
+ * already been sent.  we make sure to exit a server's dependent clients
+ * and servers before the server itself; exit_one_client takes care of
  * actually removing things off llists.   tweaked from +CSr31  -orabidoo
  */
 static void
@@ -861,11 +830,7 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
     if (IsIpHash(source_p))
       remove_one_ip(&source_p->localClient->ip);
 
-    if (source_p->localClient->auth)
-    {
-      delete_auth(source_p->localClient->auth);
-      source_p->localClient->auth = NULL;
-    }
+    delete_auth(&source_p->localClient->auth);
 
     /*
      * This source_p could have status of one of STAT_UNKNOWN, STAT_CONNECTING
@@ -897,17 +862,11 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
         free_list_task(source_p->localClient->list_task, source_p);
 
       watch_del_watch_list(source_p);
-      sendto_realops_flags(UMODE_CCONN, L_ALL, "Client exiting: %s (%s@%s) [%s] [%s]",
+      sendto_realops_flags(UMODE_CCONN, L_ALL, SEND_NOTICE,
+                           "Client exiting: %s (%s@%s) [%s] [%s]",
                            source_p->name, source_p->username, source_p->host, comment,
                            ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
                            "255.255.255.255" : source_p->sockhost);
-      sendto_realops_flags(UMODE_CCONN_FULL, L_ALL, "CLIEXIT: %s %s %s %s 0 %s",
-                           source_p->name,
-			   source_p->username,
-			   source_p->host,
-                           ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
-                           "255.255.255.255" : source_p->sockhost,
-			   comment);
       ilog(LOG_TYPE_USER, "%s (%3u:%02u:%02u): %s!%s@%s %llu/%llu",
            myctime(source_p->localClient->firsttime), (unsigned int)(on_for / 3600),
            (unsigned int)((on_for % 3600)/60), (unsigned int)(on_for % 60),
@@ -931,7 +890,7 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
       {
         /* for them, we are exiting the network */
         sendto_one(source_p, ":%s SQUIT %s :%s",
-	           ID_or_name(from, source_p), me.name, comment);
+                   ID_or_name(from, source_p), me.name, comment);
       }
 
       sendto_one(source_p, "ERROR :Closing Link: %s (%s)",
@@ -951,6 +910,11 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
     */
     close_connection(source_p);
   }
+  else if (IsClient(source_p) && HasFlag(source_p->servptr, FLAGS_EOB))
+    sendto_realops_flags(UMODE_FARCONNECT, L_ALL, SEND_NOTICE,
+                         "Client exiting at %s: %s (%s@%s) [%s]",
+                         source_p->servptr->name, source_p->name,
+                         source_p->username, source_p->host, comment);
 
   if (IsServer(source_p))
   {
@@ -961,7 +925,7 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
 
     if (ConfigServerHide.hide_servers)
       /*
-       * Set netsplit message to "*.net *.split" to still show 
+       * Set netsplit message to "*.net *.split" to still show
        * that its a split, but hide the servers splitting
        */
       strcpy(splitstr, "*.net *.split");
@@ -973,13 +937,13 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
 
     if (source_p->servptr == &me)
     {
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "%s was connected for %d seconds.  %llu/%llu sendK/recvK.",
                            source_p->name, (int)(CurrentTime - source_p->localClient->firsttime),
                            source_p->localClient->send.bytes >> 10,
                            source_p->localClient->recv.bytes >> 10);
       ilog(LOG_TYPE_IRCD, "%s was connected for %d seconds.  %llu/%llu sendK/recvK.",
-           source_p->name, (int)(CurrentTime - source_p->localClient->firsttime), 
+           source_p->name, (int)(CurrentTime - source_p->localClient->firsttime),
            source_p->localClient->send.bytes >> 10,
            source_p->localClient->recv.bytes >> 10);
     }
@@ -1034,7 +998,7 @@ dead_link_on_write(struct Client *client_p, int ierrno)
 void
 dead_link_on_read(struct Client *client_p, int error)
 {
-  char errmsg[255];
+  char errmsg[IRCD_BUFSIZE];
   int current_error;
 
   if (IsDefunct(client_p))
@@ -1048,36 +1012,36 @@ dead_link_on_read(struct Client *client_p, int error)
   if (IsServer(client_p) || IsHandshake(client_p))
   {
     int connected = CurrentTime - client_p->localClient->firsttime;
-      
+
     if (error == 0)
     {
       /* Admins get the real IP */
-      sendto_realops_flags(UMODE_ALL, L_ADMIN,
-			   "Server %s closed the connection",
-			   get_client_name(client_p, SHOW_IP));
+      sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
+                           "Server %s closed the connection",
+                           get_client_name(client_p, SHOW_IP));
 
       /* Opers get a masked IP */
-      sendto_realops_flags(UMODE_ALL, L_OPER,
-			   "Server %s closed the connection",
-			   get_client_name(client_p, MASK_IP));
+      sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
+                           "Server %s closed the connection",
+                           get_client_name(client_p, MASK_IP));
 
       ilog(LOG_TYPE_IRCD, "Server %s closed the connection",
-	   get_client_name(client_p, SHOW_IP));
+           get_client_name(client_p, SHOW_IP));
     }
     else
     {
       report_error(L_ADMIN, "Lost connection to %s: %s",
-		   get_client_name(client_p, SHOW_IP), current_error);
+                   get_client_name(client_p, SHOW_IP), current_error);
       report_error(L_OPER, "Lost connection to %s: %s",
-		   get_client_name(client_p, MASK_IP), current_error);
+                   get_client_name(client_p, MASK_IP), current_error);
     }
 
-    sendto_realops_flags(UMODE_ALL, L_ALL,
-			 "%s had been connected for %d day%s, %2d:%02d:%02d",
-			 client_p->name, connected/86400,
-			 (connected/86400 == 1) ? "" : "s",
-			 (connected % 86400) / 3600, (connected % 3600) / 60,
-			 connected % 60);
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "%s had been connected for %d day%s, %2d:%02d:%02d",
+                         client_p->name, connected/86400,
+                         (connected/86400 == 1) ? "" : "s",
+                         (connected % 86400) / 3600, (connected % 3600) / 60,
+                         connected % 60);
   }
 
   if (error == 0)
@@ -1104,7 +1068,7 @@ exit_aborted_clients(void)
 
     if (target_p == NULL)
     {
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "Warning: null client on abort_list!");
       dlinkDelete(ptr, &abort_list);
       free_dlink_node(ptr);
@@ -1118,7 +1082,7 @@ exit_aborted_clients(void)
     else
       notice = "Write error: connection closed";
 
-    exit_client(target_p, &me, notice);  
+    exit_client(target_p, &me, notice);
     free_dlink_node(ptr);
   }
 }
@@ -1146,19 +1110,18 @@ del_accept(struct split_nuh_item *accept_p, struct Client *client_p)
 
 struct split_nuh_item *
 find_accept(const char *nick, const char *user,
-            const char *host, struct Client *client_p, int do_match)
+            const char *host, struct Client *client_p,
+            int (*cmpfunc)(const char *, const char *))
 {
   dlink_node *ptr = NULL;
-  /* XXX We wouldn't need that if match() would return 0 on match */
-  int (*cmpfunc)(const char *, const char *) = do_match ? match : irccmp;
 
   DLINK_FOREACH(ptr, client_p->localClient->acceptlist.head)
   {
     struct split_nuh_item *accept_p = ptr->data;
 
-    if (cmpfunc(accept_p->nickptr, nick) == do_match &&
-        cmpfunc(accept_p->userptr, user) == do_match &&
-        cmpfunc(accept_p->hostptr, host) == do_match)
+    if (!cmpfunc(accept_p->nickptr, nick) &&
+        !cmpfunc(accept_p->userptr, user) &&
+        !cmpfunc(accept_p->hostptr, host))
       return accept_p;
   }
 
@@ -1179,7 +1142,7 @@ accept_message(struct Client *source,
   dlink_node *ptr = NULL;
 
   if (source == target || find_accept(source->name, source->username,
-                                      source->host, target, 1))
+                                      source->host, target, match))
     return 1;
 
   if (HasUMode(target, UMODE_SOFTCALLERID))
@@ -1206,17 +1169,17 @@ del_all_accepts(struct Client *client_p)
 }
 
 unsigned int
-idle_time_get(struct Client *source_p, struct Client *target_p)
+idle_time_get(const struct Client *source_p, const struct Client *target_p)
 {
   unsigned int idle = 0;
   unsigned int min_idle = 0;
   unsigned int max_idle = 0;
-  const struct ClassItem *class = get_client_class_ptr(target_p);
+  const struct ClassItem *class = get_class_ptr(&target_p->localClient->confs);
 
-  if (!(class->flags & CONF_FLAGS_FAKE_IDLE) || target_p == source_p)
+  if (!(class->flags & CLASS_FLAGS_FAKE_IDLE) || target_p == source_p)
     return CurrentTime - target_p->localClient->last_privmsg;
   if (HasUMode(source_p, UMODE_OPER) &&
-      !(class->flags & CONF_FLAGS_HIDE_IDLE_FROM_OPERS))
+      !(class->flags & CLASS_FLAGS_HIDE_IDLE_FROM_OPERS))
     return CurrentTime - target_p->localClient->last_privmsg;
 
   min_idle = class->min_idle;
@@ -1225,7 +1188,7 @@ idle_time_get(struct Client *source_p, struct Client *target_p)
   if (min_idle == max_idle)
     return min_idle;
 
-  if (class->flags & CONF_FLAGS_RANDOM_IDLE)
+  if (class->flags & CLASS_FLAGS_RANDOM_IDLE)
     idle = genrand_int32();
   else
     idle = CurrentTime - target_p->localClient->last_privmsg;
