@@ -38,7 +38,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: event.c 1672 2012-11-21 20:10:32Z michael $
+ *  $Id: event.c 2679 2013-12-15 18:37:41Z michael $
  */
 
 /*
@@ -58,6 +58,8 @@
 #include "memory.h"
 #include "log.h"
 #include "numeric.h"
+#include "rng_mt.h"
+
 
 static const char *last_event_ran = NULL;
 static struct ev_entry event_table[MAX_EVENTS];
@@ -132,10 +134,10 @@ eventDelete(EVH *func, void *arg)
 void
 eventAddIsh(const char *name, EVH *func, void *arg, time_t delta_ish)
 {
-  if (delta_ish >= 3.0)
+  if (delta_ish >= 3)
   {
     const time_t two_third = (2 * delta_ish) / 3;
-    delta_ish = two_third + ((rand() % 1000) * two_third) / 1000;
+    delta_ish = two_third + ((genrand_int32() % 1000) * two_third) / 1000;
       /*
        * XXX I hate the above magic, I don't even know if its right.
        * Grr. -- adrian
@@ -258,6 +260,7 @@ show_events(struct Client *source_p)
     me.name, RPL_STATSDEBUG, source_p->name);
 
   for (i = 0; i < MAX_EVENTS; i++)
+  {
     if (event_table[i].active)
     {
       sendto_one(source_p, ":%s %d %s : %-28s %-4d seconds",
@@ -265,9 +268,7 @@ show_events(struct Client *source_p)
                  event_table[i].name,
 		 (int)(event_table[i].when - CurrentTime));
     }
-
-  sendto_one(source_p, ":%s %d %s : ",
-    me.name, RPL_STATSDEBUG, source_p->name);
+  }
 }
 
 /*
