@@ -19,12 +19,11 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: hash.c 1397 2012-05-06 10:01:52Z michael $
+ *  $Id: hash.c 2758 2014-01-05 23:20:30Z michael $
  */
 
 #include "stdinc.h"
 #include "list.h"
-#include "balloc.h"
 #include "conf.h"
 #include "channel.h"
 #include "channel_mode.h"
@@ -39,12 +38,13 @@
 #include "numeric.h"
 #include "send.h"
 #include "memory.h"
+#include "mempool.h"
 #include "dbuf.h"
 #include "s_user.h"
 
 
-static BlockHeap *userhost_heap = NULL;
-static BlockHeap *namehost_heap = NULL;
+static mp_pool_t *userhost_pool = NULL;
+static mp_pool_t *namehost_pool = NULL;
 
 static unsigned int hashf_xor_key = 0;
 
@@ -57,7 +57,6 @@ static struct Client *idTable[HASHSIZE];
 static struct Client *clientTable[HASHSIZE];
 static struct Channel *channelTable[HASHSIZE];
 static struct UserHost *userhostTable[HASHSIZE];
-static struct ResvChannel *resvchannelTable[HASHSIZE];
 
 
 /* init_hash()
@@ -68,14 +67,10 @@ static struct ResvChannel *resvchannelTable[HASHSIZE];
  *                functions and clear the tables
  */
 void
-init_hash(void)
+hash_init(void)
 {
-  /* Default the userhost/namehost sizes to CLIENT_HEAP_SIZE for now,
-   * should be a good close approximation anyway
-   * - Dianora
-   */
-  userhost_heap = BlockHeapCreate("userhost", sizeof(struct UserHost), CLIENT_HEAP_SIZE);
-  namehost_heap = BlockHeapCreate("namehost", sizeof(struct NameHost), CLIENT_HEAP_SIZE);
+  userhost_pool = mp_pool_new(sizeof(struct UserHost), MP_CHUNK_SIZE_USERHOST);
+  namehost_pool = mp_pool_new(sizeof(struct NameHost), MP_CHUNK_SIZE_NAMEHOST);
 
   hashf_xor_key = genrand_int32() % 256;  /* better than nothing --adx */
 }
@@ -93,7 +88,7 @@ strhash(const char *name)
   const unsigned char *p = (const unsigned char *)name;
   unsigned int hval = FNV1_32_INIT;
 
-  if (*p == '\0')
+  if (EmptyString(p))
     return 0;
   for (; *p != '\0'; ++p)
   {
@@ -152,15 +147,6 @@ hash_add_channel(struct Channel *chptr)
   channelTable[hashv] = chptr;
 }
 
-void
-hash_add_resv(struct ResvChannel *chptr)
-{
-  unsigned int hashv = strhash(chptr->name);
-
-  chptr->hnext = resvchannelTable[hashv];
-  resvchannelTable[hashv] = chptr;
-}
-
 void
 hash_add_userhost(struct UserHost *userhost)
 {
@@ -304,31 +290,6 @@ hash_del_channel(struct Channel *chptr)
   }
 }
 
-void
-hash_del_resv(struct ResvChannel *chptr)
-{
-  unsigned int hashv = strhash(chptr->name);
-  struct ResvChannel *tmp = resvchannelTable[hashv];
-
-  if (tmp != NULL)
-  {
-    if (tmp == chptr)
-    {
-      resvchannelTable[hashv] = chptr->hnext;
-      chptr->hnext = chptr;
-    }
-    else
-    {
-      while (tmp->hnext != chptr)
-        if ((tmp = tmp->hnext) == NULL)
-          return;
-
-      tmp->hnext = tmp->hnext->hnext;
-      chptr->hnext = chptr;
-    }
-  }
-}
-
 /* hash_find_client()
  *
  * inputs       - pointer to name
@@ -493,9 +454,6 @@ hash_get_bucket(int type, unsigned int hashv)
     case HASH_TYPE_USERHOST:
       return userhostTable[hashv];
       break;
-    case HASH_TYPE_RESERVED:
-      return resvchannelTable[hashv];
-      break;
     default:
       assert(0);
   }
@@ -503,42 +461,6 @@ hash_get_bucket(int type, unsigned int hashv)
   return NULL;
 }
 
-/* hash_find_resv()
- *
- * inputs       - pointer to name
- * output       - NONE
- * side effects - New semantics: finds a reserved channel whose name is 'name',
- *                if can't find one returns NULL, if can find it moves
- *                it to the top of the list and returns it.
- */
-struct ResvChannel *
-hash_find_resv(const char *name)
-{
-  unsigned int hashv = strhash(name);
-  struct ResvChannel *chptr;
-
-  if ((chptr = resvchannelTable[hashv]) != NULL)
-  {
-    if (irccmp(name, chptr->name))
-    {
-      struct ResvChannel *prev;
-
-      while (prev = chptr, (chptr = chptr->hnext) != NULL)
-      {
-        if (!irccmp(name, chptr->name))
-        {
-          prev->hnext = chptr->hnext;
-          chptr->hnext = resvchannelTable[hashv];
-          resvchannelTable[hashv] = chptr;
-          break;
-        }
-      }
-    }
-  }
-
-  return chptr;
-}
-
 struct UserHost *
 hash_find_userhost(const char *host)
 {
@@ -579,8 +501,8 @@ hash_find_userhost(const char *host)
  * side effects	-
  */
 void
-count_user_host(const char *user, const char *host, int *global_p,
-                int *local_p, int *icount_p)
+count_user_host(const char *user, const char *host, unsigned int *global_p,
+                unsigned int *local_p, unsigned int *icount_p)
 {
   dlink_node *ptr;
   struct UserHost *found_userhost;
@@ -620,7 +542,9 @@ find_or_add_userhost(const char *host)
   if ((userhost = hash_find_userhost(host)) != NULL)
     return userhost;
 
-  userhost = BlockHeapAlloc(userhost_heap);
+  userhost = mp_pool_get(userhost_pool);
+
+  memset(userhost, 0, sizeof(*userhost));
   strlcpy(userhost->host, host, sizeof(userhost->host));
   hash_add_userhost(userhost);
 
@@ -671,7 +595,8 @@ add_user_host(const char *user, const char *host, int global)
     }
   }
 
-  nameh = BlockHeapAlloc(namehost_heap);
+  nameh = mp_pool_get(namehost_pool);
+  memset(nameh, 0, sizeof(*nameh));
   strlcpy(nameh->name, user, sizeof(nameh->name));
 
   nameh->gcount = 1;
@@ -730,13 +655,13 @@ delete_user_host(const char *user, const char *host, int global)
       if (nameh->gcount == 0 && nameh->lcount == 0)
       {
         dlinkDelete(&nameh->node, &found_userhost->list);
-        BlockHeapFree(namehost_heap, nameh);
+        mp_pool_release(nameh);
       }
 
       if (dlink_list_length(&found_userhost->list) == 0)
       {
         hash_del_userhost(found_userhost);
-        BlockHeapFree(userhost_heap, found_userhost);
+        mp_pool_release(found_userhost);
       }
 
       return;
@@ -767,9 +692,9 @@ delete_user_host(const char *user, const char *host, int global)
  * Sendq limit is fairly conservative at 1/2 (In original anyway)
  */
 static int
-exceeding_sendq(struct Client *to)
+exceeding_sendq(const struct Client *to)
 {
-  if (dbuf_length(&to->localClient->buf_sendq) > (get_sendq(to) / 2))
+  if (dbuf_length(&to->localClient->buf_sendq) > (get_sendq(&to->localClient->confs) / 2))
     return 1;
   else
     return 0;
@@ -778,7 +703,7 @@ exceeding_sendq(struct Client *to)
 void
 free_list_task(struct ListTask *lt, struct Client *source_p)
 {
-  dlink_node *dl, *dln;
+  dlink_node *dl = NULL, *dln = NULL;
 
   if ((dl = dlinkFindDelete(&listing_client_list, source_p)) != NULL)
     free_dlink_node(dl);
@@ -810,16 +735,16 @@ free_list_task(struct ListTask *lt, struct Client *source_p)
  * side effects -
  */
 static int
-list_allow_channel(const char *chname, struct ListTask *lt)
+list_allow_channel(const char *chname, const struct ListTask *lt)
 {
-  dlink_node *dl = NULL;
+  const dlink_node *dl = NULL;
 
   DLINK_FOREACH(dl, lt->show_mask.head)
-    if (!match_chan(dl->data, chname))
+    if (match(dl->data, chname) != 0)
       return 0;
 
   DLINK_FOREACH(dl, lt->hide_mask.head)
-    if (match_chan(dl->data, chname))
+    if (match(dl->data, chname) == 0)
       return 0;
 
   return 1;
@@ -837,7 +762,12 @@ static void
 list_one_channel(struct Client *source_p, struct Channel *chptr,
                  struct ListTask *list_task)
 {
-  if (SecretChannel(chptr) && !IsMember(source_p, chptr))
+  char listbuf[MODEBUFLEN] = "";
+  char modebuf[MODEBUFLEN] = "";
+  char parabuf[MODEBUFLEN] = "";
+
+  if (SecretChannel(chptr) &&
+      !(IsMember(source_p, chptr) || HasUMode(source_p, UMODE_ADMIN)))
     return;
   if (dlink_list_length(&chptr->members) < list_task->users_min ||
       dlink_list_length(&chptr->members) > list_task->users_max ||
@@ -851,8 +781,19 @@ list_one_channel(struct Client *source_p, struct Channel *chptr,
 
   if (!list_allow_channel(chptr->chname, list_task))
     return;
+
+  if (HasUMode(source_p, UMODE_ADMIN))
+  {
+    channel_modes(chptr, source_p, modebuf, parabuf);
+
+    if (chptr->topic[0])
+      snprintf(listbuf, sizeof(listbuf), "[%s] ", modebuf);
+    else
+      snprintf(listbuf, sizeof(listbuf), "[%s]",  modebuf);
+  }
+
   sendto_one(source_p, form_str(RPL_LIST), me.name, source_p->name,
-             chptr->chname, dlink_list_length(&chptr->members),
+             chptr->chname, dlink_list_length(&chptr->members), listbuf,
              chptr->topic);
 }
 
