@@ -19,17 +19,12 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: irc_string.c 1802 2013-04-01 11:54:23Z michael $
+ *  $Id: irc_string.c 1920 2013-04-30 14:51:30Z michael $
  */
 
 #include "config.h"
-#ifdef HAVE_LIBPCRE
-#include <pcre.h>
-#endif
-
 #include "stdinc.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 
 
 int
@@ -240,24 +235,3 @@ strlcpy(char *dst, const char *src, size_t siz)
   return s - src - 1; /* count does not include NUL */
 }
 #endif
-
-#ifdef HAVE_LIBPCRE
-void *
-ircd_pcre_compile(const char *pattern, const char **errptr)
-{
-  int erroroffset = 0;
-  int options = PCRE_EXTRA;
-
-  assert(pattern);
-
-  return pcre_compile(pattern, options, errptr, &erroroffset, NULL);
-}
-
-int
-ircd_pcre_exec(const void *code, const char *subject)
-{
-  assert(code && subject);
-
-  return pcre_exec(code, NULL, subject, strlen(subject), 0, 0, NULL, 0) < 0;
-}
-#endif
