@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: ircd.c 1871 2013-04-26 08:53:04Z michael $
+ *  $Id: ircd.c 2690 2013-12-17 18:55:43Z michael $
  */
 
 #include "stdinc.h"
@@ -36,6 +36,7 @@
 #include "ircd_signal.h"
 #include "s_gline.h"
 #include "motd.h"
+#include "conf.h"
 #include "hostmask.h"
 #include "numeric.h"
 #include "packet.h"
@@ -45,7 +46,6 @@
 #include "rng_mt.h"
 #include "s_auth.h"
 #include "s_bsd.h"
-#include "conf.h"
 #include "log.h"
 #include "s_misc.h"
 #include "s_serv.h"      /* try_connections */
@@ -53,14 +53,18 @@
 #include "whowas.h"
 #include "modules.h"
 #include "memory.h"
+#include "mempool.h"
 #include "hook.h"
 #include "ircd_getopt.h"
-#include "balloc.h"
-#include "motd.h"
 #include "supported.h"
 #include "watch.h"
+#include "conf_db.h"
+#include "conf_class.h"
 
 
+#ifdef HAVE_LIBGEOIP
+GeoIP *geoip_ctx;
+#endif
 /* /quote set variables */
 struct SetOptions GlobalSetOptions;
 
@@ -114,7 +118,7 @@ int rehashed_klines = 0;
 static void
 print_startup(int pid)
 {
-  printf("ircd: version %s\n", ircd_version);
+  printf("ircd: version %s(%s)\n", ircd_version, serno);
   printf("ircd: pid %d\n", pid);
   printf("ircd: running in %s mode from %s\n", !server_state.foreground ? "background"
          : "foreground", ConfigFileEntry.dpath);
@@ -142,14 +146,18 @@ make_daemon(void)
 static int printVersion = 0;
 
 static struct lgetopt myopts[] = {
-  {"dlinefile",  &ConfigFileEntry.dlinefile, 
-   STRING, "File to use for dline.conf"},
   {"configfile", &ConfigFileEntry.configfile, 
    STRING, "File to use for ircd.conf"},
+  {"glinefile",  &ConfigFileEntry.glinefile,
+   STRING, "File to use for gline database"},
   {"klinefile",  &ConfigFileEntry.klinefile, 
-   STRING, "File to use for kline.conf"},
+   STRING, "File to use for kline database"},
+  {"dlinefile",  &ConfigFileEntry.dlinefile,
+   STRING, "File to use for dline database"},
   {"xlinefile",  &ConfigFileEntry.xlinefile, 
-   STRING, "File to use for xline.conf"},
+   STRING, "File to use for xline database"},
+  {"resvfile",  &ConfigFileEntry.resvfile,
+   STRING, "File to use for resv database"},
   {"logfile",    &logFileName, 
    STRING, "File to use for ircd.log"},
   {"pidfile",    &pidFileName,
@@ -165,7 +173,7 @@ static struct lgetopt myopts[] = {
 void
 set_time(void)
 {
-  static char to_send[200];
+  static char to_send[IRCD_BUFSIZE];
   struct timeval newtime;
   newtime.tv_sec  = 0;
   newtime.tv_usec = 0;
@@ -174,7 +182,7 @@ set_time(void)
   {
     ilog(LOG_TYPE_IRCD, "Clock Failure (%s), TS can be corrupted",
          strerror(errno));
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "Clock Failure (%s), TS can be corrupted",
                          strerror(errno));
     restart("Clock Failure");
@@ -242,9 +250,9 @@ io_loop(void)
     }
     if (doremotd)
     {
-      read_message_file(&ConfigFileEntry.motd);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
-                           "Got signal SIGUSR1, reloading ircd motd file");
+      motd_recache();
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                           "Got signal SIGUSR1, reloading motd files");
       doremotd = 0;
     }
   }
@@ -288,24 +296,6 @@ initialize_global_set_options(void)
   /* End of global set options */
 }
 
-/* initialize_message_files()
- *
- * inputs       - none
- * output       - none
- * side effects - Set up all message files needed, motd etc.
- */
-static void
-initialize_message_files(void)
-{
-  init_message_file(USER_MOTD, MPATH, &ConfigFileEntry.motd);
-  init_message_file(USER_LINKS, LIPATH, &ConfigFileEntry.linksfile);
-
-  read_message_file(&ConfigFileEntry.motd);
-  read_message_file(&ConfigFileEntry.linksfile);
-
-  init_isupport();
-}
-
 /* initialize_server_capabs()
  *
  * inputs       - none
@@ -318,6 +308,7 @@ initialize_server_capabs(void)
   add_capability("EOB", CAP_EOB, 1);
   add_capability("TS6", CAP_TS6, 0);
   add_capability("CLUSTER", CAP_CLUSTER, 1);
+//  add_capability("FAKEHOST", CAP_FAKEHOST, 1);
   add_capability("SVS", CAP_SVS, 1);
 #ifdef HALFOPS
   add_capability("HOPS", CAP_HOPS, 1);
@@ -337,7 +328,7 @@ write_pidfile(const char *filename)
 
   if ((fb = fopen(filename, "w")))
   {
-    char buff[32];
+    char buff[IRCD_BUFSIZE];
     unsigned int pid = (unsigned int)getpid();
 
     snprintf(buff, sizeof(buff), "%u\n", pid);
@@ -366,7 +357,7 @@ static void
 check_pidfile(const char *filename)
 {
   FILE *fb;
-  char buff[32];
+  char buff[IRCD_BUFSIZE];
   pid_t pidfromfile;
 
   /* Don't do logging here, since we don't have log() initialised */
@@ -420,6 +411,14 @@ setup_corefile(void)
 #endif
 }
 
+#ifdef HAVE_LIBCRYPTO
+static int
+always_accept_verify_cb(int preverify_ok, X509_STORE_CTX *x509_ctx)
+{
+  return 1;
+}
+#endif
+
 /* init_ssl()
  *
  * inputs       - nothing
@@ -427,7 +426,7 @@ setup_corefile(void)
  * side effects - setups SSL context.
  */
 static void
-init_ssl(void)
+ssl_init(void)
 {
 #ifdef HAVE_LIBCRYPTO
   SSL_load_error_strings();
@@ -444,7 +443,8 @@ init_ssl(void)
 
   SSL_CTX_set_options(ServerInfo.server_ctx, SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3|SSL_OP_NO_TLSv1);
   SSL_CTX_set_options(ServerInfo.server_ctx, SSL_OP_TLS_ROLLBACK_BUG|SSL_OP_ALL);
-  SSL_CTX_set_verify(ServerInfo.server_ctx, SSL_VERIFY_NONE, NULL);
+  SSL_CTX_set_verify(ServerInfo.server_ctx, SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE,
+                     always_accept_verify_cb);
 
   if ((ServerInfo.client_ctx = SSL_CTX_new(SSLv23_client_method())) == NULL)
   {
@@ -457,29 +457,15 @@ init_ssl(void)
 
   SSL_CTX_set_options(ServerInfo.client_ctx, SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3|SSL_OP_NO_TLSv1);
   SSL_CTX_set_options(ServerInfo.client_ctx, SSL_OP_TLS_ROLLBACK_BUG|SSL_OP_ALL);
-  SSL_CTX_set_verify(ServerInfo.client_ctx, SSL_VERIFY_NONE, NULL);
+  SSL_CTX_set_verify(ServerInfo.client_ctx, SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE,
+                     always_accept_verify_cb);
 #endif /* HAVE_LIBCRYPTO */
 }
 
-/* init_callbacks()
- *
- * inputs       - nothing
- * output       - nothing
- * side effects - setups standard hook points
- */
-static void
-init_callbacks(void)
-{
-  iorecv_cb = register_callback("iorecv", iorecv_default);
-  iosend_cb = register_callback("iosend", iosend_default);
-}
-
 int
 main(int argc, char *argv[])
 {
-  /* Check to see if the user is running
-   * us as root, which is a nono
-   */
+  /* Check to see if the user is running us as root, which is a nono */
   if (geteuid() == 0)
   {
     fprintf(stderr, "Don't run ircd as root!!!\n");
@@ -496,18 +482,19 @@ main(int argc, char *argv[])
   init_genrand(SystemTime.tv_sec ^ (SystemTime.tv_usec | (getpid() << 20)));
 
   me.localClient = &meLocalUser;
-  dlinkAdd(&me, &me.node, &global_client_list);	/* Pointer to beginning
+  dlinkAdd(&me, &me.node, &global_client_list);  /* Pointer to beginning
 						   of Client list */
   /* Initialise the channel capability usage counts... */
   init_chcap_usage_counts();
 
   ConfigFileEntry.dpath      = DPATH;
-  ConfigFileEntry.configfile = CPATH;  /* Server configuration file */
-  ConfigFileEntry.klinefile  = KPATH;  /* Server kline file         */
-  ConfigFileEntry.xlinefile  = XPATH;  /* Server xline file         */
-  ConfigFileEntry.dlinefile  = DLPATH; /* dline file                */
-  ConfigFileEntry.cresvfile  = CRESVPATH; /* channel resv file      */
-  ConfigFileEntry.nresvfile  = NRESVPATH; /* nick resv file         */
+  ConfigFileEntry.configfile = CPATH;    /* Server configuration file */
+  ConfigFileEntry.klinefile  = KPATH;    /* Server kline file         */
+  ConfigFileEntry.glinefile  = GPATH;    /* Server gline file         */
+  ConfigFileEntry.xlinefile  = XPATH;    /* Server xline file         */
+  ConfigFileEntry.dlinefile  = DLPATH;   /* dline file                */
+  ConfigFileEntry.resvfile   = RESVPATH; /* resv file                 */
+
   myargv = argv;
   umask(077);                /* better safe than sorry --SRB */
 
@@ -515,7 +502,7 @@ main(int argc, char *argv[])
 
   if (printVersion)
   {
-    printf("ircd: version %s\n", ircd_version);
+    printf("ircd: version %s(%s)\n", ircd_version, serno);
     exit(EXIT_SUCCESS);
   }
 
@@ -525,7 +512,7 @@ main(int argc, char *argv[])
     exit(EXIT_FAILURE);
   }
 
-  init_ssl();
+  ssl_init();
 
   if (!server_state.foreground)
   {
@@ -539,34 +526,40 @@ main(int argc, char *argv[])
 
   /* Init the event subsystem */
   eventInit();
+
   /* We need this to initialise the fd array before anything else */
   fdlist_init();
   log_set_file(LOG_TYPE_IRCD, 0, logFileName);
   check_can_use_v6();
-  init_comm();         /* This needs to be setup early ! -- adrian */
+  init_netio();         /* This needs to be setup early ! -- adrian */
+
   /* Check if there is pidfile and daemon already running */
   check_pidfile(pidFileName);
 
-  initBlockHeap();
+  mp_pool_init();
   init_dlink_nodes();
-  init_callbacks();
-  initialize_message_files();
+  init_isupport();
   dbuf_init();
-  init_hash();
+  hash_init();
   init_ip_hash_table();      /* client host ip hash table */
   init_host_hash();          /* Host-hashtable. */
-  init_client();
-  init_class();
+  client_init();
+  class_init();
   whowas_init();
   watch_init();
-  init_auth();          /* Initialise the auth code */
+  auth_init();          /* Initialise the auth code */
   init_resolver();      /* Needs to be setup before the io loop */
   modules_init();
   read_conf_files(1);   /* cold start init conf files */
   init_uid();
   initialize_server_capabs();   /* Set up default_server_capabs */
   initialize_global_set_options();
-  init_channels();
+  channel_init();
+  read_links_file();
+  motd_init();
+#ifdef HAVE_LIBGEOIP
+  geoip_ctx = GeoIP_new(GEOIP_MEMORY_CACHE);
+#endif
 
   if (EmptyString(ServerInfo.sid))
   {
@@ -608,6 +601,12 @@ main(int argc, char *argv[])
   /* add ourselves to global_serv_list */
   dlinkAdd(&me, make_dlink_node(), &global_serv_list);
 
+  load_kline_database();
+  load_dline_database();
+  load_gline_database();
+  load_xline_database();
+  load_resv_database();
+
   if (chdir(MODPATH))
   {
     ilog(LOG_TYPE_IRCD, "Could not load core modules. Terminating!");
@@ -645,6 +644,8 @@ main(int argc, char *argv[])
   /* Setup the timeout check. I'll shift it later :)  -- adrian */
   eventAddIsh("comm_checktimeouts", comm_checktimeouts, NULL, 1);
 
+  eventAddIsh("save_all_databases", save_all_databases, NULL, DATABASE_UPDATE_TIMEOUT);
+
   if (ConfigServerHide.links_delay > 0)
     eventAddIsh("write_links_file", write_links_file, NULL, ConfigServerHide.links_delay);
   else
