@@ -19,15 +19,14 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: ircd_signal.c 598 2006-05-13 09:08:47Z michael $
+ *  $Id: ircd_signal.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 #include "stdinc.h"
-#include "common.h"
+#include "list.h"
 #include "ircd_signal.h"
 #include "ircd.h"         /* dorehash */
 #include "restart.h"      /* server_die */
-#include "s_log.h"
 #include "memory.h"
 #include "s_bsd.h"
 
@@ -37,7 +36,7 @@
 static void 
 sigterm_handler(int sig)  
 {
-  server_die("received signal SIGTERM", NO);
+  server_die("received signal SIGTERM", 0);
 }
 
 /* 
