@@ -19,14 +19,14 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: list.c 1665 2012-11-18 15:56:25Z michael $
+ *  $Id: list.c 2715 2013-12-24 21:15:36Z michael $
  */
 
 #include "stdinc.h"
 #include "list.h"
-#include "balloc.h"
+#include "mempool.h"
 
-static BlockHeap *dnode_heap;
+static mp_pool_t *dnode_pool;
 
 
 /* init_dlink_nodes()
@@ -38,7 +38,7 @@ static BlockHeap *dnode_heap;
 void
 init_dlink_nodes(void)
 {
-  dnode_heap = BlockHeapCreate("dlink node", sizeof(dlink_node), DNODE_HEAP_SIZE);
+  dnode_pool = mp_pool_new(sizeof(dlink_node), MP_CHUNK_SIZE_DNODE);
 }
 
 /* make_dlink_node()
@@ -50,9 +50,10 @@ init_dlink_nodes(void)
 dlink_node *
 make_dlink_node(void)
 {
-  dlink_node *lp = BlockHeapAlloc(dnode_heap);
+  dlink_node *ptr = mp_pool_get(dnode_pool);
 
-  return lp;
+  memset(ptr, 0, sizeof(*ptr));
+  return ptr;
 }
 
 /* free_dlink_node()
@@ -64,7 +65,7 @@ make_dlink_node(void)
 void
 free_dlink_node(dlink_node *ptr)
 {
-  BlockHeapFree(dnode_heap, ptr);
+  mp_pool_release(ptr);
 }
 
 /*
@@ -243,35 +244,35 @@ dlink_move_node(dlink_node *m, dlink_list *list_del, dlink_list *list_add)
 dlink_node *
 dlinkFindDelete(dlink_list *list, void *data)
 {
-  dlink_node *m;
+  dlink_node *m = NULL;
 
   DLINK_FOREACH(m, list->head)
   {
-    if (m->data == data)
-    {
-      if (m->next)
-        m->next->prev = m->prev;
-      else
-      {
-        assert(list->tail == m);
-        list->tail = m->prev;
-      }
-      if (m->prev)
-        m->prev->next = m->next;
-      else
-      {
-        assert(list->head == m);
-        list->head = m->next;
-      }
-      /* Set this to NULL does matter */
-      m->next = m->prev = NULL;
-      list->length--;
+    if (m->data != data)
+      continue;
 
-      return m;
+    if (m->next)
+      m->next->prev = m->prev;
+    else
+    {
+      assert(list->tail == m);
+      list->tail = m->prev;
     }
+
+    if (m->prev)
+      m->prev->next = m->next;
+    else
+    {
+      assert(list->head == m);
+      list->head = m->next;
+    }
+
+    /* Set this to NULL does matter */
+    m->next = m->prev = NULL;
+    list->length--;
+
+    return m;
   }
 
   return NULL;
 }
-
-
