@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: listener.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: listener.c 2690 2013-12-17 18:55:43Z michael $
  */
 
 #include "stdinc.h"
@@ -28,7 +28,6 @@
 #include "client.h"
 #include "fdlist.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
 #include "ircd_defs.h"
 #include "s_bsd.h"
@@ -58,7 +57,7 @@ free_listener(struct Listener *listener)
 {
   assert(listener != NULL);
 
-  dlinkDelete(&listener->listener_node, &ListenerPollList);
+  dlinkDelete(&listener->node, &ListenerPollList);
   MyFree(listener);
 }
 
@@ -85,7 +84,7 @@ get_listener_name(const struct Listener *const listener)
 void 
 show_ports(struct Client *source_p)
 {
-  char buf[6];
+  char buf[IRCD_BUFSIZE];
   char *p = NULL;
   dlink_node *ptr;
 
@@ -94,7 +93,8 @@ show_ports(struct Client *source_p)
     const struct Listener *listener = ptr->data;
     p = buf;
 
-    if (listener->flags & LISTENER_HIDDEN) {
+    if (listener->flags & LISTENER_HIDDEN)
+    {
       if (!HasUMode(source_p, UMODE_ADMIN))
         continue;
       *p++ = 'H';
@@ -105,11 +105,20 @@ show_ports(struct Client *source_p)
     if (listener->flags & LISTENER_SSL)
       *p++ = 's';
     *p = '\0';
-    sendto_one(source_p, form_str(RPL_STATSPLINE),
-               me.name, source_p->name, 'P', listener->port,
-               HasUMode(source_p, UMODE_ADMIN) ? listener->name : me.name,
-               listener->ref_count, buf,
-               listener->active ? "active" : "disabled");
+
+
+    if (HasUMode(source_p, UMODE_ADMIN) &&
+        (MyClient(source_p) || !ConfigServerHide.hide_server_ips))
+      sendto_one(source_p, form_str(RPL_STATSPLINE),
+                 me.name, source_p->name, 'P', listener->port,
+                 listener->name,
+                 listener->ref_count, buf,
+                 listener->active ? "active" : "disabled");
+    else
+      sendto_one(source_p, form_str(RPL_STATSPLINE),
+                 me.name, source_p->name, 'P', listener->port,
+                 me.name, listener->ref_count, buf,
+                 listener->active ? "active" : "disabled");
   }
 }
 
@@ -268,7 +277,7 @@ add_listener(int port, const char *vhost_ip, unsigned int flags)
 
   snprintf(portname, PORTNAMELEN, "%d", port);
 
-  if (vhost_ip)
+  if (!EmptyString(vhost_ip))
   {
     if (getaddrinfo(vhost_ip, portname, &hints, &res))
       return;
@@ -299,7 +308,7 @@ add_listener(int port, const char *vhost_ip, unsigned int flags)
   else
   {
     listener = make_listener(port, &vaddr);
-    dlinkAdd(listener, &listener->listener_node, &ListenerPollList);
+    dlinkAdd(listener, &listener->node, &ListenerPollList);
     listener->flags = flags;
   }
 
@@ -384,7 +393,8 @@ accept_connection(fde_t *pfd, void *data)
        */
       if ((last_oper_notice + 20) <= CurrentTime)
       {
-        sendto_realops_flags(UMODE_ALL, L_ALL, "All connections in use. (%s)",
+        sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                             "All connections in use. (%s)",
                              get_listener_name(listener));
         last_oper_notice = CurrentTime;
       }
