@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: memory.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: memory.c 1664 2012-11-18 14:33:47Z michael $
  */
 
 
@@ -66,11 +66,30 @@ MyFree(void *x)
     free(x);
 }
 
-void
-_DupString(char **x, const char *y)
+void *
+xstrdup(const char *s)
 {
-  (*x) = malloc(strlen(y) + 1);
-  strcpy((*x), y);
+  void *ret = malloc(strlen(s) + 1);
+
+  if (ret == NULL)
+    outofmemory();
+
+  strcpy(ret, s);
+
+  return ret;
+}
+
+void *
+xstrndup(const char *s, size_t len)
+{
+  void *ret = malloc(len + 1);
+
+  if (ret == NULL)
+    outofmemory();
+
+  strlcpy(ret, s, len + 1);
+
+  return ret;
 }
 
 /* outofmemory()
@@ -91,24 +110,3 @@ outofmemory(void)
   ilog(LOG_TYPE_IRCD, "Out of memory: restarting server...");
   restart("Out of Memory");
 }
-
-#ifndef NDEBUG
-/*
- * frob some memory. debugging time.
- * -- adrian
- */
-void
-mem_frob(void *data, int len)
-{
-  /* correct for Intel only! little endian */
-  unsigned char b[4] = { 0xef, 0xbe, 0xad, 0xde };
-  int i;
-  char *cdata = data;
-
-  for (i = 0; i < len; i++)
-  {
-    *cdata = b[i % 4];
-    cdata++;
-  }
-}
-#endif
