@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: modules.c 1738 2013-01-14 17:38:34Z michael $
+ *  $Id: modules.c 2526 2013-11-02 17:07:55Z michael $
  */
 
 #include "ltdl.h"
@@ -41,8 +41,10 @@
 
 dlink_list modules_list = { NULL, NULL, 0 };
 
-static const char *unknown_ver = "<unknown>";
+static dlink_list mod_paths = { NULL, NULL, 0 };
+static dlink_list conf_modules = { NULL, NULL, 0 };
 
+static const char *unknown_ver = "<unknown>";
 static const char *core_module_table[] =
 {
   "m_die.la",
@@ -61,9 +63,6 @@ static const char *core_module_table[] =
   NULL
 };
 
-static dlink_list mod_paths = { NULL, NULL, 0 };
-static dlink_list conf_modules = { NULL, NULL, 0 };
-
 int
 modules_valid_suffix(const char *name)
 {
@@ -97,7 +96,8 @@ unload_one_module(const char *name, int warn)
   if (warn == 1)
   {
     ilog(LOG_TYPE_IRCD, "Module %s unloaded", name);
-    sendto_realops_flags(UMODE_ALL, L_ALL, "Module %s unloaded", name);
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "Module %s unloaded", name);
   }
 
   return 0;
@@ -119,10 +119,12 @@ load_a_module(const char *path, int warn)
   if (findmodule_byname((mod_basename = libio_basename(path))))
     return 1;
 
-  if (!(tmpptr = lt_dlopen(path))) {
+  if (!(tmpptr = lt_dlopen(path)))
+  {
     const char *err = ((err = lt_dlerror())) ? err : "<unknown>";
 
-    sendto_realops_flags(UMODE_ALL, L_ALL, "Error loading module %s: %s",
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "Error loading module %s: %s",
                          mod_basename, err);
     ilog(LOG_TYPE_IRCD, "Error loading module %s: %s", mod_basename, err);
     return -1;
@@ -132,7 +134,8 @@ load_a_module(const char *path, int warn)
   {
     const char *err = ((err = lt_dlerror())) ? err : "<unknown>";
 
-    sendto_realops_flags(UMODE_ALL, L_ALL, "Error loading module %s: %s",
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "Error loading module %s: %s",
                          mod_basename, err);
     ilog(LOG_TYPE_IRCD, "Error loading module %s: %s", mod_basename, err);
     lt_dlclose(tmpptr);
@@ -144,7 +147,7 @@ load_a_module(const char *path, int warn)
   if (EmptyString(modp->version))
     modp->version = unknown_ver;
 
-  DupString(modp->name, mod_basename);
+  modp->name = xstrdup(mod_basename);
   dlinkAdd(modp, &modp->node, &modules_list);
 
   if (modp->modinit)
@@ -152,7 +155,7 @@ load_a_module(const char *path, int warn)
 
   if (warn == 1)
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "Module %s [version: %s handle: %p] loaded.",
                          modp->name, modp->version, tmpptr);
     ilog(LOG_TYPE_IRCD, "Module %s [version: %s handle: %p] loaded.",
@@ -368,7 +371,7 @@ load_core_modules(int warn)
  *
  * input        - pointer to path
  *		- flagged as core module or not
- * output       - -1 if error 
+ * output       - -1 if error
  * side effects - module is loaded if found.
  */
 int
@@ -388,13 +391,13 @@ load_one_module(const char *path)
       continue;
 
     if (strstr(modpath, "../") == NULL &&
-        strstr(modpath, "/..") == NULL) 
+        strstr(modpath, "/..") == NULL)
       if (!stat(modpath, &statbuf))
         if (S_ISREG(statbuf.st_mode))  /* Regular files only please */
           return load_a_module(modpath, 1);
   }
 
-  sendto_realops_flags(UMODE_ALL, L_ALL,
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                        "Cannot locate module %s", path);
   ilog(LOG_TYPE_IRCD, "Cannot locate module %s", path);
   return -1;
