@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: packet.c 1516 2012-08-30 15:08:36Z michael $
+ *  $Id: packet.c 1798 2013-03-31 17:09:50Z michael $
  */
 #include "stdinc.h"
 #include "list.h"
@@ -39,8 +39,6 @@
 
 #define READBUF_SIZE 16384
 
-struct Callback *iorecv_cb = NULL;
-
 static char readBuf[READBUF_SIZE];
 static void client_dopacket(struct Client *, char *, size_t);
 
@@ -270,20 +268,6 @@ flood_recalc(fde_t *fd, void *data)
   }
 }
 
-/*
- * iorecv_default - append a packet to the recvq dbuf
- */
-void *
-iorecv_default(va_list args)
-{
-  struct Client *client_p = va_arg(args, struct Client *);
-  int length = va_arg(args, int);
-  char *buf = va_arg(args, char *);
-
-  dbuf_put(&client_p->localClient->buf_recvq, buf, length);
-  return NULL;
-}
-
 /*
  * read_packet - Read a 'packet' of data from a connection and process it.
  */
@@ -347,7 +331,7 @@ read_packet(fde_t *fd, void *data)
       return;
     }
 
-    execute_callback(iorecv_cb, client_p, length, readBuf);
+    dbuf_put(&client_p->localClient->buf_recvq, readBuf, length);
 
     if (client_p->localClient->lasttime < CurrentTime)
       client_p->localClient->lasttime = CurrentTime;
@@ -364,7 +348,7 @@ read_packet(fde_t *fd, void *data)
     /* Check to make sure we're not flooding */
     if (!(IsServer(client_p) || IsHandshake(client_p) || IsConnecting(client_p))
         && (dbuf_length(&client_p->localClient->buf_recvq) >
-            get_recvq(client_p)))
+            get_recvq(&client_p->localClient->confs)))
     {
       if (!(ConfigFileEntry.no_oper_flood && HasUMode(client_p, UMODE_OPER)))
       {
