@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: parse.c 1682 2012-12-18 18:45:27Z michael $
+ *  $Id: parse.c 2787 2014-01-06 22:34:10Z michael $
  */
 
 #include "stdinc.h"
@@ -28,7 +28,6 @@
 #include "channel.h"
 #include "hash.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "log.h"
@@ -55,7 +54,7 @@
  *				 'i' -> [MessageTree *] -> 'e' and matches
  *
  * BUGS (Limitations!)
- * 
+ *
  * I designed this trie to parse ircd commands. Hence it currently
  * casefolds. This is trivial to fix by increasing MAXPTRLEN.
  * This trie also "folds" '{' etc. down. This means, the input to this
@@ -65,12 +64,12 @@
  * MAXPTRLEN 128.
  *
  * This is also not a patricia trie. On short ircd tokens, this is
- * not likely going to matter. 
+ * not likely going to matter.
  *
  * Diane Bruce (Dianora), June 6 2003
  */
 
-#define MAXPTRLEN	32
+#define MAXPTRLEN 32
                                 /* Must be a power of 2, and
 				 * larger than 26 [a-z]|[A-Z]
 				 * its used to allocate the set
@@ -89,9 +88,9 @@
 
 struct MessageTree
 {
-  int links; /* Count of all pointers (including msg) at this node 
-	      * used as reference count for deletion of _this_ node.
-	      */
+  int links; /* Count of all pointers (including msg) at this node
+              * used as reference count for deletion of _this_ node.
+              */
   struct Message *msg;
   struct MessageTree *pointers[MAXPTRLEN];
 };
@@ -103,7 +102,6 @@ static struct MessageTree msg_tree;
  */
 static char *sender;
 static char *para[MAXPARA + 2]; /* <prefix> + <params> + NULL */
-static char buffer[1024];
 
 static int cancel_clients(struct Client *, struct Client *, char *);
 static void remove_unknown(struct Client *, char *, char *);
@@ -131,10 +129,10 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
     return;
 
   assert(client_p->localClient->fd.flags.open);
-  assert((bufend - pbuffer) < 512);
+  assert((bufend - pbuffer) < IRCD_BUFSIZE);
 
-  for (ch = pbuffer; *ch == ' '; ++ch) /* skip spaces */
-    /* null statement */ ;
+  for (ch = pbuffer; *ch == ' '; ++ch)  /* skip spaces */
+    /* null statement */  ;
 
   if (*ch == ':')
   {
@@ -155,11 +153,10 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
       if ((from = find_person(client_p, sender)) == NULL)
         from = hash_find_server(sender);
 
-      /* Hmm! If the client corresponding to the
-       * prefix is not found--what is the correct
-       * action??? Now, I will ignore the message
-       * (old IRC just let it through as if the
-       * prefix just wasn't there...) --msa
+      /*
+       * Hmm! If the client corresponding to the prefix is not found--what is
+       * the correct action??? Now, I will ignore the message (old IRC just
+       * let it through as if the prefix just wasn't there...) --msa
        */
       if (from == NULL)
       {
@@ -186,9 +183,10 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
     return;
   }
 
-  /* Extract the command code from the packet.  Point s to the end
+  /*
+   * Extract the command code from the packet. Point s to the end
    * of the command code and calculate the length using pointer
-   * arithmetic.  Note: only need length for numerics and *all*
+   * arithmetic. Note: only need length for numerics and *all*
    * numerics must have parameters and thus a space after the command
    * code. -avalon
    */
@@ -198,13 +196,13 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
       IsDigit(*ch) && IsDigit(*(ch + 1)) && IsDigit(*(ch + 2)))
   {
     numeric = ch;
-    paramcount = MAXPARA;
+    paramcount = 2;  /* destination, and the rest of it */
     ++ServerStats.is_num;
     s = ch + 3;  /* I know this is ' ' from above if            */
-    *s++ = '\0'; /* blow away the ' ', and point s to next part */
+    *s++ = '\0';  /* blow away the ' ', and point s to next part */
   }
   else
-  { 
+  {
     unsigned int ii = 0;
 
     if ((s = strchr(ch, ' ')) != NULL)
@@ -212,7 +210,8 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
 
     if ((msg_ptr = find_command(ch)) == NULL)
     {
-      /* Note: Give error message *only* to recognized
+      /* 
+       * Note: Give error message *only* to recognized
        * persons. It's a nightmare situation to have
        * two programs sending "Unknown command"'s or
        * equivalent to each other at full blast....
@@ -267,7 +266,7 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
        if (*s == ':')
        {
          /* The rest is a single parameter */
-         para[++parc] = s + 1;
+         para[++parc] = s + (!numeric);  /* keep the colon if it's a numeric */
          break;
        }
 
@@ -322,7 +321,7 @@ handle_command(struct Message *mptr, struct Client *client_p,
     }
     else
     {
-      sendto_realops_flags(UMODE_ALL, L_ALL, 
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "Dropping server %s due to (invalid) command '%s' "
                            "with only %d arguments (expecting %d).",
                            client_p->name, mptr->cmd, i, mptr->args_min);
@@ -369,12 +368,12 @@ add_msg_element(struct MessageTree *mtree_p, struct Message *msg_p,
   }
   else
   {
-    /* *cmd & (MAXPTRLEN-1) 
+    /*
+     * *cmd & (MAXPTRLEN-1)
      * convert the char pointed to at *cmd from ASCII to an integer
      * between 0 and MAXPTRLEN.
      * Thus 'A' -> 0x1 'B' -> 0x2 'c' -> 0x3 etc.
      */
-
     if ((ntree_p = mtree_p->pointers[*cmd & (MAXPTRLEN - 1)]) == NULL)
     {
       ntree_p = MyMalloc(sizeof(struct MessageTree));
@@ -543,21 +542,23 @@ report_messages(struct Client *source_p)
 
 /* cancel_clients()
  *
- * inputs	- 
- * output	- 
- * side effects	- 
+ * inputs	-
+ * output	-
+ * side effects	-
  */
 static int
 cancel_clients(struct Client *client_p, struct Client *source_p, char *cmd)
 {
-  /* kill all possible points that are causing confusion here,
+  /*
+   * Kill all possible points that are causing confusion here,
    * I'm not sure I've got this all right...
    * - avalon
    *
-   * knowing avalon, probably not.
+   * Knowing avalon, probably not.
    */
 
-  /* with TS, fake prefixes are a common thing, during the
+  /*
+   * With TS, fake prefixes are a common thing, during the
    * connect burst when there's a nick collision, and they
    * must be ignored rather than killed because one of the
    * two is surviving.. so we don't bother sending them to
@@ -566,39 +567,44 @@ cancel_clients(struct Client *client_p, struct Client *source_p, char *cmd)
    * servers to be dropped though, as well as the ones from
    * non-TS servers -orabidoo
    */
-  /* Incorrect prefix for a server from some connection.  If it is a
+  /*
+   * Incorrect prefix for a server from some connection. If it is a
    * client trying to be annoying, just QUIT them, if it is a server
    * then the same deal.
    */
   if (IsServer(source_p) || IsMe(source_p))
   {
-    sendto_realops_flags(UMODE_DEBUG, L_ADMIN, "Message for %s[%s] from %s",
+    sendto_realops_flags(UMODE_DEBUG, L_ADMIN, SEND_NOTICE,
+                         "Message for %s[%s] from %s",
                          source_p->name, source_p->from->name,
                          get_client_name(client_p, SHOW_IP));
-    sendto_realops_flags(UMODE_DEBUG, L_OPER,  "Message for %s[%s] from %s",
+    sendto_realops_flags(UMODE_DEBUG, L_OPER, SEND_NOTICE,
+                         "Message for %s[%s] from %s",
                          source_p->name, source_p->from->name,
                          get_client_name(client_p, MASK_IP));
-    sendto_realops_flags(UMODE_DEBUG, L_ALL,
+    sendto_realops_flags(UMODE_DEBUG, L_ALL, SEND_NOTICE,
                          "Not dropping server %s (%s) for Fake Direction",
                          client_p->name, source_p->name);
     return -1;
     /* return exit_client(client_p, client_p, &me, "Fake Direction");*/
   }
 
-  /* Ok, someone is trying to impose as a client and things are
-   * confused.  If we got the wrong prefix from a server, send out a
+  /*
+   * Ok, someone is trying to impose as a client and things are
+   * confused. If we got the wrong prefix from a server, send out a
    * kill, else just exit the lame client.
    */
-  /* If the fake prefix is coming from a TS server, discard it
+  /*
+   * If the fake prefix is coming from a TS server, discard it
    * silently -orabidoo
    *
    * all servers must be TS these days --is
    */
-  sendto_realops_flags(UMODE_DEBUG, L_ADMIN,
+  sendto_realops_flags(UMODE_DEBUG, L_ADMIN, SEND_NOTICE,
                        "Message for %s[%s@%s!%s] from %s (TS, ignored)",
                        source_p->name, source_p->username, source_p->host,
                        source_p->from->name, get_client_name(client_p, SHOW_IP));
-  sendto_realops_flags(UMODE_DEBUG, L_OPER,
+  sendto_realops_flags(UMODE_DEBUG, L_OPER, SEND_NOTICE,
                        "Message for %s[%s@%s!%s] from %s (TS, ignored)",
                        source_p->name, source_p->username, source_p->host,
                        source_p->from->name, get_client_name(client_p, MASK_IP));
@@ -608,28 +614,30 @@ cancel_clients(struct Client *client_p, struct Client *source_p, char *cmd)
 
 /* remove_unknown()
  *
- * inputs	- 
- * output	- 
- * side effects	- 
+ * inputs	-
+ * output	-
+ * side effects	-
  */
 static void
 remove_unknown(struct Client *client_p, char *lsender, char *lbuffer)
 {
-  /* Do kill if it came from a server because it means there is a ghost
+  /*
+   * Do kill if it came from a server because it means there is a ghost
    * user on the other server which needs to be removed. -avalon
    * Tell opers about this. -Taner
    */
-  /* '[0-9]something'  is an ID      (KILL/SQUIT depending on its length)
+  /*
+   * '[0-9]something'  is an ID      (KILL/SQUIT depending on its length)
    * 'nodots'          is a nickname (KILL)
    * 'no.dot.at.start' is a server   (SQUIT)
    */
   if ((IsDigit(*lsender) && strlen(lsender) <= IRC_MAXSID) ||
       strchr(lsender, '.') != NULL)
   {
-    sendto_realops_flags(UMODE_DEBUG, L_ADMIN,
+    sendto_realops_flags(UMODE_DEBUG, L_ADMIN, SEND_NOTICE,
                          "Unknown prefix (%s) from %s, Squitting %s",
                          lbuffer, get_client_name(client_p, SHOW_IP), lsender);
-    sendto_realops_flags(UMODE_DEBUG, L_OPER,
+    sendto_realops_flags(UMODE_DEBUG, L_OPER, SEND_NOTICE,
                          "Unknown prefix (%s) from %s, Squitting %s",
                          lbuffer, client_p->name, lsender);
     sendto_one(client_p, ":%s SQUIT %s :(Unknown prefix (%s) from %s)",
@@ -653,117 +661,71 @@ remove_unknown(struct Client *client_p, char *lsender, char *lbuffer)
  *      wrong with the message, just *DROP* it! Don't even think of
  *      sending back a neat error message -- big danger of creating
  *      a ping pong error message...
+ *
+ * Rewritten by Nemesi, Jan 1999, to support numeric nicks in parv[1]
+ *
+ * Called when we get a numeric message from a remote _server_ and we are
+ * supposed to forward it somewhere. Note that we always ignore numerics sent
+ * to 'me' and simply drop the message if we can't handle with this properly:
+ * the savvy approach is NEVER generate an error in response to an... error :)
  */
 static void
 handle_numeric(char numeric[], struct Client *client_p, struct Client *source_p,
                int parc, char *parv[])
 {
-  struct Client *target_p;
-  struct Channel *chptr;
-  char *t;    /* current position within the buffer */
-  int i, tl;  /* current length of presently being built string in t */
+  struct Client *target_p = NULL;
+  struct Channel *chptr = NULL;
 
+  /*
+   * Avoid trash, we need it to come from a server and have a target
+   */
   if (parc < 2 || !IsServer(source_p))
     return;
 
-  /* Remap low number numerics. */
+  /*
+   * Who should receive this message ? Will we do something with it ?
+   * Note that we use findUser functions, so the target can't be neither
+   * a server, nor a channel (?) nor a list of targets (?) .. u2.10
+   * should never generate numeric replies to non-users anyway
+   * Ahem... it can be a channel actually, csc bots use it :\ --Nem
+   */
+  if (IsChanPrefix(*parv[1]))
+    chptr = hash_find_channel(parv[1]);
+  else
+    target_p = find_person(client_p, parv[1]);
+
+  if (((!target_p) || (target_p->from == client_p)) && !chptr)
+    return;
+
+  /*
+   * Remap low number numerics, not that I understand WHY.. --Nemesi
+   */
+  /*
+   * Numerics below 100 talk about the current 'connection', you're not
+   * connected to a remote server so it doesn't make sense to send them
+   * remotely - but the information they contain may be useful, so we
+   * remap them up. Weird, but true.  -- Isomer
+   */
   if (numeric[0] == '0')
     numeric[0] = '1';
 
-  /* Prepare the parameter portion of the message into 'buffer'.
-   * (Because the buffer is twice as large as the message buffer
-   * for the socket, no overflow can occur here... ...on current
-   * assumptions--bets are off, if these are changed --msa)
-   */
-  t = buffer;
-  for (i = 2; i < (parc - 1); i++)
+  if (target_p)
   {
-    tl = ircsprintf(t, " %s", parv[i]);
-    t += tl;
-  }
-
-  ircsprintf(t, " :%s", parv[parc-1]);
-
-  if (((target_p = find_person(client_p, parv[1])) != NULL) ||
-      ((target_p = hash_find_server(parv[1])) != NULL))
-  {
-    if (IsMe(target_p)) 
-    {
-      int num;
-
-      /*
-       * We shouldn't get numerics sent to us,
-       * any numerics we do get indicate a bug somewhere..
-       */
-      /* ugh.  this is here because of nick collisions.  when two servers
-       * relink, they burst each other their nicks, then perform collides.
-       * if there is a nick collision, BOTH servers will kill their own
-       * nicks, and BOTH will kill the other servers nick, which wont exist,
-       * because it will have been already killed by the local server.
-       *
-       * unfortunately, as we cant guarantee other servers will do the
-       * "right thing" on a nick collision, we have to keep both kills.  
-       * ergo we need to ignore ERR_NOSUCHNICK. --fl_
-       */
-      /* quick comment. This _was_ tried. i.e. assume the other servers
-       * will do the "right thing" and kill a nick that is colliding.
-       * unfortunately, it did not work. --Dianora
-       */
-
-      /*
-       * Also ignoring ERR_NOTREGISTERED for now. A connecting server may send this
-       * in response to the "NOTICE AUTH ..." message upon registration. Normally
-       * the !IsServer() test in handle_numeric() would take care of this situation,
-       * but due to the fact that read_packet() in s_auth.c:release_auth_client() is
-       * called for the very first time _after_ ident/dns registration, it looks like
-       * the numeric (451) messages are queued up whereas meanwhile the serverlink
-       * succesfully establishes. Right after that, after IsServer() is true, the queued
-       * numeric messages will then be processed, and a notice is falsely sent to operators.
-       * The I/O engine will be rewritten soon (a mixture of libevent), so maybe this also
-       * will fix described behavior above, but for now we're just going to drop this message
-       * (no problem at all, we propably shouldn't process numeric destined to servers anyway)
-       * -Michael
-       */
-      /* Yes, a good compiler would have optimised this, but
-       * this is probably easier to read. -db
-       */
-      num = atoi(numeric);
-
-      if ((num != ERR_NOSUCHNICK) && (num != ERR_NOTREGISTERED))
-        sendto_realops_flags(UMODE_ALL, L_ADMIN,
-                             "*** %s(via %s) sent a %s numeric to me: %s",
-                             source_p->name, client_p->name, numeric, buffer);
-      return;
-    }
-    else if (target_p->from == client_p) 
-    {
-      /* This message changed direction (nick collision?)
-       * ignore it.
-       */
-      return;
-    }
-
-    /* csircd will send out unknown umode flag for +a (admin), drop it here. */
-    if ((atoi(numeric) == ERR_UMODEUNKNOWNFLAG) && MyClient(target_p))
-      return;
-    
     /* Fake it for server hiding, if its our client */
-    if (ConfigServerHide.hide_servers &&
-        MyClient(target_p) && !HasUMode(target_p, UMODE_OPER))
-      sendto_one(target_p, ":%s %s %s%s", me.name, numeric, target_p->name, buffer);
+    if (ConfigServerHide.hide_servers && MyClient(target_p) &&
+        !HasUMode(target_p, UMODE_OPER))
+      sendto_one(target_p, ":%s %s %s %s", me.name, numeric, target_p->name, parv[2]);
     else
-      sendto_one(target_p, ":%s %s %s%s", ID_or_name(source_p, target_p->from),
-                 numeric, ID_or_name(target_p, target_p->from), buffer);
-    return;
+      sendto_one(target_p, ":%s %s %s %s", ID_or_name(source_p, target_p->from),
+                 numeric, ID_or_name(target_p, target_p->from), parv[2]);
   }
-  else if ((chptr = hash_find_channel(parv[1])) != NULL)
+  else
     sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s %s %s %s",
-                         source_p->name,
-                         numeric, chptr->chname, buffer);
+                         source_p->name, numeric, chptr->chname, parv[2]);
 }
 
 /* m_not_oper()
- * inputs	- 
+ * inputs	-
  * output	-
  * side effects	- just returns a nastyogram to given user
  */
@@ -787,7 +749,7 @@ void
 m_registered(struct Client *client_p, struct Client *source_p,
              int parc, char *parv[])
 {
-  sendto_one(source_p, form_str(ERR_ALREADYREGISTRED),   
+  sendto_one(source_p, form_str(ERR_ALREADYREGISTRED),
              me.name, source_p->name);
 }
 
@@ -797,33 +759,3 @@ m_ignore(struct Client *client_p, struct Client *source_p,
 {
   return;
 }
-
-void
-rfc1459_command_send_error(struct Client *client_p, struct Client *source_p,
-                           int parc, char *parv[])
-{
-  const char *in_para;
-
-  in_para = (parc > 1 && *parv[1] != '\0') ? parv[1] : "<>";
-
-  ilog(LOG_TYPE_IRCD, "Received ERROR message from %s: %s",
-       source_p->name, in_para);
-
-  if (client_p == source_p)
-  {
-    sendto_realops_flags(UMODE_ALL, L_ADMIN, "ERROR :from %s -- %s",
-                         get_client_name(client_p, HIDE_IP), in_para);
-    sendto_realops_flags(UMODE_ALL, L_OPER,  "ERROR :from %s -- %s",
-                         get_client_name(client_p, MASK_IP), in_para);
-  }
-  else
-  {
-    sendto_realops_flags(UMODE_ALL, L_OPER, "ERROR :from %s via %s -- %s",
-                         source_p->name, get_client_name(client_p, MASK_IP), in_para);
-    sendto_realops_flags(UMODE_ALL, L_ADMIN, "ERROR :from %s via %s -- %s",
-                         source_p->name, get_client_name(client_p, HIDE_IP), in_para);
-  }
-
-  if (MyClient(source_p))
-    exit_client(source_p, source_p, "ERROR");
-}
