@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: restart.c 1341 2012-04-07 18:33:13Z michael $
+ *  $Id: restart.c 1631 2012-11-01 22:04:01Z michael $
  */
 
 #include "stdinc.h"
@@ -32,6 +32,7 @@
 #include "log.h"
 #include "client.h" /* for UMODE_ALL */
 #include "memory.h"
+#include "conf_db.h"
 
 void
 restart(const char *mesg)
@@ -79,6 +80,8 @@ server_die(const char *mesg, int rboot)
 
   ilog(LOG_TYPE_IRCD, "%s", buffer);
 
+  save_all_databases(NULL);
+
   send_queued_all();
   close_fds(NULL);
 
