@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_bsd.c 1871 2013-04-26 08:53:04Z michael $
+ *  $Id: s_bsd.c 2734 2014-01-03 17:30:27Z michael $
  */
 
 #include "stdinc.h"
@@ -52,8 +52,6 @@ static const char *comm_err_str[] = { "Comm OK", "Error during bind()",
   "Error during DNS lookup", "connect timeout", "Error during connect()",
   "Comm Error" };
 
-struct Callback *setup_socket_cb = NULL;
-
 static void comm_connect_callback(fde_t *, int);
 static PF comm_connect_timeout;
 static void comm_connect_dns_callback(void *, const struct irc_ssaddr *, const char *);
@@ -129,7 +127,8 @@ report_error(int level, const char* text, const char* who, int error)
 {
   who = (who) ? who : "";
 
-  sendto_realops_flags(UMODE_DEBUG, level, text, who, strerror(error));
+  sendto_realops_flags(UMODE_DEBUG, level, SEND_NOTICE,
+                       text, who, strerror(error));
   ilog(LOG_TYPE_IRCD, text, who, strerror(error));
 }
 
@@ -138,10 +137,9 @@ report_error(int level, const char* text, const char* who, int error)
  *
  * Set the socket non-blocking, and other wonderful bits.
  */
-static void *
-setup_socket(va_list args)
+static void
+setup_socket(int fd)
 {
-  int fd = va_arg(args, int);
   int opt = 1;
 
   setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &opt, sizeof(opt));
@@ -152,20 +150,6 @@ setup_socket(va_list args)
 #endif
 
   fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) | O_NONBLOCK);
-
-  return NULL;
-}
-
-/*
- * init_comm()
- *
- * Initializes comm subsystem.
- */
-void
-init_comm(void)
-{
-  setup_socket_cb = register_callback("setup_socket", setup_socket);
-  init_netio();
 }
 
 /*
@@ -176,8 +160,6 @@ init_comm(void)
 void
 close_connection(struct Client *client_p)
 {
-  struct AccessItem *aconf;
-  struct ClassItem *aclass;
   dlink_node *ptr = NULL;
 
   assert(client_p);
@@ -209,7 +191,7 @@ close_connection(struct Client *client_p)
 
     DLINK_FOREACH(ptr, server_items.head)
     {
-      struct ConfItem *conf = ptr->data;
+      struct MaskItem *conf = ptr->data;
 
       if (irccmp(conf->name, client_p->name))
         continue;
@@ -218,9 +200,7 @@ close_connection(struct Client *client_p)
        * Reset next-connect cycle of all connect{} blocks that match
        * this servername.
        */
-      aconf  = map_to_conf(conf);
-      aclass = map_to_conf(aconf->class_ptr);
-      aconf->hold = CurrentTime + aclass->con_freq;
+      conf->until = CurrentTime + conf->class->con_freq;
     }
   }
   else
@@ -242,7 +222,7 @@ close_connection(struct Client *client_p)
   dbuf_clear(&client_p->localClient->buf_recvq);
   
   MyFree(client_p->localClient->passwd);
-  detach_conf(client_p, CONF_TYPE);
+  detach_conf(client_p, CONF_CLIENT|CONF_OPER|CONF_SERVER);
   client_p->from = NULL; /* ...this should catch them! >:) --msa */
 }
 
@@ -254,27 +234,63 @@ close_connection(struct Client *client_p)
 static void
 ssl_handshake(int fd, struct Client *client_p)
 {
-  int ret = SSL_accept(client_p->localClient->fd.ssl);
+  X509 *cert = NULL;
+  int ret = 0;
+
+  if ((ret = SSL_accept(client_p->localClient->fd.ssl)) <= 0)
+  {
+    if ((CurrentTime - client_p->localClient->firsttime) > 30)
+    {
+      exit_client(client_p, client_p, "Timeout during SSL handshake");
+      return;
+    }
 
-  if (ret <= 0)
     switch (SSL_get_error(client_p->localClient->fd.ssl, ret))
     {
       case SSL_ERROR_WANT_WRITE:
         comm_setselect(&client_p->localClient->fd, COMM_SELECT_WRITE,
-	               (PF *) ssl_handshake, client_p, 0);
+	               (PF *) ssl_handshake, client_p, 30);
         return;
 
       case SSL_ERROR_WANT_READ:
         comm_setselect(&client_p->localClient->fd, COMM_SELECT_READ,
-	               (PF *) ssl_handshake, client_p, 0);
+	               (PF *) ssl_handshake, client_p, 30);
         return;
 
       default:
         exit_client(client_p, client_p, "Error during SSL handshake");
 	return;
     }
+  }
 
-  execute_callback(auth_cb, client_p);
+  comm_settimeout(&client_p->localClient->fd, 0, NULL, NULL);
+
+  if ((cert = SSL_get_peer_certificate(client_p->localClient->fd.ssl)))
+  {
+    int res = SSL_get_verify_result(client_p->localClient->fd.ssl);
+    char buf[EVP_MAX_MD_SIZE * 2 + 1] = { '\0' };
+    unsigned char md[EVP_MAX_MD_SIZE] = { '\0' };
+
+    if (res == X509_V_OK || res == X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN ||
+        res == X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE ||
+        res == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT)
+    {
+      unsigned int i = 0, n = 0;
+
+      if (X509_digest(cert, EVP_sha256(), md, &n))
+      {
+        for (; i < n; ++i)
+          snprintf(buf + 2 * i, 3, "%02X", md[i]);
+        client_p->certfp = xstrdup(buf);
+      }
+    }
+    else
+      ilog(LOG_TYPE_IRCD, "Client %s!%s@%s gave bad SSL client certificate: %d",
+           client_p->name, client_p->username, client_p->host, res);
+    X509_free(cert);
+  }
+
+  start_auth(client_p);
 }
 #endif
 
@@ -305,6 +321,15 @@ add_connection(struct Listener *listener, struct irc_ssaddr *irn, int fd)
               sizeof(new_client->sockhost), NULL, 0, NI_NUMERICHOST);
   new_client->localClient->aftype = new_client->localClient->ip.ss.ss_family;
 
+#ifdef HAVE_LIBGEOIP
+  /* XXX IPV6 SUPPORT XXX */
+  if (irn->ss.ss_family == AF_INET && geoip_ctx)
+  {
+    const struct sockaddr_in *v4 = (const struct sockaddr_in *)&new_client->localClient->ip;
+    new_client->localClient->country_id = GeoIP_id_by_ipnum(geoip_ctx, (unsigned long)ntohl(v4->sin_addr.s_addr));
+  }
+#endif
+
   if (new_client->sockhost[0] == ':' && new_client->sockhost[1] == ':')
   {
     strlcpy(new_client->host, "0", sizeof(new_client->host));
@@ -331,12 +356,13 @@ add_connection(struct Listener *listener, struct irc_ssaddr *irn, int fd)
       return;
     }
 
+    AddFlag(new_client, FLAGS_SSL);
     SSL_set_fd(new_client->localClient->fd.ssl, fd);
     ssl_handshake(0, new_client);
   }
   else
 #endif
-    execute_callback(auth_cb, new_client);
+    start_auth(new_client);
 }
 
 /*
@@ -677,7 +703,7 @@ comm_open(fde_t *F, int family, int sock_type, int proto, const char *note)
   if (fd < 0)
     return -1; /* errno will be passed through, yay.. */
 
-  execute_callback(setup_socket_cb, fd);
+  setup_socket(fd);
 
   /* update things in our fd tracking */
   fd_open(F, fd, 1, note);
@@ -718,7 +744,7 @@ comm_accept(struct Listener *lptr, struct irc_ssaddr *pn)
   pn->ss_len = addrlen;
 #endif
 
-  execute_callback(setup_socket_cb, newfd);
+  setup_socket(newfd);
 
   /* .. and return */
   return newfd;
