@@ -20,7 +20,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_bsd_devpoll.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: s_bsd_devpoll.c 2724 2013-12-29 13:00:42Z michael $
  */
 
 #include "stdinc.h"
@@ -114,7 +114,11 @@ comm_setselect(fde_t *F, unsigned int type, PF *handler,
     (F->write_handler ? POLLOUT : 0);
 
   if (timeout != 0)
+  {
     F->timeout = CurrentTime + (timeout / 1000);
+    F->timeout_handler = handler;
+    F->timeout_data = client_data;
+  }
 
   if (new_events != F->evcache)
   {
@@ -163,6 +167,7 @@ comm_select(void)
       continue;
 
     if ((dopoll.dp_fds[i].revents & POLLIN))
+    {
       if ((hdl = F->read_handler) != NULL)
       {
         F->read_handler = NULL;
@@ -170,8 +175,10 @@ comm_select(void)
         if (!F->flags.open)
           continue;
       }
+    }
 
     if ((dopoll.dp_fds[i].revents & POLLOUT))
+    {
       if ((hdl = F->write_handler) != NULL) 
       {
         F->write_handler = NULL;
@@ -179,6 +186,7 @@ comm_select(void)
         if (!F->flags.open)
           continue;
       }
+    }
 
     comm_setselect(F, 0, NULL, NULL, 0);
   }
