@@ -20,7 +20,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_bsd_epoll.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: s_bsd_epoll.c 2724 2013-12-29 13:00:42Z michael $
  */
 
 #include "stdinc.h"
@@ -141,7 +141,11 @@ comm_setselect(fde_t *F, unsigned int type, PF *handler,
     (F->write_handler ? EPOLLOUT : 0);
 
   if (timeout != 0)
+  {
     F->timeout = CurrentTime + (timeout / 1000);
+    F->timeout_handler = handler;
+    F->timeout_data = client_data;
+  }
 
   if (new_events != F->evcache)
   {
@@ -198,22 +202,26 @@ comm_select(void)
       continue;
 
     if ((ep_fdlist[i].events & (EPOLLIN | EPOLLHUP | EPOLLERR)))
+    {
       if ((hdl = F->read_handler) != NULL)
       {
         F->read_handler = NULL;
         hdl(F, F->read_data);
-	if (!F->flags.open)
-	  continue;
+        if (!F->flags.open)
+          continue;
       }
+    }
 
     if ((ep_fdlist[i].events & (EPOLLOUT | EPOLLHUP | EPOLLERR)))
+    {
       if ((hdl = F->write_handler) != NULL)
       {
         F->write_handler = NULL;
         hdl(F, F->write_data);
-	if (!F->flags.open)
-	  continue;
+        if (!F->flags.open)
+          continue;
       }
+    }
 
     comm_setselect(F, 0, NULL, NULL, 0);
   }
