@@ -20,7 +20,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_bsd_kqueue.c 1309 2012-03-25 11:24:18Z michael $
+ *  $Id: s_bsd_kqueue.c 2724 2013-12-29 13:00:42Z michael $
  */
 
 #include "stdinc.h"
@@ -50,7 +50,6 @@
 static fde_t kqfd;
 static struct kevent kq_fdlist[KE_LENGTH];  /* kevent buffer */
 static int kqoff;      /* offset into the buffer */
-void init_netio(void);
 
 /*
  * init_netio
@@ -85,7 +84,10 @@ kq_update_events(int fd, int filter, int what)
 
   if (++kqoff == KE_LENGTH)
   {
-    kevent(kqfd.fd, kq_fdlist, kqoff, NULL, 0, &zero_timespec);
+    int i;
+
+    for (i = 0; i < kqoff; ++i)
+      kevent(kqfd.fd, &kq_fdlist[i], 1, NULL, 0, &zero_timespec);
     kqoff = 0;
   }
 }
@@ -115,10 +117,14 @@ comm_setselect(fde_t *F, unsigned int type, PF *handler,
   }
 
   new_events = (F->read_handler ? COMM_SELECT_READ : 0) |
-   (F->write_handler ? COMM_SELECT_WRITE : 0);
+               (F->write_handler ? COMM_SELECT_WRITE : 0);
 
   if (timeout != 0)
+  {
     F->timeout = CurrentTime + (timeout / 1000);
+    F->timeout_handler = handler;
+    F->timeout_data = client_data;
+  }
 
   diff = new_events ^ F->evcache;
 
@@ -176,22 +182,26 @@ comm_select(void)
       continue;
 
     if (ke[i].filter == EVFILT_READ)
+    {
       if ((hdl = F->read_handler) != NULL)
       {
         F->read_handler = NULL;
         hdl(F, F->read_data);
-	if (!F->flags.open)
-	  continue;
+        if (!F->flags.open)
+          continue;
       }
+    }
 
     if (ke[i].filter == EVFILT_WRITE)
+    {
       if ((hdl = F->write_handler) != NULL)
       {
         F->write_handler = NULL;
         hdl(F, F->write_data);
-	if (!F->flags.open)
-	  continue;
+        if (!F->flags.open)
+          continue;
       }
+    }
 
     comm_setselect(F, 0, NULL, NULL, 0);
   }
