@@ -20,7 +20,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_bsd_poll.c 1732 2013-01-11 12:57:19Z michael $
+ *  $Id: s_bsd_poll.c 2724 2013-12-29 13:00:42Z michael $
  */
 
 #include "stdinc.h"
@@ -28,6 +28,7 @@
 #include <sys/poll.h>
 #include "fdlist.h"
 #include "list.h"
+#include "memory.h"
 #include "hook.h"
 #include "ircd.h"
 #include "s_bsd.h"
@@ -44,6 +45,7 @@
 static struct pollfd *pollfds;
 static int pollmax = -1;  /* highest FD number */
 
+
 /*
  * init_netio
  *
@@ -107,10 +109,14 @@ comm_setselect(fde_t *F, unsigned int type, PF *handler,
   }
 
   new_events = (F->read_handler ? POLLRDNORM : 0) |
-    (F->write_handler ? POLLWRNORM : 0);
+               (F->write_handler ? POLLWRNORM : 0);
 
   if (timeout != 0)
+  {
     F->timeout = CurrentTime + (timeout / 1000);
+    F->timeout_handler = handler;
+    F->timeout_data = client_data;
+  }
 
   if (new_events != F->evcache)
   {
@@ -121,18 +127,19 @@ comm_setselect(fde_t *F, unsigned int type, PF *handler,
 
       if (pollmax == F->comm_index)
         while (pollmax >= 0 && pollfds[pollmax].fd == -1)
-	  pollmax--;
+          pollmax--;
     }
     else
     {
       if (F->evcache == 0)
       {
         F->comm_index = poll_findslot();
-	if (F->comm_index > pollmax)
-	  pollmax = F->comm_index;
+        if (F->comm_index > pollmax)
+          pollmax = F->comm_index;
 
-	pollfds[F->comm_index].fd = F->fd;
+        pollfds[F->comm_index].fd = F->fd;
       }
+
       pollfds[F->comm_index].events = new_events;
       pollfds[F->comm_index].revents = 0;
     }
@@ -180,22 +187,26 @@ comm_select(void)
       continue;
 
     if (revents & (POLLRDNORM | POLLIN | POLLHUP | POLLERR))
+    {
       if ((hdl = F->read_handler) != NULL)
       {
         F->read_handler = NULL;
         hdl(F, F->read_data);
-	if (!F->flags.open)
-	  continue;
+        if (!F->flags.open)
+          continue;
       }
+    }
 
     if (revents & (POLLWRNORM | POLLOUT | POLLHUP | POLLERR))
+    {
       if ((hdl = F->write_handler) != NULL)
       {
         F->write_handler = NULL;
         hdl(F, F->write_data);
-	if (!F->flags.open)
-	  continue;
+        if (!F->flags.open)
+          continue;
       }
+    }
 
     comm_setselect(F, 0, NULL, NULL, 0);
   }
