@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_user.c 1901 2013-04-27 19:17:36Z michael $
+ *  $Id: s_user.c 2690 2013-12-17 18:55:43Z michael $
  */
 
 #include "stdinc.h"
@@ -32,7 +32,6 @@
 #include "fdlist.h"
 #include "hash.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "s_bsd.h"
 #include "ircd.h"
 #include "listener.h"
@@ -53,14 +52,10 @@
 #include "parse.h"
 #include "watch.h"
 
-
-struct Callback *entering_umode_cb = NULL;
-struct Callback *umode_cb = NULL;
-
 static char umode_buffer[IRCD_BUFSIZE];
 
 static void user_welcome(struct Client *);
-static void report_and_set_user_flags(struct Client *, const struct AccessItem *);
+static void report_and_set_user_flags(struct Client *, const struct MaskItem *);
 static int check_xline(struct Client *);
 static void introduce_client(struct Client *);
 static const char *uid_get(void);
@@ -69,7 +64,7 @@ static const char *uid_get(void);
  * used with init_isupport, add_isupport, delete_isupport
  */
 
-struct Isupport 
+struct Isupport
 {
   dlink_node node;
   char *name;
@@ -77,11 +72,11 @@ struct Isupport
   int number;
 };
 
-static dlink_list support_list = { NULL, NULL, 0 };
-MessageFile *isupportFile;
+static dlink_list support_list;
+static dlink_list support_list_lines;
 
 /* memory is cheap. map 0-255 to equivalent mode */
-unsigned int user_modes[256] =
+const unsigned int user_modes[256] =
 {
   /* 0x00 */ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* 0x0F */
   /* 0x10 */ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* 0x1F */
@@ -90,10 +85,10 @@ unsigned int user_modes[256] =
   0,                  /* @ */
   0,                  /* A */
   0,                  /* B */
-  UMODE_CCONN_FULL,   /* C */
+  0,                  /* C */
   UMODE_DEAF,         /* D */
   0,                  /* E */
-  0,                  /* F */
+  UMODE_FARCONNECT,   /* F */
   UMODE_SOFTCALLERID, /* G */
   UMODE_HIDDEN,       /* H */
   0,                  /* I */
@@ -106,15 +101,15 @@ unsigned int user_modes[256] =
   0,                  /* P */
   0,                  /* Q */
   UMODE_REGONLY,      /* R */
-  0,                  /* S */
+  UMODE_SSL,          /* S */
   0,                  /* T */
   0,                  /* U */
   0,                  /* V */
-  0,                  /* W */
+  UMODE_WEBIRC,       /* W */
   0,                  /* X */
   0,                  /* Y */
   0,                  /* Z 0x5A */
-  0, 0, 0, 0, 0,      /* 0x5F   */ 
+  0, 0, 0, 0, 0,      /* 0x5F   */
   0,                  /* 0x60   */
   UMODE_ADMIN,        /* a */
   UMODE_BOTS,         /* b */
@@ -139,7 +134,7 @@ unsigned int user_modes[256] =
   UMODE_UNAUTH,       /* u */
   0,                  /* v */
   UMODE_WALLOP,       /* w */
-  0,                  /* x */
+  UMODE_HIDDENHOST,   /* x */
   UMODE_SPY,          /* y */
   UMODE_OPERWALL,     /* z      0x7A */
   0,0,0,0,0,          /* 0x7B - 0x7F */
@@ -174,7 +169,7 @@ assemble_umode_buffer(void)
  * side effects - display to client user counts etc.
  */
 void
-show_lusers(struct Client *source_p) 
+show_lusers(struct Client *source_p)
 {
   const char *from, *to;
 
@@ -220,7 +215,7 @@ show_lusers(struct Client *source_p)
   {
     sendto_one(source_p, form_str(RPL_LUSERME),
                from, to, Count.total, 0);
-    sendto_one(source_p, form_str(RPL_LOCALUSERS), 
+    sendto_one(source_p, form_str(RPL_LOCALUSERS),
                from, to, Count.total, Count.max_tot);
   }
 
@@ -235,7 +230,7 @@ show_lusers(struct Client *source_p)
     Count.max_loc_cli = Count.local;
 
   if ((Count.local + Count.myserver) > Count.max_loc_con)
-    Count.max_loc_con = Count.local + Count.myserver; 
+    Count.max_loc_con = Count.local + Count.myserver;
 }
 
 /* show_isupport()
@@ -245,9 +240,13 @@ show_lusers(struct Client *source_p)
  * side effects	- display to client what we support (for them)
  */
 void
-show_isupport(struct Client *source_p) 
+show_isupport(struct Client *source_p)
 {
-  send_message_file(source_p, isupportFile);
+  const dlink_node *ptr = NULL;
+
+  DLINK_FOREACH(ptr, support_list_lines.head)
+    sendto_one(source_p, form_str(RPL_ISUPPORT), me.name,
+               source_p->name, ptr->data);
 }
 
 /*
@@ -277,8 +276,7 @@ void
 register_local_user(struct Client *source_p)
 {
   const char *id = NULL;
-  const struct AccessItem *aconf = NULL;
-  dlink_node *ptr = NULL;
+  const struct MaskItem *conf = NULL;
 
   assert(source_p != NULL);
   assert(source_p == source_p->from);
@@ -289,7 +287,7 @@ register_local_user(struct Client *source_p)
 
   if (ConfigFileEntry.ping_cookie)
   {
-    if (!IsPingSent(source_p) && source_p->localClient->random_ping == 0)
+    if (!IsPingSent(source_p) && !source_p->localClient->random_ping)
     {
       do
         source_p->localClient->random_ping = genrand_int32();
@@ -309,10 +307,10 @@ register_local_user(struct Client *source_p)
   /* Straight up the maximum rate of flooding... */
   source_p->localClient->allow_read = MAX_FLOOD_BURST;
 
-  if (!execute_callback(client_check_cb, source_p, source_p->username))
+  if (!check_client(source_p))
     return;
 
-  if (valid_hostname(source_p->host) == 0)
+  if (!valid_hostname(source_p->host))
   {
     sendto_one(source_p, ":%s NOTICE %s :*** Notice -- You have an illegal "
                "character in your hostname", me.name, source_p->name);
@@ -320,8 +318,7 @@ register_local_user(struct Client *source_p)
             sizeof(source_p->host));
   }
 
-  ptr   = source_p->localClient->confs.head;
-  aconf = map_to_conf(ptr->data);
+  conf = source_p->localClient->confs.head->data;
 
   if (!IsGotId(source_p))
   {
@@ -329,7 +326,7 @@ register_local_user(struct Client *source_p)
     const char *p = username;
     unsigned int i = 0;
 
-    if (IsNeedIdentd(aconf))
+    if (IsNeedIdentd(conf))
     {
       ++ServerStats.is_ref;
       sendto_one(source_p, ":%s NOTICE %s :*** Notice -- You need to install "
@@ -340,7 +337,7 @@ register_local_user(struct Client *source_p)
 
     strlcpy(username, source_p->username, sizeof(username));
 
-    if (!IsNoTilde(aconf))
+    if (!IsNoTilde(conf))
       source_p->username[i++] = '~';
 
     for (; *p && i < USERLEN; ++p)
@@ -351,11 +348,11 @@ register_local_user(struct Client *source_p)
   }
 
   /* password check */
-  if (!EmptyString(aconf->passwd))
+  if (!EmptyString(conf->passwd))
   {
     const char *pass = source_p->localClient->passwd;
 
-    if (!match_conf_password(pass, aconf))
+    if (!match_conf_password(pass, conf))
     {
       ++ServerStats.is_ref;
       sendto_one(source_p, form_str(ERR_PASSWDMISMATCH),
@@ -371,7 +368,7 @@ register_local_user(struct Client *source_p)
    */
 
   /* report if user has &^>= etc. and set flags as needed in source_p */
-  report_and_set_user_flags(source_p, aconf);
+  report_and_set_user_flags(source_p, conf);
 
   if (IsDead(source_p))
     return;
@@ -387,7 +384,7 @@ register_local_user(struct Client *source_p)
   if ((Count.local >= ServerInfo.max_clients + MAX_BUFFER) ||
       (Count.local >= ServerInfo.max_clients && !IsExemptLimits(source_p)))
   {
-    sendto_realops_flags(UMODE_FULL, L_ALL,
+    sendto_realops_flags(UMODE_FULL, L_ALL, SEND_NOTICE,
                          "Too many clients, rejecting %s[%s].",
                          source_p->name, source_p->host);
     ++ServerStats.is_ref;
@@ -396,11 +393,12 @@ register_local_user(struct Client *source_p)
   }
 
   /* valid user name check */
-  if (valid_username(source_p->username) == 0)
+  if (!valid_username(source_p->username, 1))
   {
     char tmpstr2[IRCD_BUFSIZE];
 
-    sendto_realops_flags(UMODE_REJ, L_ALL, "Invalid username: %s (%s@%s)",
+    sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE,
+                         "Invalid username: %s (%s@%s)",
                          source_p->name, source_p->username, source_p->host);
     ++ServerStats.is_ref;
     snprintf(tmpstr2, sizeof(tmpstr2), "Invalid username [%s]",
@@ -418,39 +416,27 @@ register_local_user(struct Client *source_p)
   strlcpy(source_p->id, id, sizeof(source_p->id));
   hash_add_id(source_p);
 
-  sendto_realops_flags(UMODE_CCONN, L_ALL,
+  sendto_realops_flags(UMODE_CCONN, L_ALL, SEND_NOTICE,
                        "Client connecting: %s (%s@%s) [%s] {%s} [%s] <%s>",
                        source_p->name, source_p->username, source_p->host,
                        ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
                        "255.255.255.255" : source_p->sockhost,
-                       get_client_class(source_p),
+                       get_client_class(&source_p->localClient->confs),
                        source_p->info, source_p->id);
 
-  sendto_realops_flags(UMODE_CCONN_FULL, L_ALL,
-                       "CLICONN %s %s %s %s %s %s %s 0 %s",
-                       source_p->name, source_p->username, source_p->host,
-                       ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
-                       "255.255.255.255" : source_p->sockhost,
-		       get_client_class(source_p),
-		       ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
-                           "<hidden>" : source_p->localClient->client_host,
-		       ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
-                           "<hidden>" : source_p->localClient->client_server,
-                       source_p->info);
-
-
   if (ConfigFileEntry.invisible_on_connect)
   {
     AddUMode(source_p, UMODE_INVISIBLE);
     ++Count.invisi;
   }
 
-  if ((++Count.local) > Count.max_loc)
+  if (++Count.local > Count.max_loc)
   {
     Count.max_loc = Count.local;
 
     if (!(Count.max_loc % 10))
-      sendto_realops_flags(UMODE_ALL, L_ALL, "New Max Local Clients: %d",
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                           "New Max Local Clients: %d",
                            Count.max_loc);
   }
 
@@ -498,7 +484,7 @@ register_remote_user(struct Client *source_p,
   assert(source_p != NULL);
   assert(source_p->username != username);
 
-  strlcpy(source_p->host, host, sizeof(source_p->host)); 
+  strlcpy(source_p->host, host, sizeof(source_p->host));
   strlcpy(source_p->username, username, sizeof(source_p->username));
 
   /*
@@ -506,13 +492,14 @@ register_remote_user(struct Client *source_p,
    */
   source_p->servptr = hash_find_server(server);
 
-  /* Super GhostDetect:
+  /*
+   * Super GhostDetect:
    * If we can't find the server the user is supposed to be on,
    * then simply blow the user away.        -Taner
    */
   if (source_p->servptr == NULL)
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "No server %s for user %s[%s@%s] from %s",
                          server, source_p->name, source_p->username,
                          source_p->host, source_p->from->name);
@@ -525,7 +512,7 @@ register_remote_user(struct Client *source_p,
 
   if ((target_p = source_p->servptr) && target_p->from != source_p->from)
   {
-    sendto_realops_flags(UMODE_DEBUG, L_ALL,
+    sendto_realops_flags(UMODE_DEBUG, L_ALL, SEND_NOTICE,
                          "Bad User [%s] :%s USER %s@%s %s, != %s[%s]",
                          source_p->from->name, source_p->name, source_p->username,
                          source_p->host, source_p->servptr->name,
@@ -554,6 +541,13 @@ register_remote_user(struct Client *source_p,
   add_user_host(source_p->username, source_p->host, 1);
   SetUserHost(source_p);
 
+  if (HasFlag(source_p->servptr, FLAGS_EOB))
+    sendto_realops_flags(UMODE_FARCONNECT, L_ALL, SEND_NOTICE,
+                         "Client connecting at %s: %s (%s@%s) [%s] <%s>",
+                         source_p->servptr->name,
+                         source_p->name, source_p->username, source_p->host,
+                         source_p->info, source_p->id);
+
   introduce_client(source_p);
 }
 
@@ -569,7 +563,7 @@ static void
 introduce_client(struct Client *source_p)
 {
   dlink_node *server_node = NULL;
-  static char ubuf[12];
+  char ubuf[IRCD_BUFSIZE];
 
   if (MyClient(source_p))
     send_umode(source_p, source_p, 0, SEND_UMODES, ubuf);
@@ -578,7 +572,7 @@ introduce_client(struct Client *source_p)
 
   watch_check_hash(source_p, RPL_LOGON);
 
-  if (*ubuf == '\0')
+  if (ubuf[0] == '\0')
   {
     ubuf[0] = '+';
     ubuf[1] = '\0';
@@ -629,6 +623,9 @@ introduce_client(struct Client *source_p)
                    ubuf, source_p->username, source_p->host,
                    source_p->servptr->name, source_p->info);
     }
+
+    if (!EmptyString(source_p->certfp))
+      sendto_one(server, ":%s CERTFP %s", ID(source_p), source_p->certfp);
   }
 }
 
@@ -648,7 +645,7 @@ valid_hostname(const char *hostname)
 
   assert(p != NULL);
 
-  if (*p == '.' || *p == ':')
+  if (EmptyString(p) || *p == '.' || *p == ':')
     return 0;
 
   for (; *p; ++p)
@@ -670,7 +667,7 @@ valid_hostname(const char *hostname)
  * style of username
  */
 int
-valid_username(const char *username)
+valid_username(const char *username, const int local)
 {
   int dots      = 0;
   const char *p = username;
@@ -680,27 +677,37 @@ valid_username(const char *username)
   if (*p == '~')
     ++p;
 
-  /* reject usernames that don't start with an alphanum
+  /*
+   * Reject usernames that don't start with an alphanum
    * i.e. reject jokers who have '-@somehost' or '.@somehost'
    * or "-hi-@somehost", "h-----@somehost" would still be accepted.
    */
   if (!IsAlNum(*p))
     return 0;
 
-  while (*++p)
+  if (local)
   {
-    if ((*p == '.') && ConfigFileEntry.dots_in_ident)
+    while (*++p)
     {
-      if (++dots > ConfigFileEntry.dots_in_ident)
-        return 0;
-      if (!IsUserChar(*(p + 1)))
+      if ((*p == '.') && ConfigFileEntry.dots_in_ident)
+      {
+        if (++dots > ConfigFileEntry.dots_in_ident)
+          return 0;
+        if (!IsUserChar(*(p + 1)))
+          return 0;
+      }
+      else if (!IsUserChar(*p))
         return 0;
     }
-    else if (!IsUserChar(*p))
-      return 0;
+  }
+  else
+  {
+    while (*++p)
+      if (!IsUserChar(*p))
+        return 0;
   }
 
-  return 1;
+  return  p - username <= USERLEN;;
 }
 
 /* clean_nick_name()
@@ -731,58 +738,57 @@ valid_nickname(const char *nickname, const int local)
 /* report_and_set_user_flags()
  *
  * inputs       - pointer to source_p
- *              - pointer to aconf for this user
+ *              - pointer to conf for this user
  * output       - NONE
  * side effects - Report to user any special flags
  *                they are getting, and set them.
  */
 static void
-report_and_set_user_flags(struct Client *source_p, const struct AccessItem *aconf)
+report_and_set_user_flags(struct Client *source_p, const struct MaskItem *conf)
 {
   /* If this user is being spoofed, tell them so */
-  if (IsConfDoSpoofIp(aconf))
-  {
+  if (IsConfDoSpoofIp(conf))
     sendto_one(source_p,
-               ":%s NOTICE %s :*** Spoofing your IP. congrats.",
+               ":%s NOTICE %s :*** Spoofing your IP. Congrats.",
                me.name, source_p->name);
-  }
 
   /* If this user is in the exception class, Set it "E lined" */
-  if (IsConfExemptKline(aconf))
+  if (IsConfExemptKline(conf))
   {
     SetExemptKline(source_p);
     sendto_one(source_p,
-               ":%s NOTICE %s :*** You are exempt from K/D/G lines. congrats.",
+               ":%s NOTICE %s :*** You are exempt from K/D/G lines. Congrats.",
                me.name, source_p->name);
   }
 
-  /* The else here is to make sure that G line exempt users
+  /*
+   * The else here is to make sure that G line exempt users
    * do not get noticed twice.
    */
-  else if (IsConfExemptGline(aconf))
+  else if (IsConfExemptGline(conf))
   {
     SetExemptGline(source_p);
-    sendto_one(source_p, ":%s NOTICE %s :*** You are exempt from G lines.",
+    sendto_one(source_p, ":%s NOTICE %s :*** You are exempt from G lines. Congrats.",
                me.name, source_p->name);
   }
 
-  if (IsConfExemptResv(aconf))
+  if (IsConfExemptResv(conf))
   {
     SetExemptResv(source_p);
-    sendto_one(source_p, ":%s NOTICE %s :*** You are exempt from resvs.",
+    sendto_one(source_p, ":%s NOTICE %s :*** You are exempt from resvs. Congrats.",
                me.name, source_p->name);
   }
 
   /* If this user is exempt from user limits set it "F lined" */
-  if (IsConfExemptLimits(aconf))
+  if (IsConfExemptLimits(conf))
   {
     SetExemptLimits(source_p);
     sendto_one(source_p,
-               ":%s NOTICE %s :*** You are exempt from user limits. congrats.",
+               ":%s NOTICE %s :*** You are exempt from user limits. Congrats.",
                me.name,source_p->name);
   }
 
-  if (IsConfCanFlood(aconf))
+  if (IsConfCanFlood(conf))
   {
     SetCanFlood(source_p);
     sendto_one(source_p, ":%s NOTICE %s :*** You are exempt from flood "
@@ -791,32 +797,6 @@ report_and_set_user_flags(struct Client *source_p, const struct AccessItem *acon
   }
 }
 
-/* change_simple_umode()
- *
- * this callback can be hooked to allow special handling of
- * certain usermodes
- */
-static void *
-change_simple_umode(va_list args)
-{
-  struct Client *client_p;
-  struct Client *source_p;
-  int what;
-  unsigned int flag;
-
-  client_p = va_arg(args, struct Client *);
-  source_p = va_arg(args, struct Client *);
-  what = va_arg(args, int);
-  flag = va_arg(args, unsigned int);
-
-  if (what == MODE_ADD)
-    AddUMode(source_p, flag);
-  else
-    DelUMode(source_p, flag);
-
-  return NULL;
-}
-
 /* set_user_mode()
  *
  * added 15/10/91 By Darren Reed.
@@ -826,7 +806,7 @@ change_simple_umode(va_list args)
  */
 void
 set_user_mode(struct Client *client_p, struct Client *source_p,
-              int parc, char *parv[])
+              const int parc, char *parv[])
 {
   unsigned int flag, setflags;
   char **p, *m, buf[IRCD_BUFSIZE];
@@ -843,13 +823,6 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  if (IsServer(source_p))
-  {
-     sendto_realops_flags(UMODE_ALL, L_ADMIN, "*** Mode for User %s from %s",
-                          parv[1], source_p->name);
-     return;
-  }
-
   if (source_p != target_p)
   {
      sendto_one(source_p, form_str(ERR_USERSDONTMATCH),
@@ -862,7 +835,7 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
     m = buf;
     *m++ = '+';
 
-    for (i = 0; i < 128; i++)
+    for (i = 0; i < 128; ++i)
       if (HasUMode(source_p, user_modes[i]))
         *m++ = (char)i;
     *m = '\0';
@@ -872,15 +845,13 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  execute_callback(entering_umode_cb, client_p, source_p);
-
   /* find flags already set for user */
   setflags = source_p->umodes;
 
   /* parse mode change string(s) */
-  for (p = &parv[2]; p && *p; p++)
+  for (p = &parv[2]; p && *p; ++p)
   {
-    for (m = *p; *m; m++)
+    for (m = *p; *m; ++m)
     {
       switch (*m)
       {
@@ -901,9 +872,6 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
           }
           else
           {
-            /* Only decrement the oper counts if an oper to begin with
-             * found by Pat Szuta, Perly , perly@xnet.com 
-             */
             if (!HasUMode(source_p, UMODE_OPER))
               break;
 
@@ -912,9 +880,9 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
 
             if (MyConnect(source_p))
             {
-              dlink_node *dm;
+              dlink_node *dm = NULL;
 
-              detach_conf(source_p, OPER_TYPE);
+              detach_conf(source_p, CONF_OPER);
               ClrOFlag(source_p);
               DelUMode(source_p, ConfigFileEntry.oper_only_umodes);
 
@@ -925,14 +893,10 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
 
           break;
 
-        /* we may not get these,
-         * but they shouldnt be in default
-         */
-        case 'r':
-        case ' ' :
-        case '\n':
-        case '\r':
-        case '\t':
+        case 'S':  /* Only servers may set +S in a burst */
+        case 'W':  /* Only servers may set +W in a burst */
+        case 'r':  /* Only services may set +r */
+        case 'x':  /* Only services may set +x */
           break;
 
         default:
@@ -940,17 +904,17 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
           {
             if (MyConnect(source_p) && !HasUMode(source_p, UMODE_OPER) &&
                 (ConfigFileEntry.oper_only_umodes & flag))
-            {
               badflag = 1;
-            }
             else
-              execute_callback(umode_cb, client_p, source_p, what, flag);
-          }
-          else
-          {
-            if (MyConnect(source_p))
-              badflag = 1;
+            {
+              if (what == MODE_ADD)
+                AddUMode(source_p, flag);
+              else
+                DelUMode(source_p, flag);
+            }
           }
+          else if (MyConnect(source_p))
+            badflag = 1;
 
           break;
       }
@@ -961,13 +925,6 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
     sendto_one(source_p, form_str(ERR_UMODEUNKNOWNFLAG),
                me.name, source_p->name);
 
-  if (HasUMode(source_p, UMODE_NCHANGE) && !HasOFlag(source_p, OPER_FLAG_N))
-  {
-    sendto_one(source_p, ":%s NOTICE %s :*** You have no nchange flag;",
-               me.name, source_p->name);
-    DelUMode(source_p, UMODE_NCHANGE);
-  }
-
   if (MyConnect(source_p) && HasUMode(source_p, UMODE_ADMIN) &&
       !HasOFlag(source_p, OPER_FLAG_ADMIN))
   {
@@ -1012,7 +969,7 @@ send_umode(struct Client *client_p, struct Client *source_p,
    * build a string in umode_buf to represent the change in the user's
    * mode between the new (source_p->umodes) and 'old'.
    */
-  for (i = 0; i < 128; i++)
+  for (i = 0; i < 128; ++i)
   {
     flag = user_modes[i];
     if (!flag)
@@ -1086,22 +1043,92 @@ send_umode_out(struct Client *client_p, struct Client *source_p,
 }
 
 void
-user_set_hostmask(struct Client *target_p, const char *hostname)
+user_set_hostmask(struct Client *target_p, const char *hostname, const int what)
 {
-  if (!valid_hostname(hostname))
+  dlink_node *ptr = NULL;
+
+  if (!strcmp(target_p->host, hostname))
     return;
 
+  switch (what)
+  {
+    case MODE_ADD:
+      AddUMode(target_p, UMODE_HIDDENHOST);
+      AddFlag(target_p, FLAGS_IP_SPOOFING);
+      break;
+    case MODE_DEL:
+      DelUMode(target_p, UMODE_HIDDENHOST);
+
+      if (!HasFlag(target_p, FLAGS_AUTH_SPOOF))
+        DelFlag(target_p, FLAGS_IP_SPOOFING);
+      break;
+    default: return;
+  }
+
+  if (ConfigFileEntry.cycle_on_host_change)
+    sendto_common_channels_local(target_p, 0, 0, ":%s!%s@%s QUIT :Changing hostname",
+                                 target_p->name, target_p->username, target_p->host);
+
   if (IsUserHostIp(target_p))
     delete_user_host(target_p->username, target_p->host, !MyConnect(target_p));
 
   strlcpy(target_p->host, hostname, sizeof(target_p->host));
-  SetIPSpoof(target_p);
 
   add_user_host(target_p->username, target_p->host, !MyConnect(target_p));
   SetUserHost(target_p);
 
   if (MyClient(target_p))
+  {
+    sendto_one(target_p, form_str(RPL_NEWHOSTIS), me.name,
+               target_p->name, target_p->host);
     clear_ban_cache_client(target_p);
+  }
+
+  if (!ConfigFileEntry.cycle_on_host_change)
+    return;
+
+  DLINK_FOREACH(ptr, target_p->channel.head)
+  {
+    char modebuf[4], nickbuf[NICKLEN * 3 + 3] = { '\0' };
+    char *p = modebuf;
+    int len = 0;
+    const struct Membership *ms = ptr->data;
+
+    if (has_member_flags(ms, CHFL_CHANOP))
+    {
+      *p++ = 'o';
+      len += snprintf(nickbuf + len, sizeof(nickbuf) - len, len ? " %s" : "%s", target_p->name);
+    }
+
+    if (has_member_flags(ms, CHFL_HALFOP))
+    {
+      *p++ = 'h';
+      len += snprintf(nickbuf + len, sizeof(nickbuf) - len, len ? " %s" : "%s", target_p->name);
+    }
+
+    if (has_member_flags(ms, CHFL_VOICE))
+    {
+      *p++ = 'v';
+      len += snprintf(nickbuf + len, sizeof(nickbuf) - len, len ? " %s" : "%s", target_p->name);
+    }
+
+    *p = '\0';
+
+    sendto_channel_local_butone(target_p, 0, 0, ms->chptr, ":%s!%s@%s JOIN :%s",
+                                target_p->name, target_p->username, target_p->host,
+                                ms->chptr->chname);
+    if (nickbuf[0])
+      sendto_channel_local_butone(target_p, 0, 0, ms->chptr, ":%s MODE %s +%s %s",
+                                  target_p->servptr->name, ms->chptr->chname,
+                                  modebuf, nickbuf);
+
+  }
+
+  if (target_p->away[0])
+    sendto_common_channels_local(target_p, 0, CAP_AWAY_NOTIFY,
+                                 ":%s!%s@%s AWAY :%s",
+                                 target_p->name, target_p->username,
+                                 target_p->host, target_p->away);
 }
 
 /* user_welcome()
@@ -1120,13 +1147,16 @@ user_welcome(struct Client *source_p)
 #endif
 
 #ifdef HAVE_LIBCRYPTO
-  if (source_p->localClient->fd.ssl != NULL)
+  if (HasFlag(source_p, FLAGS_SSL))
+  {
+    AddUMode(source_p, UMODE_SSL);
     sendto_one(source_p, ":%s NOTICE %s :*** Connected securely via %s",
                me.name, source_p->name,
                ssl_get_cipher(source_p->localClient->fd.ssl));
+  }
 #endif
 
-  sendto_one(source_p, form_str(RPL_WELCOME), me.name, source_p->name, 
+  sendto_one(source_p, form_str(RPL_WELCOME), me.name, source_p->name,
              ServerInfo.network_name, source_p->name);
   sendto_one(source_p, form_str(RPL_YOURHOST), me.name, source_p->name,
              get_listener_name(source_p->localClient->listener), ircd_version);
@@ -1141,24 +1171,7 @@ user_welcome(struct Client *source_p)
                source_p->name, source_p->id);
 
   show_lusers(source_p);
-
-  if (ConfigFileEntry.short_motd)
-  {
-    sendto_one(source_p, ":%s NOTICE %s :*** Notice -- motd was last changed at %s",
-               me.name, source_p->name, ConfigFileEntry.motd.lastChangedDate);
-    sendto_one(source_p,
-               ":%s NOTICE %s :*** Notice -- Please read the motd if you haven't "
-               "read it", me.name, source_p->name);
-    sendto_one(source_p, form_str(RPL_MOTDSTART),
-               me.name, source_p->name, me.name);
-    sendto_one(source_p, form_str(RPL_MOTD),
-               me.name, source_p->name,
-               "*** This is the short motd ***");
-    sendto_one(source_p, form_str(RPL_ENDOFMOTD),
-               me.name, source_p->name);
-  }
-  else  
-    send_message_file(source_p, &ConfigFileEntry.motd);
+  motd_signon(source_p);
 }
 
 /* check_xline()
@@ -1170,22 +1183,19 @@ user_welcome(struct Client *source_p)
 static int
 check_xline(struct Client *source_p)
 {
-  struct ConfItem *conf = NULL;
+  struct MaskItem *conf = NULL;
   const char *reason = NULL;
 
-  if ((conf = find_matching_name_conf(XLINE_TYPE, source_p->info, NULL, NULL, 0)) ||
-      (conf = find_matching_name_conf(RXLINE_TYPE, source_p->info, NULL, NULL, 0)))
+  if ((conf = find_matching_name_conf(CONF_XLINE, source_p->info, NULL, NULL, 0)))
   {
-    struct MatchItem *reg = map_to_conf(conf);
+    ++conf->count;
 
-    ++reg->count;
-
-    if (reg->reason != NULL)
-      reason = reg->reason;
+    if (conf->reason != NULL)
+      reason = conf->reason;
     else
-      reason = CONF_NOREASON;
+      reason = "No Reason";
 
-    sendto_realops_flags(UMODE_REJ, L_ALL,
+    sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE,
                          "X-line Rejecting [%s] [%s], user %s [%s]",
                          source_p->info, reason,
                          get_client_name(source_p, HIDE_IP),
@@ -1203,24 +1213,23 @@ check_xline(struct Client *source_p)
  *
  * inputs	- pointer to given client to oper
  * output	- NONE
- * side effects	- Blindly opers up given source_p, using aconf info
+ * side effects	- Blindly opers up given source_p, using conf info
  *                all checks on passwords have already been done.
- *                This could also be used by rsa oper routines. 
+ *                This could also be used by rsa oper routines.
  */
 void
 oper_up(struct Client *source_p)
 {
   const unsigned int old = source_p->umodes;
-  const struct AccessItem *oconf = NULL;
+  const struct MaskItem *conf = source_p->localClient->confs.head->data;
 
   assert(source_p->localClient->confs.head);
-  oconf = map_to_conf((source_p->localClient->confs.head)->data);
 
   ++Count.oper;
   SetOper(source_p);
 
-  if (oconf->modes)
-    AddUMode(source_p, oconf->modes);
+  if (conf->modes)
+    AddUMode(source_p, conf->modes);
   else if (ConfigFileEntry.oper_umodes)
     AddUMode(source_p, ConfigFileEntry.oper_umodes);
 
@@ -1232,14 +1241,12 @@ oper_up(struct Client *source_p)
   assert(dlinkFind(&oper_list, source_p) == NULL);
   dlinkAdd(source_p, make_dlink_node(), &oper_list);
 
-  AddOFlag(source_p, oconf->port);
+  AddOFlag(source_p, conf->port);
 
   if (HasOFlag(source_p, OPER_FLAG_ADMIN))
     AddUMode(source_p, UMODE_ADMIN);
-  if (!HasOFlag(source_p, OPER_FLAG_N))
-    DelUMode(source_p, UMODE_NCHANGE);
 
-  sendto_realops_flags(UMODE_ALL, L_ALL, "%s is now an operator",
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE, "%s is now an operator",
                        get_oper_name(source_p));
   send_umode_out(source_p, source_p, old);
   sendto_one(source_p, form_str(RPL_YOUREOPER), me.name, source_p->name);
@@ -1260,7 +1267,7 @@ valid_sid(const char *sid)
 
 /*
  * init_uid()
- * 
+ *
  * inputs	- NONE
  * output	- NONE
  * side effects	- new_uid is filled in with server id portion (sid)
@@ -1270,7 +1277,7 @@ valid_sid(const char *sid)
 void
 init_uid(void)
 {
-  int i;
+  unsigned int i;
 
   memset(new_uid, 0, sizeof(new_uid));
 
@@ -1278,18 +1285,11 @@ init_uid(void)
     strlcpy(new_uid, ServerInfo.sid, sizeof(new_uid));
 
   for (i = 0; i < IRC_MAXSID; ++i)
-    if (new_uid[i] == '\0') 
+    if (new_uid[i] == '\0')
       new_uid[i] = 'A';
 
   /* NOTE: if IRC_MAXUID != 6, this will have to be rewritten */
-  /* Yes nenolod, I have known it was off by one ever since I wrote it
-   * But *JUST* for you, though, it really doesn't look as *pretty*
-   * -Dianora
-   */
   memcpy(new_uid + IRC_MAXSID, "AAAAA@", IRC_MAXUID);
-
-  entering_umode_cb = register_callback("entering_umode", NULL);
-  umode_cb = register_callback("changing_umode", change_simple_umode);
 }
 
 /*
@@ -1349,8 +1349,6 @@ uid_get(void)
 void
 init_isupport(void)
 {
-  isupportFile = init_MessageLine();
-
   add_isupport("CALLERID", NULL, -1);
   add_isupport("CASEMAPPING", CASEMAP, -1);
   add_isupport("DEAF", "D", -1);
@@ -1363,6 +1361,8 @@ init_isupport(void)
   add_isupport("PREFIX", "(ov)@+", -1);
   add_isupport("STATUSMSG", "@+", -1);
 #endif
+  add_isupport("EXCEPTS", "e", -1);
+  add_isupport("INVEX", "I", -1);
 }
 
 /*
@@ -1397,9 +1397,9 @@ add_isupport(const char *name, const char *options, int n)
     dlinkAddTail(support, &support->node, &support_list);
   }
 
-  DupString(support->name, name);
+  support->name = xstrdup(name);
   if (options != NULL)
-    DupString(support->options, options);
+    support->options = xstrdup(options);
   support->number = n;
 
   rebuild_isupport_message_line();
@@ -1446,30 +1446,35 @@ rebuild_isupport_message_line(void)
 {
   char isupportbuffer[IRCD_BUFSIZE];
   char *p = isupportbuffer;
-  dlink_node *ptr = NULL;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
   int n = 0;
   int tokens = 0;
   size_t len = 0;
   size_t reserve = strlen(me.name) + HOSTLEN + strlen(form_str(RPL_ISUPPORT));
 
-  destroy_MessageLine(isupportFile);
+  DLINK_FOREACH_SAFE(ptr, ptr_next, support_list_lines.head)
+  {
+    dlinkDelete(ptr, &support_list_lines);
+    MyFree(ptr->data);
+    free_dlink_node(ptr);
+  }
 
   DLINK_FOREACH(ptr, support_list.head)
   {
     struct Isupport *support = ptr->data;
 
-    p += (n = ircsprintf(p, "%s", support->name));
+    p += (n = sprintf(p, "%s", support->name));
     len += n;
 
     if (support->options != NULL)
     {
-      p += (n = ircsprintf(p, "=%s", support->options));
+      p += (n = sprintf(p, "=%s", support->options));
       len += n;
     }
 
     if (support->number > 0)
     {
-      p += (n = ircsprintf(p, "=%d", support->number));
+      p += (n = sprintf(p, "=%d", support->number));
       len += n;
     }
 
@@ -1482,7 +1487,7 @@ rebuild_isupport_message_line(void)
       if (*--p == ' ')
         *p = '\0';
 
-      addto_MessageLine(isupportFile, isupportbuffer);
+      dlinkAddTail(xstrdup(isupportbuffer), make_dlink_node(), &support_list_lines);
       p = isupportbuffer;
       len = 0;
       n = tokens = 0;
@@ -1493,6 +1498,6 @@ rebuild_isupport_message_line(void)
   {
     if (*--p == ' ')
       *p = '\0';
-    addto_MessageLine(isupportFile, isupportbuffer);
+    dlinkAddTail(xstrdup(isupportbuffer), make_dlink_node(), &support_list_lines);
   }
 }
