@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: send.c 1762 2013-01-18 19:21:54Z michael $
+ *  $Id: send.c 2653 2013-12-11 17:45:22Z michael $
  */
 
 #include "stdinc.h"
@@ -34,15 +34,12 @@
 #include "fdlist.h"
 #include "s_bsd.h"
 #include "s_serv.h"
-#include "sprintf_irc.h"
 #include "conf.h"
 #include "log.h"
 #include "memory.h"
-#include "hook.h"
 #include "packet.h"
 
 
-struct Callback *iosend_cb = NULL;
 static unsigned int current_serial = 0;
 
 
@@ -65,7 +62,7 @@ send_format(char *lsendbuf, int bufsize, const char *pattern, va_list args)
    *
    * IRC messages are always lines of characters terminated with a CR-LF
    * (Carriage Return - Line Feed) pair, and these messages shall not
-   * exceed 512 characters in length,  counting all characters 
+   * exceed 512 characters in length,  counting all characters
    * including the trailing CR-LF.
    * Thus, there are 510 characters maximum allowed
    * for the command and its parameters.  There is no provision for
@@ -81,20 +78,6 @@ send_format(char *lsendbuf, int bufsize, const char *pattern, va_list args)
   return len;
 }
 
-/*
- * iosend_default - append a packet to the client's sendq.
- */
-void *
-iosend_default(va_list args)
-{
-  struct Client *to = va_arg(args, struct Client *);
-  int length = va_arg(args, int);
-  char *buf = va_arg(args, char *);
-
-  dbuf_put(&to->localClient->buf_sendq, buf, length);
-  return NULL;
-}
-
 /*
  ** send_message
  **      Internal utility which appends given buffer to the sockets
@@ -106,21 +89,21 @@ send_message(struct Client *to, char *buf, int len)
   assert(!IsMe(to));
   assert(to != &me);
 
-  if (dbuf_length(&to->localClient->buf_sendq) + len > get_sendq(to))
+  if (dbuf_length(&to->localClient->buf_sendq) + len > get_sendq(&to->localClient->confs))
   {
     if (IsServer(to))
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "Max SendQ limit exceeded for %s: %lu > %u",
                            get_client_name(to, HIDE_IP),
                            (unsigned long)(dbuf_length(&to->localClient->buf_sendq) + len),
-                           get_sendq(to));
+                           get_sendq(&to->localClient->confs));
     if (IsClient(to))
       SetSendQExceeded(to);
     dead_link_on_write(to, 0);
     return;
   }
 
-  execute_callback(iosend_cb, to, len, buf);
+  dbuf_put(&to->localClient->buf_sendq, buf, len);
 
   /*
    ** Update statistics. The following is slightly incorrect
@@ -152,9 +135,9 @@ send_message_remote(struct Client *to, struct Client *from,
 {
   if (!MyConnect(to))
   {
-    sendto_realops_flags(UMODE_ALL, L_ALL,
-			 "server send message to %s [%s] dropped from %s(Not local server)",
-			 to->name, to->from->name, from->name);
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
+                         "Server send message to %s [%s] dropped from %s(Not local server)",
+                         to->name, to->from->name, from->name);
     return;
   }
 
@@ -165,13 +148,13 @@ send_message_remote(struct Client *to, struct Client *from,
   {
     if (IsServer(from))
     {
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "Send message to %s [%s] dropped from %s(Fake Dir)",
                            to->name, to->from->name, from->name);
       return;
     }
 
-    sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "Ghosted: %s[%s@%s] from %s[%s@%s] (%s)",
                          to->name, to->username, to->host,
                          from->name, from->username, from->host,
@@ -194,9 +177,8 @@ send_message_remote(struct Client *to, struct Client *from,
                  to->host, to->from);
 
     exit_client(to, &me, "Ghosted client");
-
     return;
-  } 
+  }
 
   send_message(to, buf, len);
 }
@@ -229,8 +211,8 @@ sendq_unblocked(fde_t *fd, struct Client *client_p)
 void
 send_queued_write(struct Client *to)
 {
-  int retlen;
-  struct dbuf_block *first;
+  int retlen = 0;
+  struct dbuf_block *first = NULL;
 
   /*
    ** Once socket is marked dead, we cannot start writing to it,
@@ -240,10 +222,10 @@ send_queued_write(struct Client *to)
     return;  /* no use calling send() now */
 
   /* Next, lets try to write some data */
-
   if (dbuf_length(&to->localClient->buf_sendq))
   {
-    do {
+    do
+    {
       first = to->localClient->buf_sendq.blocks.head->data;
 
 #ifdef HAVE_LIBCRYPTO
@@ -252,22 +234,23 @@ send_queued_write(struct Client *to)
         retlen = SSL_write(to->localClient->fd.ssl, first->data, first->size);
 
         /* translate openssl error codes, sigh */
-	if (retlen < 0)
-	  switch (SSL_get_error(to->localClient->fd.ssl, retlen))
-	  {
+        if (retlen < 0)
+        {
+          switch (SSL_get_error(to->localClient->fd.ssl, retlen))
+          {
             case SSL_ERROR_WANT_READ:
-	      return;  /* retry later, don't register for write events */
-
-	    case SSL_ERROR_WANT_WRITE:
-	      errno = EWOULDBLOCK;
+              return;  /* retry later, don't register for write events */
+            case SSL_ERROR_WANT_WRITE:
+              errno = EWOULDBLOCK;
             case SSL_ERROR_SYSCALL:
-	      break;
+              break;
             case SSL_ERROR_SSL:
               if (errno == EAGAIN)
                 break;
             default:
-	      retlen = errno = 0;  /* either an SSL-specific error or EOF */
-	  }
+              retlen = errno = 0;  /* either an SSL-specific error or EOF */
+          }
+        }
       }
       else
 #endif
@@ -288,7 +271,7 @@ send_queued_write(struct Client *to)
       /* we have a non-fatal error, reschedule a write */
       SetSendqBlocked(to);
       comm_setselect(&to->localClient->fd, COMM_SELECT_WRITE,
-                     (PF *)sendq_unblocked, (void *)to, 0);
+                     (PF *)sendq_unblocked, to, 0);
     }
     else if (retlen <= 0)
     {
@@ -313,13 +296,13 @@ send_queued_all(void)
    * a notice to opers, which is to be delivered by this function.
    */
   DLINK_FOREACH(ptr, serv_list.head)
-    send_queued_write((struct Client *) ptr->data);
+    send_queued_write(ptr->data);
 
   DLINK_FOREACH(ptr, unknown_list.head)
-    send_queued_write((struct Client *) ptr->data);
+    send_queued_write(ptr->data);
 
   DLINK_FOREACH(ptr, local_client_list.head)
-    send_queued_write((struct Client *) ptr->data);
+    send_queued_write(ptr->data);
 
   /* NOTE: This can still put clients on aborted_list; unfortunately,
    * exit_aborted_clients takes precedence over send_queued_all,
@@ -349,7 +332,7 @@ sendto_one(struct Client *to, const char *pattern, ...)
     return; /* This socket has already been marked as dead */
 
   va_start(args, pattern);
-  len = send_format(buffer, IRCD_BUFSIZE, pattern, args);
+  len = send_format(buffer, sizeof(buffer), pattern, args);
   va_end(args);
 
   send_message(to, buffer, len);
@@ -379,15 +362,14 @@ sendto_channel_butone(struct Client *one, struct Client *from,
   dlink_node *ptr = NULL, *ptr_next = NULL;
 
   if (IsServer(from))
-    local_len = ircsprintf(local_buf, ":%s ",
-                           from->name);
+    local_len = snprintf(local_buf, sizeof(local_buf), ":%s ",
+                        from->name);
   else
-    local_len = ircsprintf(local_buf, ":%s!%s@%s ",
-                           from->name, from->username, from->host);
-  remote_len = ircsprintf(remote_buf, ":%s ",
-                          from->name);
-  uid_len = ircsprintf(uid_buf, ":%s ",
-                       ID(from));
+    local_len = snprintf(local_buf, sizeof(local_buf), ":%s!%s@%s ",
+                         from->name, from->username, from->host);
+  remote_len = snprintf(remote_buf, sizeof(remote_buf), ":%s ",
+                        from->name);
+  uid_len = snprintf(uid_buf, sizeof(uid_buf), ":%s ", ID(from));
 
   va_start(alocal, pattern);
   va_start(aremote, pattern);
@@ -460,7 +442,7 @@ sendto_channel_butone(struct Client *one, struct Client *from,
  * lazylinks, uids, etc.
  * -davidt
  */
-void 
+void
 sendto_server(struct Client *one,
               const unsigned int caps,
               const unsigned int nocaps,
@@ -472,7 +454,7 @@ sendto_server(struct Client *one,
   int len = 0;
 
   va_start(args, format);
-  len = send_format(buffer, IRCD_BUFSIZE, format, args);
+  len = send_format(buffer, sizeof(buffer), format, args);
   va_end(args);
 
   DLINK_FOREACH(ptr, serv_list.head)
@@ -506,7 +488,7 @@ sendto_server(struct Client *one,
  *		  used by m_nick.c and exit_one_client.
  */
 void
-sendto_common_channels_local(struct Client *user, int touser,
+sendto_common_channels_local(struct Client *user, int touser, unsigned int cap,
                              const char *pattern, ...)
 {
   va_list args;
@@ -519,14 +501,14 @@ sendto_common_channels_local(struct Client *user, int touser,
   int len;
 
   va_start(args, pattern);
-  len = send_format(buffer, IRCD_BUFSIZE, pattern, args);
+  len = send_format(buffer, sizeof(buffer), pattern, args);
   va_end(args);
 
   ++current_serial;
 
   DLINK_FOREACH(cptr, user->channel.head)
   {
-    chptr = ((struct Membership *) cptr->data)->chptr;
+    chptr = ((struct Membership *)cptr->data)->chptr;
     assert(chptr != NULL);
 
     DLINK_FOREACH(uptr, chptr->members.head)
@@ -539,6 +521,9 @@ sendto_common_channels_local(struct Client *user, int touser,
           target_p->localClient->serial == current_serial)
         continue;
 
+      if (HasCap(target_p, cap) != cap)
+        continue;
+
       target_p->localClient->serial = current_serial;
       send_message(target_p, buffer, len);
     }
@@ -546,7 +531,8 @@ sendto_common_channels_local(struct Client *user, int touser,
 
   if (touser && MyConnect(user) && !IsDead(user) &&
       user->localClient->serial != current_serial)
-    send_message(user, buffer, len);
+    if (HasCap(user, cap) == cap)
+      send_message(user, buffer, len);
 }
 
 /* sendto_channel_local()
@@ -560,24 +546,22 @@ sendto_common_channels_local(struct Client *user, int touser,
  *		  locally connected to this server.
  */
 void
-sendto_channel_local(int type, int nodeaf, struct Channel *chptr,
+sendto_channel_local(unsigned int type, int nodeaf, struct Channel *chptr,
                      const char *pattern, ...)
 {
   va_list args;
   char buffer[IRCD_BUFSIZE];
-  int len;
-  dlink_node *ptr;
-  struct Membership *ms;
-  struct Client *target_p;
+  int len = 0;
+  dlink_node *ptr = NULL;
 
   va_start(args, pattern);
-  len = send_format(buffer, IRCD_BUFSIZE, pattern, args);
+  len = send_format(buffer, sizeof(buffer), pattern, args);
   va_end(args);
 
   DLINK_FOREACH(ptr, chptr->members.head)
   {
-    ms = ptr->data;
-    target_p = ms->client_p;
+    struct Membership *ms = ptr->data;
+    struct Client *target_p = ms->client_p;
 
     if (type != 0 && (ms->flags & type) == 0)
       continue;
@@ -602,25 +586,23 @@ sendto_channel_local(int type, int nodeaf, struct Channel *chptr,
  *
  * WARNING - +D clients are omitted
  */
-void       
-sendto_channel_local_butone(struct Client *one, int type,
-			    struct Channel *chptr, const char *pattern, ...)
+void
+sendto_channel_local_butone(struct Client *one, unsigned int type, unsigned int cap,
+                            struct Channel *chptr, const char *pattern, ...)
 {
   va_list args;
   char buffer[IRCD_BUFSIZE];
-  int len;
-  struct Client *target_p;
-  struct Membership *ms;
-  dlink_node *ptr;
+  int len = 0;
+  dlink_node *ptr = NULL;
 
-  va_start(args, pattern); 
-  len = send_format(buffer, IRCD_BUFSIZE, pattern, args);
+  va_start(args, pattern);
+  len = send_format(buffer, sizeof(buffer), pattern, args);
   va_end(args);
 
-  DLINK_FOREACH(ptr, chptr->members.head)       
-  {   
-    ms = ptr->data;
-    target_p = ms->client_p;
+  DLINK_FOREACH(ptr, chptr->members.head)
+  {
+    struct Membership *ms = ptr->data;
+    struct Client *target_p = ms->client_p;
 
     if (type != 0 && (ms->flags & type) == 0)
       continue;
@@ -628,6 +610,10 @@ sendto_channel_local_butone(struct Client *one, int type,
     if (!MyConnect(target_p) || target_p == one ||
         IsDefunct(target_p) || HasUMode(target_p, UMODE_DEAF))
       continue;
+
+    if (HasCap(target_p, cap) != cap)
+      continue;
+
     send_message(target_p, buffer, len);
   }
 }
@@ -645,45 +631,45 @@ sendto_channel_local_butone(struct Client *one, int type,
  *		  remote to this server.
  */
 void
-sendto_channel_remote(struct Client *one, struct Client *from, int type,
+sendto_channel_remote(struct Client *one, struct Client *from, unsigned int type,
                       const unsigned int caps, const unsigned int nocaps,
                       struct Channel *chptr, const char *pattern, ...)
 {
   va_list args;
   char buffer[IRCD_BUFSIZE];
-  int len;
-  dlink_node *ptr;
-  struct Client *target_p;
-  struct Membership *ms;
+  int len = 0;
+  dlink_node *ptr = NULL;
 
   va_start(args, pattern);
-  len = send_format(buffer, IRCD_BUFSIZE, pattern, args);
+  len = send_format(buffer, sizeof(buffer), pattern, args);
   va_end(args);
 
   ++current_serial;
 
   DLINK_FOREACH(ptr, chptr->members.head)
   {
-    ms = ptr->data;
-    target_p = ms->client_p;
+    struct Membership *ms = ptr->data;
+    struct Client *target_p = ms->client_p;
 
     if (type != 0 && (ms->flags & type) == 0)
       continue;
 
     if (MyConnect(target_p))
       continue;
+
     target_p = target_p->from;
 
     if (target_p == one->from ||
         ((target_p->from->localClient->caps & caps) != caps) ||
         ((target_p->from->localClient->caps & nocaps) != 0))
       continue;
+
     if (target_p->from->localClient->serial != current_serial)
     {
       send_message(target_p, buffer, len);
       target_p->from->localClient->serial = current_serial;
     }
-  } 
+  }
 }
 
 /*
@@ -707,9 +693,9 @@ static int
 match_it(const struct Client *one, const char *mask, int what)
 {
   if (what == MATCH_HOST)
-    return match(mask, one->host);
+    return !match(mask, one->host);
 
-  return match(mask, one->servptr->name);
+  return !match(mask, one->servptr->name);
 }
 
 /* sendto_match_butone()
@@ -727,9 +713,10 @@ sendto_match_butone(struct Client *one, struct Client *from, char *mask,
   struct Client *client_p;
   dlink_node *ptr, *ptr_next;
   char local_buf[IRCD_BUFSIZE], remote_buf[IRCD_BUFSIZE];
-  int local_len = ircsprintf(local_buf, ":%s!%s@%s ", from->name,
-                             from->username, from->host);
-  int remote_len = ircsprintf(remote_buf, ":%s ", from->name);
+  int local_len = snprintf(local_buf, sizeof(local_buf), ":%s!%s@%s ",
+                           from->name, from->username, from->host);
+  int remote_len = snprintf(remote_buf, sizeof(remote_buf), ":%s ",
+                            from->name);
 
   va_start(alocal, pattern);
   va_start(aremote, pattern);
@@ -794,14 +781,12 @@ sendto_match_butone(struct Client *one, struct Client *from, char *mask,
  * side effects	- data sent to servers matching with capab
  */
 void
-sendto_match_servs(struct Client *source_p, const char *mask, int cap,
+sendto_match_servs(struct Client *source_p, const char *mask, unsigned int cap,
                    const char *pattern, ...)
 {
   va_list args;
-  struct Client *target_p;
-  dlink_node *ptr;
+  dlink_node *ptr = NULL;
   char buffer[IRCD_BUFSIZE];
-  int found = 0;
 
   va_start(args, pattern);
   vsnprintf(buffer, sizeof(buffer), pattern, args);
@@ -811,7 +796,7 @@ sendto_match_servs(struct Client *source_p, const char *mask, int cap,
 
   DLINK_FOREACH(ptr, global_serv_list.head)
   {
-    target_p = ptr->data;
+    struct Client *target_p = ptr->data;
 
     /* Do not attempt to send to ourselves, or the source */
     if (IsMe(target_p) || target_p->from == source_p->from)
@@ -820,14 +805,13 @@ sendto_match_servs(struct Client *source_p, const char *mask, int cap,
     if (target_p->from->localClient->serial == current_serial)
       continue;
 
-    if (match(mask, target_p->name))
+    if (!match(mask, target_p->name))
     {
       /*
        * if we set the serial here, then we'll never do a
        * match() again, if !IsCapable()
        */
       target_p->from->localClient->serial = current_serial;
-      found++;
 
       if (!IsCapable(target_p->from, cap))
         continue;
@@ -861,23 +845,19 @@ sendto_anywhere(struct Client *to, struct Client *from,
   if (MyClient(to))
   {
     if (IsServer(from))
-    {
-      if (IsCapable(to, CAP_TS6) && HasID(from))
-        len = ircsprintf(buffer, ":%s ", from->id);
-      else
-        len = ircsprintf(buffer, ":%s ", from->name);
-    }
+      len = snprintf(buffer, sizeof(buffer), ":%s ", from->name);
     else
-      len = ircsprintf(buffer, ":%s!%s@%s ",
-                       from->name, from->username, from->host);
+      len = snprintf(buffer, sizeof(buffer), ":%s!%s@%s ",
+                     from->name, from->username, from->host);
   }
-  else len = ircsprintf(buffer, ":%s ", ID_or_name(from, send_to));
+  else
+    len = snprintf(buffer, sizeof(buffer), ":%s ", ID_or_name(from, send_to));
 
   va_start(args, pattern);
   len += send_format(&buffer[len], IRCD_BUFSIZE - len, pattern, args);
   va_end(args);
 
-  if(MyClient(to))
+  if (MyClient(to))
     send_message(send_to, buffer, len);
   else
     send_message_remote(send_to, from, buffer, len);
@@ -892,60 +872,48 @@ sendto_anywhere(struct Client *to, struct Client *from,
  * side effects	- Send to *local* ops only but NOT +s nonopers.
  */
 void
-sendto_realops_flags(unsigned int flags, int level, const char *pattern, ...)
+sendto_realops_flags(unsigned int flags, int level, int type, const char *pattern, ...)
 {
+  const char *ntype = NULL;
   dlink_node *ptr = NULL;
   char nbuf[IRCD_BUFSIZE];
   va_list args;
 
   va_start(args, pattern);
-  vsnprintf(nbuf, IRCD_BUFSIZE, pattern, args);
+  vsnprintf(nbuf, sizeof(nbuf), pattern, args);
   va_end(args);
 
+  switch (type)
+  {
+    case SEND_NOTICE:
+      ntype = "Notice";
+      break;
+    case SEND_GLOBAL:
+      ntype = "Global";
+      break;
+    case SEND_LOCOPS:
+      ntype = "LocOps";
+      break;
+    default:
+      assert(0);
+  }
+
   DLINK_FOREACH(ptr, oper_list.head)
   {
     struct Client *client_p = ptr->data;
     assert(HasUMode(client_p, UMODE_OPER));
 
-    /* If we're sending it to opers and theyre an admin, skip.
-     * If we're sending it to admins, and theyre not, skip.
-     */
-    if (((level == L_ADMIN) && !HasUMode(client_p, UMODE_ADMIN)) ||
-	((level == L_OPER) && HasUMode(client_p, UMODE_ADMIN)))
-      continue;
-
-    if (HasUMode(client_p, flags))
-      sendto_one(client_p, ":%s NOTICE %s :*** Notice -- %s",
-                 me.name, client_p->name, nbuf);
-  }
-}
-
-void
-sendto_globops_flags(unsigned int flags, int level, const char *pattern, ...)
-{
-  dlink_node *ptr = NULL;
-  char nbuf[IRCD_BUFSIZE];
-  va_list args;
-
-  va_start(args, pattern);
-  vsnprintf(nbuf, IRCD_BUFSIZE, pattern, args);
-  va_end(args);
-
-  DLINK_FOREACH(ptr, oper_list.head)
-  {
-    struct Client *client_p = ptr->data;
-    assert(client_p->umodes & UMODE_OPER);
-
-    /* If we're sending it to opers and theyre an admin, skip.
-     * If we're sending it to admins, and theyre not, skip.
+    /*
+     * If we're sending it to opers and they're an admin, skip.
+     * If we're sending it to admins, and they're not, skip.
      */
     if (((level == L_ADMIN) && !HasUMode(client_p, UMODE_ADMIN)) ||
         ((level == L_OPER) && HasUMode(client_p, UMODE_ADMIN)))
       continue;
 
     if (HasUMode(client_p, flags))
-      sendto_one(client_p, ":%s NOTICE %s :*** Global -- %s",
-                 me.name, client_p->name, nbuf);
+      sendto_one(client_p, ":%s NOTICE %s :*** %s -- %s",
+                 me.name, client_p->name, ntype, nbuf);
   }
 }
 
@@ -967,10 +935,12 @@ sendto_wallops_flags(unsigned int flags, struct Client *source_p,
   int len;
 
   if (IsClient(source_p))
-    len = ircsprintf(buffer, ":%s!%s@%s WALLOPS :",
-                     source_p->name, source_p->username, source_p->host);
+    len = snprintf(buffer, sizeof(buffer), ":%s!%s@%s WALLOPS :",
+                   source_p->name, source_p->username,
+                   source_p->host);
   else
-    len = ircsprintf(buffer, ":%s WALLOPS :", source_p->name);
+    len = snprintf(buffer, sizeof(buffer), ":%s WALLOPS :",
+                   source_p->name);
 
   va_start(args, pattern);
   len += send_format(&buffer[len], IRCD_BUFSIZE - len, pattern, args);
@@ -1023,7 +993,7 @@ ts_warn(const char *pattern, ...)
   vsnprintf(buffer, sizeof(buffer), pattern, args);
   va_end(args);
 
-  sendto_realops_flags(UMODE_ALL, L_ALL, "%s", buffer);
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE, "%s", buffer);
   ilog(LOG_TYPE_IRCD, "%s", buffer);
 }
 
@@ -1048,8 +1018,9 @@ kill_client(struct Client *client_p, struct Client *diedie,
   if (IsDead(client_p))
     return;
 
-  len = ircsprintf(buffer, ":%s KILL %s :", ID_or_name(&me, client_p->from),
-                   ID_or_name(diedie, client_p));
+  len = snprintf(buffer, sizeof(buffer), ":%s KILL %s :",
+                 ID_or_name(&me, client_p->from),
+                 ID_or_name(diedie, client_p));
 
   va_start(args, pattern);
   len += send_format(&buffer[len], IRCD_BUFSIZE - len, pattern, args);
@@ -1069,8 +1040,8 @@ kill_client(struct Client *client_p, struct Client *diedie,
  *		  client being unknown to leaf, as in lazylink...
  */
 void
-kill_client_ll_serv_butone(struct Client *one, struct Client *source_p,
-                           const char *pattern, ...)
+kill_client_serv_butone(struct Client *one, struct Client *source_p,
+                        const char *pattern, ...)
 {
   va_list args;
   int have_uid = 0;
@@ -1082,16 +1053,16 @@ kill_client_ll_serv_butone(struct Client *one, struct Client *source_p,
   {
     have_uid = 1;
     va_start(args, pattern);
-    len_uid = ircsprintf(buf_uid, ":%s KILL %s :", me.id, ID(source_p));
-    len_uid += send_format(&buf_uid[len_uid], IRCD_BUFSIZE - len_uid, pattern,
-                           args);
+    len_uid = snprintf(buf_uid, sizeof(buf_uid), ":%s KILL %s :",
+                       me.id, ID(source_p));
+    len_uid += send_format(&buf_uid[len_uid], IRCD_BUFSIZE - len_uid, pattern, args);
     va_end(args);
   }
 
   va_start(args, pattern);
-  len_nick = ircsprintf(buf_nick, ":%s KILL %s :", me.name, source_p->name);
-  len_nick += send_format(&buf_nick[len_nick], IRCD_BUFSIZE - len_nick, pattern,
-                          args);
+  len_nick = snprintf(buf_nick, sizeof(buf_nick), ":%s KILL %s :",
+                      me.name, source_p->name);
+  len_nick += send_format(&buf_nick[len_nick], IRCD_BUFSIZE - len_nick, pattern, args);
   va_end(args);
 
   DLINK_FOREACH(ptr, serv_list.head)
@@ -1108,4 +1079,4 @@ kill_client_ll_serv_butone(struct Client *one, struct Client *source_p,
     else
       send_message(client_p, buf_nick, len_nick);
   }
-} 
+}
