@@ -18,7 +18,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: version.c 1748 2013-01-15 18:16:36Z michael $
+ *  $Id: version.c 2731 2014-01-01 14:13:47Z michael $
  */
 
 #include "patchlevel.h"
@@ -30,10 +30,10 @@ const char *serno = SERIALNUM;
 
 const char *infotext[] =
 {
-  "IRC --",
+  "ircd-hybrid --",
   "Based on the original code written by Jarkko Oikarinen",
   "Copyright 1988, 1989, 1990, 1991 University of Oulu, Computing Center",
-  "Copyright (c) 1997-2013 Hybrid Development Team",
+  "Copyright (c) 1997-2014 Hybrid Development Team",
   "",
   "This program is free software; you can redistribute it and/or",
   "modify it under the terms of the GNU General Public License as",
@@ -57,7 +57,7 @@ const char *infotext[] =
   "cryogen, Stuart Walsh <stu@ipng.org.uk>",
   "Dianora, Diane Bruce <db@db.net>",
   "metalrock, Jack Low <jclow@csupomona.edu>",
-  "Michael, Michael Wobst <michael@wobst.at>",
+  "Michael, Michael Wobst <michael@ircd-hybrid.de>",
   "Rodder, Jon Lusky <lusky@blown.net>",
   "Wohali, Joan Touzet <joant@ieee.org>",
   "",
@@ -71,6 +71,7 @@ const char *infotext[] =
   "bysin, Ben Kittridge <bkittridge@cfl.rr.com>",
   "cosine, Patrick Alken <wnder@uwns.underworld.net>",
   "David-T, David Taylor <davidt@yadt.co.uk>",
+  "Dom, Dominic Hargreaves <dom@earth.li>",
   "fgeek, Henri Salo <henri@nerv.fi>",
   "fl, Lee Hardy <lee@leeh.co.uk>",
   "Garion, Joost Vunderink <garion@efnet.nl>",
@@ -84,7 +85,6 @@ const char *infotext[] =
   "knight, Alan LeVee <alan.levee@prometheus-designs.net>",
   "kre, Dinko Korunic <kreator@fly.srk.fer.hr>",
   "madmax, Paul Lomax <madmax@efnet.org>",
-  "nenolod, William Pitcock <nenolod@nenolod.net>",
   "Riedel, Dennis Vink, <riedel@chaotic.nl>",
   "scuzzy, David Todd <scuzzy@aniverse.net>",
   "spookey, David Colburn <spookey@spookey.org>",
