@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: whowas.c 1362 2012-04-22 19:05:08Z michael $
+ *  $Id: whowas.c 2747 2014-01-05 19:19:06Z michael $
  */
 
 #include "stdinc.h"
@@ -36,7 +36,16 @@ dlink_list WHOWASHASH[HASHSIZE];
 
 
 void
-add_history(struct Client *client_p, int online)
+whowas_init(void)
+{
+  unsigned int idx;
+
+  for (idx = 0; idx < NICKNAMEHISTORYLENGTH; ++idx)
+    WHOWAS[idx].hashv = -1;
+}
+
+void
+whowas_add_history(struct Client *client_p, const int online)
 {
   static unsigned int whowas_next = 0;
   struct Whowas *who = &WHOWAS[whowas_next];
@@ -55,6 +64,7 @@ add_history(struct Client *client_p, int online)
   }
 
   who->hashv = strhash(client_p->name);
+  who->shide = IsHidden(client_p->servptr) != 0;
   who->logoff = CurrentTime;
 
   strlcpy(who->name, client_p->name, sizeof(who->name));
@@ -75,7 +85,7 @@ add_history(struct Client *client_p, int online)
 }
 
 void
-off_history(struct Client *client_p)
+whowas_off_history(struct Client *client_p)
 {
   dlink_node *ptr = NULL, *ptr_next = NULL;
 
@@ -89,7 +99,7 @@ off_history(struct Client *client_p)
 }
 
 struct Client *
-get_history(const char *nick, time_t timelimit)
+whowas_get_history(const char *nick, time_t timelimit)
 {
   dlink_node *ptr = NULL;
 
@@ -110,33 +120,17 @@ get_history(const char *nick, time_t timelimit)
 }
 
 void
-count_whowas_memory(unsigned int *wwu, uint64_t *wwum)
+whowas_count_memory(unsigned int *const count, uint64_t *const bytes)
 {
-  const struct Whowas *tmp;
-  int i;
-  unsigned int u = 0;
-  uint64_t um = 0;
+  const struct Whowas *tmp = &WHOWAS[0];
+  unsigned int i = 0;
 
-  /* count the number of used whowas structs in 'u'   */
-  /* count up the memory used of whowas structs in um */
-  for (i = 0, tmp = &WHOWAS[0]; i < NICKNAMEHISTORYLENGTH; ++i, ++tmp)
+  for (; i < NICKNAMEHISTORYLENGTH; ++i, ++tmp)
   {
     if (tmp->hashv != -1)
     {
-      ++u;
-      um += sizeof(struct Whowas);
+      (*count)++;
+      (*bytes) += sizeof(struct Whowas);
     }
   }
-
-  *wwu = u;
-  *wwum = um;
-}
-
-void
-whowas_init(void)
-{
-  unsigned int idx;
-
-  for (idx = 0; idx < NICKNAMEHISTORYLENGTH; ++idx)
-    WHOWAS[idx].hashv = -1;
 }
